/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <nex_P2p.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "Avatar.h"
#include "World.h"

#include "../common/DemoFramework.h"
#include "../common/DemoColor.h"

#include "SceneSessionGame.h"
#include "ControlInfo.h"


namespace
{
}

static DemoColor sColorList[AVATAR_PLAYER_MAX] =
{
    DEMO_COLOR_RED,
    DEMO_COLOR_GREEN,
    DEMO_COLOR_GRAY,
    DEMO_COLOR_WHITE,
    //  DEMO_COLOR_BLACK,
    DEMO_COLOR_LIME,

    DEMO_COLOR_RED,
    DEMO_COLOR_GREEN,
    DEMO_COLOR_GRAY,
    DEMO_COLOR_WHITE,
    //  DEMO_COLOR_BLACK,
    DEMO_COLOR_LIME,

    DEMO_COLOR_RED,
    DEMO_COLOR_GREEN
};

// Constructor
Avatar::Avatar()
{
    m_vSpeed.Set(QREAL(0.0), QREAL(0.0));

    m_dsInfo.playerId = 0;
}

Avatar::~Avatar()
{
}

void Avatar::HandleError()
{
    nn::nex::qChar szError[256] = _T("");
    nn::nex::SystemError::GetErrorString(nn::nex::SystemError::GetLast(), szError, 256);
    _UTIL_TRACE(
        SAMPLE_TRACE_ERROR,
        (
            _T("ERROR: ") SZ_FMT _T("!"), szError
        )
    );
}

void Avatar::InitDO()
{
    if(!IsADuplicationMaster())
    {
        return;
    }

    m_dsInfo.playerId = nn::nex::Station::GetLocalInstance()->GetHandle().GetDOID() % AVATAR_PLAYER_MAX;

    _UTIL_TRACE(
        SAMPLE_TRACE_AVATAR,
        (
            _T("Avatar::InitDO:%x, %d"),
            nn::nex::Station::GetLocalInstance()->GetStationID(),
            m_dsInfo.playerId
        )
    );
}

void Avatar::DoPhysic()
{
    UTIL_TRACE(SAMPLE_TRACE_UPDATE, _T("Avatar::DoPhysic:start"));

    if(IsADuplicationMaster())
    {
        ControlInfo* controlInfo = ControlInfo::GetInstance();
        m_vSpeed = controlInfo->m_vMove;

        (void)MovePosition(m_vSpeed.m_x, m_vSpeed.m_y);

        qBool result;

        //n2788:memo: Update only positions of dead-reckoning being used
        World::Ref refWorld(nn::nex::g_hWorld);
        switch(refWorld->m_dsInfo.extrapolationType)
        {
        case WorldInfo::ExtrapolationTypeNone:
            result = Update(m_dsPosition);
            break;
        case WorldInfo::ExtrapolationTypeExtrapolated:
            result = Update(m_dsExtrapolatedPosition);
            break;
        case WorldInfo::ExtrapolationTypeLoopback:
            result = Update(m_dsExtrapolatedLoopbackPosition);
            break;
        default:
            _UTIL_TRACE(
                SAMPLE_TRACE_ERROR,
                (
                    _T("AvatarDo::DoPhysic:unexpected:%d"),
                    refWorld->m_dsInfo.extrapolationType
                )
            );
            result = false;
            break;
        }

        if (!result)
        {
            HandleError();
        }
    }
    else
    {
        Refresh();
    }

    UTIL_TRACE(SAMPLE_TRACE_UPDATE, _T("Avatar::DoPhysic:end"));
}

void Avatar::Render()
{
    nn::nex::ScopedCS oCS(nn::nex::Scheduler::GetInstance()->SystemLock());

    qReal x, y;
    (void)GetExtrapolationPosition(&x, &y);

    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::Render:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(x),
            nn::nex::FormatCast(y)
        )
    );

    DemoFramework::GetInstance()->DrawRect((int)x, (int)y, AVATAR_SIZE, AVATAR_SIZE, sColorList[m_dsInfo.playerId]);
}

qBool Avatar::MovePosition(qReal x, qReal y)
{
    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::MovePosition:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(x),
            nn::nex::FormatCast(y)
        )
    );

    m_dsPosition.Move(x, y, 0);
    m_dsExtrapolatedPosition.Set(m_dsPosition.x, m_dsPosition.y, 0);
    m_dsExtrapolatedLoopbackPosition.Set(m_dsPosition.x, m_dsPosition.y, 0);

    return true;
}

qBool Avatar::SetPosition(qReal x, qReal y)
{
    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::SetPosition:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(x),
            nn::nex::FormatCast(y)
        )
    );

    m_dsPosition.Set(x, y, 0);
    m_dsExtrapolatedPosition.Set(m_dsPosition.x, m_dsPosition.y, 0);
    m_dsExtrapolatedLoopbackPosition.Set(m_dsPosition.x, m_dsPosition.y, 0);

    return true;
}

qBool Avatar::GetExtrapolationPosition(qReal* x, qReal* y)
{
    qReal returnX, returnY;

    World::Ref refWorld(nn::nex::g_hWorld);
    switch(refWorld->m_dsInfo.extrapolationType)
    {
    case WorldInfo::ExtrapolationTypeNone:
        returnX = m_dsPosition.x;
        returnY = m_dsPosition.y;
        break;
    case WorldInfo::ExtrapolationTypeExtrapolated:
        returnX = m_dsExtrapolatedPosition.x;
        returnY = m_dsExtrapolatedPosition.y;
        break;
    case WorldInfo::ExtrapolationTypeLoopback:
        returnX = m_dsExtrapolatedLoopbackPosition.x;
        returnY = m_dsExtrapolatedLoopbackPosition.y;
        break;
    default:
        _UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            (
                _T("Avatar::GetExtrapolationPosition:unexpected:%d"),
                refWorld->m_dsInfo.extrapolationType
            )
        );
        return false;
    }

    if(x != NULL)
    {
        *x = returnX;
    }
    if(y != NULL)
    {
        *y = returnY;
    }

    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::GetExtrapolationPosition:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(returnX),
            nn::nex::FormatCast(returnY)
        )
    );

    return true;
}
