/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/



#include "SceneSelectNetZSession.h"

#include <NetZ/src/Core/DOCore/Session.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "../common/DemoFramework.h"

#include "../common/NetZManager.h"
#include "../common/Config.h"
#include "../common/HeapManager.h"

#include "SceneSessionCreating.h"
#include "SceneSessionJoining.h"
#include "CommonData.h"
#include "GlobalStatus.h"
#include "SampleCommon.h"
#include "../common/Config.h"

namespace
{

}  // End of anonymous namespace


SceneSelectNetZSession::SceneSelectNetZSession(void)
{
    mSceneId = SCENE_ID_SELECT_NETZ_SESSION;
}


SceneSelectNetZSession::~SceneSelectNetZSession(void)
{
}


SceneBase* SceneSelectNetZSession::GetInstance(void)
{
    static SceneSelectNetZSession s_instance;
    return &s_instance;
}


void SceneSelectNetZSession::Initialize(void)
{
    SceneBase::Initialize();

    SampleConfig cfg;
    cfg.threadMode = THREAD_MODE;
    cfg.useUserDO = true;

    // Stream related?
    cfg.maxSilenceTime   = SAMPLE_MAX_SILENCE_TIME;
    cfg.keepAliveTimeout = SAMPLE_KEEP_ALIVE_TIMEOUT;
    cfg.maxWindowMapSize = SAMPLE_MAX_WINDOW_MAP_SIZE;

    cfg.networkTopology  = NETWORK_TOPOLOGY;

    // Message bundle related
    cfg.enableMessageBundle           = g_gsStatus.m_bEnableMessageBundle;
    cfg.messageBundleFlushDelay       = g_gsStatus.m_uiMessageBundleFlushDelay;
    cfg.messageBundleFlushCheckPeriod = g_gsStatus.m_uiMessageBundleFlushCheckPeriod;

    // Packet bundle related
    cfg.enablePacketBundle            = g_gsStatus.m_bEnablePacketBundle;
    cfg.packetBundleFlushDelay        = g_gsStatus.m_uiPacketBundleFlushDelay;

    bool ret = NetZManager::GetInstance()->CreateNetZ(cfg);

    if(!ret)
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("ERROR: NetZManager::CreateNetZ() failed.")
        );
    }

    m_context.Reset();
}


void SceneSelectNetZSession::Finalize(void)
{
}

// Executes a session search.
void SceneSelectNetZSession::updateQuery(void)
{
    // List up session

    //    const qUnsignedInt32 TIMEOUT = 500;  // I am not sure what is the appropriate value to set.
    const qUnsignedInt32 TIMEOUT = 2000;  // I am not sure what is the appropriate value to set.
    switch(m_context.GetState())
    {
    case nn::nex::CallContext::Available:
        m_workingSessionList.clear();
        nn::nex::Session::QuerySessions(&m_context, &m_workingSessionList, false, TIMEOUT);
        break;
    case nn::nex::CallContext::CallInProgress:
        // Do nothing.
        break;
    case nn::nex::CallContext::CallSuccess:
        m_sessionList = m_workingSessionList;
        m_context.Reset();
        break;
    case nn::nex::CallContext::CallFailure:
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("#CallFailure.")
        );
        m_context.Reset();
        break;
    case nn::nex::CallContext::CallCancelled:
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("#CallCancelled.")
        );
        break;
    default:
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("ERROR: Unknown CallContext state.")
        );
        break;
    }
}

SceneBase* SceneSelectNetZSession::Update(void)
{
    (void)SceneBase::Update();

    updateQuery();

    if(DemoFramework::GetInstance()->IsTrigger(DEMO_BUTTON_A))
    {
        UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSelectNetZSession::Update:BUTTON_A"));

        // Create session
        m_context.Cancel();
        return SceneSessionCreating::GetInstance();
    }
    else if(DemoFramework::GetInstance()->IsTrigger(DEMO_BUTTON_B))
    {
        UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSelectNetZSession::Update:BUTTON_B"));

        if(!m_sessionList.empty())
        {
            // Set session to join
            m_context.Cancel();
            CommonData::GetInstance()->SetSessionToJoin(*m_sessionList.begin());
            return SceneSessionJoining::GetInstance();
        }
        else
        {
            UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSelectNetZSession::Update:no session found"));
            return this;
        }
    }

    return this;
}


void SceneSelectNetZSession::Render(void) const
{
    RenderInfo("<SceneSelectNetZSession>");

    u16 y = DRAW_POSITION_OPERATION_Y;

    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "A: create session");
    y += DRAW_FONT_HEIGHT;
    if(m_sessionList.empty())
    {
        DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "Session is not found.");
        y += DRAW_FONT_HEIGHT;
    }
    else
    {
        DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "B: join session");
        y += DRAW_FONT_HEIGHT;

        for(nn::nex::qList<nn::nex::SessionDescription>::const_iterator it = m_sessionList.begin(); it!=m_sessionList.end(); ++it)
        {
            DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "Session ID: %x", it->GetSessionID());
            y += DRAW_FONT_HEIGHT;
        }
    }
}
