/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#include <nex_P2p.h>

#include "Avatar.h"
#include "World.h"

#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "../common/DemoFramework.h"
#include "../common/StatisticValue.h"

#include "../common/NetZManager.h"
#include "../common/ErrorHandling.h"

#include "SceneExitNetZSession.h"
#include "SceneSessionGame.h"
#include "GlobalStatus.h"


SceneSessionGame::SceneSessionGame(void)
{
}

SceneSessionGame::~SceneSessionGame(void)
{
}

void SceneSessionGame::Initialize(void)
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionGame::Initialize"));
    SceneBase::Initialize();

    UTIL_TRACE(
        SAMPLE_TRACE_AVATAR,
        _T("Create avatar")
    );

    Avatar* pNewAvatar = Avatar::Create();
    if (pNewAvatar == NULL)
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("SceneSessionGame::Initialize:ERROR: Could not create Avatar.")
        );
        nn::nex::ConsoleIO::Error();
        return;
    }

    UTIL_TRACE(
        SAMPLE_TRACE_AVATAR,
        _T("Publish avatar")
    );

    if (!pNewAvatar->Publish())
    {
        UTIL_TRACE(
            SAMPLE_TRACE_ERROR,
            _T("Sample:ERROR: Could not publish Avatar.")
        );
        nn::nex::ConsoleIO::Error();
        return;
    }

    g_gsStatus.m_uiJoinCount ++;
}

void SceneSessionGame::Finalize()
{
    UTIL_TRACE(SAMPLE_TRACE_SCENE, _T("SceneSessionGame::Finalize"));
    SceneBase::Finalize();

    Avatar::SelectionIterator itAvatarDM(nn::nex::DUPLICATION_MASTER);
    while (!itAvatarDM.EndReached())
    {
        itAvatarDM->DeleteMainRef();
        itAvatarDM++;
    }
}

SceneBase* SceneSessionGame::Update(void)
{
    //  (void)SceneBase::Update();

    //n1589
    if(SceneBase::CheckNexError())
    {
        return SceneExitNetZSession::GetInstance();
    }

    ControlInfo* controlInfo = ControlInfo::GetInstance();
    controlInfo->m_vMove.Set(QREAL(0.0), QREAL(0.0));

    //n1589
    if(
        ! DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_L) &&
        ! DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_R)
    )
    {
        f32 analogX;
        f32 analogY;
        DemoFramework::GetInstance()->GetAnalog(&analogX, &analogY);
        if(analogX != 0.0f || analogY != 0.0f)
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:AnalogInput"));
            controlInfo->m_vMove.SetX( CONTROL_INFO_MOVE_SPEED * analogX );
            controlInfo->m_vMove.SetY( -CONTROL_INFO_MOVE_SPEED * analogY );
        }
        else
        {
            if(DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_LEFT))
            {
                UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_LEFT"));
                controlInfo->m_vMove.SetX( -CONTROL_INFO_MOVE_SPEED );
            }

            if(DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_RIGHT))
            {
                UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_RIGHT"));
                controlInfo->m_vMove.SetX( CONTROL_INFO_MOVE_SPEED );
            }

            if(DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_UP))
            {
                UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_UP"));
                controlInfo->m_vMove.SetY( -CONTROL_INFO_MOVE_SPEED );
            }

            if(DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_DOWN))
            {
                UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_DOWN"));
                controlInfo->m_vMove.SetY( CONTROL_INFO_MOVE_SPEED );
            }
        }

        if(DemoFramework::GetInstance()->IsTrigger(DEMO_BUTTON_Y) && nn::nex::Session::GetRole() == nn::nex::SESSION_ROLE_MASTER)
        {
            // The dead reckoning method can be changed for the master station only.
            World::Ref refWorld(nn::nex::g_hWorld);
            refWorld->m_dsInfo.extrapolationType = (refWorld->m_dsInfo.extrapolationType + 1) % WorldInfo::ExtrapolationTypeLast;
            UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("refWorld->Update:") << refWorld->m_dsInfo.extrapolationType);
            refWorld->Update(refWorld->m_dsInfo.extrapolationType);
        }

        if(DemoFramework::GetInstance()->IsTrigger(DEMO_BUTTON_B))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_B"));
            return SceneExitNetZSession::GetInstance();
        }
    }
    else if(
        DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_L) &&
        ! DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_R)
    )
    {
    }
    else if(
        !DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_L) &&
        DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_R)
    )
    {

    }
    else if(
        DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_L) &&
        DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_R)
    )
    {
        if(DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_UP))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_L & BUTTON_R & BUTTON_UP"));
            nn::nex::SocketTransport* transport = (nn::nex::SocketTransport*)nn::nex::Network::GetInstance()->GetRootTransport();
            transport->SetTransportThreadPriority(NN_NEX_PRIORITY_HIGH);
        }

        if(DemoFramework::GetInstance()->IsHold(DEMO_BUTTON_DOWN))
        {
            UTIL_TRACE(SAMPLE_TRACE_BUTTON, _T("SceneSessionGame::Update:BUTTON_L & BUTTON_R & BUTTON_DOWN"));
            nn::nex::SocketTransport* transport = (nn::nex::SocketTransport*)nn::nex::Network::GetInstance()->GetRootTransport();
            transport->SetTransportThreadPriority(NN_NEX_PRIORITY_LOW);
        }
    }

    Avatar::SelectionIterator itAvatarALL;
    itAvatarALL.GotoStart();
    while(!itAvatarALL.EndReached())
    {
        itAvatarALL->DoPhysic();
        ++itAvatarALL;
    }

    //n2788: Dispatch after reflecting controller input
    (void)SceneBase::Update();

    return this;
}

void SceneSessionGame::Render(void) const
{
    //    RenderInfo("<SceneSessionGame>");

    World::Ref refWorld(nn::nex::g_hWorld);
    refWorld->Render();

    Avatar::SelectionIterator itAvatarALL;
    itAvatarALL.GotoStart();
    while(!itAvatarALL.EndReached())
    {
        itAvatarALL->Render();
        ++itAvatarALL;
    }

    u16 y = DRAW_POSITION_OPERATION_Y;
    DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "B: exit session");
    y += DRAW_FONT_HEIGHT;

    if(nn::nex::Session::GetRole() == nn::nex::SESSION_ROLE_MASTER)
    {
        DemoFramework::GetInstance()->DrawText(DRAW_POSITION_OPERATION_X, y, "Y: change extrapolation type");
        y += DRAW_FONT_HEIGHT;
    }
}
