/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include <NetZ/src/Core/DOCore/Station.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>

#include "../common/DemoFramework.h"
#include "../common/DemoColor.h"
#include "../common/NexUtil.h"

#include "Avatar.h"
#include "SceneSessionGame.h"


static DemoColor sColorList[AVATAR_PLAYER_MAX] =
{
    DEMO_COLOR_WHITE,
    //  DEMO_COLOR_BLACK,
    DEMO_COLOR_GREEN,
    DEMO_COLOR_RED,
    DEMO_COLOR_LIME,
    //  DEMO_COLOR_BLUE,
    DEMO_COLOR_GREEN,

    //  DEMO_COLOR_DARKBLUE,
    DEMO_COLOR_GREEN,
    DEMO_COLOR_GRAY,
    DEMO_COLOR_GREEN,
    DEMO_COLOR_GREEN,
    DEMO_COLOR_GREEN,

    DEMO_COLOR_GREEN,
    DEMO_COLOR_GREEN
};

void Avatar::Trace(qUnsignedInt64 flag, CommunicationAnalyzeInfo* pAnalyzeInfo)
{
    UTIL_TRACE(
        flag,
        _T("CommunicationAnalyzeInfo:LocalSendTime:") << nn::nex::dec << pAnalyzeInfo->uiLocalSendTime
    );

    for(u32 index = 0; index < AVATAR_PEER_MAX; index++)
    {
        UTIL_TRACE(
            flag,
            _T("CommunicationAnalyzeInfo:[") << index << _T("]:") << nn::nex::hex << pAnalyzeInfo->sendTime[index].uiStationId
            << _T(" ,") << nn::nex::dec << pAnalyzeInfo->sendTime[index].uiPeerSendTime
        );
    }

}

Avatar* Avatar::Create()
{
    return new Avatar();
}

void Avatar::Delete(Avatar* avatar)
{
    delete avatar;
}

// Constructor
Avatar::Avatar()
{
    m_vSpeed.Set(QREAL(0.0), QREAL(0.0));
    m_hStation = INVALID_DOHANDLE;
    m_bAlive = false;

    m_uiUnreliableState = 0;
    m_pPosition.Set(QREAL(0.0), QREAL(0.0), QREAL(0.0));
    m_uiReliableState = 0;

    memset(&m_pUnreliableSendBuffer, 0, sizeof(m_pUnreliableSendBuffer));
    memset(&m_pUnreliableReceiveBuffer, 0, sizeof(m_pUnreliableReceiveBuffer));
    memset(&m_pReliableSendBuffer, 0, sizeof(m_pReliableSendBuffer));
    memset(&m_pReliableReceiveBuffer, 0, sizeof(m_pReliableReceiveBuffer));

    memset(&m_CommunicationAnalyzeInfo, 0, sizeof(m_CommunicationAnalyzeInfo));
    m_uiSenderSendTime = 0;

    m_uiTestUnreliableSize = TEST_UNRELIABLE_DATA_SIZE;
}

Avatar::~Avatar()
{
    m_hStation = INVALID_DOHANDLE;
    m_bAlive = false;
}

void Avatar::HandleError(nn::nex::qResult result)
{
    _UTIL_TRACE(
        SAMPLE_TRACE_ERROR,
        (
            _T("ERROR: ") SZ_FMT _T("!"), result.GetErrorName()
        )
    );

    nn::nex::SystemError::ClearLast();
}

void Avatar::Initialize(nn::nex::DOHandle hStation)
{
    m_hStation = hStation;
    m_bAlive = true;

    UTIL_TRACE(SAMPLE_TRACE_AVATAR, _T("Avatar::Initialize:") << nn::nex::hex << hStation);

    m_iSendDelayStartTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
    m_iReceiveDelayStartTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
}

void Avatar::Finalize()
{
    UTIL_TRACE(SAMPLE_TRACE_AVATAR, _T("Avatar::Finalize:"));

    m_hStation = INVALID_DOHANDLE;
    m_bAlive = false;
}

void Avatar::Control(ControlInfo* controlInfo)
{
    m_vSpeed = controlInfo->m_vMove;

    m_iSendDelayStartTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
}

void Avatar::DoPhysic()
{
    UTIL_TRACE(SAMPLE_TRACE_UPDATE, _T("Avatar::DoPhysic:start"));

    if(IsLocal())
    {
        (void)MovePosition(m_vSpeed.m_x, m_vSpeed.m_y);

        nn::nex::qResult result;
        result = Update();

        if (!result)
        {
            HandleError(result);
        }
    }
    else
    {
        Refresh();
    }

    UTIL_TRACE(SAMPLE_TRACE_UPDATE, _T("Avatar::DoPhysic:end"));
}

const CommunicationAnalyzeInfo* Avatar::ConvertHtoLECommunicationAnalyzeInfo(CommunicationAnalyzeInfo* pLECommunicationAnalyzeInfo, const CommunicationAnalyzeInfo* pHCommunicationAnalyzeInfo) const
{
    // Converts `pHCommunicationAnalyzeInfo` into data in NEX network byte order (little endian).
    // If conversion is not necessary, `pHCommunicationAnalyzeInfo` is returned as is.
    // If conversion is needed, data is stored in `pLECommunicationAnalyzeInfo` and returned.
#if NN_NEX_BYTE_ORDER == NN_NEX_BIG_ENDIAN
    MemcpyHtoLE32(&pLECommunicationAnalyzeInfo->uiLocalSendTime, &pHCommunicationAnalyzeInfo->uiLocalSendTime);
    for(int i = 0; i < sizeof(pHCommunicationAnalyzeInfo->sendTime) / sizeof(*pHCommunicationAnalyzeInfo->sendTime); ++i)
    {
#if ANALYZE_INFO_STATION_ID_BYTE
        pLECommunicationAnalyzeInfo->sendTime[i].uiStationId = pHCommunicationAnalyzeInfo->sendTime[i].uiStationId;
#else
        MemcpyHtoLE32(&pLECommunicationAnalyzeInfo->sendTime[i].uiStationId, &pHCommunicationAnalyzeInfo->sendTime[i].uiStationId);
#endif
        MemcpyHtoLE32(&pLECommunicationAnalyzeInfo->sendTime[i].uiPeerSendTime, &pHCommunicationAnalyzeInfo->sendTime[i].uiPeerSendTime);
    }
    return pLECommunicationAnalyzeInfo;
#else
    (void)pLECommunicationAnalyzeInfo;
    return pHCommunicationAnalyzeInfo;
#endif
}

const CommunicationAnalyzeInfo* Avatar::ConvertLEtoHCommunicationAnalyzeInfo(CommunicationAnalyzeInfo* pHCommunicationAnalyzeInfo, const CommunicationAnalyzeInfo* pLECommunicationAnalyzeInfo) const
{
    return ConvertHtoLECommunicationAnalyzeInfo(pHCommunicationAnalyzeInfo, pLECommunicationAnalyzeInfo);
}


nn::nex::qResult Avatar::Update()
{
    nn::nex::DirectStream* directStream = nn::nex::DirectStream::GetInstance();

    SYSTEMCHECK(directStream != NULL);

    qUnsignedInt16 magicCode = UNRELIABLE_DATA_MAGIC_CODE;
    MemcpyHtoLE16(&m_pUnreliableSendBuffer[UNRELIABLE_DATA_MAGIC_OFFSET], &magicCode);

    m_uiUnreliableState++;
    MemcpyHtoLE32(&m_pUnreliableSendBuffer[UNRELIABLE_DATA_STATE_OFFSET], &m_uiUnreliableState);

    qUnsignedInt64 currentTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
    m_svTimeSendDelay.AddValue(currentTime - m_iSendDelayStartTime);
    m_CommunicationAnalyzeInfo.uiLocalSendTime = currentTime;

    CommunicationAnalyzeInfo leCommunicationAnalyzeInfo;
    const CommunicationAnalyzeInfo* pLECommunicationAnalyzeInfo = ConvertHtoLECommunicationAnalyzeInfo(&leCommunicationAnalyzeInfo, &m_CommunicationAnalyzeInfo);
    memcpy(&m_pUnreliableSendBuffer[UNRELIABLE_DATA_ANALYZE_INFO_OFFSET], pLECommunicationAnalyzeInfo, UNRELIABLE_DATA_ANALYZE_INFO_SIZE);

    NEX_DEMO_ASSERT(sizeof(qReal)==4);
    MemcpyHtoLE32(&m_pUnreliableSendBuffer[UNRELIABLE_DATA_POSITION_OFFSET_X], &m_pPosition.x);
    MemcpyHtoLE32(&m_pUnreliableSendBuffer[UNRELIABLE_DATA_POSITION_OFFSET_Y], &m_pPosition.y);

    nn::nex::qResult result;
#if ENABLE_SEND_UNRELIABLE_ALL
    result = directStream->SendAll(nn::nex::DirectStream::Unreliable, m_pUnreliableSendBuffer, UNRELIABLE_DATA_SIZE);
#else
    nn::nex::qResult eachResult;
    result = (qBool)true;
    for(nn::nex::Station::SelectionIterator station; !station.EndReached(); ++station)
    {
        if(!station->IsLocal())
        {
            eachResult = directStream->Send(station->GetStationID(), nn::nex::DirectStream::Unreliable, m_pUnreliableSendBuffer, UNRELIABLE_DATA_SIZE);
            if(eachResult.Equals(false))
            {
                result = eachResult;
            }
        }
    }
#endif //ENABLE_SEND_UNRELIABLE_ALL
    return result;
}

qBool Avatar::UpdateReliable()
{
    nn::nex::DirectStream* directStream = nn::nex::DirectStream::GetInstance();

    if(directStream == NULL)
    {
        return false;
    }

    qUnsignedInt16 magicCode = RELIABLE_DATA_MAGIC_CODE;
    MemcpyHtoLE16(&m_pReliableSendBuffer[RELIABLE_DATA_MAGIC_OFFSET], &magicCode);
    m_uiReliableState++;
    MemcpyHtoLE32(&m_pReliableSendBuffer[RELIABLE_DATA_STATE_OFFSET], &m_uiReliableState);

    qBool result;
    nn::nex::Station::SelectionIterator itStations;

    for (itStations.GotoStart(); !itStations.EndReached(); ++itStations)
    {
        if (!itStations->IsLocal() && itStations->IsConnected())
        {
            // Send to stations other than the local host
            nn::nex::StationID dstID = itStations->GetStationID();
            result = directStream->Send(dstID, nn::nex::DirectStream::Reliable, m_pReliableSendBuffer, RELIABLE_DATA_SIZE);
            if (!result)
            {
                UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("Send Reliable error. dstID:") << dstID);
            }
        }
    }
    return result;
}

void Avatar::Receive(const qByte* buffer, qUnsignedInt32 size)
{
    if(!m_bAlive)
    {
        UTIL_TRACE(SAMPLE_TRACE_ALWAYS, _T("Avatar::Receive:!m_bAlive"));
        return;
    }

    qUnsignedInt16 magicCode = 0;
    MemcpyLEtoH16(&magicCode, &buffer[0]);

    if(magicCode == UNRELIABLE_DATA_MAGIC_CODE)
    {
        NEX_DEMO_ASSERT(size == UNRELIABLE_DATA_SIZE);

        memcpy(m_pUnreliableReceiveBuffer, buffer, UNRELIABLE_DATA_SIZE);

        CommunicationAnalyzeInfo lePeerAnalyzeInfo;
        memcpy(&lePeerAnalyzeInfo, &m_pUnreliableReceiveBuffer[UNRELIABLE_DATA_ANALYZE_INFO_OFFSET], UNRELIABLE_DATA_ANALYZE_INFO_SIZE);
        CommunicationAnalyzeInfo hostPeerAnalyzeInfo;

        const CommunicationAnalyzeInfo* pPeerAnalyzeInfo = ConvertLEtoHCommunicationAnalyzeInfo(&hostPeerAnalyzeInfo, &lePeerAnalyzeInfo);

        m_uiSenderSendTime = pPeerAnalyzeInfo->uiLocalSendTime;

#if ANALYZE_INFO_STATION_ID_BYTE
        u8 stationId = (u8)nn::nex::Station::GetLocalStation();
#else
        u32 stationId = (u32)nn::nex::Station::GetLocalStation();
#endif
        qUnsignedInt64 currentTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
        m_iReceiveDelayStartTime = currentTime;
        for(u32 i = 0; i < AVATAR_PEER_MAX; i++)
        {
            if(pPeerAnalyzeInfo->sendTime[i].uiStationId == stationId)
            {
                m_svLatency.AddValue((qUnsignedInt64)nn::nex::SystemClock::GetTime() - pPeerAnalyzeInfo->sendTime[i].uiPeerSendTime);
                break;
            }
        }
    }

    if(magicCode == RELIABLE_DATA_MAGIC_CODE)
    {
        NEX_DEMO_ASSERT(size == RELIABLE_DATA_SIZE);
        memcpy(&m_pReliableReceiveBuffer, buffer, RELIABLE_DATA_SIZE);
    }

    if(magicCode == TEST_UNRELIABLE_DATA_MAGIC_CODE)
    {
        UTIL_TRACE(SAMPLE_TRACE_TRANSPORT, _T("Avatar::Receive:") << size);
    }

}

void Avatar::Refresh()
{
    MemcpyLEtoH32(&m_uiUnreliableState, &m_pUnreliableReceiveBuffer[UNRELIABLE_DATA_STATE_OFFSET]);

    qUnsignedInt64 currentTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
    m_svTimeReceiveDelay.AddValue(currentTime - m_iReceiveDelayStartTime);

    NEX_DEMO_ASSERT(sizeof(qReal) == 4);
    MemcpyLEtoH32(&(m_pPosition.x), &m_pUnreliableReceiveBuffer[UNRELIABLE_DATA_POSITION_OFFSET_X]);
    MemcpyLEtoH32(&(m_pPosition.y), &m_pUnreliableReceiveBuffer[UNRELIABLE_DATA_POSITION_OFFSET_Y]);

    MemcpyLEtoH32(&m_uiReliableState, &m_pReliableReceiveBuffer[RELIABLE_DATA_STATE_OFFSET]);
}

void Avatar::Render()
{
    qReal x, y;
    (void)GetExtrapolationPosition(&x, &y);

    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::Render:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(x),
            nn::nex::FormatCast(y)
        )
    );

    DemoFramework::GetInstance()->DrawRect((int)x, (int)y, AVATAR_DRAW_SIZE, AVATAR_DRAW_SIZE, sColorList[GetHandle() % AVATAR_PLAYER_MAX]);
}

qBool Avatar::MovePosition(qReal x, qReal y)
{
    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::MovePosition:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(x),
            nn::nex::FormatCast(y)
        )
    );

    m_pPosition.Move(x, y, 0);

    return true;
}

qBool Avatar::SetPosition(qReal x, qReal y)
{
    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::SetPosition:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(x),
            nn::nex::FormatCast(y)
        )
    );

    m_pPosition.Set(x, y, 0);

    return true;
}

qBool Avatar::GetExtrapolationPosition(qReal* x, qReal* y)
{
    qReal returnX, returnY;

    //n1589:return direct value
    returnX = m_pPosition.x;
    returnY = m_pPosition.y;

    if(x != NULL)
    {
        *x = returnX;
    }
    if(y != NULL)
    {
        *y = returnY;
    }

    _UTIL_TRACE(
        SAMPLE_TRACE_POSITION,
        (
            _T("Avatar::GetPosition:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(returnX),
            nn::nex::FormatCast(returnY)
        )
    );

    return true;
}

qBool Avatar::IsLocal()
{
    //n1589
#if 0
    nn::nex::Station::Ref rStation(m_hStation);
    if(!rStation.IsValid())
    {
        return false;
    }

    return rStation->IsLocal();
#else
    nn::nex::DOHandle hLocalStation = nn::nex::Station::GetLocalStation();
    return m_hStation == hLocalStation;
#endif
}

qBool Avatar::TestUnreliableSend(nn::nex::StationID id, qUnsignedInt32 size)
{
    qByte pTestUnreliableSendBuffer[TEST_UNRELIABLE_DATA_SIZE];

    nn::nex::DirectStream* directStream = nn::nex::DirectStream::GetInstance();

    if(directStream == NULL)
    {
        return false;
    }

    if(size == 0)
    {
        size = m_uiTestUnreliableSize;
        m_uiTestUnreliableSize -= 2;
        if(m_uiTestUnreliableSize < 1200)
        {
            m_uiTestUnreliableSize = TEST_UNRELIABLE_DATA_SIZE;
        }
    }

    qUnsignedInt16 magicCode = TEST_UNRELIABLE_DATA_MAGIC_CODE;
    MemcpyHtoLE16(&pTestUnreliableSendBuffer[TEST_UNRELIABLE_DATA_MAGIC_OFFSET], &magicCode);

    UTIL_TRACE(SAMPLE_TRACE_TRANSPORT, _T("Avatar::TestUnreliableSend:id:") << id << _T(" size:") << size);

    qBool result;
    if(id == 0)
    {
        result = directStream->SendAll(nn::nex::DirectStream::Unreliable, pTestUnreliableSendBuffer, size);
    }
    else
    {
        result = directStream->Send(id, nn::nex::DirectStream::Unreliable, pTestUnreliableSendBuffer, size);
    }
    return result;
}

void Avatar::Trace()
{
    _UTIL_TRACE(
        SAMPLE_TRACE_AVATAR,
        (
            _T("Avatar:%X, (%f,%f)"),
            (qUnsignedInt32)(GetHandle()),
            nn::nex::FormatCast(m_pPosition.x),
            nn::nex::FormatCast(m_pPosition.y)
        )
    );
}
