/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef SAMPLE_COMMON_H_
#define SAMPLE_COMMON_H_

#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <nex_P2p.h>

#include "../common/UtilTrace.h"

#include "../common/Defines.h"

#define SAMPLE_VERSION_STRING "Ver:0831"
#define SAMPLE_TITLE _T("DirectStream")


// ****** Macros, etc., where changes by the application developer are assumed *******************
#define FRAME_PER_SECOND 30

//n1589: Cycle for performing NEX processes
#define PROCESS_NEX_PERIOD 1
//#define PROCESS_NEX_PERIOD 30

#define SAFE_USER_THREAD_DISPATCH_UPDATE_FREQENCY FRAME_PER_SECOND

#define THREAD_MODE THREAD_MODE_UNSAFE_TRANSPORT_BUFFER
//#define THREAD_MODE THREAD_MODE_SAFE_USER_THREAD

#define TRANSPORT_BUFFER_THREAD_ON  ((THREAD_MODE == THREAD_MODE_UNSAFE_TRANSPORT_BUFFER) || (THREAD_MODE == THREAD_MODE_SAFE_TRANSPORT_BUFFER))

//Sets the dispatch time. Setting 0 will make the program dispatch until the process is complete.
#define DISPATCH_TIMEOUT 0

#define SAMPLE_MAX_SILENCE_TIME (10 * 1000)
#define SAMPLE_KEEP_ALIVE_TIMEOUT (2 * 1000)
#define SAMPLE_MAX_WINDOW_MAP_SIZE 128

#define NETWORK_TOPOLOGY nn::nex::NetworkTopology::Peer2Peer
//#define NETWORK_TOPOLOGY nn::nex::NetworkTopology::Hybrid


#define SAMPLE_AUTO_STACK_SIZE (4096 * 8)

//Bundling DO messages will decrease the number of packets and increase the number of devices connected. (Default value, recommended)
#define ENABLE_MESSAGE_BUNDLING 1
//Setting 0 will flush after each dispatch. (Default value, recommended)
#define MESSAGE_BUNDLING_FLUSH_DELAY 0
#define MESSAGE_BUNDLING_FLUSH_CHECK_PERIOD 0

//Bundles all packets including DO and DirectStream packets. (recommended)
#define ENABLE_PACKET_BUNDLING 1
//Setting 0 will flush after each dispatch. (Default value, recommended)
#define PACKET_BUNDLING_FLUSH_DELAY 0
//n1589: The flush time check for packet bundles is performed automatically each time the transport send process (DeliverOutgoing) is called.

#define ENABLE_SEND_UNRELIABLE_ALL 1
//#define ENABLE_SEND_UNRELIABLE_ALL 0

//Specifies the amount of data sent per packet (in bytes). The maximum size is 1250 bytes.
#define UNRELIABLE_DATA_SIZE 128
//#define UNRELIABLE_DATA_SIZE 256
//#define UNRELIABLE_DATA_SIZE 512
//#define UNRELIABLE_DATA_SIZE 1024

#define RELIABLE_DATA_SIZE 16
//#define RELIABLE_DATA_SIZE 1600

#define ENABLE_AUTO_RELIABLE_SEND 1
#if ENABLE_AUTO_RELIABLE_SEND
#define AUTO_RELIABLE_SEND_PERIOD 20
#endif

#define MAX_ENTRY 12

// ****** Macros, etc., which may be edited by the application developer in some cases *******************
// ****** (Those for which the necessity to make changes is generally low) ********************************************


enum
{
    SCENE_ID_BASE,

    SCENE_ID_SELECT_NETZ_SESSION,
    SCENE_ID_SESSION_CREATING,
    SCENE_ID_SESSION_JOINING,
    SCENE_ID_SESSION_MASTER,
    SCENE_ID_SESSION_PARTICIPANT,

    SCENE_ID_EXIT_NETZ_SESSION,

    SCENE_ID_ERROR
};

#define SCREEN_WIDTH 400
#define SCREEN_HEIGHT 240

#define DRAW_FONT_WIDTH 8
#define DRAW_FONT_HEIGHT 8

#define DRAW_POSITION_SCENE_TITLE_X 0
#define DRAW_POSITION_SCENE_TITLE_Y 0

#define DRAW_POSITION_VERSION_X (SCREEN_WIDTH - 9 * DRAW_FONT_WIDTH)
#define DRAW_POSITION_VERSION_Y 0

#define DRAW_POSITION_STATUS_X (SCREEN_WIDTH - 12 * DRAW_FONT_WIDTH)
#define DRAW_POSITION_STATUS_Y 0

#define DRAW_POSITION_SESSION_X (SCREEN_WIDTH - 20 * DRAW_FONT_WIDTH)
#define DRAW_POSITION_SESSION_Y 0

#define DRAW_POSITION_OPERATION_X 0
#define DRAW_POSITION_OPERATION_Y (DRAW_POSITION_SCENE_TITLE_Y + 1 * DRAW_FONT_HEIGHT)

#define DRAW_POSITION_AVATAR_STATE_X 0
#define DRAW_POSITION_AVATAR_STATE_Y (DRAW_POSITION_OPERATION_Y + 4 * DRAW_FONT_HEIGHT)

#define DRAW_POSITION_BUNDLE_X 0
#define DRAW_POSITION_BUNDLE_Y (SCREEN_HEIGHT - 2 * DRAW_FONT_HEIGHT)

#define DRAW_POSITION_FRAME_X 0
#define DRAW_POSITION_FRAME_Y (SCREEN_HEIGHT - 1 * DRAW_FONT_HEIGHT)

// **************** Macros not for the application developer *******************
// **************** (Macros for the NEX library creator) ************************

#define IS_TRIGGER(button) DemoFramework::GetInstance()->IsTrigger((button))
#define IS_HOLD(button) DemoFramework::GetInstance()->IsHold((button))

//n1769
void PrintSampleCommonDescription(void);

#endif
