/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef NN_SCENE_H_
#define NN_SCENE_H_

#include <OnlineCore/src/Platform/Core/qList.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>
#include <OnlineCore/src/Platform/Core/Type.h>

#include "SampleCommon.h"

/* Please see man pages for details 
    
*/
class SceneBase
{
public:
    SceneBase(void);
    virtual ~SceneBase(void);

    // Describe the process to execute when entering this state.
    virtual void Initialize(void);

    // Describe the process to execute when leaving this state.
    virtual void Finalize(void);

    virtual void SetParentScene(SceneBase* parent);

    // Updates the state.
    // When a state transition occurs, the pointer to the next state is returned.
    // If no state transition occurs, pointer to oneself (this pointer) is returned.
    virtual SceneBase* Update(void);

    virtual void BeginFrame(void);
    virtual void EndFrame(void);
    virtual void EndGameProcess(void);

    virtual void ProcessNex(void);
    virtual void ProcessCommunicationSend();
    virtual void ProcessCommunicationReceive();

    // Executes rendering.
    virtual void Render(void) const = 0;

    u16 GetSceneId()
    {
        return mSceneId;
    }

    qUnsignedInt64 GetCurrentSceneTime()
    {
        qUnsignedInt64 currentTime = (qUnsignedInt64)nn::nex::SystemClock::GetTime();
        return currentTime - m_iInitializeTime;
    }

protected:
    u16 mSceneId;
    qUnsignedInt64 m_iInitializeTime;

    SceneBase* m_pParentScene;

    // Display all convenient information
    void RenderInfo(const char *sceneName) const;

private:
    SceneBase           (const SceneBase&);
    SceneBase& operator=(const SceneBase&);

    void RenderSceneTitle(const char *name) const;

    // Station role (displays either MASTER or DUPLICA)
    void RenderInfoTop(void) const;

    // Renders for the number of Vsync
    void RenderInfoBottom(void) const;
};

/*
  Template.
#include "SceneBase.h"

class SceneTMPL : public SceneBase{
public:
    static SceneBase* GetInstance(void);

    virtual ~SceneTMPL(void);

    virtual SceneBase* Update(void);
    virtual void Render(void);

Private:
    SceneTMPL(void);   // Forces use of GetInstance function.
    SceneTMPL           (const SceneTMPL&);
    SceneTMPL& operator=(const SceneTMPL&);
};
*/

#endif  // end of NN_SCENE_H_
