/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef SCENESESSIONGAME_H_
#define SCENESESSIONGAME_H_

#include "SceneBase.h"
#include "ControlInfo.h"

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DirectStream.h>

class Avatar;

class SceneSessionGame : public SceneBase
{
public:
    static SceneSessionGame* GetInstance(void);
    static void SetInstance(SceneSessionGame* instance);

    static void GameStationEventCallback(nn::nex::StationOperation* pOperation);
    static void GameUserReceiveCallback(nn::nex::StationID sourceId, qByte* buffer, qUnsignedInt32 size);

    virtual ~SceneSessionGame(void);
    virtual SceneSessionGame* GetSceneSessionGameInstance() = 0;

    virtual void Initialize(void);
    virtual void Finalize();

    virtual SceneBase* Update(void);
    virtual void Render(void) const;

    void MakeLinkWithStations();
    void MakeLinkWithStation(nn::nex::DOHandle hStation);
    void ReleaseLinkWithStation(nn::nex::DOHandle hStation);

    void OnStationEventUpdated(nn::nex::StationOperation* pOperation);

    Avatar* GetAvatar(nn::nex::DOHandle hStation);
    Avatar* GetLocalAvatar();

    void LogLatency();
protected:
    static SceneSessionGame* s_pSessionGameInstance;

    SceneSessionGame(void);   // Force use of GetInstance function.
    SceneSessionGame           (const SceneSessionGame&);
    SceneSessionGame& operator=(const SceneSessionGame&);

    virtual void ProcessCommunicationSend();
    virtual void ProcessCommunicationReceive();

    void UpdateReceiveInfo(const nn::nex::DirectStream::Data& data);

    nn::nex::qList<Avatar*> m_AvatarList;

    nn::nex::qVector<nn::nex::DirectStream::Data> m_recvData;
};

#endif  // end of SCENESESSIONGAME_H_
