/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#include "ErrorUtil.h"

#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <OnlineCore/src/Platform/Core/SystemErrorResult.h>
#include <OnlineCore/src/Platform/Core/Result.h>


BEGIN_NAMESPACE_NEX


// Passes a qResult value.
enum ErrorUtil::Severity ErrorUtil::GetSeverity(const qResult &result)
{
    if(result.IsProgrammingError())
    {
        // Suspected application or library error.
        return SEVERITY_PROGRAMMING_ERROR;
    }

    if(result.IsResetRequired())
    {
        // The CTR must be reset or turned off and on.
        return SEVERITY_CRITICAL;
    }

    if(result.IsReinitializeRequired())
    {
        // The application must reinitialize NetZ.
        return SEVERITY_HIGH;
    }

    if(result.IsWarning())
    {
        // The application can ignore the warning.
        return SEVERITY_LOW;
    }

    if(result.IsError())
    {
        // Must be bundled appropriately by the application.
        return SEVERITY_MIDDLE;
    }
    else
    {
        // This is for when a qResult for which the severity cannot be determined is passed.
        return SEVERITY_VOID;
    }
}



// Obtains a string representation.
const qChar* ErrorUtil::ToString(enum Severity s)
{
    static const qChar* STR[] =
    {
        _T("VOID"),
        _T("LOW"),
        _T("MIDDLE"),
        _T("HIGH"),
        _T("CRITICAL"),
        _T("PROGRAMMING_ERROR"),
        _T("MAX")
    };

    if((SEVERITY_VOID <= s) && (s <= SEVERITY_MAX))
    {
        return STR[s];
    }
    else
    {
        return _T("(Invalid Severity)");
    }
}



END_NAMESPACE_NEX
