﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef _MultiUnitHeapTemplate_H_
#define _MultiUnitHeapTemplate_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <OnlineCore/src/Platform/Core/Result.h>

#include <nn/fnd.h>


//n2781: Speed up Allocate
#define NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION 1

//n1769: Attention when memory is depleted
#define NEX_ENABLE_MEMORY_STARVATION_CARE 1


BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
  

  
  
  
  
  

  
  
  
  
  

  
  
  

  
     

  
     
     

  
     
     
     
     
     
     

  
     
     
     
     
     
     

  

  

  
     
     

  
  
  
  
 */
template<class Allocator> class MultiUnitHeapTemplate
{
public:
    static const size_t LEVEL = 8; //

    /* Please see man pages for details 
      

      
      
      

      
      */
    static size_t GetRequiredMemorySize(const size_t unitSize[], const size_t unitObjs[], size_t level);


    /* Please see man pages for details 
    

    

    
    */
    static bool IsInitialized(void)
    {
        return s_bInitialized;
    }


    /* Please see man pages for details 
    

    
    
    
    
    
    

    

    
            
    */
#if NEX_ENABLE_MEMORY_STARVATION_CARE
    static bool Initialize(void *pMem, size_t size, const size_t unitSize[LEVEL], const size_t unitObjs[LEVEL], size_t level, size_t reserved);
#else
    static bool Initialize(void *pMem, size_t size, const size_t unitSize[LEVEL], const size_t unitObjs[LEVEL], size_t level);
#endif

    /* Please see man pages for details 
    

           
           
           

    

     */
    static void Finalize  (void);

    /* Please see man pages for details 
    

           
           
           

    
     */
    static void Invalidate(void);

    /* Please see man pages for details 
    
    */
    static void PrintDebugInfo(void);


    /* Please see man pages for details 
    
    */
    static void PrintUsage(void);


    /* Please see man pages for details 
    

    
    */
    static bool IsClean(void);


    /* Please see man pages for details 
    

    

    
    */
    static bool GetUnitSizes(size_t size[LEVEL]);


    /* Please see man pages for details 
    

    

    
    */
    static bool GetAllocatableCounts(size_t count[LEVEL]);


    /* Please see man pages for details 
    

    

    
    */
    static bool GetMaximumAllocatableCounts(size_t count[LEVEL]);


    /* Please see man pages for details 
    

           

    
    */
    static void EnablePeakRecording(void);


    /* Please see man pages for details 
    

    
     */
    static void DisablePeakRecording(void);


    /* Please see man pages for details 
    

    

    
    */
    static bool IsPeakRecordingEnabled(void)
    {
        return s_bPeakRecording;
    }


    /* Please see man pages for details 
    

    

    

    
    */
    static bool GetPeakCounts(size_t count[LEVEL]);


    /* Please see man pages for details 
    

           

    
    */
    static void ResetPeakCounts(void);


    // {secret}
    // This is a utility function that returns the total size of the memory region in use by MultiUnitHeapTemplate.
    // A value of 0xffffffff is returned when MultiUnitHeapTemplate is not enabled.
    // Am a little concerned whether it is OK to make this function public, so for now will leave it secret.
    static size_t GetTotalMemoryUsage(void);


    // {secret}
    // This function allocates/deallocates memory. These functions are passed to SetBasicMemoryFunctions function.
    // The application program should not call these functions directly.
#if NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION
    template<bool PeakRecording> static void* Allocate (size_t sz);
#else
    static void* Allocate (size_t sz);
#endif
    static void  Free     (void *pMem);


private:
    // Seal these because they all are static functions.
    MultiUnitHeapTemplate(void) {}
    virtual ~MultiUnitHeapTemplate(void) {}
    MultiUnitHeapTemplate           (const MultiUnitHeapTemplate&);
    MultiUnitHeapTemplate& operator=(const MultiUnitHeapTemplate&);

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    static void* emergencyAllocate(size_t sz);
    static bool isEmergencyAllocatorAvailable(void)
    {
        return s_bExpHeapAvailable;
    }
#endif

    typedef Allocator TUnitHeap;

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    typedef nn::fnd::ThreadSafeExpHeap TExpHeap;  //n1769 Will use thread-safe ExpHeap for now, but it may be good to allow to be specified with the template.
#endif

    static const size_t ALIGN = 4;       // Alignment.
    NN_STATIC_ASSERT(ALIGN==4);

    static size_t s_level;               // Which stage unit heap to use? (1 <= s_level <= LEVEL)
    static size_t UNIT_SIZE[LEVEL];      // Was const before.
    static size_t UNIT_OBJS[LEVEL];      // Was const before.
    static size_t MAX_UNIT_OBJS[LEVEL];  // Maximum number that can be allocated. Value is set at Initialization.

    static size_t s_aPeakCount[LEVEL];   // Record the peak value of number of unit heaps used here.

    static TUnitHeap s_aUnitHeap[LEVEL]; // Allocator system.

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    static TExpHeap s_expHeap;           // Emergency use (when memory is depleted) allocator.
    static size_t s_maxExpHeapFreeSize;  // Maximum memory size that can be allocated from the emergency-use memory allocator.
    static bool s_bExpHeapAvailable;     // Can the emergency-use memory allocator be used?
#endif

    static bool s_bInitialized;          // Flag variable to indicate whether initialized.
    static bool s_bPeakRecording;        // Whether to record peak value
};



// "Header" to attach to the start of the memory allocated with the Allocate function.
// Insert in the header the pointer to the allocator required when deallocating this memory.
class MultiUnitHeapTemplateHeader
{
public:
    MultiUnitHeapTemplateHeader(void) : m_pHeap(NULL) {}
    void SetHeapBasePtr(nn::fnd::HeapBase *p)
    {
        m_pHeap = p;
    }
    nn::fnd::HeapBase* GetHeapBasePtr(void)
    {
        return m_pHeap;
    }

private:
    nn::fnd::HeapBase *m_pHeap;
};

NN_STATIC_ASSERT(sizeof(MultiUnitHeapTemplateHeader)%4==0);
static const size_t HEADER_SIZE = sizeof(MultiUnitHeapTemplateHeader);




// Static member group of MultiUnitHeapTemplate
template<class T> size_t MultiUnitHeapTemplate<T>::s_level = 0;
template<class T> size_t MultiUnitHeapTemplate<T>::UNIT_SIZE    [MultiUnitHeapTemplate::LEVEL] = {};
template<class T> size_t MultiUnitHeapTemplate<T>::UNIT_OBJS    [MultiUnitHeapTemplate::LEVEL] = {};
template<class T> size_t MultiUnitHeapTemplate<T>::MAX_UNIT_OBJS[MultiUnitHeapTemplate::LEVEL] = {};  // Maximum number that can be allocated. Value is set at Initialization.
template<class T> size_t MultiUnitHeapTemplate<T>::s_aPeakCount [MultiUnitHeapTemplate::LEVEL] = {};  // Record the peak value of number of unit heaps used here.

// Allocator instance.
template<class T> MultiUnitHeapTemplate<T>::TUnitHeap MultiUnitHeapTemplate<T>::s_aUnitHeap[MultiUnitHeapTemplate::LEVEL];
#if NEX_ENABLE_MEMORY_STARVATION_CARE
template<class T> MultiUnitHeapTemplate<T>::TExpHeap  MultiUnitHeapTemplate<T>::s_expHeap;
template<class T> size_t MultiUnitHeapTemplate<T>::s_maxExpHeapFreeSize = 0;
template<class T> bool MultiUnitHeapTemplate<T>::s_bExpHeapAvailable = false;
#endif

// All flag variables types.
template<class T> bool MultiUnitHeapTemplate<T>::s_bInitialized   = false;  // Flag variable to indicate whether initialized.
template<class T> bool MultiUnitHeapTemplate<T>::s_bPeakRecording = false;  // Whether to record peak value


// Function for calculating how much memory is required to build `MultiUnit` with the settings provided by `cfg`.
// 
template<class T>
size_t MultiUnitHeapTemplate<T>::GetRequiredMemorySize(const size_t unitSize[], const size_t unitObjs[], size_t level)
{
    size_t total = 0;
    size_t required[LEVEL] = {};

    for(size_t i=0; i<level; ++i)
    {
        required[i] = TUnitHeap::GetRequiredHeapSize(unitSize[i], unitObjs[i]);
        NN_ASSERT(required[i]%ALIGN == 0);
        total += required[i];
    }

    return total;
}


// If initialization was successful, TRUE.
// If it was a NULL pointer or if the size was insufficient, FALSE.
template<class T>
#if NEX_ENABLE_MEMORY_STARVATION_CARE
bool MultiUnitHeapTemplate<T>::Initialize(void *pMem, size_t size, const size_t unitSize[], const size_t unitObjs[], size_t level, size_t reserved)
#else
bool MultiUnitHeapTemplate<T>::Initialize(void *pMem, size_t size, const size_t unitSize[], const size_t unitObjs[], size_t level)
#endif
{
    if(IsInitialized())
    {
        return true;
    }

    // n2343: Add SYSTEMCHECK because it's inconceivable that Initialize would have been called with a NULL pointer
    SYSTEMCHECK(pMem!=NULL);

    // Null pointer detection
    if(pMem==NULL)
    {
        return false;
    }

    // Check whether the level value is valid
    SYSTEMCHECK((1 <= level) && (level <= LEVEL));
    if(!((1 <= level) && (level <= LEVEL)))
    {
        return false;
    }

    // Calculate size required for each unit heap.
    size_t required[LEVEL] = {};
    {
        for(size_t i=0; i<level; ++i)
        {
            required[i] = TUnitHeap::GetRequiredHeapSize(unitSize[i], unitObjs[i]);
            NN_ASSERT(required[i]%ALIGN == 0);
        }
    }

    // Is the size sufficient?
    {
        size_t totalRequired = 0;
        for(size_t i=0; i<level; ++i)
        {
            totalRequired += required[i];
        }

        NN_ASSERT(totalRequired == GetRequiredMemorySize(unitSize, unitObjs, level));

#if NEX_ENABLE_MEMORY_STARVATION_CARE
        totalRequired += reserved;
#endif

        if(totalRequired > size)
        {
            return false;
        }
    }

    // The size problem is cleared if we arrived here.

    // Copy the size and count settings.
    // In the past, this was set with const, so we didn't need all this code.
    s_level = level;
    for(size_t i=0; i<s_level; ++i)
    {
        UNIT_SIZE[i] = unitSize[i];
        UNIT_OBJS[i] = unitObjs[i];
    }

    // Initialize each unit heap.
    {
        const bit32 OPTION = 0;
        u8 *p = static_cast<u8*>(pMem);
        for(size_t i=0; i<s_level; ++i)
        {
            s_aUnitHeap[i].Initialize(UNIT_SIZE[i], reinterpret_cast<uptr>(p), required[i], ALIGN, OPTION);
            p += required[i];

            // While still not used, get the maximum number that can be allocated.
            MAX_UNIT_OBJS[i] = s_aUnitHeap[i].GetAllocatableCount();
        }

#if NEX_ENABLE_MEMORY_STARVATION_CARE
        // Initialize ExpHeap.
        if(reserved > 0)
        {
            s_expHeap.Initialize(reinterpret_cast<uptr>(p), reserved);
            s_maxExpHeapFreeSize = s_expHeap.GetTotalFreeSize();
            s_bExpHeapAvailable = true;
        }
#endif
    }

    // Register the Allocate and Free functions of MultiUnitHeapTemplate
#if NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION
    if(s_bPeakRecording)
    {
        MemoryManager::SetBasicMemoryFunctions(Allocate<true>, Free);
    }
    else
    {
        MemoryManager::SetBasicMemoryFunctions(Allocate<false>, Free);
    }
#else
    MemoryManager::SetBasicMemoryFunctions(Allocate, Free);
#endif

    s_bInitialized = true;
    return true;
}

// Clean up.
template<class T>
void MultiUnitHeapTemplate<T>::Finalize(void)
{
    if(IsInitialized())
    {
        s_bInitialized = false;

        // Release the Allocate and Free functions of MultiUnitHeapTemplate
        MemoryManager::SetBasicMemoryFunctions(NULL, NULL);

#if NEX_ENABLE_MEMORY_STARVATION_CARE
        // ExpHeap's Finalize function.
        if(isEmergencyAllocatorAvailable())
        {
            s_bExpHeapAvailable = false;
            s_expHeap.Finalize();
        }
#endif

        for(size_t i=0; i<s_level; ++i)
        {
            size_t idx = s_level - 1 - i;
            // Finalize in reverse order.
            s_aUnitHeap[idx].Finalize();

            UNIT_SIZE[idx]     = 0;
            UNIT_OBJS[idx]     = 0;
            MAX_UNIT_OBJS[idx] = 0;
        }
        s_level = 0;  // Just in case, clear to zero.
    }
}


template<class T>
void MultiUnitHeapTemplate<T>::Invalidate(void)
{
    for(size_t i=0; i<s_level; ++i)
    {
        s_aUnitHeap[i].Invalidate();
    }

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    if(isEmergencyAllocatorAvailable())
    {
        s_expHeap.Invalidate();
    }
#endif
}


// Allocate memory from unit heap that is within the size.
// Write header information at the beginning of the memory and equip with Free.
template<class T>
#if NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION
template<bool PeakRecording>
#endif
void* MultiUnitHeapTemplate<T>::Allocate(size_t sz)
{
    void *ret = NULL;
    size_t actual = sz + HEADER_SIZE;  // Size of the memory that is actually allocated (size with the header added)

    for(size_t i=0; i<s_level; ++i)
    {
        if(actual <= UNIT_SIZE[i])
        {
            ret = s_aUnitHeap[i].Allocate();
            if(ret)
            {
#if NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION
                if(PeakRecording)
#else
                if(IsPeakRecordingEnabled())
#endif
                {
                    // Update peak value
                    s_aPeakCount[i] = std::max(MAX_UNIT_OBJS[i] - s_aUnitHeap[i].GetAllocatableCount(), s_aPeakCount[i]);
                }

                static_cast<MultiUnitHeapTemplateHeader*>(ret)->SetHeapBasePtr(&s_aUnitHeap[i]);

                void *pMem = static_cast<u8*>(ret) + HEADER_SIZE;  // Shift header portion.
                return pMem;
            }
            else
            {
                _QLOG(EventLog::Always, _T("MultiUnitHeapTemplate failed to alloc... (%u bytes)"), sz);
                PrintUsage();

#if NEX_ENABLE_MEMORY_STARVATION_CARE
                return emergencyAllocate(actual);
#else
                return NULL;
#endif
            }
        }
    }

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    return emergencyAllocate(actual);
#else
    _QLOG(EventLog::Always, _T("MultiUnitHeapTemplate failed to alloc %u bytes.  Too large memory is requested!"), sz);
    return NULL;
#endif
}


#if NEX_ENABLE_MEMORY_STARVATION_CARE
template<class T>
void* MultiUnitHeapTemplate<T>::emergencyAllocate(size_t sz)
{
    if(!isEmergencyAllocatorAvailable())
    {
        //n1769 Case when an emergency-use memory allocator is not set. Can do nothing except return NULL.
        _QLOG(EventLog::Always, _T("NULL will be returned because reserved memory is not specified."));
        PrintUsage();
        return NULL;
    }

    void *ret = s_expHeap.Allocate(sz);
    if(ret)
    {
        static_cast<MultiUnitHeapTemplateHeader*>(ret)->SetHeapBasePtr(&s_expHeap);
        void *pMem = static_cast<u8*>(ret) + HEADER_SIZE;  // Shift header portion.
        return pMem;
    }
    else
    {
        //n1769 Case when nothing can be done at all. Allocation also fails from the emergency-use memory allocator.
        _QLOG(EventLog::Always, _T("NULL will be returned because MultiUnitHeapTemplate failed to alloc... (%u bytes)"), sz);
        PrintUsage();
        return NULL;
    }
}
#endif


// Version that gets allocator information written in the header, and appropriately uses Free.
// Ignored when a NULL pointer is given.
// Be aware that threads that allocate memory may not deallocate that memory.
template<class T>
void MultiUnitHeapTemplate<T>::Free(void *pMem)
{
    if(pMem)
    {
        // Be aware that the virtual function, FreeV(), is used for the deallocation process!
        MultiUnitHeapTemplateHeader *pHead = reinterpret_cast<MultiUnitHeapTemplateHeader*>(static_cast<u8*>(pMem) - HEADER_SIZE);
        pHead->GetHeapBasePtr()->FreeV(pHead);
    }
}


// Print debugging information.
template<class T>
void MultiUnitHeapTemplate<T>::PrintDebugInfo(void)
{
    for(int i=0; i<s_level; ++i)
    {
        const TUnitHeap *pUnit = &s_aUnitHeap[i];
        u8 *pTop = static_cast<u8*>(pUnit->GetStartAddress());
        size_t total = pUnit->GetTotalSize();
        _QLOG(EventLog::Always, _T("[%d] Addr: 0x%p - 0x%p (%7u bytes)"), i, pTop, pTop + total, total);
    }

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    //n1769: Print ExpHeap debugging information.
    if(isEmergencyAllocatorAvailable())
    {
        u8 *pTop = static_cast<u8*>(s_expHeap.GetStartAddress());
        size_t total = s_expHeap.GetTotalSize();
        _QLOG(EventLog::Always, _T("[Exp] Addr: 0x%p - 0x%p (%7u bytes)"), pTop, pTop + total, total);
    }
#endif
}


template<class T>
void MultiUnitHeapTemplate<T>::PrintUsage(void)
{
    if(!IsInitialized())
    {
        return;
    }

    size_t unitSize   [LEVEL] = {};
    size_t allocatable[LEVEL] = {};
    size_t maximum    [LEVEL] = {};
    size_t peak       [LEVEL] = {};

    MultiUnitHeapTemplate::GetUnitSizes(unitSize);
    MultiUnitHeapTemplate::GetAllocatableCounts(allocatable);
    MultiUnitHeapTemplate::GetMaximumAllocatableCounts(maximum);
    MultiUnitHeapTemplate::GetPeakCounts(peak);

    _QLOG(EventLog::Always, _T("-------- MultiUnitHeapTemplate report begin --------"));
    for(int i=0; i<s_level; ++i)
    {
        _QLOG(EventLog::Always, _T("[%6d] %5d / %5d (%5.1f%% free, %5.1f%% peak)"),
              unitSize[i],
              allocatable[i],
              maximum[i],
              100.0f * allocatable[i] / maximum[i],
              100.0f * peak[i] / maximum[i]
             );
    }
#if NEX_ENABLE_MEMORY_STARVATION_CARE
    if(isEmergencyAllocatorAvailable())
    {
        size_t totalSize = s_expHeap.GetTotalSize();
        size_t freeSize  = s_expHeap.GetTotalFreeSize();
        _QLOG(EventLog::Always, _T("[Exp] %7d / %7d (%5.1f%% free) "),
              freeSize, totalSize, 100.0f * freeSize / totalSize);
    }
#endif
    _QLOG(EventLog::Always, _T("-------- MultiUnitHeapTemplate report end   --------"));
}


// Brand new state?
template<class T>
bool MultiUnitHeapTemplate<T>::IsClean(void)
{
    for(int i=0; i<s_level; ++i)
    {
        const TUnitHeap *pUnit = &s_aUnitHeap[i];
        if(pUnit->GetAllocatableCount() < UNIT_OBJS[i])
        {
            return false;
        }
    }

#if NEX_ENABLE_MEMORY_STARVATION_CARE
    if(isEmergencyAllocatorAvailable())
    {
        if(s_expHeap.GetTotalFreeSize()!=s_maxExpHeapFreeSize)
        {
            return false;
        }
    }
#endif

    return true;
}


// Get unit size in an array.
template<class T>
bool MultiUnitHeapTemplate<T>::GetUnitSizes(size_t size[LEVEL])
{
    if(IsInitialized())
    {
        NN_NULL_ASSERT(size);

        for(int i=0; i<s_level; ++i)
        {
            size[i] = UNIT_SIZE[i];
        }
        return true;
    }
    else
    {
        return false;
    }
}


// Get the number of units that can be allocated
template<class T>
bool MultiUnitHeapTemplate<T>::GetAllocatableCounts(size_t count[LEVEL])
{
    if(IsInitialized())
    {
        NN_NULL_ASSERT(count);

        for(int i=0; i<s_level; ++i)
        {
            count[i] = s_aUnitHeap[i].GetAllocatableCount();
        }
        return true;
    }
    else
    {
        return false;
    }
}


// Get the maximum number of units that can be allocated
template<class T>
bool MultiUnitHeapTemplate<T>::GetMaximumAllocatableCounts(size_t count[LEVEL])
{
    if(IsInitialized())
    {
        NN_NULL_ASSERT(count);

        for(int i=0; i<s_level; ++i)
        {
            count[i] = MAX_UNIT_OBJS[i];
        }
        return true;
    }
    else
    {
        return false;
    }
}


// Return the peak values for numbers of unit heaps used.
template<class T>
bool MultiUnitHeapTemplate<T>::GetPeakCounts(size_t count[LEVEL])
{
    if(IsInitialized())
    {
        NN_NULL_ASSERT(count);

        for(int i=0; i<s_level; ++i)
        {
            count[i] = s_aPeakCount[i];
        }
        return true;
    }
    else
    {
        return false;
    }
}


// Reset the counter that stores the peak values (restore all to zero)
template<class T>
void MultiUnitHeapTemplate<T>::ResetPeakCounts(void)
{
    for(int i=0; i<s_level; ++i)
    {
        s_aPeakCount[i] = 0;
    }
}


// Return the total for memory regions being used for MultiUnitHeapTemplate.
// A value of 0xffffffff is returned when MultiUnitHeapTemplate is not enabled.
template<class T>
size_t MultiUnitHeapTemplate<T>::GetTotalMemoryUsage(void)
{
    if(MultiUnitHeapTemplate::IsInitialized())
    {
        size_t unitSize[MultiUnitHeapTemplate::LEVEL] = {};
        size_t unitObjs[MultiUnitHeapTemplate::LEVEL] = {};
        size_t maxObjs [MultiUnitHeapTemplate::LEVEL] = {};

        MultiUnitHeapTemplate::GetUnitSizes(unitSize);
        MultiUnitHeapTemplate::GetAllocatableCounts(unitObjs);
        MultiUnitHeapTemplate::GetMaximumAllocatableCounts(maxObjs);

        size_t sum = 0;
        for(int i=0; i<s_level; ++i)
        {
            sum += (maxObjs[i] - unitObjs[i]) * unitSize[i];
        }

        return sum;
    }
    else
    {
        return 0xffffffff;
    }
}


template<class T>
void MultiUnitHeapTemplate<T>::EnablePeakRecording(void)
{
    s_bPeakRecording = true;
#if NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION
    if(IsInitialized())
        MemoryManager::SetBasicMemoryFunctions(Allocate<true>, Free);
#endif
}

template<class T>
void MultiUnitHeapTemplate<T>::DisablePeakRecording(void)
{
    s_bPeakRecording = false;
#if NEX_ENABLE_PEAK_RECORDING_OPTIMIZATION
    if(IsInitialized())
        MemoryManager::SetBasicMemoryFunctions(Allocate<false>, Free);
#endif
}



// It is convenient for the application to use this typedef.
typedef MultiUnitHeapTemplate<nn::fnd::ThreadSafeUnitHeap> ThreadSafeMultiUnitHeap;
typedef MultiUnitHeapTemplate<nn::fnd::UnitHeap>           ThreadUnsafeMultiUnitHeap;


END_NAMESPACE_NEX


#endif  // end of _MultiUnitHeapTemplate_H_
