﻿/*--------------------------------------------------------------------------
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Vector2D_H_
#define _Vector2D_H_

#include "MathUtil.h"

class Vector2D
{
public:
    Vector2D(qReal x = 0, qReal y = 0)
    {
        m_x = x;
        m_y = y;
    }
    ~Vector2D()
    {
    };

    const Vector2D& operator=(const Vector2D& oVector2D)
    {
        m_x = oVector2D.m_x;
        m_y = oVector2D.m_y;
        return *this;
    }

    // Basic methods.

    void Set(qReal x, qReal y)
    {
        m_x = x;
        m_y = y;
    }

    qReal GetX() const
    {
        return m_x;
    }
    qReal GetY() const
    {
        return m_y;
    }
    void SetX(qReal sx)
    {
        m_x = sx;
    }
    void SetY(qReal sy)
    {
        m_y = sy;
    }

    // Polar coordinates methods.

    qReal GetAngle() const
    {
        return Direction(m_x, m_y);
    }
    qReal GetLength() const
    {
        return Length(m_x, m_y);
    }

    //n1589:math.h
#if 1
    void SetFromPolar(qReal dAngle, qReal dLength)
    {
        m_x = static_cast<qReal>(cos(dAngle)) * dLength;
        m_y = static_cast<qReal>(sin(dAngle)) * dLength;
    }

    // Transformations.

    void Rotate(qReal dRotationAngle)
    {
        qReal dAngle = GetAngle();
        qReal dLength = GetLength();
        SetFromPolar(dAngle + dRotationAngle, dLength);
    }
#endif

    void Scale(qReal dScale)
    {
        m_x *= dScale;
        m_y *= dScale;
    }

    void Add(qReal x, qReal y)
    {
        m_x += x;
        m_y += y;
    }

    void Add(const Vector2D& oVector2D)
    {
        m_x += oVector2D.m_x;
        m_y += oVector2D.m_y;
    }

    //n1589
    qReal m_x;
    qReal m_y;

private:
};

#endif
