/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include "Graphics.h"

namespace nexdemo
{

namespace
{
// Heap
nn::fnd::ExpHeap s_gxHeap;
// Buffer
uptr s_pBuffer = NULL;

// Renderer
demo::RenderSystemDrawing s_renderSystem;
}

// Performs initialization.
void Graphics::Initialize(size_t heapSize)
{
    // Initialize the heap
    s_gxHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    // Allocates the heap
    s_pBuffer = reinterpret_cast<uptr>(s_gxHeap.Allocate(heapSize));

    // Initialize the renderer
    s_renderSystem.Initialize(s_pBuffer, heapSize);
}

// Performs shutdown.
void Graphics::Finalize()
{
    // Finalize the renderer
    s_renderSystem.Finalize();

    // Destroys heap.
    s_gxHeap.Free(reinterpret_cast<void*>(s_pBuffer));
    // Perform heap finalization
    s_gxHeap.Finalize();
}

// Get the renderer.
demo::RenderSystemDrawing* Graphics::GetRenderSystem()
{
    return &s_renderSystem;
}

} // namespace nexdemo
