﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef _MultiUnitHeapTemplate_H_
#define _MultiUnitHeapTemplate_H_

#include <nn/fnd.h>

/* Please see man pages for details 
  

  

  
  
  

  
  

  
  
  


  

  
    
    
    
    

    
    
    
    

  
    
    
    

  
    
    
    

  
    
    
    
    
    
    

  
    
    
    

 */

#define NEX_SAMPLE_MUH_PRINT NN_LOG
#define NEX_SAMPLE_MUH_ASSERT NN_ASSERT

template<size_t UnitHeapNum, class TUnitHeap, class TReservedHeap>
class MultiUnitHeapTemplate
{
public:

    /* Please see man pages for details 
     
     */
    struct DebugCount
    {
        size_t peak;        //
        size_t use;         //
        size_t unitSize;    //
    };

    MultiUnitHeapTemplate() :
        m_pReservedHeap(NULL),
        m_bInitialized(false),
        m_bOutOfMemory(false)
    {}

    /* Please see man pages for details 
      
      
      
      
      */
    size_t GetRequiredMemorySize(const size_t unitSize[UnitHeapNum], const size_t unitObjs[UnitHeapNum])
    {
        size_t total = 0;
        for(size_t i=0; i<UnitHeapNum; ++i)
        {
            size_t required = TUnitHeap::GetRequiredHeapSize(unitSize[i] + HEADER_SIZE, unitObjs[i], ALIGN);
            total += required;
        }
        return total;
    }

    /* Please see man pages for details 
    
    
    
    */
    bool IsInitialized(void) const
    {
        return m_bInitialized;
    }
	
	
    /* Please see man pages for details 
    
           
    
    
    
    
    
    
    */
    void Initialize(void *pMem, size_t size, const size_t unitSize[UnitHeapNum], const size_t unitObjs[UnitHeapNum], TReservedHeap* pReservedHeap)
    {
        (void)size;
        NEX_SAMPLE_MUH_ASSERT(pMem);
        NEX_SAMPLE_MUH_ASSERT(pReservedHeap);
        NEX_SAMPLE_MUH_ASSERT(IsInitialized()==false);
        NEX_SAMPLE_MUH_ASSERT(size%ALIGN==0);
        NEX_SAMPLE_MUH_ASSERT(GetRequiredMemorySize(unitSize, unitObjs) <= size);

        // Initialize each unit heap.
        {
            u8 *p = static_cast<u8*>(pMem);
            for(size_t i=0; i<UnitHeapNum; ++i)
            {
                const size_t realSize = unitSize[i] + HEADER_SIZE;
                const size_t required = TUnitHeap::GetRequiredHeapSize(realSize, unitObjs[i], ALIGN);
                m_UnitHeapInstance[i].Initialize(realSize, reinterpret_cast<uptr>(p), required);
                p += required;
            }
        }

        m_pReservedHeap = pReservedHeap;
        m_bOutOfMemory = false;
        m_bInitialized = true;
    }

    /* Please see man pages for details 
    
           
           
           
           
    
     */
    void Finalize(void)
    {
        NEX_SAMPLE_MUH_ASSERT(m_bInitialized);
        m_bInitialized = false;
        for(size_t i=0; i<UnitHeapNum; ++i)
        {
            m_UnitHeapInstance[i].Finalize();
        }
        m_pReservedHeap = NULL;
    }

    /* Please see man pages for details 
    
           
           
           
    
     */
    void Invalidate(void)
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            m_UnitHeapInstance[i].Invalidate();
        }
    }

    /* Please see man pages for details 
    
    */
    void Dump(void)
    {
        DebugCount debugCount[UnitHeapNum];
        GetDebugCount(debugCount);
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            NEX_SAMPLE_MUH_PRINT("unit[%d], use:%d, peak:%d\n",
                                 i, debugCount[i].use, debugCount[i].peak
                                );
        }
    }


    /* Please see man pages for details 
    
    
    */
    bool IsClean(void)
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            if (m_UnitHeapInstance[i].IsClean() == false)
                return false;
        }
        return true;
    }

    /* Please see man pages for details 
    
    */
    void GetDebugCount(DebugCount dst[UnitHeapNum])
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            dst[i] = m_UnitHeapInstance[i].GetDebugCount();
        }
    }

    /* Please see man pages for details 
    
    */
    void ResetDebugCount()
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            m_UnitHeapInstance[i].ResetDebugCount();
        }
    }

    /* Please see man pages for details 
    
           
    */
    void* Allocate(size_t size)
    {
        void* ret = NULL;
        size_t realSize = size + HEADER_SIZE;
        nn::fnd::HeapBase* pAllocatedHeap = NULL;
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            ret = m_UnitHeapInstance[i].Allocate(realSize);
            if (ret)
            {
                pAllocatedHeap = m_UnitHeapInstance[i].GetHeapPtr();
                break;
            }
        }
        if (ret == NULL)
        {
            // MultiUnitHeap is exhausted, or the request is too large.
            // Allocate memory from the reserved heap and set IsOutOfMemory to return true.
            NEX_SAMPLE_MUH_PRINT("MultiUnitHeap is failed to Allocate %u byte. Use the reserved heap.\n", realSize);
            NEX_SAMPLE_MUH_ASSERT(m_pReservedHeap);
            ret = m_pReservedHeap->Allocate(realSize);
            NEX_SAMPLE_MUH_ASSERT(ret);
            pAllocatedHeap = m_pReservedHeap;

            // If the request is too large and memory cannot be allocated in MultiUnitHeap, this does not necessarily mean that MultiUnitHeap is exhausted.
            // 
            // If in this case you do not want to set IsOutOfMemory to return true, uncomment the following if statement.
            // While using UnitHeap, it is possible to also use ExpHeap for large requests.
            // However, if MultiUnitHeap is actually exhausted, make sure to use a sufficiently large reserve heap so that memory can always be allocated successfully.
            // 
            //
            // if ( m_UnitHeapInstance[UnitHeapNum].GetUnitSize() <= realSize ) // Sets IsOutOfMemory to return true only when MultiUnitHeap is actually exhausted.
            {
                m_bOutOfMemory = true;
            }
        }

        if (ret)
        {
            // Header preparation
            NEX_SAMPLE_MUH_ASSERT(pAllocatedHeap);
            static_cast<MultiUnitHeapHeader*>(ret)->SetHeapBasePtr(pAllocatedHeap);
            ret = static_cast<u8*>(ret) + HEADER_SIZE;  // Shift header portion.
        }

        NEX_SAMPLE_MUH_ASSERT(ret);
        return ret;
    }

    /* Please see man pages for details 
    
    */
    void  Free(void *pMem)
    {
        MultiUnitHeapHeader *pHead = reinterpret_cast<MultiUnitHeapHeader*>(static_cast<u8*>(pMem) - HEADER_SIZE);
        pHead->GetHeapBasePtr()->FreeV(pHead);
    }

    /* Please see man pages for details 
    
    
    */
    bool IsOutOfMemory() const
    {
        return m_bOutOfMemory;
    }

private:
    static const size_t ALIGN = 4;

    class MultiUnitHeapHeader
    {
    public:
        MultiUnitHeapHeader(void) : m_pHeap(NULL) {}
        void SetHeapBasePtr(nn::fnd::HeapBase *p)
        {
            m_pHeap = p;
        }
        nn::fnd::HeapBase* GetHeapBasePtr(void)
        {
            return m_pHeap;
        }
    private:
        nn::fnd::HeapBase *m_pHeap;
    };

    NN_STATIC_ASSERT(sizeof(MultiUnitHeapHeader)%ALIGN==0);
    static const size_t HEADER_SIZE = sizeof(MultiUnitHeapHeader);

    class UnitHeapInstance
    {
    public:
        UnitHeapInstance() :
            m_unitSize(0)
        {
            ResetDebugCount();
        }
        TUnitHeap *GetHeapPtr()
        {
            return &m_unitHeap;
        }
        void Initialize(size_t unitSize, uptr addr, size_t memSize)
        {
            m_unitSize = unitSize;
            m_unitHeap.Initialize(unitSize, addr, memSize, ALIGN);
        }
        void Invalidate()
        {
            m_unitHeap.Invalidate();
        }
        void Finalize()
        {
            m_unitHeap.Finalize();
        }
        void* Allocate(size_t size)
        {
            if (size > m_unitSize) return NULL;
            void* ret = m_unitHeap.Allocate();
            if (ret)
            {
                size_t use = m_unitHeap.GetAllocatedCount();
                if ( use > m_dbgCount.peak )
                    m_dbgCount.peak = use;
            }
            return ret;
        }
        DebugCount& GetDebugCount()
        {
            m_dbgCount.unitSize = m_unitSize;
            m_dbgCount.use = m_unitHeap.GetAllocatedCount();
            return m_dbgCount;
        }
        void ResetDebugCount()
        {
            memset(&m_dbgCount, 0, sizeof(m_dbgCount));
        }
        bool IsClean() const
        {
            return m_unitHeap.GetAllocatedCount() == 0;
        }
        size_t GetUnitSize() const
        {
            return m_unitSize;
        }
    private:
        DebugCount m_dbgCount;
        size_t m_unitSize;
        TUnitHeap m_unitHeap;
    };

    UnitHeapInstance m_UnitHeapInstance[UnitHeapNum];
    TReservedHeap *m_pReservedHeap;

    bool m_bInitialized;
    bool m_bOutOfMemory;
};

#endif  // end of _MultiUnitHeapTemplate_H_
