/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DEMOMEM_H_
#define _DEMOMEM_H_

#include "NexUtil.h"

NEXUTIL_BEGIN

class DemoMemBase
{
public:
    static const int ALIGNMENT = 32;

    DemoMemBase();
    virtual ~DemoMemBase() {};

    // Initialize
    void Initialize(void* memBlock, size_t memSize, size_t reserveSize);

    // Finalize
    void Finalize();

    // Returns whether emergency memory was used
    qBool IsLowMemory() const;

    // Returns whether memory was allocated when the function was called
    qBool IsClean() const;

    // mem alloc
    void* AllocFunction(size_t size);

    // mem free
    void FreeFunction(void* p);

    // Gets amount of memory currently used
    size_t GetUsage()const;

    // Gets peak of current memory usage
    size_t GetMaxUsage()const;

    // Gets current number of memory allocations
    size_t GetNowAllocCount()const;

    // Destroy heap
    void Invalidate();

    // Dump memory state
    virtual void Dump() = 0;


private:
    size_t m_memUsage;
    size_t m_memMaxUsage;
    size_t m_nowAllocCount;
    void* m_pReservedMemory;

    void AllocReservedMemory(size_t size);
    void FreeReservedMemory();

    virtual size_t GetBlockSize(const void* p) const = 0;
    virtual void InnerInitialize(void* memBlock, size_t memSize) = 0;
    virtual void InnerFinalize() = 0;
    virtual void* InnerAlloc(size_t size) = 0;
    virtual void InnerFree(void* p) = 0;
    virtual void InnerInvalidate() = 0;
};

class DemoMem : public DemoMemBase
{
public:
    DemoMem() {};
    virtual ~DemoMem() {};
    void Dump();
private:

    size_t GetBlockSize(const void* p) const ;
    void InnerInitialize(void* memBlock, size_t memSize);
    void InnerFinalize();
    void* InnerAlloc(size_t size);
    void InnerFree(void* p);
    void InnerInvalidate();
};

NEXUTIL_END

#endif // _DEMOMEM_H_
