﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#include "DemoMem.h"

NEXUTIL_BEGIN

DemoMemBase::DemoMemBase():
    m_memUsage(0),
    m_memMaxUsage(0),
    m_nowAllocCount(0),
    m_pReservedMemory(NULL)
{
}

void DemoMemBase::Initialize(void* memBlock, size_t memSize, size_t reserveSize)
{
    InnerInitialize(memBlock, memSize);
    m_memUsage = 0;
    m_memMaxUsage = 0;
    m_nowAllocCount = 0;
    AllocReservedMemory(reserveSize);
}

void DemoMemBase::Finalize()
{
    FreeReservedMemory();
    InnerFinalize();
}

qBool DemoMemBase::IsLowMemory() const
{
    return m_pReservedMemory == NULL;
}
qBool DemoMemBase::IsClean() const
{
    return m_memUsage == 0;
}

void* DemoMemBase::AllocFunction(size_t size)
{
    void* ret = InnerAlloc(size);
    if (!ret)
    {
        FreeReservedMemory();
        ret = InnerAlloc(size);
    }
    m_nowAllocCount++;

#if defined(CTR_TARGET) || defined(CAFE_TARGET)
    m_memUsage += GetBlockSize(ret);
    if (m_memUsage > m_memMaxUsage)
    {
        m_memMaxUsage = m_memUsage;
    }
#endif

    return ret;
}

void DemoMemBase::FreeFunction(void* p)
{
    m_nowAllocCount--;
#if defined(CTR_TARGET) || defined(CAFE_TARGET)
    m_memUsage -= GetBlockSize(p);
#endif
    InnerFree(p);
}

size_t DemoMemBase::GetUsage()const
{
    return m_memUsage;
}
size_t DemoMemBase::GetMaxUsage()const
{
    return m_memMaxUsage;
}
size_t DemoMemBase::GetNowAllocCount()const
{
    return m_nowAllocCount;
}

void DemoMemBase::Invalidate()
{
    FreeReservedMemory();
    InnerInvalidate();
}

void DemoMemBase::AllocReservedMemory(size_t size)
{
    m_pReservedMemory = InnerAlloc(size);
}

void DemoMemBase::FreeReservedMemory()
{
    if ( m_pReservedMemory )
    {
        InnerFree(m_pReservedMemory);
        m_pReservedMemory = NULL;
    }
}

NEXUTIL_END
