﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ActiveDOCallContext_H_
#define _ActiveDOCallContext_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <OnlineCore/src/Platform/Core/EventHandler.h>
#include <NetZ/src/Core/DOCore/DOCallContext.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX
class Event;

// Group=Results Classes

/*
Summary: This class defines the context of a call to a duplicated object.
Remarks: This class is used to define the call context of calls made to fetch and migrate
duplicated objects and of RMCs.
*/
class ActiveDOCallContext: public DOCallContext
{
    //Group=Type Definitions
public:
    ActiveDOCallContext(DOHandle hTargetStation=INVALID_DOHANDLE,
                        qBool bSynchronousCall=true);

    virtual ~ActiveDOCallContext();

    // {secret}
    inline qBool KeepCallMessage() const;   //n1769: Added const.

    // {secret}
    qBool PerformCallAndWait();
    // {secret}
    qBool PerformCall();
    // {secret}
    virtual qResult SendCallMessage(Message* pCallMessage);

    // {secret}
    qResult SendCallMessageToStation(Message* pCallMessage, DOHandle hStation, qUnsignedInt32 uiFlags);

    // {secret}
    Message* GetCallMessage();

private:
    // Deactivate copy-ctor + op=
    // {secret}
    ActiveDOCallContext(const ActiveDOCallContext&);
    // {secret}
    ActiveDOCallContext& operator=(const ActiveDOCallContext&);

    qResult _SendCallMessage(Message* pCallMessage);

    qBool IsOverReliableSendBufferLimit(const Station *station, qUnsignedInt32 uiFlags) const;
    qBool IsOverReliableSendBufferLimit(const DORef &station, qUnsignedInt32 uiFlags) const;
    qBool IsOverReliableSendBufferLimit(const Selection *pTargets, qUnsignedInt32 uiFlags) const;

    qBool IsInvalidSubStreamID(const Station *station) const;
    qBool IsInvalidSubStreamID(const DORef &station) const;
    qBool IsInvalidSubStreamID(const Selection *pTargets) const;


    qBool IsUserDO();
#ifdef NEX_CLIENT_PBPOOL
    void SetMessageTargetType(Message*);
#endif // NEX_CLIENT_PBPOOL

protected:
    // Messages...
    // {secret}
    Message* m_pCallMessage;
};


END_NAMESPACE_NEX

#endif
