﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _CreateDuplicaProcess_H_
#define _CreateDuplicaProcess_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/DOCore/DOCore.h>
#include <OnlineCore/src/Core/Job.h>
#include <NetZ/src/Core/DOCore/DOClassesTable.h>
#include <OnlineCore/src/Platform/Core/MemoryManager.h>

BEGIN_NAMESPACE_NEX

class CreateDuplicaProcess : public RootObject
{
public:
    CreateDuplicaProcess(DOHandle hObject, DOHandle hMasterStation, LogicalClock lcMasterVersion, Message* pMessage, DOHandle hSender, qBool bOwnMessage = false);
    CreateDuplicaProcess(const CreateDuplicaProcess& oCreateDuplicaProcess);

    virtual ~CreateDuplicaProcess();

    CreateDuplicaProcess* Clone();

    void PromoteAfterCreation(qUnsignedInt16 uiCallID, const qList<DOHandle>* plstDupSet);

    qBool ValidateSchedulingContext();
    qBool Execute(qBool bCheckSchedulingContext = true);

protected:
    qBool CreateDuplicaImpl();

private:
    DOHandle         m_hObject;
    DOHandle         m_hMasterStation;
    LogicalClock     m_lcMasterVersion;
    Message*         m_pMessage;
    DOHandle         m_hSender;
    qBool            m_bOwnMessage;
    qBool            m_bPromoteAfterCreation;
    qUnsignedInt16   m_uiCallID;
    const qList<DOHandle>* m_plstDupSet;
    qBool            m_bCheckSchedulingContext;
};

class JobExecuteCreateDuplicaProcess: public Job
{
public:
    JobExecuteCreateDuplicaProcess(CreateDuplicaProcess* pProcess);
    virtual ~JobExecuteCreateDuplicaProcess();
    virtual void Execute();

private:
    CreateDuplicaProcess* m_pProcess;
};


CreateDuplicaProcess::CreateDuplicaProcess(DOHandle hObject, DOHandle hMasterStation, LogicalClock lcMasterVersion, Message* pMessage, DOHandle hSender, qBool bOwnMessage)
    : m_hObject(hObject),
      m_hMasterStation(hMasterStation),
      m_lcMasterVersion(lcMasterVersion),
      m_pMessage(pMessage),
      m_hSender(hSender),
      m_bOwnMessage(bOwnMessage),
      m_bPromoteAfterCreation(false),
      m_uiCallID(0),
      m_plstDupSet(NULL),
      m_bCheckSchedulingContext(true)
{
    if (pMessage && bOwnMessage)
    {
        m_pMessage = qNew Message(*pMessage);
    }
}

CreateDuplicaProcess::CreateDuplicaProcess(const CreateDuplicaProcess& oCreateDuplicaProcess)
    : m_hObject(oCreateDuplicaProcess.m_hObject),
      m_hMasterStation(oCreateDuplicaProcess.m_hMasterStation),
      m_lcMasterVersion(oCreateDuplicaProcess.m_lcMasterVersion),
      m_pMessage(oCreateDuplicaProcess.m_pMessage),
      m_hSender(oCreateDuplicaProcess.m_hSender),
      m_bOwnMessage(true),
      m_bPromoteAfterCreation(oCreateDuplicaProcess.m_bPromoteAfterCreation),
      m_uiCallID(oCreateDuplicaProcess.m_uiCallID),
      m_plstDupSet(NULL),
      m_bCheckSchedulingContext(oCreateDuplicaProcess.m_bCheckSchedulingContext)
{
    if (oCreateDuplicaProcess.m_pMessage)
    {
        m_pMessage = qNew Message(*oCreateDuplicaProcess.m_pMessage);
    }
    if (oCreateDuplicaProcess.m_plstDupSet)
    {
        m_plstDupSet = qNew qList<DOHandle>(*oCreateDuplicaProcess.m_plstDupSet);
    }
}

CreateDuplicaProcess::~CreateDuplicaProcess()
{
    if (m_pMessage && m_bOwnMessage)
    {
        qDelete m_pMessage;
    }
    if (m_plstDupSet)
    {
        qDelete m_plstDupSet;
    }
}

CreateDuplicaProcess* CreateDuplicaProcess::Clone()
{
    return qNew CreateDuplicaProcess(*this);
}

void CreateDuplicaProcess::PromoteAfterCreation(qUnsignedInt16 uiCallID, const qList<DOHandle>* plstDupSet)
{
    m_bPromoteAfterCreation = true;
    m_uiCallID = uiCallID;
    if (plstDupSet)
    {
        m_plstDupSet = qNew qList<DOHandle>(*plstDupSet);
    }
}

qBool CreateDuplicaProcess::ValidateSchedulingContext()
{

    if (m_bCheckSchedulingContext)
    {
        // Check if we must queue the operation in a different scheduler
        DOClass* pDOClass = DOClassesTable::GetDOClass(m_hObject.GetDOClassID());
        SYSTEMCHECK(pDOClass!=NULL);
        Scheduler* pSchedulerToUse = pDOClass->GetAlternateSchedulerToUse();
        if (pSchedulerToUse)
        {
            CreateDuplicaProcess* pClone = Clone();
            JobExecuteCreateDuplicaProcess* pJob = qNew JobExecuteCreateDuplicaProcess(pClone);
            pSchedulerToUse->Queue(pJob);
            return false;
        }
    }

    return true;
}

qBool CreateDuplicaProcess::Execute(qBool bCheckSchedulingContext)
{

    m_bCheckSchedulingContext = bCheckSchedulingContext;

    if (!ValidateSchedulingContext())
    {
        return true;
    }

    qBool bResult = CreateDuplicaImpl();

    if (bResult && m_bPromoteAfterCreation)
    {
        ChangeMasterStationOperation opChangeMasterStation(
#ifdef TRACE_ENABLED
            _T(__FILE__),
            __LINE__,
#endif
            m_hMasterStation,
            m_hObject,
            m_hMasterStation,
            MasterStationRef(Station::GetLocalStation(), m_lcMasterVersion+1),
            m_plstDupSet,
            ChangeMasterStationOperation::Unknown);
        opChangeMasterStation.SendOutcomeAfterExecution(m_uiCallID);
        bResult = opChangeMasterStation.Execute(false);
    }

    return bResult;
}

qBool CreateDuplicaProcess::CreateDuplicaImpl()
{

    DOHandle hDiscoveredObject = m_hObject;

    // Can probably be replaced with a SYSTEMCHECK.
    if (DOCore::GetInstance()->HasStartedTermination())
    {
        return false;
    }

    // Check class category.
    if (DOClass::GetCategory(hDiscoveredObject.GetDOClassID())==DOClass::Unknown)
    {
        if (ObjDupProtocol::GetInstance()->FlagIsSet(ObjDupProtocol::IgnoreUnknownClasses))
        {
            return false;
        }
        else
        {
            SYSTEMCHECK(false);
            return false;
        }
    }

    // SECURITY ISSUE.  It would be possible for a cheater to fake a
    // discovery message - to a station where the object exists.  This
    // would have the effect of changing the master station of the
    // object.

    // Check if a duplica is already there.
    {
        DuplicatedObject::Ref refDiscoveredObject(hDiscoveredObject);
        if (refDiscoveredObject.IsValid())
        {
            QTRACE(TRACE_DUPSPACE,_T("Processing DISCOVER message: ") << (refDiscoveredObject->GetClassNameString()) << _T(" ") << hex << ( (qUnsignedInt32)hDiscoveredObject) << _T(" already exists"));
            // It is valid to redundant receive duplicas of objects,
            // because of migrations or faults.

            // We take this opportunity to update the manager ID.  The
            // one I've got should be correct - but I never know...
            // (and in fact, it does happen, at bootstrap - see bug
            // 269.
            if (m_lcMasterVersion!=refDiscoveredObject->GetMasterVersion())
            {
                // Discovery message comes from a
                // master whose version is not the one
                // I have.  The master is typically not
                // the same, in which case we call
                // perform migration, that will ensure
                // that the master is the most recent.
                // The master could also be the same,
                // if two quick migrations occurs (A to
                // B to A), or when an orphan duplica
                // is created (version 1), and then
                // discovered (version 2, same master).

                // TODO.  Should we complete relevant migration
                // context?
                refDiscoveredObject->ChangeMasterStation(m_hSender, refDiscoveredObject->GetMasterStation(), MasterStationRef(m_hMasterStation, m_lcMasterVersion), NULL, ChangeMasterStationOperation::Unknown);
            }
            if (refDiscoveredObject->MainRefIsDeleted())
            {
                if (!refDiscoveredObject.GetDOPtr()->AddToStoreAsDuplica(m_hSender, m_pMessage, false))
                {
                    return false;
                }
            }
            else
            {
                if (refDiscoveredObject->IsADuplicationMaster())
                {
                    // TODO - At some point, when migration is correct - we should
                    // systemcheck here (or simply call the operation - it will
                    // systemcheck within the state machine).
                    // This may occur if the migration just occured.  The master sent
                    // a message and a migration occurs at the same time.  But the migration
                    // completes before the message is received.
                    // see bug 4381 for further details.
                    QTRACE(
                        D_TRACE_MIGRATION,
                        _T("Ignore UpdateDataSetOperation in CreateDuplica. Discovered object : '") << (refDiscoveredObject->GetHandle().GetClassNameString()) <<
                        _T("' from station: ") << hex << ( refDiscoveredObject->GetHandle().GetValue()) <<
                        _T(" is the master."));

                }
                else
                {

                    // I also use the message to update the object...
                    UpdateDataSetOperation opUpdate(
#ifdef TRACE_ENABLED
                        _T(__FILE__),
                        __LINE__,
#endif
                        m_hSender, refDiscoveredObject.GetHandle(), m_pMessage);
                    opUpdate.Execute(false);
                    // When we connect an orphan duplica, we fall in this
                    // case: the object's main ref isn't deleted, thus
                    // there is no need to add it.  We must still, however,
                    // signal any fetch contexts and clear the orphan
                    // duplica flag.
                    SYSTEMCHECK(!refDiscoveredObject->FlagIsSet(DuplicatedObject::OrphanDuplica)); // This was cleared while updating
                    CallRegister::GetInstanceRef().SignalRelevantFetchContextes(refDiscoveredObject.GetHandle(), DOCallContext::Success);
                }
            }
            return true;
        }
    }

    QTRACE(TRACE_DUPSPACE,_T("Processing DISCOVER message: A duplica of ") << (hDiscoveredObject.GetClassNameString()) << _T(" ") << hex << ( (qUnsignedInt32)hDiscoveredObject) << _T(" will be created (master is on Station ") << hex << ( (qUnsignedInt32)m_hMasterStation) << _T(")"));

    DuplicatedObject* pDO=DuplicatedObject::CreateDuplica(hDiscoveredObject, MasterStationRef(m_hMasterStation, m_lcMasterVersion));
    {
        if (!pDO->AddToStoreAsDuplica(m_hSender, m_pMessage, false))
        {
            // We checked earlier that the object does not exist.  Thus
            // the only reason this operation fails is if the call is
            // denied.
            // SYSTEMCHECK(SystemError::GetLast()==SYSTEMERROR_GEN_ACCESS_DENIED);
            pDO->ReleaseMainReference();
            return false;
        }
    }
    return true;
}

JobExecuteCreateDuplicaProcess::JobExecuteCreateDuplicaProcess(CreateDuplicaProcess* pProcess)
    : Job(Job::NetZ, _T("JobExecuteCreateDuplicaProcess"))
{
    m_pProcess=pProcess;
}

JobExecuteCreateDuplicaProcess::~JobExecuteCreateDuplicaProcess()
{
    if (m_pProcess!=NULL)
    {
        qDelete m_pProcess;
        m_pProcess = NULL;
    }
}

void JobExecuteCreateDuplicaProcess::Execute()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
    m_pProcess->Execute(false);
    qDelete m_pProcess;
    m_pProcess = NULL;
}


END_NAMESPACE_NEX

#endif
