// *---------------------------------------------------------------------------*
//  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
//  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.
//
//  Copyright (C)2011-2014 Nintendo  All rights reserved.
//
//  These coded instructions, statements, and computer programs contain
//  proprietary information of Nintendo and/or its licensors and are 
//  protected by national and international copyright laws.
//  They may not be disclosed to third parties or copied or duplicated
//  in any form, in whole or in part, without the prior written consent
//  of Nintendo.
//  The content herein is highly confidential and should be handled
//  accordingly.
// *---------------------------------------------------------------------------*

#ifndef _DOCore_DDL_
#define _DOCore_DDL_

//Compatibility with DDL compiler
#define CAFE_NEX_DDL_COMPATIBILITY

#ifdef CAFE_NEX_DDL_COMPATIBILITY
// This symbol is used to indicate to the DDL compiler that
// DUPLICATED_OBJECTS are allowed - this will open up all DO related
// keywords.
// It is absolutly requried to include this file to declare any
// dataset since the DDL compiler will attempt to add a default
// dataset property to all datasets if required - and this dataset
// property is declared here (basic_update_protocol).
#define ALLOW_DUPLICATED_OBJECTS
#endif //CAFE_NEX_DDL_COMPATIBILITY

#use <OnlineCore/OnlineCore.ddl>
#include <NetZ/src/Core/DOCore/DOCoreDSProperties.ddl>
#include <NetZ/src/Core/DOCore/DOCoreTypes.ddl>
#include <NetZ/src/Core/DOCore/DOCoreDefines.h>

// Dead reckoning.  Should eventually be moved in an extension.
dsproperty extrapolation_filter: code, dsdecl, updateprotocol;

// The default update protocol
dsproperty basic_update_protocol: updateprotocol;

doclassproperty bootstrapdo: code;

// These two properties are automatically set by the compiler.  They
// are builtin the DDL compiler - and do not need to be declared here.
#ifdef CAFE_NEX_DDL_COMPATIBILITY
doclassproperty coredo: code;
#else //CAFE_NEX_DDL_COMPATIBILITY
//doclassproperty coredo: code;
#endif //CAFE_NEX_DDL_COMPATIBILITY
// doclassproperty userdo: code;

#ifdef CAFE_NEX_DDL_COMPATIBILITY
doclassproperty nonglobal: code;
#endif //CAFE_NEX_DDL_COMPATIBILITY

// Note.  The order in which the classes appear in this file is
// important.  The order will define the index, which in turns defines
// the order in which object discovery are sent accross the network.
// In some cases (e.g. MulticastGroup vs Session), this order is
// important.

// This object is the root of the DO hiearchy.

doclass RootDO {
	action AddDuplicaLocation(dohandle hOperationOrigin, dohandle hLocation, bool bForceRedundantAdd, dohandle hAvailableRouting);
	// This action ends up called by the DupSpace extension, when a duplica is not
	// relevant.
	action DeleteDuplica(dohandle hLocation);
	action RemoveFromCachedDuplicationSet(dohandle hLocation);
	coreclassid = 2;
};

// Station.  

dataset StationInfo {
	dohandle  m_hObserver;	
    uint32    m_uiMachineUID;
};

dataset StationState {
	uint16 m_ui16State;
};

dataset ConnectionInfo {
	// Connection info.
	string m_strStationURL1;
	string m_strStationURL2;
	string m_strStationURL3;
	string m_strStationURL4;
	string m_strStationURL5;

	// Speed.
	uint32 m_uiInputBandwidth;
	uint32 m_uiOutputBandwidth;
	uint32 m_uiInputLatency;
	uint32 m_uiOutputLatency;
	
	// Initialization state
	bool m_bURLInitialized;
};

dataset StationIdentification {
	uint32 m_uiProcessType; // Tool, Client, Server.
	string m_strProcessName;
	uint32 m_uiProductVersion;
	string m_strIdentificationToken;
	string m_strPlayerName;
	uint32 m_uiPrincipalId;
	byte  m_uiLanguageCode;
	uint32 m_uiNetZVersion;
	uint32 m_uiApplicationVersion;
	uint32 m_uiNATProperties;
	REVISION = 1;
	uint32 m_uiPlatformID;
} constant;

doclass Station: RootDO {
	StationInfo m_dsInfo;
	StationState m_dsStationState;
	ConnectionInfo m_dsConnectionInfo;
	StationIdentification m_dsIdentification;

	void SignalAsFaulty(uint32 uiErrorCode);
    coreclassid = 3;
#ifdef CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, coredo;
#else //CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo;
#endif //CAFE_NEX_DDL_COMPATIBILITY

// Session.

dataset UserDefinedState {
	uint32 m_uiUserDefinedState;
};

dataset SessionState {
	byte m_bySessionState;
};


dataset SessionInfo {
	cstr	 m_szSessionName[SESSIONNAME_LEN];
	dohandle m_dohRootMulticastGroup;
	uint32   m_uiSessionID;
};

dataset SharedSessionDescription {
	cstr m_szAttributes[SHAREDSESSIONDESCRIPTION_ATTRIBUTES_LEN];
	cstr m_szSessionDiscoveryType[SHAREDSESSIONDESCRIPTION_TYPE_LEN];
	cstr m_szSessionDiscoveryContext[SHAREDSESSIONDESCRIPTION_CONTEXT_LEN];
};

doclass Session: RootDO {
	SessionState m_dsSessionState;
	UserDefinedState m_dsUserDefinedState;
	SessionInfo m_dsSessionInfo;

	SharedSessionDescription m_dsSessionDiscoveryDescription;

	bool SynchronizeTermination(dohandle hLeavingStation);
	void RetrieveURLs(dohandle hStation, out qlist<stationurl> lstURLs);
	
    void SyncStationList(in qvector<uint32> stationHandleValues);
	
	void DestroySessionFromRemote();
    void DestroySessionWithMasterCheckFromRemote();
    coreclassid = 4;
#ifdef CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, globaldo, coredo;
#else //CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, globaldo;
#endif //CAFE_NEX_DDL_COMPATIBILITY

//----------------------------------------------------------------------------------------

dataset Range {
	uint32 m_uiFirst;
	uint32 m_uiLast;
};

//The object ID Generator
doclass IDGenerator: RootDO {
	void RequestIDRangeFromMaster(in uint32 uiNbRequestedID, out uint32 iFirst, out uint32 iLast);
	Range m_oMasterRange;
	coreclassid = 5;
#ifdef CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, globaldo, coredo;
#else //CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, globaldo;
#endif //CAFE_NEX_DDL_COMPATIBILITY

// Fault recovery election mecanism.

doclass PromotionReferee: RootDO {
	void ConfirmElection(in dohandle hObject, in dohandle hFaultyMaster, in dohandle hNewMaster);
	void ElectNewMaster(in dohandle hObject,
						in dohandle hFaultyMaster, in byte byMasterVersion,
						in dohandle hCalleeStation, in byte byAffinity, uint16 uiWaitTime,
						out dohandle hNewMaster, out byte byNewMasterVersion);
	void DeclinePromotion(in dohandle hObject, in dohandle hFaultyMaster);
	void RequestConfirmFaultyMaster(in dohandle hFaultyMaster);
	void ReplyConfirmFaultyMaster(in dohandle hFaultyMaster, in byte byResult);
	coreclassid = 6;
#ifdef CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, globaldo, coredo;
#else //CAFE_NEX_DDL_COMPATIBILITY
} bootstrapdo, globaldo;
#endif //CAFE_NEX_DDL_COMPATIBILITY


// All well known instances.

wellknown Session g_hdoSession;
wellknown PromotionReferee g_hdoPromotionReferee;

#endif
