﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DOCoreErrors_H_
#define _DOCoreErrors_H_

//group=Error Code Enumerations
BEGIN_NAMESPACE_NEX


// Description: This enumeration defines the success codes that may be returned for the DOCore facility.
/* Please see man pages for details 
 
 */
namespace DOCoreSuccessCode
{

/*
Description: Enumerates the success codes for the DOCore facility that an operation may return.
Remarks: Details as to the cause of each SuccessCode are detailed in the documentation
of each individual $PRODUCTNAME$ method that may return a result.
*/
/* Please see man pages for details 
 
*/
enum SuccessCode
{
    // The call terminated successfully.
    Success         = 1,
    // The call has been postponed. A call may be postponed via CallMethodOperation::PostponeOperation.
    CallPostponed   = 2

};
}

//group=Error Code Enumerations

// Description: This enumeration defines the error codes that may be returned for the DOCore facility.
/* Please see man pages for details 
 
 */
namespace DOCoreErrorCode
{

/*
Description: Enumerates the error codes for the DOCore facility that a call may return.
Remarks: Reasons as to the cause of each ErrorCode are detailed in the documentation
of each individual $PRODUCTNAME$ method that may return a result.
*/
/* Please see man pages for details 
 
*/
enum ErrorCode
{

    // The target station of the call could not be reached.
    StationNotReached               = 1,  //
    // The call was invoked but the target station was disconnected before a response was received.
    TargetStationDisconnect         = 2,  //
    // The call was invoked but the local station is leaving the session and a response
    // will never be received.
    LocalStationLeaving             = 3, //
    // The object on which the call was called could not be found on the target station of the call.
    ObjectNotFound                  = 4, //
    // The object was found, but the role of the object was not
    // the one expected.  For example, this occurs if the flag
    // TargetObjectMustBeMaster was set but an RMC was called
    // on a Duplica instead of a Duplication master.
    InvalidRole                     = 5, //
    // The call failed because it did not complete before the time-out (set for the call) expired.
    CallTimeout                     = 6, //
    // The method invocation of an RMC failed.  This can be caused by
    // conflicting versions of the DDL between the caller and the
    // callee. This outcome is only valid if the context is an RMCContext.
    RMCDispatchFailed               = 7, //
    // The migration of an object is in progress. This outcome is only valid if the context
    // is a MigrationContext.
    MigrationInProgress             = 8, //
    // The object was found, but the TargetObjectMustHaveAuthority flag was set and the
    // object did not have authority. Refer to DuplicatedObject::HasAuthority for
    // more details about object authority.
    NoAuthority                     = 9, //
    // No target station was specified for this call.
    NoTargetStationSpecified        = 10, //
    // The call to JoinSession failed.
    JoinFailed                      = 11, //
    // The call to JoinSession was denied.
    JoinDenied                      = 12, //
    // JoinSession failed because the connectivity test failed.
    ConnectivityTestFailed          = 13, //
    // The result of the call is unknown.
    Unknown                         = 14, //
    // Upon tear down, some references or selection iterators are still pointing to duplicated
    // objects that are being destroyed.
    // Use DORef::TraceReferences() to find them.
    UnfreedReferences               = 15, //

    //n1769
    JobTerminationFailed            = 16, //


    //n1769
    InvalidState                    = 17, //

    //n1769
    FaultRecoveryFatal              = 18, //

    FaultRecoveryJobProcessFailed   = 19, //

    StationInconsistency = 20, //

    AbnormalMasterState = 21, //

    VersionMismatch  = 22  //
};
//Note: When adding new results, it is necessary to change NEX_QRESULT_TO_STR in OnlineCore/src/Platform/Core/Result.cpp.
//See operations done when adding error codes to the wiki.
//If the errors the application should handle have increased, add them to the list of errors the application should handle in the overall.rst of the common manual.

}

END_NAMESPACE_NEX

#endif // _DOCoreErrors_H_

