﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DOFilters_H_
#define _DOFilters_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOFilter.h>

BEGIN_NAMESPACE_NEX

// Only let CoreDO pass.
class CoreDOFilter: public DOFilter
{
public:
    CoreDOFilter() {}
    virtual ~CoreDOFilter() {}

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);

#ifdef TEST_ENABLED
    static qBool Test(DOClassID idTestClass);
#endif
};

// Only let UserDO pass.
class UserDOFilter: public DOFilter
{
public:
    UserDOFilter() {}
    virtual ~UserDOFilter() {}

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);
};

// Only let some DO pass, based on ClassID.
class IsADOFilter: public DOFilter
{
public:
    IsADOFilter(DOClassID idDOCID);
    virtual ~IsADOFilter() {}

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);

    virtual DOHandle GetMinimumValidHandle();
    virtual DOHandle GetMaximumValidHandle();

private:
    DOClassID m_idDOCID;
};

// Only let some DO pass, based on being ans instance
//of a subclass.
class IsAKindOfDOFilter: public DOFilter
{
public:
    IsAKindOfDOFilter(DOClassID idDOCID);
    virtual ~IsAKindOfDOFilter() {}

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);

    virtual DOHandle GetMinimumValidHandle();
    virtual DOHandle GetMaximumValidHandle();

    DOClassID GetDOClassID() const
    {
        return m_idDOCID;
    }

#ifdef TEST_ENABLED
    static qBool Test(DOClassID idFirst,  DOHandle hFirst,
                      DOClassID idSecond, DOHandle hSecond,
                      DOClassID idThird,  DOHandle hThird);
#endif
private:
    // Result cache
    DOClassID m_idLastDOCID;
    qBool m_bLastResult;
    DOClassID m_idDOCID;
};

class ConstantDOFilter: public DOFilter
{
public:
    // If true, all DO pass through, if false, all are blocked.
    ConstantDOFilter(qBool bDesiredAnswer);
    virtual ~ConstantDOFilter() {};

    virtual qBool Filter(DuplicatedObject* pDO)
    {
        DUMMY_PARAM(pDO);
        return m_bDesiredAnswer;
    }
    virtual qBool Filter(DOHandle hDO)
    {
        DUMMY_PARAM(hDO);
        return m_bDesiredAnswer;
    }

    virtual DOHandle GetMaximumValidHandle();

private:
    qBool m_bDesiredAnswer;
};

class OrDOFilter: public DOFilter
{
public:
    OrDOFilter(DOFilter *pfFilter1, DOFilter *pfFilter2);
    virtual ~OrDOFilter();

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);

    virtual DOHandle GetMinimumValidHandle();
    virtual DOHandle GetMaximumValidHandle();

#ifdef TEST_ENABLED
    static qBool Test();
#endif
private:
    DOFilter *m_pfFilter1, *m_pfFilter2;
};

class AndDOFilter: public DOFilter
{
public:
    AndDOFilter(DOFilter *pfFilter1, DOFilter *pfFilter2);
    virtual ~AndDOFilter();

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);

    virtual DOHandle GetMinimumValidHandle();
    virtual DOHandle GetMaximumValidHandle();

#ifdef TEST_ENABLED
    static qBool Test();
#endif
private:
    DOFilter *m_pfFilter1, *m_pfFilter2;
};

class NotDOFilter: public DOFilter
{
public:
    NotDOFilter(DOFilter *pfFilter1);
    virtual ~NotDOFilter();

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);

private:
    DOFilter *m_pfFilter1;
};


class IsADuplicationMasterDOFilter: public DOFilter
{
public:
    IsADuplicationMasterDOFilter();
    virtual ~IsADuplicationMasterDOFilter();

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);
private:
};

class IsADuplicaDOFilter: public DOFilter
{
public:
    IsADuplicaDOFilter();
    virtual ~IsADuplicaDOFilter();

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);
private:
};

class IsAWellKnownDOFilter: public DOFilter
{
public:
    IsAWellKnownDOFilter(qBool bIsOrIsNot);
    virtual ~IsAWellKnownDOFilter();

    virtual qBool Filter(DuplicatedObject* pDO);
    virtual qBool Filter(DOHandle hDO);
private:
    qBool m_bIsOrIsNot;
};

END_NAMESPACE_NEX

#endif //_DOFilters_H_

