/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DOID_H_
#define _DOID_H_

#include <OnlineCore/src/Platform/Core/Type.h>

BEGIN_NAMESPACE_NEX

// Group = ID Classes

/*
Description: Defines a duplicated object ID.
Remarks: A DOID is a unique identification (22-bit value) within a duplicated object class. It is
assigned to an object upon creation and may be either generated automatically by
$PRODUCTNAME$ or set by the user.

If DOID s are set by the user, the user must ensure that within each duplicated object
class each DOID is unique.

The DOHandle of a duplicated object is made up of the object's DOClassID and its DOID.
See Also: DuplicatedObject::Create
*/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
class DOID : public RootObject
{
public:
    enum
    {
        //
        InvalidID = 0
    };

    /*
    Parameters:
    nID: A duplicated object ID.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    DOID() : m_nID(static_cast<ID>(InvalidID)) {}

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    DOID(const DOID& doid) : m_nID(doid.m_nID) {}

    /* Please see man pages for details 
     
     
     
     
     
     */
    explicit DOID(qUnsignedInt32 nID) : m_nID(static_cast<ID>(nID)) {}

    const DOID &operator=( const DOID &idOther )
    {
        m_nID = idOther.m_nID;
        return *this;
    }

    /* Please see man pages for details 
     
     
     */
    operator qUnsignedInt32 () const
    {
        return GetValue();
    }

    /* Please see man pages for details 
     
     
     */
    qUnsignedInt32 GetValue() const
    {
        return static_cast<qUnsignedInt32>(m_nID);
    }

private:
    ID m_nID;
};

END_NAMESPACE_NEX

#endif

