/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DORef_H_
#define _DORef_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/Scheduler.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>


BEGIN_NAMESPACE_NEX


class Message;
class DuplicatedObject;

// Group=Template Classes

#ifdef DEBUG_VERSION
class DOSelections;
#endif

//n1589:memo:DORef operation requires SystemLock
/*
Description: Creates a reference (accessor) to a duplicated object.
Remarks: References are implemented via a template that is detailed under RefTemplate.
See Also: RefTemplate
*/
/* Please see man pages for details 
 
 
 
 
 
 */
class DORef: public RootObject
{
    // Group=Methods
public:

    /*
    Description: Creates a reference to a duplicated object.
    Parameters:
    hDO: A DOHandle that refers to the duplicated object to which a
    reference is wanted.
    oRef: A reference to a duplicated object.
    pDO: A pointer to a duplicated object.
    Remarks: The method IsValid should be used to check if the reference was correctly constructed.
    See Also: Duplicated Object Overview, RefTemplate
    */
    /* Please see man pages for details 
     
     
     
     */
    DORef();
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     */
    DORef(DOHandle hDO);
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    DORef(const DORef& oRef);
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    DORef(DuplicatedObject* pDO);

    virtual ~DORef();

    // {secret}
    void EmptyInit();

    DORef& operator=(const DOHandle& hDO);
    DORef& operator=(const DORef& oRef);
    DORef& operator=(DuplicatedObject* pDO);

    // Description: Returns the DOHandle used to construct this reference.
    // Returns: The DOHandle used to construct this reference.
    // See Also: Duplicated Object Overview, RefTemplate
    /* Please see man pages for details 
     
     
     */
    DOHandle GetHandle() const
    {
        return m_hReferencedDO;
    }

    // {secret}
    qUnsignedInt32 GetHandleValue() const
    {
        return m_hReferencedDO.GetValue();
    }

    /*
    Description: Returns whether this DORef is a Soft reference to a duplicated object.
    Returns: true if this DORef is a Soft reference to a duplicated object, false if it is a Hard reference or
    an error occurred.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. Refer to SetSoft for further information about hard and soft references.
    See Also: IsHard, SetSoft, SetHard, Soft reference
    */
    /* Please see man pages for details 
    

           
           

    
    
     */
    qBool IsSoft() const
    {
        return !m_bLockRelevance;
    }

    /*
    Description: Returns whether this DORef is a Hard reference to a duplicated object.
    Returns: true if this DORef is a Hard reference to a duplicated object, false if it is a Soft reference or
    an error occurred.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. Refer to SetHard for further information about hard and soft references.
    See Also: IsSoft, SetSoft, SetHard, Hard reference
    */
    /* Please see man pages for details 
    

           
           
    
    
     */
    qBool IsHard() const
    {
        return m_bLockRelevance;
    }

    /*
    Description: Sets this DORef to a Soft reference.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether 
    you want to keep the referenced object updated.

    For example, if you want to keep a list of your teammates and their current status (position,
    inventory, and so on) even when they are in a different part of the virtual world to your avatar,
    you will want to keep Hard reference s to their avatars so that the avatar duplicas will always
    be updated. On the other hand, if you wish to keep a list of all the objects of a given type,
    say enemy avatars, that exist in the world, you will probably want to use Soft references
    so you do not update these objects unnecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station, due to a Duplication space match condition.
    See Also: IsHard, IsSoft, SetHard, Soft reference
    */
    /* Please see man pages for details 
    

      
      
      
      
      
      
      
      
      
      

      
      
      
      
      
      
      

     
     */
    void SetSoft();

    /*
    Description: Sets this DORef to a Hard reference.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and, therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your
    teammates and their current status (position, inventory, and so on) even when they are in a different part
    of the virtual world to your avatar, you will want to keep Hard reference s to their avatars so that
    the avatar duplicas will always be updated. On the other hand, if you wish to keep a list of all the objects
    of a given type, say enemy avatars, that exist in the world, you will probably want to use Soft references
    so you do not update these objects unnecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station, due to a Duplication space match condition.
    See Also: IsHard, IsSoft, SetSoft, Hard reference
    */
    /* Please see man pages for details 
    

      
      
      
      
      
      
      
      
      
      

      
      
      
      
      
      
      

     
     */
    void SetHard();

    /* Please see man pages for details 
     
     
     */
    DuplicatedObject* GetDOPtr() const
    {
        return m_poReferencedDO;
    }

    /*
    Summary: Returns whether the DORef instance is valid.
    Returns: true if the reference construction is valid, false if not or an error occurred.
    Remarks: This method will return false if you pass a reference or a DOHandle to an invalid or
    unknown object or to an object of the wrong type. The possible errors that will be returned
    are SYSTEMERROR_OBJDUP_DO_NOT_FOUND and SYSTEMERROR_GEN_INVALID_OPERATION. If you create a
    reference by specifying a DOHandle to the object, the error SYSTEMERROR_OBJDUP_MISMATCHED_DOCLASS
    is also possible.
    See Also: Duplicated Object Overview, RefTemplate
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    qBool IsValid() const
    {
        return GetDOPtr()!=NULL;
    }

    //Description: Tests whether the reference is valid.
    //
    //Returns: true if the DORef is now valid, false if not or an error occurred.
    //See Also: Wait, IsValid
    /* Please see man pages for details 
     
     
     
     
     */
    qBool Poll()
    {
        if (m_poReferencedDO==NULL)
        {
            Acquire();
        }
        return IsValid();
    }


    /*
    Description: Cause the system to wait until the DORef is valid or the timeout is exceeded.
    Parameters:
    uiTimeout: The maximum time in milliseconds that the system will wait. The default uiTimeout is infinite.
    Returns: true if the DORef is valid, false if not or the timeout is exceeded.
    Remarks: The error SYSTEMERROR_GEN_TIMEOUT will be signalled if the timeout is exceeded.
    See Also: Poll
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test(DOClassID idTestObjectClass);
#endif

public:
    // TODO: Should be protected.

    // {secret}
    void Release();


#ifdef DEBUG_REFERENCES
    /*
    Description: Traces all the references to a given duplicated object.
    Parameters:
    uiTraceFlag: The trace flag used when tracing. To ensure tracing always takes
    place TRACE_ALWAYS can be used.
    hDO: A handle to the duplicated object for which the references will be traced.
    Remarks: In the debug version, to help debug reference leaks, the name of a DORef can be traced
    via this method. For each reference, the name specified via NameReference will be traced.
    In the release version, this method has no effect.
    See Also: NameReference
    */
    static void TraceReferences(qUnsignedInt64 uiTraceFlag, DOHandle hDO);
    // {secret}
    static void TraceAllReferences(qUnsignedInt64 uiTraceFlag);
    //{secret}
    static qUnsignedInt32 CountReferences(DOHandle hDO);
    //{secret}
    static void TraceReferencesImpl(qUnsignedInt64 uiTraceFlag, DOHandle hDO);
    //{secret}
    static qUnsignedInt32 CountReferencesImpl(DOHandle hDO);

    /*
    Description: Sets the name of a reference.
    Remarks: In the debug version, to help debug reference leaks, the name of a DORef can be traced
    via TraceReferences. In the release version, this method has no effect.
    See Also: TraceReferences
    */
    void NameReference(const qChar* szReferenceName)
    {
        m_szReferenceName = szReferenceName;
    }
#else
    //{secret}
    void NameReference(const qChar* /*szReferenceName*/) {}
    //{secret}
    static void TraceReferences(qUnsignedInt64 /*uiTraceFlag*/, DOHandle /*hDO*/) {}
#endif

protected:
    void SetPointer(DuplicatedObject* pDO);
    void ReleasePointer();
    void Acquire();

private:
    inline void AcquireImpl();

#ifdef DEBUG_REFERENCES
    const qChar* m_szReferenceName;
    static ForcedCriticalSection s_debugCS;
#endif

    DuplicatedObject* m_poReferencedDO;
    DOHandle m_hReferencedDO;

    qBool m_bLockRelevance;

#ifdef DEBUG_VERSION
    DOSelections* m_poStore;
#endif
};

END_NAMESPACE_NEX

#endif
