﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DatasetCGMacros_H_
#define _DatasetCGMacros_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

////////////////////////////////////////////////////////////////////////////////
// Macros to implement specific dataset and dataset::buffer.
////////////////////////////////////////////////////////////////////////////////

#define _DS_ARRAY_OP(OP, SIZE)              {qInt i; for (i=0; i<SIZE; i++) {OP;}}

#define _DS_ARRAY_OP_BEGIN(SIZE)            {qInt i; for (i=0; i<SIZE; i++)
#define _DS_ARRAY_OP_END()                  }

#define _DS_IMPLEMENT(DS)

#define _DS_GENERAL_DECLARATIONS(DS)    \
    DATASET(DS)() {InitVariables();}    \
    static const qChar* GetDatasetNameString() { return _T(#DS); }              \
    qBool FormatVariableValue(Variable* pVariable, String *pstrResult) const;   \
    void AddSourceTo(Message *pMsg, Time tTimestamp, qBool bDiscovery);         \
    void CallOperationOnVars(Operation::Event eEvent, void* pDOOperation);

#define _DS_DECLARE_VAR(type,name)                  type name
#define _DS_DECLARE_VECTOR(type,name)               MemberVector<type> name; \
                                                    typedef MemberVector<type>::Iterator name##Iterator;
#define _DS_DECLARE_LIST(type,name)                 MemberList<type> name;    \
                                                    typedef MemberList<type>::Iterator name##Iterator;
#define _DS_DECLARE_QUEUE(type,name)                MemberQueue<type> name;

#define _DS_DECLARE_EXTRAPOLATION_VAR(type,name)    PHBDRVAR(type) name;

#define _DS_DECLARE_ARRAY(type,name,size)           type name[size];
#define _DS_DECLARE_EXTRAPOLATION_ARRAY(type,name,size) PHBDRVAR(type) name[size];


#define _DS_UPDATE_TIME(T)                  qUnsignedInt32 uiPosition=m_vectorTime.NewValue(T, oContinuityBreak);

#define _DS_UPDATE_VAR(name)                Buffer::name.NewValue(name, uiPosition);  \
                                            Buffer::name.ComputeParameters(s_oPHBDRParameters, m_vectorTime, byRole, uiPosition, oContinuityBreak);



#define _DS_UPDATE_ARRAY(name,size)         {for (qInt i=0; i<size; i++) {_DS_UPDATE_VAR(name[i]);}}

#define _DS_COPYDATASET_VAR(VAR)            VAR=oOther.VAR
#define _DS_ASSIGN_OPERATOR(DS)             void operator =( const DATASET(DS) &oOther )
#define _DS_COPYDATASET_ARRAY(VAR, SIZE)    _DS_ARRAY_OP(_DS_COPYDATASET_VAR(VAR[i]),SIZE)

#define _DS_TMPL_EQUAL_OPERATOR(DS) qBool operator ==( const DataSetTemplate<DATASET(DS)> &oOther ){                \
                                    return *this==( static_cast<const DATASET(DS) &>(oOther) );         \
                                    }
#define _DS_EQUAL_OPERATOR(DS)      qBool operator ==( const DATASET(DS) &oOther )

#define _DS_EQUAL_VAR(VAR)                  if(!(VAR == oOther.VAR)){ return false; }
#define _DS_EQUAL_ARRAY(VAR, SIZE)          _DS_ARRAY_OP(_DS_EQUAL_VAR(VAR[i]),SIZE)

#define _DS_READY_TO_USE()  m_vectorTime.DelayedValueReadyToUse()
#define _DS_DELAYED_UPDATE_TIME()                               \
                                                                qByte byDelayedContinuityBreak;                 \
                                                                qUnsignedInt32 uiPosition=m_vectorTime.UseDelayedValue(&byDelayedContinuityBreak);
#define _DS_DELAYED_UPDATE_VAR(name)                                \
                                                                    Buffer::name.UseDelayedValue(uiPosition);       \
                                                                    Buffer::name.ComputeParameters(s_oPHBDRParameters, m_vectorTime, DUPLICA, uiPosition, byDelayedContinuityBreak);
#define _DS_DELAYED_UPDATE_ARRAY(name,size) {for (qInt i=0; i<size; i++) {_DS_DELAYED_UPDATE_VAR(name[i]);}}

#define _DS_ADDESE_VAR(VAR)                 dESE+=pBuffer->ComputeESE(s_oPHBDRParameters, VAR, pBuffer->VAR, tPredictionTime);
#define _DS_ADDESE_ARRAY(VAR,SIZE)          _DS_ARRAY_OP(_DS_ADDESE_VAR(VAR[i]),SIZE)

#define _DS_REFRESHVARS(DS)                 qBool Refresh(DATASET(DS)* pDS, const Time& tRefreshTime)
#define _DS_COPY_VAR(VAR)                   pDS->VAR=VAR
#define _DS_EXTRAPOLATE_VAR(VAR)            pDS->VAR=PredictValue(s_oPHBDRParameters, VAR, tRefreshTime)

#define _DS_COPY_ARRAY(VAR,SIZE)            _DS_ARRAY_OP(_DS_COPY_VAR(VAR[i]), SIZE)
#define _DS_EXTRAPOLATE_ARRAY(VAR,SIZE)     _DS_ARRAY_OP(_DS_EXTRAPOLATE_VAR(VAR[i]), SIZE)

#define _DS_PREDICT_NEWPOINT_VAR(VAR)       VAR=m_dsBuffer.VAR.PredictValue(s_oPHBDRParameters, m_dsBuffer.GetTimeVector(), tTimestamp);
#define _DS_PREDICT_NEWPOINT_ARRAY(VAR,size) {for (qInt i=0; i<size; i++) {_DS_PREDICT_NEWPOINT_VAR(VAR[i]);}}

#define _DS_FormatMatchingVariableValue(DDLTYPE, VAR)       if (pVariable->GetName()==_T(#VAR)) {return DDLTYPE::FormatVariableValue((void*)&VAR, pstrResult);}

// Buffered...

//n1589:memo:not used
#define _DS_DECLARE_EXTRAPOLATED()              Buffer m_dsBuffer;  \
                                            void ExtractFrom(Message* pMsg) {SetSubStreamID(pMsg->GetSubStreamID());m_dsBuffer.ExtractFrom(pMsg);} \
                                            qBool Refresh() {m_dsBuffer.Refresh(this, GetTimestamp()); return true;}    \
                                            qBool Refresh(const Time& tRefreshTime) {m_dsBuffer.Refresh(this, tRefreshTime); return true;}  \
                                            qBool UsesBufferedRefresh() const {return true;}
#define _DS_IMPLEMENT_EXTRAPOLATED(DS)

// Loopback

#define _DS_DECLARE_LOOPBACK() static TimeInterval s_tiLoopbackDelay; \
                               static TimeInterval s_tiLoopbackSamplingInterval; \
                               Time m_tLastLoopbackUpdate; \
                               Buffer m_oLoopback;      \
                               qBool HasLoopback() const {return true;}     \
                               Buffer* GetLoopbackModel() {return &m_oLoopback;}    \
                               static TimeInterval GetLoopbackDelay() {return s_tiLoopbackDelay;}   \
                               static void SetLoopbackDelay(TimeInterval tiLoopbackDelay) {s_tiLoopbackDelay=tiLoopbackDelay;}  \
                               static TimeInterval GetLoopbackSamplingInterval() {return s_tiLoopbackSamplingInterval;} \
                               static void SetLoopbackSamplingInterval(TimeInterval tiSamplingInterval) {s_tiLoopbackSamplingInterval=tiSamplingInterval;} \
                               void UpdateLoopback();   \
                               qBool RefreshLoopback();



#define _DS_DECLARE_NOLOOPBACK() qBool HasLoopback() const {return false;}      \
                                 Buffer* GetLoopbackModel() {return NULL;}



#define _DS_IMPLEMENT_UPDATELOOPBACK(DS)    \
void DATASET(DS)::UpdateLoopback() {                    \
    Time tNow=Time::GetSessionTime();       \
    if (tNow-m_tLastLoopbackUpdate > s_tiLoopbackSamplingInterval) {    \
        m_tLastLoopbackUpdate=tNow;         \
        UpdateModel(&m_oLoopback, tNow + s_tiLoopbackDelay, true);  \
    }   \
}

//n1589:mark:TransportLock:off
#define _DS_IMPLEMENT_REFRESHLOOPBACK(DS)   \
qBool DATASET(DS)::RefreshLoopback() {                  \
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock());       \
    DSCHECK_ROLE_IS_DM(false);                                                      \
    m_oLoopback.DelayedUpdateModel();                                               \
    return true;            \
}



#define _DS_IMPLEMENT_LOOPBACK(DS) TimeInterval DATASET(DS)::s_tiLoopbackDelay=0;   \
                                   TimeInterval DATASET(DS)::s_tiLoopbackSamplingInterval=50;   \
                                   _DS_IMPLEMENT_UPDATELOOPBACK(DS)    \
                                   _DS_IMPLEMENT_REFRESHLOOPBACK(DS)

// The following macros are used to check if the role of the
// DuplicatedObject that contains the dataset is appropriate for the
// operation beeing called.  These checks can only be performed in
// DEBUG mode, where an extra variable is kept.

#ifdef DEBUG_VERSION

#define DSCHECK_ROLE_IS_DUPLICA(ERROR) {if (m_pDOContainer && m_pDOContainer->IsADuplicationMaster()) {SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_INVALID_ON_MASTER,0); return ERROR;}}
#define DSCHECK_ROLE_IS_DM(ERROR) {if (m_pDOContainer && m_pDOContainer->IsADuplica()) {SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA,0); return ERROR;}}
#define EDSCHECK_ROLE_IS_DUPLICA(ERROR) {if (DataSetTemplate<SpecificDataSet>::m_pDOContainer && DataSetTemplate<SpecificDataSet>::m_pDOContainer->IsADuplicationMaster()) {SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_INVALID_ON_MASTER,0); return ERROR;}}
#define EDSCHECK_ROLE_IS_DM(ERROR) {if (DataSetTemplate<SpecificDataSet>::m_pDOContainer && DataSetTemplate<SpecificDataSet>::m_pDOContainer->IsADuplica()) {SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA,0); return ERROR;}}

#else

#define DSCHECK_ROLE_IS_DUPLICA(ERROR)
#define DSCHECK_ROLE_IS_DM(ERROR)
#define EDSCHECK_ROLE_IS_DUPLICA(ERROR)
#define EDSCHECK_ROLE_IS_DM(ERROR)

#endif


#endif
