﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DirectStream_H_
#define _DirectStream_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/RefBuffer.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DOCore.h>

BEGIN_NAMESPACE_NEX

class Job;
/* Please see man pages for details 

*/
class DirectStream: public SystemComponent
{
public:
    /* Please see man pages for details 
    
    */
    enum StreamType
    {
        Unreliable, //
        Reliable //
    };
    /* Please see man pages for details 
    
    */
    struct Data : public RootObject
    {
        StationID id; //
        StreamType type; //
        SubStreamID subStreamID; //!< SubStreamID during reliable communication.
        RefBuffer buffer; //
    };
public:

    DECLARE_SYSCOMPONENT_METHODS(DirectStream,SystemComponent);

    /* Please see man pages for details 
    
     */
    DirectStream();
    /* Please see man pages for details 
    
     */
    virtual ~DirectStream();
    /* Please see man pages for details 
    
    
    
    

    
     */
    static DirectStream* GetInstance()
    {
        if (DOCore::GetInstance() == NULL)
        {
            return NULL;
        }
        return DOCore::GetInstance()->GetDirectStream();
    }

public:
    /* Please see man pages for details 
    

    
    
    
    */
    void Enable();
    /* Please see man pages for details 
    

    
    
    */
    void Disable();

    /* Please see man pages for details 
    

    

    
    
    
    

    
    
    
    
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    */
    qResult Send(StationID id, StreamType type, const qByte* pBuffer, qUnsignedInt32 size)
    {
        return Send(id, type, 0, pBuffer, size);
    }
    /* Please see man pages for details 
    

    

    
    
    
    
    
    
    

    
    
    
    
    
    
    
    */
    qBool Send(CallContext *pContext, StationID id, StreamType type, const qByte* pBuffer, qUnsignedInt32 size)
    {
        return Send(pContext, id, type, 0, pBuffer, size);
    }

    /* Please see man pages for details 
    

    

    
    
    

    
    
    
    
    
    
    
    
    
    
    
    */
    qResult SendUnreliable(StationID id, const qByte* pBuffer, qUnsignedInt32 size)
    {
        return Send(id, Unreliable, 0, pBuffer, size);
    }

    /* Please see man pages for details 
    

    

    
        
    
    

    
    
    
    
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    

    

    */
    qResult SendReliable(StationID id, SubStreamID subStreamID, const qByte* pBuffer, qUnsignedInt32 size)
    {
        return Send(id, Reliable, subStreamID, pBuffer, size);
    }
    /* Please see man pages for details 
    

    

    
    
    
    
    
    

    
    
    
    
    
    
    

    
    */
    qBool SendReliable(CallContext *pContext, StationID id, SubStreamID subStreamID, const qByte* pBuffer, qUnsignedInt32 size)
    {
        return Send(pContext, id, Reliable, subStreamID, pBuffer, size);
    }

    /* Please see man pages for details 
    

    
    
    
    
    

    
    
    
    
    
    
    
    
    
    */
    qResult SendAll(StreamType type, const qByte* pBuffer, qUnsignedInt32 size);

    /* Please see man pages for details 
    

    
    
    

    
    
    
    
    
    
    
    
    */
    qResult SendUnreliableAll(const qByte* pBuffer, qUnsignedInt32 size)
    {
        return SendAll(Unreliable, pBuffer, size);
    }

    /* Please see man pages for details 
    

           
           
           
           
           
           
    
    */
    void GetReceivedData(qVector<DirectStream::Data>& recvData);

    /* Please see man pages for details 
    
    
    */
    qUnsignedInt32 GetReceivedDataNum();

    /* Please see man pages for details 
    

    
    

    
    */
    qUnsignedInt32 GetMaxReliableSendBufferNum() const
    {
        return m_maxReliableSendBufferNum;
    }
    /* Please see man pages for details 
    

    
    
    

    
    */
    void SetMaxReliableSendBufferNum(qUnsignedInt32 bufferNum)
    {
        m_maxReliableSendBufferNum = bufferNum;
    }

    /* Please see man pages for details 
    
    

    
    
    

    

    */
    qInt32 GetPendingReliableBufferNum(StationID id, SubStreamID subStreamID =  SubStreamIDDefine::SYSTEM);

    /* Please see man pages for details 
    
    
    */
    qUnsignedInt32 GetMaxReliableSendBufferSize() const
    {
        return m_maxReliableSendBufferSize;
    }
    /* Please see man pages for details 
    
    */
    void SetMaxReliableSendBufferSize(qUnsignedInt32 bufferSize)
    {
        m_maxReliableSendBufferSize = bufferSize;
    }

    /* Please see man pages for details 
    
    
    */
    qUnsignedInt32 GetMaxUnreliableSendBufferSize() const
    {
        return m_maxUnreliableSendBufferSize;
    }
    /* Please see man pages for details 
    
    */
    void SetMaxUnreliableSendBufferSize(qUnsignedInt32 bufferSize)
    {
        m_maxUnreliableSendBufferSize = bufferSize;
    }

    /* Please see man pages for details 
    
    
    */
    qUnsignedInt32 GetMaxRecvBufferNum() const
    {
        return m_maxRecvBufferNum;
    }
    /* Please see man pages for details 
    
    
    */
    void SetMaxRecvBufferNum(qUnsignedInt32 bufferNum)
    {
        m_maxRecvBufferNum = bufferNum;
    }

private: // private method

    // Common Send function with or without call context.
    qResult SendCommon(CallContext *pContext, StationID id, StreamType type, qByte subStreamID, const qByte* pBuffer, qUnsignedInt32 size);

    // Receive callback function passed to PRUDPProtocol.
    static void ReceiveCallback(nn::nex::StationID srcId, Buffer* pBuffer, qUnsignedInt32 packetTypeFlags, qByte subStreamID);

    void SetFatalError(qResult err);
    void SetRecvOverflow(qBool overflow)
    {
        m_recvOverflow = overflow;
    }
    qBool IsRecvOverflow()
    {
        return m_recvOverflow;
    }

    /* Please see man pages for details 
    
    
    
    
    
    */
    qUnsignedInt32 CheckSendBufferSize(DirectStream::StreamType type, qUnsignedInt32 bufferSize);

private: // private members
    qResult Send(StationID id, StreamType type, qByte subStreamID, const qByte* pBuffer, qUnsignedInt32 size);
    qBool Send(CallContext *pContext, StationID id, StreamType type, qByte subStreamID, const qByte* pBuffer, qUnsignedInt32 size);

    qVector<DirectStream::Data> m_recvData; // Received data
    CriticalSection m_csRecvData;
    class AckCallBackJob;
    Job* m_pAckCallBackJob; // Job that updates the status of a call context that has completed receiving Ack.

    // Variable parameter such that tuning is possible for the application
    qUnsignedInt32 m_maxReliableSendBufferNum; // Number of reliable send buffers
    qUnsignedInt32 m_maxReliableSendBufferSize; // Maximum size of reliable transmission
    qUnsignedInt32 m_maxUnreliableSendBufferSize; // Maximum size of unreliable transmission
    qUnsignedInt32 m_maxRecvBufferNum; // Maximum number of receive buffers
    qBool m_recvOverflow;
};

END_NAMESPACE_NEX

#endif // _DirectStream_H_
