﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ElectionTable_H_
#define _ElectionTable_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

BEGIN_NAMESPACE_NEX

class ElectionTable: public RootObject
{
public:
    enum {TABLEENTRY_EXPIRATION=45000};


    ElectionTable();
    virtual ~ElectionTable();

    // Entry point.
    DOHandle ElectNewMaster(DOHandle hObject,
                            DOHandle hFaultyMaster, qByte byFaultyMasterVersion,
                            DOHandle hCalleeStation,
                            TimeInterval tiDecisionDelay, qByte byAffinity,
                            TimeInterval* ptiWaitHint, qByte *pbyNewMasterVersion);

#ifdef TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags) const;
#endif  // TRACE_ENABLED

    void ConfirmElection(DOHandle hObject, DOHandle hFaultyMaster, DOHandle hNewMaster);
    void DeclinePromotion(DOHandle hObject, DOHandle hFaultyMaster);
    void ProcessLeavingStation(DOHandle hStation);
    qBool ElectionIsInProgress();
    qBool ElectionIsInProgressForTermination();
#ifdef TEST_ENABLED
    DOHandle TestElectNewMaster(DOHandle hObject, DOHandle hFaultyMaster,
                                DOHandle hCalleeStation, TimeInterval tiDecisionDelay,
                                qByte byAffinity, TimeInterval* ptiWaitHint=NULL);
    static qBool Test();
#endif

private:
    /* Types */

    typedef std::pair<DOHandle,DOHandle> ElectionID;

    // TODO- should eventually become more responsible of itself!
    class TableEntry
    {
    public:
        TableEntry(ElectionID id);

        void Trace(qUnsignedInt64 uiTraceFlags) const;

        void SetDeadline(Time tNewDeadline);

        void ConfirmResult();
        qBool IsConfirmed() const;

        void CloseElection();
        qBool IsClosed() const;

        const Time& GetExpiration() const
        {
            return m_tExpiration;
        }
        const Time& GetElectionDeadline() const
        {
            return m_tElectionDeadline;
        }

        TimeInterval ComputeWaitHint() const;

        qBool HasExpired() const;

        const ElectionID& ID() const
        {
            return m_id;
        }
        ElectionID& ID()
        {
            return m_id;
        }

        qByte GetAffinity() const
        {
            return m_byAffinity;
        }

        qBool PreferNewValue(qByte byAffinity) const;
        void UpdateEntry(DOHandle hNewMaster,qByte byNewMasterVersion,qByte byAffinity,TimeInterval tiDecisionDelay);
        void TestDeadline();

    private:
        ElectionID m_id;

    public:

        DOHandle m_hNewMaster;
        qByte m_byNewMasterVersion;

    private:

        qByte m_byAffinity;
        Time m_tExpiration;
        Time m_tElectionDeadline;
        qBool m_bConfirmed;
    };

    typedef qMap<ElectionID, TableEntry>::iterator iterator;
    typedef qMap<ElectionID, TableEntry>::const_iterator const_iterator;


private:
    // Implementation.

    DOHandle UpdateElectionTable(ElectionID idElection, DOHandle hProposedNewMaster, qByte byProposedNewMasterVersion, TimeInterval tiDecisionDelay, qByte byAffinity,
                                 TimeInterval* ptiWaitHint, qByte *pbyNewMasterVersion);
    void CleanupTable();

private:

    qMap<ElectionID, TableEntry> m_mapTableEntries;
};

END_NAMESPACE_NEX

#endif
