/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Selection_H_
#define _Selection_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class DORef;

// Group=Utility Subsystem

class Store;

class Selection: public RootObject
{
    // Group=
public:
    typedef enum _Flags
    {
        OwnsReference=1,
        LocksRelevance=2
    } Flags;

    Selection(qByte byFlags=0);
    virtual ~Selection();

    void SetFlags(qByte byFlags);

    typedef qMap<DOHandle, DuplicatedObject*>::iterator iterator;
    typedef qMap<DOHandle, DuplicatedObject*>::const_iterator const_iterator;

    iterator begin()
    {
        return m_map.begin();
    }
    const_iterator begin() const
    {
        return m_map.begin();
    }

    iterator end()
    {
        return m_map.end();
    }
    const_iterator end() const
    {
        return m_map.end();
    }

    const_iterator lower_bound(DOHandle hValue) const
    {
        return m_map.lower_bound(hValue);
    }
    const_iterator find(DOHandle hValue) const
    {
        return m_map.find(hValue);
    }
    iterator find(DOHandle hValue)
    {
        return m_map.find(hValue);
    }

    qUnsignedInt size() const
    {
        return m_map.size();
    }
    qBool empty() const
    {
        return m_map.empty();
    }

    qBool Contains(DOHandle hValue) const
    {
        return m_map.find(hValue)!=end();
    }

    void Clear();
    void ClearOnCorruption();

    qBool Add(DOHandle hDO);
    qBool Add(DuplicatedObject* pDO);
    qBool Remove(DOHandle hDO);
    qBool Remove(const DuplicatedObject* pDO);

    // Cache stuff
    // Creates virtual member functions in the Selection class so that we can access via a Selection pointer.
    virtual void AddCache(DuplicatedObject* pDO)
    {
        DUMMY_PARAM(pDO);    // Do not update the cache unless this is a SelectionMaster instance.
    }
    virtual void RemoveCache(DOHandle hDO)
    {
        DUMMY_PARAM(hDO);    // Do not update the cache unless this is a SelectionMaster instance.
    }
    virtual qBool IsCache() const
    {
        return false;    // Always false because it does not have a cache.
    }
    virtual Selection* GetMasterSelection() const
    {
        SYSTEMCHECK(false);    // Get rid of this because it is never called.
        return NULL;
    }
    virtual Selection* GetCacheSelection(DOClassID docid) const
    {
        DUMMY_PARAM(docid);    // Cache is always NULL.
        return NULL;
    }

public:

#ifdef DEBUG_REFERENCES
    static void TraceReferencesImpl(qUnsignedInt64 uiTraceFlag, DOHandle hDO);
    static qUnsignedInt32 CountReferencesImpl(DOHandle hDO);

    void NameSelection(const qChar* szName)
    {
        m_szName = szName;
    }
#else
    void NameSelection(const qChar* /*szName*/) {}
#endif

private:

    void ApplyRemovalRules(DuplicatedObject* pDO) const;
    void ApplyInsertionRules(DuplicatedObject* pDO) const;

protected:
#ifdef DEBUG_REFERENCES
    const qChar* m_szName;
#endif

    qMap<DOHandle, DuplicatedObject*> m_map;
    qByte m_byFlags;
};

/* Please see man pages for details 
 
 */
class SelectionCache: public Selection
{
public:
    SelectionCache(qByte byFlags=0);
    virtual ~SelectionCache();

    // Cache stuff
    virtual qBool IsCache() const
    {
        return true;    // Always true because it has a cache.
    }
    void SetMasterSelection(Selection* pMaster)
    {
        m_pMasterSelection = pMaster;    // Set the cache source.
    }
    virtual Selection* GetMasterSelection() const
    {
        return m_pMasterSelection;    // If it has a cache, we can get back to the source.
    }
    virtual Selection* GetCacheSelection(DOClassID docid) const
    {
        DUMMY_PARAM(docid);    // We can never get the cache.
        SYSTEMCHECK(false);
        return NULL;
    }

protected:
    Selection* m_pMasterSelection;
};

/* Please see man pages for details 
 
 
 
 */
class SelectionMaster: public SelectionCache
{
public:
    SelectionMaster(qByte byFlags=0);
    virtual ~SelectionMaster();

    // Cache stuff
    virtual void AddCache(DuplicatedObject* pDO); // Implement because we update the cache.
    virtual void RemoveCache(DOHandle hDO); // Implement because we update the cache.
    virtual qBool IsCache() const
    {
        return false;    // Always false because it does not have a cache.
    }
    virtual Selection* GetMasterSelection() const
    {
        SYSTEMCHECK(false);
        return NULL;
    }
    virtual Selection* GetCacheSelection(DOClassID docid) const; // Implement so that we can get the cache.
    void AddCacheSelection(DOClassID docid, SelectionCache* pSelection); // Add a cache.
protected:
    qMap<DOClassID, Selection*> m_cacheSelectionMap; // This is so we can have a collection of class objects.
};

END_NAMESPACE_NEX

#endif
