﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// This file must have the name of the DOClass that it implements, Station

#ifndef _Station_H_
#define _Station_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX
class Message;
class EndPoint;
class StationOperation;
class StationDetectionAlgorithm;
END_NAMESPACE_NEX
#include <NetZ/src/Core/DOCore/DOClass.h>
#include <NetZ/src/Core/DOCore/StationDDL.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/UpdateContextMap.h>
#include <NetZ/src/Core/DOCore/MessageBundle.h>
#include <NetZ/src/Core/DOCore/DistanceComputationCache.h>
#include <OnlineCore/src/Transport/Interface/EndPointInfoInterface.h>
#include <OnlineCore/src/Transport/Interface/EndPointInfo.h>


#define NEX_STATION_PLAYER_NAME_LEN 16  //

/* Please see man pages for details 
 
 */
BEGIN_NAMESPACE_NEX

namespace StationEvent
{
typedef enum
{
    None,
    Participated,
    Left
} Type;
}

//n2788:
/* Please see man pages for details 
 
 

        

        
        

        
*/
typedef void (*StationEventCallback)(DOHandle hStation, StationEvent::Type oEvent);

enum
{
    USER_STATION_MESSAGE=0
};

//Group=Miscellaneous Types and Macros

/*
Description: The round trip time for a message is unknown.
Remarks: The round trip time (RTT) for a message between two stations is unknown if the stations
are not connected directly.
See Also: Station::GetRTT, Station::GetRTTAverage
*/


class Observer;
class PRUDPEndPoint;

//Group=Session Management

// This macro is never defined.  It is used only to get around a compilation problem with
// DocJet, which does not like the inheritance to the DOCLASS(Station).
#ifdef FOR_DOC_PURPOSE
#include <NetZ/src/Core/DOCore/SelectionIteratorTemplate.h>
/*
Description: Retrieves specific information about a Station.
Remarks: This class is used to retrieve specific information about a Station and to set
the observer used when distance-based dead reckoning is invoked.
*/
/* Please see man pages for details 
 
 
 
 
 */
class Station : DuplicatedObject
{
public:
    /* Please see man pages for details 
     
     */
    typedef SelectionIteratorTemplate<Station> SelectionIterator;
#else
class Station: public DOCLASS(Station)
{
#endif
public:
    //n1589:moved static methods to here
    /* Please see man pages for details 
     
       
     */
    static const TimeInterval STATION_UNKNOWN_RTT=-1;

    /*
    Description: The input or output bandwidth for this Station is unknown.
    Remarks: The input and output bandwidth for a station's connection must be defined
    via Station::SetBandwidthInfo.
    See Also: Station::SetBandwidthInfo, Station::GetInputBandwidth, Station::GetOutputBandwidth
    */
    static const qUnsignedInt STATION_UNKNOWN_BANDWIDTH=0;
    /*
    Description: The input or output latency for this Station is unknown.
    Remarks: The input and output latency due to a station's connection must be defined
    via Station::SetLatencyInfo.
    See Also: Station::SetLatencyInfo, Station::GetInputLatency, Station::GetOutputLatency
    */
    static const qUnsignedInt STATION_UNKNOWN_LATENCY =0;

    // {secret}
    static void SetLocalStation(DOHandle hStation);

    //Group=Methods

    //Description: Returns the DOHandle of the local station.
    //Returns     : The DOHandle of the local station.
    /* Please see man pages for details 
     
     
     
     */
    // n2967 Moved this from Station.cpp and made it inline to speed it up
    inline static DOHandle GetLocalStation()
    {
        return s_hLocalStation.GetValue(PseudoSingleton::GetCurrentContext());
    }

    //Description: Returns a pointer to the instance of the local
    //Station.
    //Returns     : A pointer to the local station object, NULL if the
    //object does not exist.
    /* Please see man pages for details 
     
     
     
     */
    // n2967 Moved this from Station.cpp and made it inline to speed it up
    inline static Station* GetLocalInstance()
    {
        // Get from cache.
        Station* pStation = s_hLocalStationCache.GetValue(PseudoSingleton::GetCurrentContext());
        if (!pStation)
        {
            // Get from DOHandle when there is no cache.
            Station::Ref refStation(GetLocalStation());
            pStation = *refStation;

            // Update the cache.
            s_hLocalStationCache.SetValue(PseudoSingleton::GetCurrentContext(), pStation);
        }
        return pStation;
    }

    static void GetStationHandles(Role rSelectionType, qVector<DOHandle> &handles);
    static void GetStationHandles(qVector<DOHandle> &handles);

    /* Please see man pages for details 
     
     
     
     */
    static DOHandle ConvertStationIDToDOHandle(StationID stationId);
    /* Please see man pages for details 
     
     
     
     
     */
    static DOHandle ConvertConnectionIDToDOHandle(StationID stationId);

    // {secret}
    static DOHandle ConvertIDToDOHandle(StationID idStation);
    // {secret}
    static DOHandle ConvertDOIDToDOHandle(DOID idStation);

    /* Please see man pages for details 
     
     
     
     
     */
    static StationID ConvertDOHandleToID(DOHandle hStation);

    /* Please see man pages for details 
     
     
     
     */
    static StationID ConvertDOHandleToStationID(DOHandle hStation);

    /* Please see man pages for details 
     
     
     
     */
    static DOID ConvertDOHandleToDOID(DOHandle hStation);

    /*
    Description: Sets the process details for this Station.
    Parameters:
    uiType: The ProcessType of this Station, which may be one of UnknownProcess, ClientProcess, ServerProcess, or ToolProcess.
    strProcessName: The name of this process. This value is user-defined.
    uiProductVersion: The product version number of this process. This value is user-defined.
    Remarks: This method is used to define the type of process that a Station is and to define its name and version number.
    This information may be used to restrict the activities that given types of processes can perform. For example, $PRODUCTNAME$
    does not permit ToolProcesses to become a session master.

    Note that this method should only be called after the $PRODUCTNAME$ object has been created and before a Session is created or joined.
    See Also: ProcessType, GetProcessType, GetProcessName, GetProductVersion
    */
    static void SetLocalProcessInfo(qUnsignedInt32 uiType, const String& strProcessName, qUnsignedInt32 uiProductVersion=0);

    /*
    Description: Sets the identification token details for this Station.
    Parameters:
    strIdentificationToken: A user-defined string that is associated with this Station.
    Remarks: An identification token is a user-defined string that is associated with a given Station. One way
    to use an identification token is to use it to pass a password or other identifying tag when a Station attempts to
    join a Session to determine whether the Station has the right to join the Session. In this case, the
    information is passed to the Connection point Station so that that Station can make a decision about the join
    request. The identification token is set on the joining Station object and is then accessible via GetIdentificationToken.
    Refer to Session::RegisterJoinApprovalCallback for an example of how an identification token could be used.

    Note that this method should only be called after the $PRODUCTNAME$ object has been created and before a Session is created or joined.
    See Also: GetIdentificationToken
    */
    /* Please see man pages for details 
     
     
     
     
     */
    static void SetLocalIdentificationToken(const String& strIdentificationToken);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    static void SetLocalPlayerName(const String& playerName, qUnsignedInt8 languageCode);

    /* Please see man pages for details 
     
     
     
     
     
     */
    static void SetLocalApplicationVersion(qUnsignedInt32 applicationVersion);


    /*
    Description: Returns a DOHandle to the observer on the local station.
    Returns     : A DOHandle to the object on the local station that is the registered observer, or
    INVALID_DOHANDLE if there is no registered observer.
    See Also: RegisterObserver, UnregisterObserver, GetObserver, Dead Reckoning Overview
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    static DOHandle GetLocalObserver();

    /*
    Description: Registers the specified object as the observer on this Station.
    Parameters:
    idObserver: The DOHandle of the object that you want to register as the observer on this Station.
    Returns     : true if the observer was successfully registered, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION is signaled if an observer is already registered.
    Remarks: Either a Duplication master or a Duplica may be selected as the observer for dead reckoning
    calculations. Refer to the Dead Reckoning Overview for details as to the calculations performed.
    If no observer is registered and distance-based dead reckoning is invoked, then the default
    distance defined by ErrorToleranceFunction::SetDefaultDistance is used to calculate the error tolerance
    used in the dead reckoning algorithm.

    As an example, the following code creates two instances of spheres and defines the user-controlled sphere on
    the local station as the observer for that station:

           // Create and publish UserSphere
           Sphere *pUserSphere = Sphere::Create();
           pUserSphere->Publish();
           // Create and publish AISphere
           Sphere* pAISphere = Sphere::Create();
           pAISphere->Publish();
           // Register UserSphere as the observer on the local station
           Station::RegisterObserver(pUserSphere->GetHandle());

    Although only one object may be registered as an observer for a specified Station at any time, this
    does not restrict how many individual observer avatars can be set on a Station. To
    implement multiple observers you simply register an object that contains a list of observer
    avatars. For example, say that you have a split-screen mode with your game. Then when in split-screen
    mode, rather than registering a player's avatar as the observer you would register a "SplitScreen"
    object, which contains an "Observers" dataset that is a list of all the avatars
    that you want to be considered as observers. Then in the ComputeDistance callback for the SplitScreen
    object you would calculate the distance between a specific object and each of the observers, returning
    the minimum value. This ensures that as long as one of the observers is close enough to the object
    for it to require an update, an update will be sent.

    Note: As the distance between objects is calculated on a remote Station to that where the observer
    object is registered, the idObserver object must be published before it can be registered.
       See Also: GetObserver, UnregisterObserver, ErrorToleranceFunction::SetDefaultDistance,
       Dead Reckoning, Dead Reckoning Overview
           */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    static qBool RegisterObserver(DOHandle idObserver);

    /*
    Description: Unregisters the current observer.
    Returns     : true if the observer was successfully unregistered, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION is signaled if no observer is currently registered.
    See Also: GetObserver, RegisterObserver, Dead Reckoning Overview
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    static qBool UnregisterObserver();

    /*
    Description: Sends all messages stored in all message bundles.
    Remarks: When this method is called all messages in all message bundles on the local station will be sent. Once
    all bundles are sent all messages at the transport level are then also flushed.
    See Also: FlushBundle, BundlingPolicy
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     */
    static void FlushAllBundles(qBool bFlushTransportBundles = true);

    // {secret}
    static qBool SignalCompleteFault(qBool bProcessNow = true);
    // {secret}
    static void InitiateFaultProcessingForStation(DOHandle hFaultyStation, qUnsignedInt32 uiFault);
    // {secret}
    static qUnsignedInt GetNbLocalStations();

    static qUnsignedInt GetNbDuplicaStations();

    /*
    Remarks: A Station object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    when a station joins the session.
    */
    /* Please see man pages for details 
     
     
     */
    Station();

    /*
    Remarks: A Station object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when
    the associated station leaves the session.
    */
    /* Please see man pages for details 
     
     
     */
    virtual ~Station();

    //Group=Type Definitions

    //Description: Enumerates various Station states.
    //See Also: State
    /* Please see man pages for details 
     
     */
    enum State
    {
        // The State of the Station is unknown.
        //
        Unknown = 0,

        // The Station is in the process of joining the session.
        //
        JoiningSession = 1,

        // The Station is in the process of creating the session.
        //
        CreatingSession = 2,

        // The Station is currently participating in the session.
        //
        Participating = 3,

        // The Station is in the process of leaving the session.
        //
        Leaving = 4,

        // The Station is in the process of leaving the session due to a fault.
        //
        LeavingOnFault = 5
    };

    // {last three bits are reserved}

    // Description: Enumerates the process types that a Station may be.
    /* Please see man pages for details 
     
     */
    enum ProcessType
    {
        // The Station process type is unknown. This is the default process type.
        //
        UnknownProcess  =0,

        // The Station is a client process.
        //
        ClientProcess   =1,

        // The Station is a server process.
        //
        ServerProcess   =2,

        // The Station is a tool process. An example of a tool process is the $PRODUCTNAME$ Object
        // Monitor tool. A tool process cannot become a session master.
        //
        ToolProcess     =4
    };

    // {secret}
    enum ConnectionState
    {
        Disconnected,
        Connecting,
        Connected,
        Disconnecting
    };

    //Group=Methods

    /*
    Description: Returns the current State of the station.
    Returns     : The current State of the station, which is one of: Unknown, JoiningSession,
    CreatingSession, Participating, Leaving, or LeavingOnFault.
    See Also: State
    */
    /* Please see man pages for details 
     
     
     */
    State GetState() const
    {
        return (State) m_dsStationState.Get();
    }

    // {secret}
    void SetState(State uNewState);

    /*
    Description: Returns whether or not a Station is a peer.
    Returns     : true if the Station is a peer station, false if it is the local station.

    $ifdef NETZ$
    Remarks: In Net-Z, all remote stations in the Session are peer stations to a given Station.
    $endif$

    $ifdef ETERNA$
    Remarks: The peers of a given station depend on the implementation of the distributed
    broadcast. If a distributed broadcast is implemented the peers of a given station will
    consist of: the stations where the duplication masters of the duplicas on the local
    station reside, the stations where the duplicas of the duplication masters of the
    local station reside, and the station where the Duplication master of the
    RootMulticastGroup object resides. In addition, if the local station contains a
    Duplication master of a MulticastGroup, its peers will also include the stations of
    all of the MulticastGroup members. In contrast, if a distributed broadcast is
    disabled, the peers will consist of all the stations in the session.
    $endif$

    See Also: IsLocal
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    qBool IsAPeer()const;

    /*
    Description: Returns whether or not a Station is the local station.
    Returns     : true if the Station is the local station, false if it is a peer station.
    Remarks: The local station is typically the Duplication master of the Station
    but this is not always the case.
    See Also: IsAPeer
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    qBool IsLocal()const;

    qBool IsNotLeavingOrLeft() const;

    // {secret}
    void ReleaseOwnReference();
    // {secret}
    void AcquireOwnReference();
    // {secret}
    qBool OwnsReferenceToItself();

    /*
    Description: Returns the type of this process.
    Returns     : The type of this process as a qUnsignedInt32. The ProcessType may be one of UnknownProcess,
    ClientProcess, ServerProcess, or ToolProcess and is set via SetLocalProcessInfo.
    Remarks: Refer to DataSet::UpdateIsRequired for an example of this method in use.
    See Also: ProcessType, SetLocalProcessInfo, GetProcessName, GetProductVersion
    */
    /* Please see man pages for details 
     
     
     */
    qUnsignedInt32 GetProcessType() const;

    /*
    Description: Returns the name of this process.
    Returns     : The name of this process as a String. The ProcessName is user-defined and is set via
    SetLocalProcessInfo.
    See Also: ProcessType, SetLocalProcessInfo, GetProcessType, GetProductVersion
    */
    const String& GetProcessName() const;

    /*
    Description: Returns the product version number of this process.
    Returns     : The product version number of this process as a qUnsignedInt32. The ProductVersion is user-defined
    and is set via SetLocalProcessInfo.
    See Also: ProcessType, SetLocalProcessInfo, GetProcessType, GetProcessName
    */
    qUnsignedInt32 GetProductVersion() const;

    /*
    Description: Returns the identification token of this process.
    Returns     : The identification token of this process. An identification token is a user-defined string that is
    associated to a given Station.
    Remarks: Refer to Session::RegisterJoinApprovalCallback for an example of this method in use.
    See Also: SetLocalIdentificationToken
    */
    /* Please see man pages for details 
     
     
     */
    const String& GetIdentificationToken() const;

    /* Please see man pages for details 
     
     
     */
    const String& GetPlayerName() const;

    /* Please see man pages for details 
     
     
     */
    qUnsignedInt8 GetLanguageCode() const;

    /* Please see man pages for details 
     
     
     */
    qUnsignedInt32 GetPrincipalID() const;

    /* Please see man pages for details 
     
     
     */
    qUnsignedInt32 GetApplicationVersion() const;

    // {secret}
    qResult Send(Message* pMsg, qUnsignedInt32 uiFlags);

    //n1589
    void SetLinkObject(void* object)
    {
        m_pLinkObject = object;
    }

    void* GetLinkObject()
    {
        return m_pLinkObject;
    }

    // {secret}
    virtual qBool ApproveEmigration(qUnsignedInt32 uiReason);
    // {secret}
    virtual qBool ApproveFaultRecovery();

    // {secret}
    virtual qBool ValidOperation(DOOperation* pOperation);

    // n2370
    // {secret}
    virtual void OperationBegin(DOOperation* pOperation);

    // {secret}
    virtual void OperationEnd(DOOperation* pOperation);
    // {secret}
    void ReleaseSystemReferences();

    // {secret}
    void LeaveSessionNormally();
    // {secret}
    void LeaveSessionOnFault();

    //Group=Connection Information

    /*
    Description: Returns whether or not this Station is connected to the target Station.
    Returns     : true if this Station is connected to the target Station, false if not or an error occurred.
    */
    /* Please see man pages for details 
     
     
     
     */
    qBool IsConnected() const;

    //n1589
    qBool IsConnectedWithoutLock() const;

    // {secret}
    qBool IsFaulty() const;

    // {secret}
    qBool DiscoversGlobalObject(DuplicatedObject* pDO);

    // {secret}
    // Called in ProcessJoinRequest
    void SetConnection(EndPoint* pEndPoint);

    // {secret}
    UpdateContextMap& UpdateContextes() const
    {
        return m_mapUpdateContext;
    }

    /*
    Description: Returns the currently registered observer on this Station.
    Parameters:
    Returns     : A DOHandle to the object on the given station that is the registered observer, or
    INVALID_DOHANDLE if there is no registered observer.
    Remarks: If an observer, which is used in dead reckoning calculations, is registered then
    this method returns which object is the observer.
    See Also: RegisterObserver, UnregisterObserver, GetLocalObserver, Dead Reckoning Overview
    */
    /* Please see man pages for details 
     
     
     
     
     
     */
    DOHandle GetObserver()const ;

    //{secret}
    qReal ComputeDistanceTo(DOHandle hDO);

    //Group=Connection Information

    /*
    Description: Returns the current round trip time for a message between this Station and the local station.
    Returns     : The current round trip time (RTT) in milliseconds or STATION_UNKNOWN_RTT if this Station
    is not directly connected to the local station. The default initial value is 250 ms.
    Remarks: The round trip time is the time taken for a message to reach another Station
    and the acknowledgement to be received by the sender. The RTT can only be calculated
    if the two stations are directly connected to each other. The maximum time between messages that
    are used to calculate the RTT is defined by the PRUDP_KEEPALIVETIMEOUT.
    See Also: GetRTTAverage
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    TimeInterval GetRTT();

    /*
    Description: Returns the average round trip time for a message between this Station and the local station.
    Returns     : The average round trip time (RTT) in milliseconds or STATION_UNKNOWN_RTT if this Station is not
    directly connected to the local station. The default initial value is 250 ms.
    Remarks: The average round trip time is the average time taken for a message to reach another
    station and the acknowledgement to be received by the sender. The RTT can only be calculated
    if two stations are directly connected to each other. The maximum time between messages that are
    used to calculate the RTT is defined by the PRUDP_KEEPALIVETIMEOUT.

    The average round trip time is calculated using smoothed averages based on V. Jacobson's well-known algorithms.
    See Also: GetRTT
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     */
    TimeInterval GetRTTAverage();

    /*
    Description: Sets the input and output latency induced by this station's connection.
    Parameters:
    tiInputLatency: The input latency in milliseconds of this station's connection.
    tiOutputLatency: The output latency in milliseconds of this station's connection.
    Returns     : true if the station's latency was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION is signaled if this method is called on a Duplica.
    Remarks: SetLatencyInfo can only be called on a Duplication master. The method is used to
    account for asymmetric connection speeds between stations. $PRODUCTNAME$ will use this
    user-supplied information to improve time synchronization between stations. Refer to
    SessionClock::AdjustForStationLatency for more information on how the input and output
    latency affects time synchronization.
    See Also: GetInputLatency, GetOutputLatency, SessionClock::AdjustForStationLatency
    */
    qBool SetLatencyInfo(TimeInterval tiInputLatency, TimeInterval tiOutputLatency);

    /*
    Description: Returns the input latency of this station's connection.
    Returns     : The input latency in milliseconds or STATION_UNKNOWN_LATENCY if there is no
    information known about the station's connection.
    Remarks: The input latency of this station's connection is set via SetLatencyInfo.
    See Also: SetLatencyInfo, GetOutputLatency
    */
    TimeInterval GetInputLatency();

    /*
    Description: Returns the output latency of this station's connection.
    Returns     : The output latency in milliseconds or STATION_UNKNOWN_LATENCY if there is no
    information known about the station's connection.
    Remarks: The output latency of this station's connection is set via SetLatencyInfo.
    See Also: SetLatencyInfo, GetInputLatency
    */
    TimeInterval GetOutputLatency();

    /*
    Description: Sets the input and output bandwidth of this station's connection.
    Parameters:
    uiInputBandwidth: The input bandwidth in bits per second of this station's connection.
    uiOutputBandwidth: The output bandwidth in bits per second of this station's connection.
    Returns     : true if the station's bandwidth was successfully set, false if not or an error occurred.
    Remarks: This method can only be called on a Duplication master. SYSTEMERROR_GEN_INVALID_OPERATION
    is signaled if it is called on a Duplica.
    See Also: GetInputBandwidth, GetOutputBandwidth
    */
    qBool SetBandwidthInfo(qUnsignedInt uiInputBandwidth, qUnsignedInt uiOutputBandwidth);

    /*
    Description: Returns the input bandwidth of this station's connection.
    Returns     : The input bandwidth in bits per second or STATION_UNKNOWN_BANDWIDTH if there is no
    information known about the station's connection.
    Remarks: The input bandwidth of this station's connection is set via SetBandwidthInfo.
    See Also: SetBandwidthInfo, GetOutputBandwidth
    */
    TimeInterval GetInputBandwidth();

    /*
    Description: Returns the output bandwidth of this Station.
    Returns     : The output bandwidth in bits per second or STATION_UNKNOWN_BANDWIDTH if there is no
    information known about the station's connection.
    Remarks: The output bandwidth of this station's connection is set via SetBandwidthInfo.
    See Also: SetBandwidthInfo, GetInputBandwidth
    */
    TimeInterval GetOutputBandwidth();

    // {secret}
    void SetAtEOS();
    // {secret}
    void ClearAtEOS();
    // {secret}
    qBool AtEOS();

    // {secret}
    qUnsignedInt32 GetMachineUniqueID() const;

    // {secret}
    qBool TestAndSetFaultFlag();
    // {secret}
    qBool Faulty();

#ifdef TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;

    //n1589
    void TraceID(qUnsignedInt64 uiTraceFlags) const;

#endif  // TRACE_ENABLED

    // {secret}
    void InitLocalStationInfo();

    /*
     
     
     
     
     
     
     
     
     */
    PRUDPEndPoint* GetPRUDPEndPoint()const
    {
        return reinterpret_cast<PRUDPEndPoint*>(GetEndPoint());
    }

    EndPoint* GetEndPoint()const;


    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     */
    EndPointInfoInterface* GetEndPointInfoInterface() const;


    // {secret}
    MessageBundle *GetSharedMessageBundle()
    {
        return &m_oSharedMessageBundle;
    }
    // {secret}
    MessageBundle *GetUnreliableMessageBundle()
    {
        return &m_oUnreliableMessageBundle;
    }

#if NEX_ENABLE_HYBRID
    static void SessionMasterDisconnected();
#endif

    //Group=Methods

    /*
    Description: Returns a list of valid URLs for this Station.
    Parameters:
    plstURLs: A pointer to the list of valid StationURL s for this Station.
    Returns     : A list of valid StationURL s for this Station.
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     */
    void GetStationURLs(qList<StationURL> *plstURLs)const;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    const qChar* GetStationURL(qInt iIndex)const;

    /*
    Description: Sends all messages stored in a given message bundle.
    Parameters:
    bFlushTransportBundles: If true (the default), the transport message bundles will be flushed
    once the message bundle is flushed.
    Remarks: When this method is called all messages in the message bundle on the local station associated
    with this Station will be sent. If bFlushTransportBundles is set to true, transport bundles will be sent
    after the message bundle has been sent. Note that transport bundles need to be flushed periodically, so if
    you call this method you must set the bFlushTransportBundles parameter to true at regular intervals.
    See Also: FlushAllBundles, BundlingPolicy
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    qResult FlushBundle(qBool bFlushTransportBundles = true);

#ifdef TEST_ENABLED
    // {secret}
    void ForceStationURL(qInt iIndex, const qChar *szTransportURL);
#endif
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    qBool SignalFault(qBool bProcessNow = true);

    // {secret}
    void SignalAsFaulty(qUnsignedInt32 uiFaultEvent);

    static void SignalAsFaultyAllStations();

    // {secret}
    void InitIdentification(StationIdentification* pIdentification);

    //Description: Returns the ID of the Station.
    //Returns     : The StationID of the Station.
    /* Please see man pages for details 
     
     
     
     */
    StationID GetStationID() const;

    //n1589
    /* Please see man pages for details 
     
     
     
     */
    StationID GetConnectionID() const;

    //n1589
    qBool ConnectionIsPossible() const;

#if NEX_ENABLE_HYBRID
    //n2781
    qBool HasRoutingConnection(const InetAddress *router = NULL) const;
#endif
#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    qUnsignedInt32 GetNumOfRoutingConnections() const;
#endif

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    qBool KickStation();

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    static qBool KickMe();

    //n2788:
    // {secret}
    // static qBool IsFatalReliableSendBufferFull(Station::Ref &refStation);

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    void SetRoutingLimit(qBool b);
    qBool GetRoutingLimit()
    {
        return m_bRoutingLimit;
    }
#endif
    qUnsignedInt32 GetNetZVersion() const;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    static void RegisterOperationBeginCallback(void (*pfOperationBegin)(StationOperation* pOperation));

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    static void RegisterOperationEndCallback(void (*pfOperationEnd)(StationOperation* pOperation));

    static qBool GetAutoAddRecentPlayRecord();

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     */
    static void SetAutoAddRecentPlayRecord(qBool flag);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    static qUnsignedInt32 GetApprovedStationNum();

    /* Please see man pages for details 
     
     
     
     
     */
    qBool IsApproved() const
    {
        return IsPublished() || m_isApproved;
    }

    /* Please see man pages for details 
     
     
     */
    void SetApproved()
    {
        m_isApproved = true;
    }


    //n1589:moved private methods to here
private:
    qResult SendImpl(Message *pMessage, qUnsignedInt32 uiFlags);
    qResult QueueLocalMessage(Message* pMsg);
    qResult SendRemoteMessage(Message* pMsg, qUnsignedInt32 uiFlags);

    qResult FlushBundleImpl(qBool bFlushTransportBundles);

    static void FlushAllBundlesImpl();

    // {secret}
    EndPoint* ReleaseConnection();

    // {secret}
    qBool RegisterObserverImpl(DOHandle idObserver);

    virtual void TestInvariants();

    //n1769
#if NEX_ENABLE_SEND_TRACELOG_OPTIMIZATION
    __declspec(noinline) void sendTrace(qResult retVal, Message *pMsg);
#endif

private:
    friend class JobDisconnectStation;
    friend class JobConnectStation;

    // Local station.
    // {secret}
    static PseudoGlobalVariable<DOHandle> s_hLocalStation;
    static PseudoGlobalVariable<Station*> s_hLocalStationCache;

    // {secret}
    EndPoint* m_poEndPoint;

    // Info used to implement different functionality of the system.
    // {secret}
    mutable UpdateContextMap m_mapUpdateContext;

    DistanceComputationCache m_oDistanceComputation;

    qBool m_bAtEOS;
    AtomicValue m_bFaulty;

    // This message bundle either for reliable messages (when not
    // mixing reliability) or for both type of messages (when it is
    // mixed)
    MessageBundle  m_oSharedMessageBundle;
    // This message bundle is used for unreliable messages, when we
    // do not mix reliability.
    MessageBundle  m_oUnreliableMessageBundle;

    qUnsignedInt32 m_uiLastConnectionAttempt;

    //n1589
    void* m_pLinkObject;

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    qBool m_bRoutingLimit;
#endif
    qBool m_isApproved; // Whether joining the session was approved.
    // Setting to automatically add players to Recent Players list. Only has effect in Cafe.
    static qBool s_bAutoAddRecentPlayRecord;

    static void (*s_pfOperationBegin)(StationOperation* pOperation);
    static void (*s_pfOperationEnd)(StationOperation* pOperation);
    StationDetectionAlgorithm* m_pStationDetectionAlgorithm;

public:
    static void CallOperationBeginCallback(StationOperation* pOperation);
    static void CallOperationEndCallback(StationOperation* pOperation);
};

// For the TraceOutput.
// {secret}
extern qUnsignedInt32 GetLocalStationHandle();

END_NAMESPACE_NEX

#endif
