﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DuplicationSpaceTable_H_
#define _DuplicationSpaceTable_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <NetZ/src/Core/DOCore/DupSpace.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DupSpace/DuplicationSpace.h>

BEGIN_NAMESPACE_NEX

class DOOperation;
class RootDO;

class IteratorOverDOs;
class DuplicatedObject;
class JobThreadPool;
class MatchOperation;
class PeriodicJob;

class DuplicationSpaceTable: public PseudoSingleton
{
public:
    DuplicationSpaceTable();
    virtual ~DuplicationSpaceTable();

    static DuplicationSpaceTable* GetInstance();

    DuplicationSpace* FindDuplicationSpace(DupSpace::ID idDS);

    void RegisterDuplicationSpaces();
    void UnregisterDuplicationSpaces(qInt iDummy=0);

    void UpdateDuplicationSpaces();

    void StartPeriodicMatch();
    void StopPeriodicMatch();

    void TraceMatchJobQueue(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);
    qUnsignedInt GetJobQueueSize();

    void OperationEndMatchTrigger(DOOperation* pOperation);

    void CheckRelevance(RootDO* pDuplicaToCheck);

    void CreateCells();

    class AllocationContext: public RootObject
    {
    public:
        DupSpace::ID* m_pDSArray;
        qUnsignedInt m_uiArrayLength;
    };

    void CheckPeriodicMatch(qInt iDummy=0);
    void QueueMatchOperation(MatchOperation* pMatchOperation, TimeInterval tiWhenFromNow=0);

    void IncNbCheckRelevanceOperations()
    {
        m_uiNbCheckRelevanceOperations++;
    }
    void DecNbCheckRelevanceOperations()
    {
        SYSTEMCHECK(m_uiNbCheckRelevanceOperations>0);
        m_uiNbCheckRelevanceOperations--;
    }
    qInt GetNbCheckRelevanceOperations()
    {
        return m_uiNbCheckRelevanceOperations;
    }

private:
    void RegisterDuplicationSpace(DuplicationSpace* pDS);
    void UnregisterDuplicationSpace(DuplicationSpace* pDS);

    qMap<DupSpace::ID, DuplicationSpace*> m_mapDS;

    PeriodicJob* m_pPeriodicMatchJob;

    qInt m_uiNbCheckRelevanceOperations;
};

END_NAMESPACE_NEX

#endif


