/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ProductFacade_H_
#define _ProductFacade_H_

#include <NetZ/src/Core/Facade/FacadesDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <NetZ/src/Core/DOCore/LANSessionDiscovery.h>
#include <OnlineCore/src/Core/Scheduler.h>
#include <OnlineCore/src/Core/Holder.h>
#include <NetZ/src/Core/DOCore/DOCore.h>
#include <OnlineCore/src/Core/Core.h>
#include <OnlineCore/src/Core/FirstErrorHolder.h>

BEGIN_NAMESPACE_NEX

class ProductSpecifics;
class UtilitySubsystem;
class DOSelections;
class SystemComponents;
class CallContext;

typedef void (*InitDOClassesFunction)(void);

//==============================================================================
// ProductFacade is the class that encapsulates the run-time of the product.
//==============================================================================
class ProductFacade: public PseudoSingleton
{
public:

    ProductFacade(InitDOClassesFunction fInitDOClasses);
    virtual ~ProductFacade();

    virtual qBool Terminate(CallContext* pContext);
    virtual qResult Terminate();

    static ProductFacade* GetInstance();

    void InitSubsystems();
    void DestroySubsystems();

    virtual qBool IsValid() const
    {
        return m_bIsValid;
    }

#ifdef TRACE_ENABLED    // n2370
    void TraceSystemState(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);
#else
    void TraceSystemState(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS)
    {
        (void)uiTraceFlag;
    }
#endif  // TRACE_ENABLED

    static qBool DecrementDOCoreRefCount();
    LANSessionDiscovery *GetLANSessionDiscovery()
    {
        return m_pLANSessionDiscovery;
    }

    static void SetPartialFaultProcessingMode(qBool bTreatFaultAsPartial);
#if NEX_ENABLE_HYBRID
    static void SetHybridPartialFaultProcessingMode(qBool bRecover);
#endif
    static void SetConnectionErrorProcessingMode(qBool bDisconnectOnError);

    Job::JobType GetAssociatedJobsType();

    //n1589
    qBool IsTerminated()
    {
        return m_bIsTerminated;
    }

    qResult GetFatalError()const
    {
        return m_FirstFatalErrorHolder.Get();
    }
    TimeInterval GetTerminateTimeout() const
    {
        if (Core::GetInstance()->GetTerminateImmediately() || GetTerminateImmediately())
        {
            // Specify 0 for the timeout to use terminate-immediately mode
            return 0;
        }
        else
        {
            return m_terminateTimeout;
        }
    }
    void SetTerminateTimeout(TimeInterval timeout)
    {
        m_terminateTimeout = timeout;
    }


    qUnsignedInt32 GetVersion() const;
    void SetVersion(qUnsignedInt32 version);

    void SetTerminateImmediately(qBool immediately);
    qBool GetTerminateImmediately() const;

protected:

    static UtilitySubsystem* s_poUtilitySubsystem;
    static CriticalSection s_csGlobalLock;

    static qUnsignedInt s_uiSubsystemRefCount;
    static qUnsignedInt s_uiDOCoreRefCount;

    qBool m_bIsValid;
    qBool m_bIsTerminated;

    LANSessionDiscovery *m_pLANSessionDiscovery;

private:
    // no copying and copy assignment allowed
    ProductFacade (const ProductFacade&);
    ProductFacade& operator= (const ProductFacade&);

    FirstErrorHolder m_FirstFatalErrorHolder;
    TimeInterval m_terminateTimeout;
    qBool m_terminateImmediately;
};

END_NAMESPACE_NEX

#endif
