﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _HostMigrationExtension_H_
#define _HostMigrationExtension_H_

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/SystemComponent.h>

BEGIN_NAMESPACE_NEX

//Group=Host Migration

/*
Description: This class indicates to Net-Z that a client-side Host Migration extension package will be used.
Remarks: If you wish to use the Host Migration extension package to synchronise the Net-Z session master and Rendez-Vous
Gathering host for a given session, then you must register HostMigrationExtension with NetZ using the Register
method of this class and the HostMigrationCallback must be registered with a specific Net-Z-based gathering using the Register
method of its class.
*/

/* Please see man pages for details 
 

 
 
 

 

 
 
 
 
 

 

 
 */
class HostMigrationExtension: public SystemComponent
{
public:
    DECLARE_SYSCOMPONENT_METHODS(HostMigrationExtension,SystemComponent);

    //group=Methods

    HostMigrationExtension();
    virtual ~HostMigrationExtension();

    //{secret}
    virtual qBool BeginInitialization();
    //{secret}
    virtual qBool EndInitialization();
    //{secret}
    virtual qBool BeginTermination();

    /*
    Description: Registers HostMigrationExtension with Net-Z.
    Returns: true if HostMigrationExtension was successfully registered with Net-Z, false if not or an error occurred.
    Remarks: This method must be called after the NetZ object is created and before a session is either created or joined.
    HostMigrationExtension is automatically unregistered when the NetZ object is destroyed.
    Example:
            #include <Services/Extensions/HostMigration.h>

            // Create NetZ object
            NetZ oNetZ;
            // Register the Host Migration extension
            HostMigrationExtension::Register()
            // Create the Net-Z session
            Session::CreateSession();
    See Also: HostMigrationCallback,
    */

    /* Please see man pages for details 
     

     
     
     
     

     
     

     
     */
    static qBool Register();

    /* Please see man pages for details 
     

     
     
     

     
     

     
     */
    static qBool Disable();
};

END_NAMESPACE_NEX

#endif
