﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// This file will be included several times.

#include <NetZ/src/Extensions/PHBDR/PHBDRTimeVector.h>
#include <NetZ/src/Extensions/PHBDR/PHBDRParameters.h>
#include <NetZ/src/Extensions/PHBDR/ContinuityBreak.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qQueue.h>
#include <OnlineCore/src/Core/Statistics.h>

// Group=Object Duplication Subsystem

BEGIN_NAMESPACE_NEX


class PHBDRParameters;

// Description:
//
class PHBDRVar: public PHBDRVarRoot
{
    // Group=
public:
    PHBDRVar(AttributeType x=0);
    virtual ~PHBDRVar() {}

    // {secret}
    typedef enum _ModelType {Unknown, Tracking, Convergence, LocalCorrection} ModelType;

    // Accessors...
    void PredictValues(const PHBDRParameters& oParams, const PHBDRTimeVector& t,
                       InternalValue* pdValue, InternalValue *pdRateOfChange, InternalValue *pdAcceleration,
                       Time tFuture, qBool bCommitToThisFuture=false);

    // Accessors to add values...

    void NewValue(AttributeType x, qUnsignedInt32 uiPosition=INSERT_VALUE_AT_THE_END);

    AttributeType GetValue(const PHBDRParameters& oParams, const PHBDRTimeVector& t);
    AttributeType PredictValue(const PHBDRParameters& oParams, const PHBDRTimeVector& t,
                               Time tSomeFuture=SessionClock::GetTime());

    void DelayValue(AttributeType x);
    void UseDelayedValue(qUnsignedInt32 uiPosition);

#ifdef TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);
#endif  // TRACE_ENABLED

    // {secret}
    qBool LocalCorrectionInProgress() const
    {
        return m_tLocalCorrectionExpiration!=Time(0);
    }
    // {secret}
    inline qBool MustRecomputeTrackingModel(Time tSomeFuture) const;

    // Accessors.

    InternalValue GetInitialPosition() const;
    InternalValue GetInitialSpeed() const;
    InternalValue GetAcceleration() const;

    void SetModel(InternalValue dInitialValue, InternalValue dRateOfChange, InternalValue dAcceleration,
                  Time tTimestamp=SessionClock::GetTime());
    void SetLocalCorrection(const PHBDRParameters& oParams, InternalValue dInitialValue, InternalValue dRateOfChange, InternalValue dAcceleration,
                            Time tTimestamp=SessionClock::GetTime());

    void ComputeParameters(const PHBDRParameters& oParams, PHBDRTimeVector& t,
                           qByte byRole, qUnsignedInt32 uiPos=INSERT_VALUE_AT_THE_END,
                           ContinuityBreak oContinuityBreak=CONTINUITY_BREAK_NONE,
                           Time tNow=SessionClock::GetTime());

    // {secret}
    AttributeType GetPoint(qUnsignedInt uiIndex)
    {
        SYSTEMCHECK(uiIndex<=2);
        return m_x[uiIndex];
    }

private:
    class ComputationContext: public RootObject
    {
    public:
        ComputationContext();

        void SetComputationInfo(const PHBDRParameters& oParams,
                                const PHBDRTimeVector& t,
                                Time tNow);
        void SetResultLocation(Time* ptInitialTime, InternalValue *pdInitialPos,
                               InternalValue* pdSpeed, InternalValue *pdAcceleration,
                               Time* ptNextParametersUpdate, ModelType* peModelType);

        // Info to perform the computation...
        const PHBDRParameters* m_pParams;
        const PHBDRTimeVector* m_pTimeVector;
        Time m_tNow;
        // Where to store the result...
        Time* m_ptInitialTime;
        InternalValue* m_pdInitialPos;
        InternalValue* m_pdSpeed;
        InternalValue* m_pdAcceleration;
        Time* m_ptNextParametersUpdate;
        ModelType* m_peModelType;
    };

    void PredictValuesImpl(const PHBDRParameters& oParams, const PHBDRTimeVector& t,
                           InternalValue* pdValue, InternalValue *pdRateOfChange, InternalValue *pdAcceleration,
                           Time tFuture, qBool bCommitToThisFuture);


    // Basic computations.

    static InternalValue ComputePosition(InternalValue dInitialPos, InternalValue dSpeed,
                                         InternalValue dAcceleration, TimeInterval tiElapsed);
    InternalValue ComputePosition(TimeInterval tiElapsed) const
    {
        return ComputePosition(m_dInitialPosition, m_dSpeed, m_dAcceleration, tiElapsed);
    }

    InternalValue ComputeEmbraceAngle(const PHBDRTimeVector& t);

    void ComputeTrackingParameters(const ComputationContext& oContext);
    qByte SelectTrackingType(const ComputationContext& oContext);
    void ComputeLinearTrackingParam(const ComputationContext& oContext);
    void ComputeNonLinearTrackingParam(const ComputationContext& oContext);
    TimeInterval ComputeCP(const ComputationContext& oContext,
                           Time tNewNextTrackingUpdate,
                           Time* ptCP);
    void ComputeLinearConvergenceParam(const ComputationContext& oContext);
    void ComputeNonLinearConvergenceParam(const ComputationContext& oContext);

    String FormatQueueTrace() const;

    qUnsignedInt GetQueueSize() const
    {
        return m_queueDelayedValues.size();
    }
    AttributeType GetQueuedValue(qUnsignedInt iIndex) const
    {
        return m_queueDelayedValues.at(iIndex);
    }

#ifdef TEST_ENABLED
public:
    static qBool Test(qBool bHideBug212=false);
#endif

private:
    AttributeType m_x[3];

    Time m_tInitialTime;
    Time m_tNextParametersUpdate;
    Time m_tLocalCorrectionExpiration;
    InternalValue m_dInitialPosition;
    InternalValue m_dSpeed;
    InternalValue m_dAcceleration;

    // Delayed info.
    qQueue<AttributeType>   m_queueDelayedValues;
};


END_NAMESPACE_NEX
