/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _KeyedChecksumAlgorithm_H
#define _KeyedChecksumAlgorithm_H

#include <OnlineCore/src/Core/Checksum/Interface/ChecksumDecl.h>
#include <OnlineCore/src/Core/PluginObject.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Core/Checksum/Interface/ChecksumAlgorithm.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class Plugin;

//Group=Checksum Algorithms

/*
Description: Specifies attributes for the keyed checksum algorithms.
Remarks: This class is used to specify specific atributes of the keyed checksum algorithm used by
$PRODUCTNAME$. The available keyed checksum algorithms are HMAC and UMAC. These two algorithms
function in a similar manner except that they use different hashing algorithms resulting in
the UMAC algorithm being an order of magnitude faster than the HMAC algorithm.
See Also: Key
*/
class KeyedChecksumAlgorithm: public ChecksumAlgorithm
{
public:
    //Group=Methods

    //{secret}
    KeyedChecksumAlgorithm(qUnsignedInt uiMinimumKeyLength=6, qUnsignedInt uiMaximumKeyLength=64);
    //{secret}
    virtual ~KeyedChecksumAlgorithm();

    /*
    Summary: Sets the key used by the checksum algorithm.
    Parameters:
    pbyContent: A pointer to a buffer of Bytes that contains the key.
    uiLength: The length of the key as a qUnsignedInt.
    oKey: The key as a Key object.

    Remarks:
    See Also: Key
    */
    qBool SetKey(const Key& oKey);
    qBool SetKey(const qByte *pbyContent, qUnsignedInt uiLength)
    {
        return SetKey(Key(pbyContent, uiLength));   //n1769: Added const to pbyContent.
    }

    //Summary: Returns the minimum key length used by the checksum algorithm.
    //Returns: The minimum key length as an unsigned integer.
    //See Also: GetMaximumKeyLength
    qUnsignedInt GetMinimumKeyLength() const
    {
        return m_uiMinimumKeyLength;
    }

    //Summary: Returns the maximum key length used by the checksum algorithm.
    //Returns: The maximum key length as an unsigned integer.
    //See Also: GetMinimumKeyLength
    qUnsignedInt GetMaximumKeyLength() const
    {
        return m_uiMaximumKeyLength;
    }

protected:
    virtual qBool IsReady() const;
    virtual void KeyHasChanged() {};  //n1769 memo: Called from the SetKey function. Called when the new key finishes replacing the old key.

public:
#ifdef TEST_ENABLED
    //{secret}
    qBool GenericTest();
#endif

protected:
    Key m_keyChecksum;
    qUnsignedInt m_uiMinimumKeyLength;
    qUnsignedInt m_uiMaximumKeyLength;
};

END_NAMESPACE_NEX

#endif




