﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _InstanceControl_H_
#define _InstanceControl_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Core/InstanceTable.h>

BEGIN_NAMESPACE_NEX

class InstanceControl: public RootObject
{
public:
    InstanceControl(ICType icInstanceType, ICContext icInstanceContext);
    virtual ~InstanceControl();

#ifdef TEST_ENABLED
    static qBool Test();
#endif

    void SetDelegatorInstance(void*);
    void* GetDelegatorInstance()
    {
        return m_pDelegatorInstance;
    }

    // InstanceTable static functions
    static void AllocateExtraContexts(qUnsignedInt uiNewSize);
    static void FreeExtraContexts();

    static ICContext GetHighestID();
    static qBool ContextIsValid(ICContext icContext);

    static ICContext CreateContext();
    static qBool DeleteContext(ICContext icInstanceContext);

    static InstanceControl* FindInstance(ICType icInstanceType, ICContext icInstanceContext)
    {
        return s_oInstanceTable.FindInstance(icInstanceType, icInstanceContext);
    }
    static void* FindDelegatorInstance(ICType icInstanceType, ICContext icInstanceContext)
    {
        InstanceControl* pInstanceControl = FindInstance(icInstanceType,icInstanceContext);
        if (pInstanceControl!=NULL)
        {
            return pInstanceControl->GetDelegatorInstance();
        }
        else
        {
            return NULL;
        }
    }

    static ICContext FindInstanceContext(InstanceControl* icElement, ICType icInstanceType);

    qBool IsValid() const
    {
        return m_bIsValid;
    }

protected:
    static InstanceTable s_oInstanceTable;

    ICContext m_icInstanceContext;
    ICType m_icInstanceType;
    void* m_pDelegatorInstance;
    qBool   m_bIsValid;
};


END_NAMESPACE_NEX

#endif
