/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _InstantiationContext_H_
#define _InstantiationContext_H_

#include <OnlineCore/src/Core/CoreDecl.h>
// n2973
#include <OnlineCore/src/Platform/Core/SystemError.h>

BEGIN_NAMESPACE_NEX

class InstanceControl;

// For NEX Developer <wiki/Pseudo Singleton#IDs>
#define IC_TYPE_FACADE                      0
#define IC_TYPE_NETWORK                     1
#define IC_TYPE_DUPLICATION_SPACE_TABLE     2
#define IC_TYPE_CORE                        3
#define IC_TYPE_DOCORE                      4
#define IC_TYPE_ADAPTER_EXT                 5
#define IC_TYPE_RV_FRAMEWORK                6
#define IC_TYPE_SERVICECORE                 7
#define IC_TYPE_SPARK_UIFLOW                8
#define IC_TYPE_SPARK_LOBBY_BINDING         9

#define IC_TYPE_CONNECTIVITY_MANAGER        11
#define IC_TYPE_RVCLIENTCORE                12
#define IC_TYPE_PRUDPUSERPROTOCOL           13
#define NB_INSTANCE_TYPES                   14

#define INVALID_INSTANCE            (NAMESPACE_NEX InstanceControl*)0xFFFFFFFF

typedef qUnsignedInt ICType;
typedef qUnsignedInt ICContext;


class InstantiationContext: public RootObject
{
public:
    InstantiationContext();
    InstantiationContext(InstanceControl* icElement, ICType icInstanceType);
    virtual ~InstantiationContext();

    qBool AddInstance(InstanceControl* icElement, ICType icInstanceType);
    qBool DelInstance(InstanceControl* icElement, ICType icInstanceType);
    // For NEX Developer <wiki/Pseudo Singleton#GetPseudoSingleton>
    // n2973 Inlined to speed up code.
    inline InstanceControl* GetInstance(ICType icInstanceType);

    void InitContext();
    qBool IsValid()
    {
        return m_IsValid;
    }

private:
    // For NEX Developer <wiki/Pseudo Singleton#PseudoSingletonArray>
    InstanceControl* m_icInstanceContext[NB_INSTANCE_TYPES];
    qBool m_IsValid;
};

// n2973 Moved from "InstantiationContext.cpp" to speed up code.
// Return the instance if we found it. INVALID_INSTANCE if a problem occurred. Set error flag.
InstanceControl* InstantiationContext::GetInstance(ICType icInstanceType)
{
    if(icInstanceType >= NB_INSTANCE_TYPES)
    {
        SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_INDEX_OOB,0);
        return INVALID_INSTANCE;
    }

    return m_icInstanceContext[icInstanceType];
}


END_NAMESPACE_NEX

#endif


