﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------
*/

#ifndef NEX_ONLINECORE_SRC_CORE_JOBCALLCONTEXTCALLBACK_H_
#define NEX_ONLINECORE_SRC_CORE_JOBCALLCONTEXTCALLBACK_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>

BEGIN_NAMESPACE_NEX

class CallContext;

class JobCallContextCallback : public StepSequenceJob, public NonCopyable
{
public:

    JobCallContextCallback(JobType type, const qChar* szDescription, CallContext* associatedCallContext);
    virtual ~JobCallContextCallback();

    //Set a Job and start. When false, delete this object with RELEASEREF. When true, you do not need to delete it because Scheduler will do it.
    //The CallContext that can be retrieved using GetCallContext must already be initialized and registered to CallContextRegister before starting.
    qBool StartJob();

    void CompleteJob(const qResult& result);

    void StepCallStart();
    void StepCallCompletion() { StepCallCompletionImp(); }
    void StepFinish();

    //After instantiating the CallContext object, it must be initialized using this GetCallContext function.
    CallContext* GetCallContext()
    {
        if (m_pCallContext == NULL)
        {
            m_pCallContext = CreateCallContext();
        }
        return m_pCallContext;
    }

protected:
    virtual CallContext* CreateCallContext();

    //StepCallCompletion is the step that is called on a CallCompletion event. We implement StepCallCompletion here so that even if it is made virtual, the implementation in any derived classes will not be called.
    //
    virtual void StepCallCompletionImp() { StepFinish(); }
    //The pCC called on success is the same as the associatedCallContext of the constructor.
    virtual void ProccessSuccess(CallContext* pCC) { DUMMY_PARAM(pCC); }

    //The pCC called on failure is the same as the associatedCallContext of the constructor.
    virtual void ProccessFailure(CallContext* pCC) { DUMMY_PARAM(pCC); }
    virtual void ProccessCancel() {}
    qUnsignedInt32 m_AssociatedCallID;

private:

    CallContext* m_AssociatedCallContext;
    CallContext* m_pCallContext;
};

END_NAMESPACE_NEX

#endif /* NEX_ONLINECORE_SRC_CORE_JOBCALLCONTEXTCALLBACK_H_ */
