/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#ifndef _LatestMedian_H_
#define _LatestMedian_H_

#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <algorithm>


BEGIN_NAMESPACE_NEX

/*
 * Class that returns the median value for the latest N number of values.
 */

template<class Type> class LatestMedian: public RootObject
{
public:
    LatestMedian(qInt size) : m_values(size), m_sortedTmp(size)
    {
        SYSTEMCHECK(size > 0);
        Reset();
    }
    virtual ~LatestMedian()
    {
    }

    void Reset()
    {
        m_values.clear();
    }

    void Add(Type v)
    {
        if (m_values.full())
        {
            m_values.pop();
        }
        m_values.push(v);
    }

    Type GetMedian()
    {
        m_sortedTmp.clear();
        for(qUnsignedInt i = 0; i < m_values.size(); i++)
        {
            m_sortedTmp.push_back(m_values.at(i));
        }

        std::sort(m_sortedTmp.begin(), m_sortedTmp.end());

        qInt index = m_sortedTmp.size() / 2;
        if (m_sortedTmp.size() % 2 == 0 && index > 0)
        {
            return static_cast<Type>(m_sortedTmp[index - 1] + m_sortedTmp[index]) / static_cast<Type>(2);
        }
        else
        {
            return m_sortedTmp[index];
        }
    }

private:
    qReservedQueue<Type> m_values;
    qVector<Type> m_sortedTmp;
};

#ifdef TEST_ENABLED
qBool LatestMedianTest();
#endif

END_NAMESPACE_NEX

#endif /* _LatestMedian_H_ */
