﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MethodCallJob_H_
#define _MethodCallJob_H_

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/Job.h>

BEGIN_NAMESPACE_NEX

// Group=Core Subsystem
template<class TargetClass, class ArgumentType, class BaseJobClass=Job> class MethodCallJob: public BaseJobClass
{
    // Group=
public:
    MethodCallJob(
        Job::JobType type,
        const String& strJobDescription,
        TargetClass* pTargetObject,
        void (TargetClass::*pMethod)(ArgumentType arg),
        ArgumentType arg
    ): BaseJobClass(type, strJobDescription)
    {
        m_pTargetObject=pTargetObject;
        m_pMethod=pMethod;
        m_arg = arg;
    }

    virtual ~MethodCallJob()
    {
    }

    ArgumentType& GetArgumentRef()
    {
        return m_arg;
    }
    TargetClass* GetTarget()
    {
        return m_pTargetObject;
    }

    virtual String GetTraceInfo() const
    {
        StringStream oStream;
        oStream << BaseJobClass::GetTraceInfo();
        oStream << _T("\tArgument: ") << m_arg;

        return String(oStream.CStr());
    }

private:

    virtual void Execute()
    {
        TargetClass* pTargetObject=m_pTargetObject;
        void (TargetClass::*pMethod)(ArgumentType arg)=m_pMethod;
        (pTargetObject->*pMethod)(m_arg);
    }

private:
    void (TargetClass::*m_pMethod)(ArgumentType arg);
    TargetClass* m_pTargetObject;
    ArgumentType m_arg;

    static qBool MethodCallJobTest();
};

#ifdef TEST_ENABLED
qBool MethodCallJobTest();
#endif

END_NAMESPACE_NEX

#endif
