﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Statistics_H_
#define _Statistics_H_

#include <OnlineCore/src/Core/UtilityDecl.h>
#include <OnlineCore/src/Core/SmoothedAverage.h>

BEGIN_NAMESPACE_NEX

class Statistics: public RootObject
{
public:
    Statistics();
    virtual ~Statistics();

    void AddValue(qReal dNewValue);
    void Reset();

    qReal GetMinimum() const
    {
        return m_dMinimumValue;
    }
    qReal GetMaximum() const
    {
        return m_dMaximumValue;
    }
    qReal GetAverage() const;
    qReal GetTotal() const
    {
        return m_dTotal;
    }
    qUnsignedInt GetNbEntries() const
    {
        return m_uiNbEntries;
    }

    void SetSmoothedAverageWeight(qReal dSmoothedAverageWeight)
    {
        m_oSmoothedAverage.SetWeight(dSmoothedAverageWeight);
    }
    qReal GetSmoothedAverage() const
    {
        return m_oSmoothedAverage.GetAverage();
    }

    void Trace(qUnsignedInt64 uiTraceFlags, qChar* strName);

private:
    qUnsignedInt m_uiNbEntries;
    qReal m_dTotal;
    qReal m_dMaximumValue;
    qReal m_dMinimumValue;

    SmoothedAverage<qReal> m_oSmoothedAverage;
};

END_NAMESPACE_NEX

#endif
