﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _PropertyDeclaration_H_
#define _PropertyDeclaration_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/ParseTree/Declaration.h>
#include <OnlineCore/src/ParseTree/NameSpaceRef.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/ParseTree/PropertyTarget.h>

BEGIN_NAMESPACE_NEX

// Categories - for generic properties
#define PROP_DDL_BUILTIN            (1<<0)
#define PROP_CUSTOM_CODE            (1<<1)
#define PROP_INHERITABLE            (1<<2)

// Categories for DS specific properties
#define DSPROP_CUSTOM_VAR           (1<<5)
#define DSPROP_CUSTOM_DSDECL        (1<<6)
#define DSPROP_DSFUN                (1<<7)
#define DSPROP_DOFUN                (1<<8)
#define DSPROP_UPDATEPROTOCOL       (1<<9)
#define DSPROP_UPDATEFILTER         (1<<10)
#define DSPROP_DEADRECKONING        (1<<11)

#define EXCLUSIVE_CATEGORIES    (DSPROP_UPDATEPROTOCOL | DSPROP_CUSTOM_VAR | DSPROP_CUSTOM_DSDECL)

class OSDK_API PropertyDeclaration: public Declaration
{
public:
    PropertyDeclaration(const String& strName, qUnsignedInt32 uiCategories, qUnsignedInt32 uiAllowedTargets);
    virtual ~PropertyDeclaration();

    DECLARE_NAMESPACE_ITEM(PropertyDeclaration, Declaration);

    qBool HasCategory(qUnsignedInt32 uiCat) const
    {
        return (m_uiCategories & uiCat)==uiCat;
    }
    qBool AllowsTarget(qUnsignedInt32 uiTarget) const
    {
        return (m_uiAllowedTargets & uiTarget)==uiTarget;
    }

#ifdef TRACE_ENABLED
    virtual void Trace(qUnsignedInt64 uiTraceFlag) const;
#endif

    qResult StreamIn(BinaryRepresentation *pStream) const;
    qResult StreamOut(BinaryRepresentation *pStream);

    virtual qBool IsEqual(const ParseTreeItem* pOther) const;

    virtual qBool ResolveReferences();

    static qBool HasTargetConflicts(
        const qVector<PropertyDeclaration*>& vecDecl,
        qUnsignedInt32 uiTarget,
        PropertyDeclaration** ppConflictingProp);

    static qBool HasExclusivityConflicts(
        const qVector<PropertyDeclaration*>& vecDecl,
        PropertyDeclaration** ppPropA,
        PropertyDeclaration** ppPropB);

private:

    qUnsignedInt32 m_uiCategories;
    qUnsignedInt32 m_uiAllowedTargets;
};

END_NAMESPACE_NEX

#endif
