﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NN_NEX_GLOBAL_VARIABLES_H_
#define NN_NEX_GLOBAL_VARIABLES_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/ThreadVariable.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>
#include <OnlineCore/src/Platform/Core/EventLog.h>
#include <OnlineCore/src/Platform/Core/TraceLog.h>
#include <OnlineCore/src/Platform/Core/PacketBufferManager.h>
#include <OnlineCore/src/Core/CacheManager.h>

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    

           
           
           

           
*/
class GlobalVariables : public RefCountedObject
{
public:

    /* Please see man pages for details 
        

               
               
    */
    static void AcquireInstance();

    /* Please see man pages for details 
        

               

               
     */
    static void ReleaseInstance();

    /* Please see man pages for details 
        

        
        
        
        

        
     */
    static qInt32 GetCount();

    //{secret}
    static GlobalVariables* GetInstance()
    {
#ifdef DEBUG_VERSION
        // SYSTEMCHECK can't be used before an instance of GlobalVariables is created.
        if (!s_pGv)
        {
            NEX_NO_ALLOC_PRINT("(FATAL) You must call nn::nex::GlobalVariables::AcquireInstance() before using almost all NEX functions.\n");
        }
#endif
        return s_pGv;
    }

    //{secret}
    // Destroys cacheable data. This normally does not need to be called. Called by Core::~Core().
    static void ClearCache();
private:
    //{secret}
    GlobalVariables();
    //{secret}
    virtual ~GlobalVariables();

    //{secret}
    GlobalVariables(const GlobalVariables &);
    //{secret}
    GlobalVariables& operator=(const GlobalVariables &);

public:
    static GlobalVariables* s_pGv;
    static ThreadVariableList* GetThreadVariableListPtr();

    // for LANSessionDiscovery
    String s_broadcastAddress;

    // for SystemError
    ThreadVariable<qUnsignedInt32> s_uiLastError;
    ThreadVariable<qUnsignedInt32> s_uiLastExtError;

    // for ObjectThreadRoot
    ThreadVariable<ObjectThreadRoot*> t_pCurrentThreadRoot;

#if defined(CTR_TARGET) || defined(CAFE_TARGET)
    // for RootCaReader
    qMap<String, qVector<qByte> > s_RootCaFileCache;
#endif

#if defined(CAFE_TARGET)
    qMap<qUnsignedInt32, String> s_AuthTokenCache;
    qMap<qUnsignedInt32, String> s_PasswordCache;
    qMap<qUnsignedInt32, String> s_ServerIPCache;
    qMap<qUnsignedInt32, qUnsignedInt16> s_ServerPortCache;
#endif

    // for EventLog
    EventLog s_oEventLog;
#ifndef LOG_ENABLED
    qByte padding1[4];  // Avoids the compiler warning.
#endif

    // for TraceLog
    TraceLog s_oTraceLog;

    CacheManager s_oCacheManager;

#ifdef NEX_CLIENT_PBPOOL
    qVector<PacketBufferManager::UnitHeapParam> s_unitHeapParam;
    qBool s_isEnablePacketBufferPool;
#endif // NEX_CLIENT_PBPOOL
};

END_NAMESPACE_NEX

#endif // NN_NEX_GLOBAL_VARIABLES_H_
