﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _HighResolutionChrono_H_
#define _HighResolutionChrono_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// The high resolution chronometer provides time values in microseconds.
//
// Future enhancements could include a StopWatch-like feature. This would be
// useful to prevent glitches in cumulative arithmetics that can lead to
// unexpected results:
//
// Computing time intervals A-B and B-C may give an higher value than the time
// interval recorded between A-C. This is due to the correction algorithm
// attempting to reliable elapbsed time values as the microsecond values is
// not reliable in all platform/contexts. (See also HighResolutionClock.h)
//==============================================================================
class HighResolutionChrono: public RootObject
{
public:
    HighResolutionChrono(qBool reset=true);
    virtual ~HighResolutionChrono();

    HighResolutionTime GetElapsed(qBool bReset=false);
    void Reset();

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    HighResolutionTime  m_HPStartTime;  // Preferred source
#if NEX_ENABLE_HIGHRESOLUTIONCHRONO_OPTIMIZATION == 0 || !defined ( CTR_TARGET )
    Time                m_tStartTime;   // Reliable source
#endif  // NEX_ENABLE_HIGHRESOLUTIONCHRONO_OPTIMIZATION == 0 || !defined ( CTR_TARGET )
};

END_NAMESPACE_NEX

#endif
