//
// (c) Copyright 1998-2009, Quazal Technologies Inc.
// ALL RIGHTS RESERVED
//
// Net-Z, Eterna, Rendez-Vous, Duplicated Object and Duplication Space
// are trademarks of Quazal.
//
// Use of this software indicates acceptance of the End User license
// Agreement for this product. A copy of the EULA is available from
// www.quazal.com.
//

#ifndef _LogDeviceMemory_H_
#define _LogDeviceMemory_H_

#include <OnlineCore/src/Platform/Core/LogDevice.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>

BEGIN_NAMESPACE_NEX

/*
Write the log to the specified memory.
The log is written as a ring buffer so the old log will be overwritten.
Written in ASCII. Wide characters will be converted to ASCII and then the log will be written.
Up to a maximum of 256 characters per log.
The log will be written with qChar8 characters, so in the Unicode version, make sure that the log will not exceed 256 bytes after ASCII conversion.
The last buffer entry will be a delimiter. The delimiter must be 4 characters or less.
*/
class LogDeviceMemory : public LogDevice
{
public:
    // Maximum length per log.
    static const qInt32 MAX_ONE_LOG_LEN = 256;

public:
    LogDeviceMemory(qChar8* pBuffer, qInt32 bufferSize, const qChar8* delimiter, qChar8** ppLogTop=NULL);
    virtual ~LogDeviceMemory();

    virtual void Output(const qChar *pLog);

    /* Please see man pages for details 
    

    
    
    
    */
    void SetBuffer(qChar8* pBuffer, qInt32 bufferSize, qChar8** ppLogTop=NULL);
    /* Please see man pages for details 
    

    
    */
    qChar8* GetBuffer()
    {
        return m_pBuffer;
    }
    /* Please see man pages for details 
    

    
    */
    qChar8* GetLogTop()
    {
        return m_pBuffer + m_offsetTop;
    }


    // Starting log position offset
    qInt32 GetOffsetTop()
    {
        return m_offsetTop;
    }
    // Log write position offset
    qInt32 GetOffsetWrite()
    {
        return m_offsetWrite;
    }

private:
    qChar8* m_pBuffer;
    qInt32 m_bufferSize;
    const qChar8* m_delimiter;
    qChar8** m_ppLogTop;
    ForcedCriticalSection m_cs;

    qInt32 m_offsetTop;
    qInt32 m_offsetWrite;
    qBool m_rotated; // The log has made one full rotation.

    qChar8 m_tmpBuffer[ MAX_ONE_LOG_LEN+8 ]; // Temporary region new line and delimiter
};

END_NAMESPACE_NEX

#endif

