﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _MutexPrimitive_H_
#define _MutexPrimitive_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Platform.h>
// Group=Platform Subsystem

#if defined(WIN_FAMILY) || defined(UNIX_FAMILY) || defined (WII_TARGET) || defined(CAFE_TARGET) || defined(CAFEIOP_TARGET) || defined (CTR_TARGET)
#define MUTEX_PRIMITIVE_IS_REENTRANT
#endif

#if defined( CTR_TARGET )
#include <nn/os/os_CriticalSection.h>
#elif defined( CAFE_TARGET )
#include <cafe/os.h>
#elif defined(CAFEIOP_TARGET)
#include <utl_mutex.h>
#endif

BEGIN_NAMESPACE_NEX

//==============================================================================
// Mutex primitive (Quazal internal class)
//==============================================================================
class MutexPrimitive: public RootObject
{
    class Attributes;
    friend class Condition;

    // Group=
public:
    MutexPrimitive();
    virtual ~MutexPrimitive();

    void Enter()
    {
        if (!s_bNoOp)
            EnterImpl();
    }
    void Leave()
    {
        if (!s_bNoOp)
            LeaveImpl();
    }
    // Forced lock.
    void ForceEnter()
    {
        EnterImpl();
    }
    // Forced unlock.
    void ForceLeave()
    {
        LeaveImpl();
    }

#if !(defined(WIN_FAMILY) || defined(UNIX_FAMILY))
    void EnterImpl()
    {
#if defined(WII_TARGET)
        OSLockMutex(static_cast<OSMutex*>(m_hMutex));
#elif defined(CAFE_TARGET)
#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        OSLockMutex(&m_hMutex);
#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        OSLockMutex(static_cast<OSMutex*>(m_hMutex));
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
#elif defined(CAFEIOP_TARGET)
#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        UTL_LockMutex(&m_hMutex);
#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        UTL_LockMutex(static_cast<UTLMutex*>(m_hMutex));
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
#elif defined(CTR_TARGET)

#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        m_hMutex.Enter();
#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        static_cast<nn::os::CriticalSection*>(m_hMutex)->Enter();
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION

#endif //<platform>
#ifdef LOCK_DEBUG_ENABLED
        m_iCounter++;
#endif
    }
    void LeaveImpl()
    {
#ifdef LOCK_DEBUG_ENABLED
        // DO NOT use Leave if the mutex has not been entered.
        SYSTEMCHECK(m_iCounter != 0 && _T("MutexPrimitive::LeaveImpl Not Entered Mutex"));
        m_iCounter--;
#endif
#if defined(WII_TARGET)
        OSUnlockMutex(static_cast<OSMutex*>(m_hMutex));
#elif defined(CAFE_TARGET)
#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        OSUnlockMutex(&m_hMutex);
#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        OSUnlockMutex(static_cast<OSMutex*>(m_hMutex));
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
#elif defined(CAFEIOP_TARGET)
#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        UTL_UnlockMutex(&m_hMutex);
#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        UTL_UnlockMutex(static_cast<UTLMutex*>(m_hMutex));
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
#elif defined(CTR_TARGET)

#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        m_hMutex.Leave();
#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
        static_cast<nn::os::CriticalSection*>(m_hMutex)->Leave();
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION

#endif // <platform>
    }
#else   // #if !(defined(WIN_FAMILY) || defined(UNIX_FAMILY))
    void EnterImpl();
    void LeaveImpl();
#endif  // #if !(defined(WIN_FAMILY) || defined(UNIX_FAMILY))

#if defined(TEST_ENABLED)
    static qBool Test();
#endif

    static qBool IsANoOp()
    {
        return s_bNoOp;
    }
    //n1589:memo:called from Core::Core
    static void SetNoOpFlag()
    {
        s_bNoOp=true;
    }
    static void ClearNoOpFlag()
    {
        s_bNoOp=false;
    }
#ifdef LOCK_DEBUG_ENABLED
    qInt32 GetCount() const
    {
        return m_iCounter;
    }
#endif

private:

#if NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION

#if defined(CTR_TARGET)
    nn::os::CriticalSection m_hMutex;
#elif defined(CAFE_TARGET)
    OSMutex m_hMutex;
#elif defined(CAFEIOP_TARGET)
    UTLMutex m_hMutex;
#else
#endif

#else  // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION
    void    *m_hMutex;
#endif // NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION

#ifdef LOCK_DEBUG_ENABLED
    qInt32 m_iCounter; // For debugging.
#endif
    static qBool s_bNoOp;

    // Cannot copy
    MutexPrimitive( const MutexPrimitive & );
    MutexPrimitive &operator=( const MutexPrimitive & );
};

END_NAMESPACE_NEX

#endif
