﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ObjectPointer_H_
#define _ObjectPointer_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>


BEGIN_NAMESPACE_NEX

// This class is used to hold pointer to variables, in a
// ThreadVariable.  It ensures that when it is deleted, the pointer is
// safely destroyed.

// Note that copy construction and assignment are only valid when the
// source ObjectPointer currently points to NULL.  This ensures we do
// not need to clone objects, and that we never have mupliple pointers
// ot the same location.

template<class VariableType> class ObjectPointer
{
public:
    ObjectPointer()
    {
        m_pObject = NULL;
    }

    ObjectPointer(const ObjectPointer& oValue)
    {
#ifndef SYSTEMCHECK_ENABLED
        DUMMY_PARAM(oValue);
#endif
        // Checks that the source object is pointing to
        // NULL.
        SYSTEMCHECK(oValue.m_pObject==NULL);
        m_pObject = NULL;
    }

    ObjectPointer(VariableType* pObject)
    {
        m_pObject = pObject;
    }

    ~ObjectPointer()
    {
        DeleteInstance();
    }

    ObjectPointer &operator=(const VariableType& oValue)
    {
        // Checks that the source object is pointing to
        // NULL.
        SYSTEMCHECK(oValue.m_pObject==NULL);
        DeleteInstance();
        return *this;
    }

    VariableType* GetInstance()
    {
        if (m_pObject == NULL)
        {
            m_pObject = qSpecialNew(VariableType);
        }
        return m_pObject;
    }

    void DeleteInstance()
    {
        if (m_pObject != NULL)
        {
            qSpecialDelete(m_pObject);
            m_pObject = NULL;
        }
    }

public:
    VariableType    *m_pObject;
};


#ifdef TEST_ENABLED
qBool ObjectPointerTest();
#endif


END_NAMESPACE_NEX

#endif
