/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// <desc: Class that abstracts the notion of threads>

#ifndef _ObjectThread_H_
#define _ObjectThread_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>

BEGIN_NAMESPACE_NEX

// Group=Platform Subsystem
template<class TargetClass, class ArgumentType> class ObjectThread: public ObjectThreadRoot
{
    // Group=
public:
    ObjectThread(const String& strName): ObjectThreadRoot(strName)
    {
        m_pTargetObject=NULL;
        m_pMethod=NULL;
    }

    //n1589
    ObjectThread(
        const String& strName,
        qUnsignedInt threadPrio
    ): ObjectThreadRoot(strName, threadPrio)
    {
        m_pTargetObject=NULL;
        m_pMethod=NULL;
    }

    virtual ~ObjectThread()
    {
    }

    /* Please see man pages for details 
    

    
    
    
    
    */
    qBool Launch(TargetClass* pTargetObject,
                 void (TargetClass::*pMethod)(ArgumentType arg),
                 ArgumentType arg,
                 qUnsignedInt32 uiStackSize = 0)
    {
        m_pTargetObject=pTargetObject;
        m_pMethod=pMethod;
        m_arg=arg;
        return ObjectThreadRoot::Launch(uiStackSize);
    }

private:

    virtual void CallObjectMethod()
    {
        //SYSTEMCHECK(m_pTargetObject!=NULL);
        //SYSTEMCHECK(m_pMethod!=NULL);
        TargetClass* pTargetObject=m_pTargetObject;
        void (TargetClass::*pMethod)(ArgumentType arg)=m_pMethod;
        ArgumentType arg=m_arg;
        ReadyToRun(); // When this is called, we cannot use any class variables anymore...
        (pTargetObject->*pMethod)(arg);
    }

private:
    void (TargetClass::*m_pMethod)(ArgumentType arg);
    TargetClass* m_pTargetObject;
    ArgumentType m_arg;

    // n2370: need TEST_ENABLED ?
    static qBool ObjectThreadTest();
};

#ifdef TEST_ENABLED
qBool ObjectThreadTest();
#endif

END_NAMESPACE_NEX

#endif
