﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ObjectThreadRoot_H_
#define _ObjectThreadRoot_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>
#include <OnlineCore/src/Platform/Core/String.h>

//n1589
#ifdef DEBUG_VERSION
#define NEX_ENABLE_THREAD_DEBUG 1
#else
#define NEX_ENABLE_THREAD_DEBUG 0
#endif

#ifdef NON_SUPPORT_C_TARGET
#error "Not supported platform."
#elif defined(NON_SUPPORT_A_TARGET)
#error "Not supported platform."
#endif

#if defined(WII_TARGET) || defined(CAFE_TARGET)
#define NN_NEX_PRIORITY_LOW         31
// The default if 16, but we want that the main thread has a higher priority
// such that it is able to SPINTEST / Wait and timeout the processing of the
// background thread. This solution is similar to DWC_LoginAsync which spawn
// a thread with a priority of 17...
// Because threads are non-preemptive, the thread of the game must have a higher
// priority than the Quazal-thread in order to control how much CPU they
// give us.
#define NN_NEX_PRIORITY_NORMAL          17
#define NN_NEX_PRIORITY_HIGH            0

#elif defined(CAFEIOP_TARGET)

#define NN_NEX_PRIORITY_LOW         0
#define NN_NEX_PRIORITY_NORMAL      50 // The level of BG process priority, such as FPD or BOSS, that has been defined by <priority.h>.
#define NN_NEX_PRIORITY_HIGH        127

#elif defined(DS_TARGET)
#define NN_NEX_PRIORITY_LOW            0
#define NN_NEX_PRIORITY_NORMAL         16
#define NN_NEX_PRIORITY_HIGH           31

#elif defined(CTR_TARGET)
#define NN_NEX_PRIORITY_LOW            24
#define NN_NEX_PRIORITY_NORMAL         16
#define NN_NEX_PRIORITY_HIGH           8

#else
#define NN_NEX_PRIORITY_LOW         0
#define NN_NEX_PRIORITY_NORMAL          1
#define NN_NEX_PRIORITY_HIGH            2
#endif

#define NN_NEX_THREAD_STACK_CHECK_WORD      0xCC

BEGIN_NAMESPACE_NEX

typedef qUnsignedInt ThreadID;

// For the TraceOutput.
extern const qChar* GetThreadName();

class ObjectThreadRoot: public RootObject
{

    class Handle;

public:
    static const qInt32 DEFAULT_CORE_NO = QINT32_MIN;

    ObjectThreadRoot(const String& strName);

    //n1589
    ObjectThreadRoot(
        const String& strName,
        qUnsignedInt threadPrio
    );

    virtual ~ObjectThreadRoot();

    static ObjectThreadRoot* GetInstance();

    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    static qUnsignedInt32 NbRunningThreads()
    {
        return s_avRunningThreads.Get();
    }
    ThreadID GetThreadId() const
    {
        return m_uiThreadID;
    }

    void Attach(qUnsignedInt32 uiThreadID);
    void Detach();

    void SetPriority(qUnsignedInt32 uiThreadPrio);
    qUnsignedInt32 GetPriority() const
    {
        return m_uiThreadPrio;
    }
    static void SetDefaultPrio(qUnsignedInt32 uiDefaultPrio);
    static qUnsignedInt32 GetDefaultPrio();
    static void SetThreadStackSize(qUnsignedInt32 uiThreadSize);
    static qUnsignedInt32 GetThreadStackSize();

    static void UseAutoStack(qBool usingAutoStack = true);
    static qBool IsUsingAutoStack();

    qBool Launched() const
    {
        return m_bLaunched;
    }
    qBool Finished() const
    {
        return m_bFinished;
    }
    Handle *GetThreadHandle();

    static ThreadID GetCurrentThreadID();

    void SetThreadName(const String& strName);
    String m_strName;

#ifdef THREAD_AFFINITY_SUPPORT
    static void SetDefaultThreadAffinity(qUnsignedInt32 uiThreadAffinityMask);
    static qUnsignedInt32 GetDefaultThreadAffinity()
    {
        return s_uiDefaultThreadAffinityMask;
    }
    static void SetThreadAffinity(qUnsignedInt32 uiThreadAffinityMask);
#endif
    qBool CheckStack();

    //n2967
    void SetCoreNo(qInt32 no)
    {
        m_coreNo = no;    //n2967: If the Core No setting is DEFAULT_CORE_NO, use the NEX default CPU
    }
    qInt32 GetCoreNo()
    {
        return m_coreNo;    //n2967: If the Core No setting is DEFAULT_CORE_NO, use the NEX default CPU
    }


#if defined(DEBUG_VERSION)
    static volatile qBool s_isObjectThreadProcNewSection;
#endif
protected:
    /* Please see man pages for details 
    
    
    */
    qBool Launch(qUnsignedInt32 uiStackSize = 0);
    void ReadyToRun();

private:
    qBool LaunchImpl();

private:
#if defined(UNIX_FAMILY) || defined(WIN32_TARGET)
    static qUnsignedInt STDCALL ObjectThreadProc(void *lpParameter);
#endif
#if defined(WII_TARGET)
    static void* ObjectThreadProc(void *lpParameter);
#endif
#if defined(CAFE_TARGET)
    static int ObjectThreadProc(int intArg, void *lpParameter);
#endif
#if defined(CAFEIOP_TARGET)
    static void ObjectThreadProc(u32 lpParameter);
#endif
#if defined(DS_TARGET) || defined(CTR_TARGET)
    static void ObjectThreadProc(void *lpParameter);
#endif

    void CallObjectMethodWrapper();
    virtual void CallObjectMethod()=0;
    void WrapExceptionHandling();

private:
    Handle       *m_pHandle;
    ThreadID m_uiThreadID;
    qUnsignedInt m_uiThreadPrio;
    qUnsignedInt32 m_uiThreadStackSize;

    volatile qBool m_bLaunched;
    volatile qBool m_bRunning;
    volatile qBool m_bFinished;
    qByte padding1;  // Avoids the compiler warning.

    static qUnsignedInt32 s_uiDefaultThreadStackSize;

    static AtomicValue s_avRunningThreads;
    static qUnsignedInt32 s_uiDefaultPrio;

#ifdef THREAD_AFFINITY_SUPPORT
    static qUnsignedInt32 s_uiDefaultThreadAffinityMask;
#endif

    qInt32 m_coreNo; //n2967

#if defined(CAFE_TARGET) && defined(UNICODE_VERSION)
    static const qUnsignedInt OS_THREAD_NAME_LEN = 256;
    qChar8 m_OSThreadNameChar8[OS_THREAD_NAME_LEN];
#endif
};

END_NAMESPACE_NEX


#endif
