﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NN_NEX_OPTIMIZATION_DEFS_H_
#define NN_NEX_OPTIMIZATION_DEFS_H_


//n1769
#ifdef NEX_CLIENT_OPTIMIZATION

//n1769: The original MemoryManager prepared headers when memory was allocated and wrote the timestamp, signature, and other information there, making it convenient for debugging.
//However, this feature is expensive, so optimizations have been made to omit this convenient debugging feature entirely.
// 
//By cutting it out, the GetDefaultMemoryManager function is also not called.
#ifndef DEBUG_VERSION
#define NEX_ENABLE_MEMORYMANAGER_SIMPLIFY_OPTIMIZATION 1
#undef LEAK_DETECTION_ENABLED
#undef HEAP_DEBUGGING_ENABLED
#else
#define NEX_ENABLE_MEMORYMANAGER_SIMPLIFY_OPTIMIZATION 1
#undef LEAK_DETECTION_ENABLED
#undef HEAP_DEBUGGING_ENABLED
#endif

//n1769:begin
#define NEX_DISABLE_PERFCOUNTER_UPDATE_OPTIMIZATION 1
// Disable the calculation of when the time goes around in the Time::operator- function.
#define NEX_ENABLE_TIME_SUBTRACT_OPERATOR_OPTIMIZATION 1
//n1769:end

// n1769: Because the return value from the MessageBundle::GetMaximumSize function does not change very often, optimize this by always returning the fixed value.
// 
#define NEX_ENABLE_FIX_BUNDLE_MAXSIZE_OPTIMIZATION 1

#define NEX_ENABLE_STRING_FORMAT_STACKSIZE_OPTIMIZATION 0
// n2370: Optimization for high precision timer for CTR
#define NEX_ENABLE_HIGHRESOLUTIONCHRONO_OPTIMIZATION 1
// n2370: Optimization of the portion to determine whether to connect to a station
#define NEX_ENABLE_OBJDUP_CONNECTSTATION_OPTIMIZATION 1
// n2370: Increase speed by disabling scheduler performance measurement
#define NEX_DISABLE_SCHEDULER_PERFCOUNTER_OPTIMIZATION 1
// n2370: Optimization by using SetCurrentItem for the return value of the IteratorOverDOs' GetNext function
#define NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION 1
// n2370: Optimization that caches maximum values and minimum values with IsAKindOfDOFilter instead of calculating them each time
#define NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION 1
// n2370: Optimization to not access member variables frequently in AdvanceToValidItem
#define NEX_ENABLE_ADVANCETOVALIDITEM_LOOP_OPTIMIZATION 1
// n2370: Optimization to remove overhead to prevent a lock occurring twice when a valid pointer is set in DORef
#define NEX_ENABLE_DOREF_LOCK_OPTIMIZATION 1

// n2494: Prevent calling MemoryManager::Allocate by replacing String that can only substitute for string constants with const qChar*.
#define NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION 1

// n2967: Prevent new with QLOGSTR. It is a problem when using a ThreadUnSafe memory allocator.
#define NEX_ENABLE_AVOID_NEW_IN_QLOGSTR 1

// n2967: Since the logger type buffer size was large and the stack was consumed (4096), it was made to be decreased.
#define NEX_LOG_STRING_BUFFER_SIZE 256

// n2343: Flag for the caching and increased speed of the Time::GetTime function
#define NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION 1

#else // n2619: When not NEX_CLIENT_OPTIMIZATION, defined as all 0's.
//n1769: The original MemoryManager prepared headers when memory was allocated and wrote the timestamp, signature, and other information there, making it convenient for debugging.
//However, this feature is expensive, so optimizations have been made to omit this convenient debugging feature entirely.
// 
//By cutting it out, the GetDefaultMemoryManager function is also not called.
#define NEX_ENABLE_MEMORYMANAGER_SIMPLIFY_OPTIMIZATION 0

//n1769:begin
#define NEX_DISABLE_PERFCOUNTER_UPDATE_OPTIMIZATION 0
#define NEX_ENABLE_TIME_SUBTRACT_OPERATOR_OPTIMIZATION 0
//n1769:end

// n1769: Because the return value from the MessageBundle::GetMaximumSize function does not change very often, optimize this by always returning the fixed value.
// 
#define NEX_ENABLE_FIX_BUNDLE_MAXSIZE_OPTIMIZATION 0

// n2370: Workaround for a bug that causes a large amount of stack to be consumed with String::Format (and slows down during debug build).
#define NEX_ENABLE_STRING_FORMAT_STACKSIZE_OPTIMIZATION 0
// n2370: Optimization for high precision timer for CTR
#define NEX_ENABLE_HIGHRESOLUTIONCHRONO_OPTIMIZATION 0
// n2370: Optimization of the portion to determine whether to connect to a station
#define NEX_ENABLE_OBJDUP_CONNECTSTATION_OPTIMIZATION 0
// n2370: Increase speed by disabling scheduler performance measurement
#define NEX_DISABLE_SCHEDULER_PERFCOUNTER_OPTIMIZATION 0
// n2370: Optimization by using SetCurrentItem for the return value of the IteratorOverDOs' GetNext function
#define NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION 0
// n2370: Optimization that caches maximum values and minimum values with IsAKindOfDOFilter instead of calculating them each time
#define NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION 0
// n2370: Optimization to not access member variables frequently in AdvanceToValidItem
#define NEX_ENABLE_ADVANCETOVALIDITEM_LOOP_OPTIMIZATION 0
// n2370: Optimization to remove overhead to prevent a lock occurring twice when a valid pointer is set in DORef
#define NEX_ENABLE_DOREF_LOCK_OPTIMIZATION 0

// n2494: Prevent calling MemoryManager::Allocate by replacing String that can only substitute for string constants with const qChar*.
#define NEX_ENABLE_CONST_STRING_TO_QCHAR_OPTIMIZATION 0

// n2967: Prevent new with QLOGSTR. It is a problem when using a ThreadUnSafe allocator.
#define NEX_ENABLE_AVOID_NEW_IN_QLOGSTR 0

// n2967: Since the logger type buffer size was large and the stack was consumed (4096), it was made to be decreased.
// For Linux, Unittest fails, so with Linux and Windows, 4K is allocated
#define NEX_LOG_STRING_BUFFER_SIZE 4096

// n2343: Flag for the caching and increased speed of the Time::GetTime function
#define NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION 0

#endif  // end of NEX_CLIENT_OPTIMIZATION

//  Platform-specific optimizations
#if defined(CTR_TARGET)
// n2494: Revised so that new/delete does not occur with the MutexPrimitive constructor/destructor.
#define NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION 1

#define NEX_ENABLE_SEND_TRACELOG_OPTIMIZATION 1

#elif defined(CAFE_TARGET)
// n2494: Revised so that new/delete does not occur with the MutexPrimitive constructor/destructor.
#define NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION 1

#define NEX_ENABLE_SEND_TRACELOG_OPTIMIZATION 0

#elif defined(CAFEIOP_TARGET)
// n2494: Revised so that new/delete does not occur with the MutexPrimitive constructor/destructor.
#define NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION 1

#define NEX_ENABLE_SEND_TRACELOG_OPTIMIZATION 0

#else
// n2494: Revised so that new/delete does not occur with the MutexPrimitive constructor/destructor.
#define NEX_ENABLE_MUTEX_PRIMITIVE_OBJECT_OPTIMIZATION 0

#define NEX_ENABLE_SEND_TRACELOG_OPTIMIZATION 0

#endif

// n2690 Suppress the number of times DeliverIncoming is called for a single Dispatch.
// By doing so, the amount of memory used when receiving a large volume of Notifications for a single Dispatch is suppressed
#if defined( FPD_BUILD )
#define NEX_ENABLE_TRANSPORT_JOB_LIMITATION 1
//n1589
#elif defined(NEX_CLIENT_OPTIMIZATION)
#define NEX_ENABLE_TRANSPORT_JOB_LIMITATION 1
#else
#define NEX_ENABLE_TRANSPORT_JOB_LIMITATION 0
#endif

//n2690: Cache NAT determination results
#if defined(CTR_TARGET)
// Disabled until the ac private header is inserted in the NEX-Kit
#define NEX_ENABLE_NAT_CHECK_CACHE 0
#else
#define NEX_ENABLE_NAT_CHECK_CACHE 0
#endif

#endif  // NN_NEX_OPTIMIZATION_DEFS_H_
