﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _PacketBufferManager_H_
#define _PacketBufferManager_H_

#if defined(NEX_CLIENT_PBPOOL) || defined(FOR_DOC_PURPOSE) // Client only.

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/MutexPrimitive.h>
#include <OnlineCore/src/Platform/Core/MemoryManager.h>



BEGIN_NAMESPACE_NEX


namespace
{
// The default packet buffer pool has 300 kbytes allocated. 
const qUnsignedInt32 DEFAULT_PBPOOL_SIZE=300*1024;
// The smalllest pool size in GetSimpleUnitHeapParam.
const qUnsignedInt32 MINIMUM_SIMPLE_UNIT_HEAP_SIZE=64*1024;
}

/* Please see man pages for details 
 

 

 
 

 

 
 

 
 
 
 

 
 
 

 
 

 
 
 */
class PacketBufferManager: public RootObject
{
public:
    /* Please see man pages for details 
     
     
     
     
     
     
     */
    struct UnitHeapParam : public RootObject
    {
        qUnsignedInt32 unitSize;  //
        qUnsignedInt32 unitNum;   //

        bool operator<(const UnitHeapParam& param) const
        {
            return unitSize < param.unitSize;
        }
        bool operator==(const UnitHeapParam& param) const
        {
            return unitSize == param.unitSize;
        }
    };
    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    struct UnitHeapDebugInfo : public RootObject
    {
        qUnsignedInt32 peak;     //
        qUnsignedInt32 use;      //
        qUnsignedInt32 max;      //
        qUnsignedInt32 unitSize; //
        qUnsignedInt32 realUnitSize; //
        qUnsignedInt32 alloc; //
        qUnsignedInt32 free;  //
        qUnsignedInt32 outOfMemory;  //
    };

private:
    /* Please see man pages for details 
     
     */
    PacketBufferManager()
    {
        SYSTEMCHECK(false);
    }
public:
    /* Please see man pages for details 
     

            
            

            

            
            
            
            
            

     
     
     */
    static qUnsignedInt32 GetRequiredMemorySize(const qVector<UnitHeapParam>& param);
    static qUnsignedInt32 GetRequiredMemorySizeImpl(const qVector<UnitHeapParam>& param);
    /* Please see man pages for details 
     

            
            
            

            

            
            

     
     
     
     
     
     */
    static qBool GetSimpleUnitHeapParam(qUnsignedInt32 poolSize, qUnsignedInt32 avgSendSize, qVector<UnitHeapParam>& param);
    /* Please see man pages for details 
     

               
               

               

               
               
               
               
               

               

     
     
     
     
     */
    static qBool SetUnitHeapParam(const qVector<UnitHeapParam>& param);
    /* Please see man pages for details 
     
               

               
               
     
     
     */
    static void GetUnitHeapParam(qVector<UnitHeapParam>& param);

    /* Please see man pages for details 
     
               
     
     
     */
    static void GetUnitHeapDebugInfo(qVector<UnitHeapDebugInfo>& debugInfo);
    /* Please see man pages for details 
     
               
               
     
     
     */
    static void GetAllUnitHeapDebugInfo(qVector<UnitHeapDebugInfo>& debugInfo);
    /* Please see man pages for details 
     
     
     */
    static void ResetUnitHeapDebugInfo();

    /* Please see man pages for details 
     
     
     
     */
    static qBool IsActive();
    /* Please see man pages for details 
     
               
               
               
     
     
     
     
     */
    static qBool EnablePacketBufferPool(qBool bEnable);
    /* Please see man pages for details 
     
     
     
     
     */
    static qBool IsEnablePacketBufferPool();

public:
    /* Please see man pages for details 
     
            
            
     */
    static qBool CheckUnitHeapParam(const qVector<UnitHeapParam>& param);
    /* Please see man pages for details 
     
            
            

     
     
     */
    static void GetPacketManagementUnitHeapParam(qVector<UnitHeapParam>& paramOutput, const qVector<UnitHeapParam>& paramInput);
    /* Please see man pages for details 
     
            
            

     
     
     
     */
    static qUnsignedInt32 GetUnitNum(qUnsignedInt32 memorySize, qUnsignedInt32 unitSize);
    /* Please see man pages for details 
     

     
     
     */
    static void SetPacketBufferMemoryFunctions(MemoryManager::MallocFunction fcnMalloc, MemoryManager::MallocFunction fcnManagementMalloc)
    {
        s_fcnPacketBufferMalloc = fcnMalloc;
        s_fcnPacketManagementObjectMalloc = fcnManagementMalloc;
    }

    /* Please see man pages for details 
     
     */
    static void Dump(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
    /* Please see man pages for details 
     
     
     */
    static qBool IsClean();

    /* Please see man pages for details 
     
     */
    static void Initialize();

    /* Please see man pages for details 
     
     */
    static void Finalize();

    /* Please see man pages for details 
     
     */
    static qUnsignedInt32 GetMaxUnitSize();
    /* Please see man pages for details 
     
     */
    static qUnsignedInt32 GetProperUnitSize(qUnsignedInt32 required);
    /* Please see man pages for details 
     
               
     */
    static qBool IsAllocatable(qUnsignedInt32 requiredSize, qUnsignedInt32 requiredNum);

    /* Please see man pages for details 
     
     */
    static void EnableReceive(qBool isEnable)
    {
        s_isEnableReceive = isEnable;
    }
    static qBool IsEnableReceive()
    {
        return s_isEnableReceive;
    }
    /* Please see man pages for details 
     
     */
    static void SetRequiredPacketSpace(qUnsignedInt32 space)
    {
        s_requiredPacketSpace = space;
    }
    static qUnsignedInt32 GetRequiredPacketSpace()
    {
        return s_requiredPacketSpace;
    }
    /* Please see man pages for details 
     
     */
    static void SetRequiredSendPacketSpace(qUnsignedInt32 space)
    {
        s_requiredSendPacketSpace = space;
    }
    static qUnsignedInt32 GetRequiredSendPacketSpace()
    {
        return s_requiredSendPacketSpace;
    }
    /* Please see man pages for details 
     
     */
    static qUnsignedInt32 GetRefCount()
    {
        return s_refCount;
    }
    /* Please see man pages for details 
     
     */
    static void *Allocate(size_t stSize);
    /* Please see man pages for details 
     
     */
    static void *AllocateManagementObject(size_t stSize);
    /* Please see man pages for details 
     
     */
    static void SetAllocateFailProbability(qReal prob)
    {
        s_allocateFailProbability = prob;
    }
private:
    static MemoryManager::MallocFunction s_fcnPacketBufferMalloc;
    static MemoryManager::MallocFunction s_fcnPacketManagementObjectMalloc;
    static void* s_pPacketBuffer;
    static qUnsignedInt32 s_packetBufferSize;
    static qUnsignedInt32 s_refCount;
    static qUnsignedInt32 s_requiredPacketSpace;
    static qUnsignedInt32 s_requiredSendPacketSpace;
    static const qUnsignedInt32 DEFAULT_REQUIRED_PACKET_SPACE = 8;
    static const qUnsignedInt32 DEFAULT_REQUIRED_SEND_PACKET_SPACE = 4;
    static qBool s_isEnableReceive;
    static qReal s_allocateFailProbability;
};

END_NAMESPACE_NEX

#endif // NEX_CLIENT_PBPOOL

#endif /* _PacketBufferManager_H_ */
