/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Platform_H_
#define _Platform_H_
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RandomNumberGenerator.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <OnlineCore/src/Platform/Core/String.h>


#if defined(UNIX_FAMILY) && defined(UNICODE)
#error NO SUPPORT FOR UNICODE UNDER LINUX
#endif

#ifdef MSC_COMPILER
#pragma warning(disable : 4786)
#pragma warning(disable : 4291)
#endif

#ifdef APPLY_PATCHES
#include <OnlineCore/src/Platform/Core/bugs.patches>
#endif

BEGIN_NAMESPACE_NEX

class ErrorDescriptionTable;

//group=Miscellaneous Classes

/* Description: Encapsulates the methods used by $PRODUCTNAME$ to monitor itself.
     A Platform object should never be directly instantiated. It is created
     by $PRODUCTNAME$ when required.
   Remarks: Encapsulates the methods used by $PRODUCTNAME$ to monitor itself.
*/
/* Please see man pages for details 
    
*/
class Platform: public RootObject
{
public:
    Platform();
    virtual ~Platform();

    // Description: Returns a reference to the Platform instance.
    static Platform *Instance();

    //{secret}
    static void CreateInstance();

    //{secret}
    static void DeleteInstance();

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
#endif

    static qInt64 GetTick();

    static qUnsignedInt32 GetCurrentTimeMilli(qInt64 startTick);

    /* Please see man pages for details 
        

        
    */
    static void Sleep(qUnsignedInt32 uiTime);

    //{secret}
    static void YieldThread();

    /* Please see man pages for details 
        

        
    */
    static void SetRandomNumberSeed(qUnsignedInt32 uiSeed);

    /* Please see man pages for details 
        

        

        

        
    */
    static qReal GetRealRandomNumber(qReal fMaxValue);

    /* Please see man pages for details 
        

        

        

        
    */
    static qUnsignedInt32 GetRandomNumber(qUnsignedInt32 uiMaxValue);


    static qUnsignedInt32 GetRandomSeed();


    //{secret}
    static void Breakpoint();

    //{secret}
    static void NetworkToHost(const qByte *ucNet, qUnsignedInt16 *ucHost);

    //{secret}
    static void NetworkToHost(const qByte *ucNet, qUnsignedInt32 *ucHost);

    //{secret}
    static void NetworkToHost(const qByte *ucNet, qUnsignedInt64 *ucHost);

    //{secret}
    static void HostToNetwork(const qUnsignedInt16 *ucHost, qByte *ucNet);

    //{secret}
    static void HostToNetwork(const qUnsignedInt32 *ucHost, qByte *ucNet);

    //{secret}
    static void HostToNetwork(const qUnsignedInt64 *ucHost, qByte *ucNet);

    //{secret}
    static void WarnObsoleteMethod(const qChar* strOldMethod, const qChar* strNewMethod);

#if defined(FILE_SUPPORT) && (defined(WIN32_TARGET) || defined(LINUX_TARGET))

    static qBool ChangeCurrentDirectory(const qChar* szNewDir);

    //  Build a file path with the appropriate delimiter for the OS (\ or /)
    static String JoinPath(const qChar* szFirstPart, const qChar* szSecondPart);

    static qBool GetCWD(qChar* szNewDir, qUnsignedInt32 szlen);
#endif



private:
    static qInt AssertTranslator( qInt ReportType, qChar8 *UserMsg, qInt *RetVal );

private:
    static  Platform        *_Instance;
    static ErrorDescriptionTable m_oErrorTable;
    static RandomNumberGenerator    s_oRNG;
    static qBool s_bSeedIsInitialized;
    static qUnsignedInt64 GetPlatformID();
};

END_NAMESPACE_NEX

#endif
