﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef CTR_PROFILING_H_
#define CTR_PROFILING_H_

#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>

#include <stdlib.h>  // For mbstowcs().

/* Please see man pages for details 
    

    
    

    
    
        
    

    
    
    
    
    
    

    
    
        
        
    

    
    

    
    
        
        

        
        
            

            
            
        
    


    
    
    
    
    
    
        

        
        
    


    
    
 */
//#define NEX_DISABLE_USING_CTR_PROFILING_UNIT 1

#if NEX_DISABLE_USING_CTR_PROFILING_UNIT

#define CTR_PROFILING_UNIT(order)                   (void)0
#define CTR_PROFILING_UNIT_WITH_NAME(order, name)   (void)0
#define CTR_PROFILING_UNIT_WITH_SIZE(order, amount) (void)0

#else
//n1769
// Some failure with UNICODE build.
// the _T() macro was developed and became the L macro, but this L macro was combined with __PRETTY_FUNCTION__ which ended up generating strange symbols.
// 
// http://social.msdn.microsoft.com/Forums/en-US/vcgeneral/thread/82213969-1796-4000-9656-3e692e3ec634
// Tried to use the technique described in the site above and wrote the following macro, but this didn't work either.
// 
//  #define MY_TEXT_MACRO2(x) L##x
//  #define MY_TEXT_MACRO(x) MY_TEXT_MACRO2(x)
//  #define MY_PRETTY_FUNCTION MY_TEXT_MACRO(__PRETTY_FUNCTION__)
// Tried introducing a case statement according to whether a _UNICODE macro was present and then copying to a wide character array once during the UNICODE build.
//  For now, this gets through the build.

// Ordered. SetOrder function is called several times and is a waste, but it is ignored here.
#define CTR_PROFILING_UNIT_WITH_NAME(order, name)                               \
          static NAMESPACE_NEX ProfilingUnit s_profilingUnitWithNameInstance(_T(name)); \
          s_profilingUnitWithNameInstance.SetOrder(order);                        \
          NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnitWithName(s_profilingUnitWithNameInstance);

#ifndef _UNICODE
#define CTR_PROFILING_UNIT(order)                                                  \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitInstance(__PRETTY_FUNCTION__); \
            s_profilingUnitInstance.SetOrder(order);                                   \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnit(s_profilingUnitInstance);

#define CTR_PROFILING_UNIT_WITH_SIZE(order, amount)                                                \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitWithSizeInstance(__PRETTY_FUNCTION__);         \
            s_profilingUnitWithSizeInstance.SetOrder(order);                                           \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnitWithSize(s_profilingUnitWithSizeInstance); \
            s_profilingUnitWithSizeInstance.IncAmount(amount);

#else

#define CTR_PROFILING_UNIT(order)                                                    \
            static bool  s_first_time_flag = true;                                       \
            static NAMESPACE_NEX qChar s_function_name[128];                                   \
            if(s_first_time_flag)mbstowcs(s_function_name, __PRETTY_FUNCTION__, 128);    \
            s_first_time_flag = false;                                                   \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitInstance(s_function_name);       \
            s_profilingUnitInstance.SetOrder(order);                                     \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnit(s_profilingUnitInstance);

#define CTR_PROFILING_UNIT_WITH_SIZE(order, amount)                                                \
            static bool  s_first_time_flag = true;                                                     \
            static NAMESPACE_NEX qChar s_function_name[128];                                                 \
            if(s_first_time_flag)mbstowcs(s_function_name, __PRETTY_FUNCTION__, 128);                  \
            s_first_time_flag = false;                                                                 \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitWithSizeInstance(s_function_name);             \
            s_profilingUnitWithSizeInstance.SetOrder(order);                                           \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnitWithSize(s_profilingUnitWithSizeInstance); \
            s_profilingUnitWithSizeInstance.IncAmount(amount);
#endif  // end of _UNICODE

#endif  // end of NEX_DISABLE_USING_CTR_PROFILING_UNIT

#endif  // end of CTR_PROFILING_H_
