/*---------------------------------------------------------------------------*
  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Readwritelock_H_
#define _Readwritelock_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/LockMasks.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/Condition.h>


BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
 
 
 
 
 
 */
class ReadWriteLock: public RootObject
{
public:
    ReadWriteLock(qUnsignedInt32 uiLockGroup);
    virtual ~ReadWriteLock();

    void BeginRead();
    void EndRead();
    void BeginWrite();
    void EndWrite();

    class ScopedWriteLock: public RootObject
    {
    public:
        ScopedWriteLock(ReadWriteLock *lock);
        ~ScopedWriteLock();
    private:
        ReadWriteLock* m_pReadWriteLock;
    };

    class ScopedReadLock : public RootObject
    {
    public:
        ScopedReadLock(ReadWriteLock *lock);
        ~ScopedReadLock();
    private:
        ReadWriteLock* m_pReadWriteLock;
    };


private:

    volatile qInt m_readerCounts;
    volatile qInt m_waitingWriterCounts;
    volatile qBool m_isWriting;

    Condition m_condition;
    ForcedCriticalSection m_lock;
};

END_NAMESPACE_NEX

#endif /* _Readwritelock_H_ */
