/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _RefCountedObject_H_
#define _RefCountedObject_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>

#ifdef TEST_ENABLED
#define ACQUIREREF(a) a->AcquireRef(_T(__FILE__),__LINE__)
#define RELEASEREF(a) a->ReleaseRef(_T(__FILE__),__LINE__)
#else
#define ACQUIREREF(a) a->AcquireRef()
#define RELEASEREF(a) a->ReleaseRef()
#endif

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
 

 
 
 
 */
class RefCountedObject: public RootObject
{

public:
    RefCountedObject();
    virtual ~RefCountedObject();

    class Ref
    {
    private:
        void AcquireRef()
        {
            if (m_pObject!=NULL)
            {
                ACQUIREREF(m_pObject);
            }
        }
        void ReleaseRef()
        {
            if (m_pObject!=NULL)
            {
                RELEASEREF(m_pObject);
                m_pObject=NULL;
            }
        }
    public:
        Ref(RefCountedObject* pObject=NULL): m_pObject(pObject)
        {
        }
        Ref(const Ref& oRef): m_pObject(oRef.m_pObject)
        {
            AcquireRef();
        }
        ~Ref()
        {
            ReleaseRef();
        }

        const Ref& operator=(const Ref& oRef)
        {
            ReleaseRef();
            m_pObject=oRef.m_pObject;
            AcquireRef();
            return *this;
        }

        RefCountedObject* operator->()
        {
            return m_pObject;
        }
        const RefCountedObject* operator->() const
        {
            return m_pObject;
        }
    private:
        RefCountedObject* m_pObject;
    };


#ifdef TEST_ENABLED
    virtual RefCountedObject    *AcquireRef(qChar *szFile, qUnsignedInt32 uiFileNo);
    virtual void                ReleaseRef(qChar *szFile, qUnsignedInt32 uiFileNo);
#else
    /* Please see man pages for details 
     
     */
    inline RefCountedObject *AcquireRef()
    {
        m_RefCount++;
        return this;
    }
    /* Please see man pages for details 
     
     */
    inline void ReleaseRef()
    {
        qInt32 ref = (--m_RefCount);
        SYSTEMCHECK(ref >= 0);
        if (ref == 0 && !m_bIsDeleting)
        {
            m_bIsDeleting = true;
            qDelete this;
        }
    }
#endif

#ifdef TEST_ENABLED
    virtual RefCountedObject    *AcquireRef(qChar *szFile, qUnsignedInt32 uiFileNo);
    virtual void                ReleaseRef(qChar *szFile, qUnsignedInt32 uiFileNo);
    static qBool Test();
#endif

    inline void TraceRefs( qBool boTraceRefs )
    {
#ifdef DEBUG_VERSION
        m_boTraceRefs = boTraceRefs;
#else
        DUMMY_PARAM(boTraceRefs);
#endif
    }

    qInt32 GetRefCount() const
    {
        return m_RefCount.Get();
    }



private:
#ifdef DEBUG_VERSION
    qBool m_boTraceRefs;
    qByte padding1[3];  // to avoid compiler warning.
#endif

    AtomicValue m_RefCount;

    qBool m_bIsDeleting;
};


END_NAMESPACE_NEX

#endif
