﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StringStream_H_
#define _StringStream_H_

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <stdarg.h>

//n2788: Added 0-padded hexadecimal display
#define ENABLE_ZERO_FILL_HEX 1

BEGIN_NAMESPACE_NEX

#define STRINGSTREAM_INITIAL_SIZE   256

//==============================================================================
// Internal class for text strings stream.
//==============================================================================
/* Please see man pages for details

 */
class StringStream: public RootObject
{
public:
    StringStream();
    virtual ~StringStream();

    // Returns the stream length
    qUnsignedInt32 GetLength() const;

    // Returns the m_szBuffer size (capacity)
    qUnsignedInt32 GetSize() const
    {
        return m_uiSize;
    }

    /* Please see man pages for details


     */
    const qChar* CStr() const
    {
        return m_szBuffer;
    }

    void Clear();

    // Streaming of basic types
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(const qChar* szText);
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(const void* pPointer);
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(qBool bValue);
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(qDouble dValue);
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(qFloat fValue);
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(qByte uiValue)
    {
        return StreamNumber(uiValue);
    }

    /* Please see man pages for details
	
	
	
     */
    StringStream& operator<<(qInt8 iValue)
    {
        return StreamNumber(iValue);
    }
    /* Please see man pages for details
	
	
	
     */
    StringStream& operator<<(qUnsignedInt16 uiValue)
    {
        return StreamNumber(uiValue);
    }
    /* Please see man pages for details
	
	
	
     */
    StringStream& operator<<(qInt16 iValue)
    {
        return StreamNumber(iValue);
    }
    /* Please see man pages for details
	
	
	
     */
    StringStream& operator<<(qUnsignedInt32 uiValue)
    {
        return StreamNumber(uiValue);
    }
    /* Please see man pages for details
	
	
	
     */
    StringStream& operator<<(qInt32 iValue)
    {
        return StreamNumber(iValue);
    }

    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(qUnsignedInt64 uiValue);
    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(qInt64 uiValue);

    /* Please see man pages for details
         
         
         
     */
    StringStream& operator<<(const StringStream& oSS);

#if defined(MACOSX_TARGET) || defined(WII_TARGET) || defined (CAFEIOP_TARGET)
    // How is size_t mapped in Unix targets??
    StringStream& operator<<(size_t uiValue)
    {
        return StreamNumber(qUnsignedInt32(uiValue));
    }
#endif

    // Manipulators
    StringStream& operator<<(StringStream& (*pf)(StringStream&))
    {
        return (*pf)(*this);
    }

#if !ENABLE_ZERO_FILL_HEX
    StringStream& Manip_hex()
    {
        m_bHex = true;
        return *this;
    }
    StringStream& Manip_dec()
    {
        m_bHex = false;
        return *this;
    }
#else // !ENABLE_ZERO_FILL_HEX
    StringStream& Manip_hex()
    {
        m_bHex = true;
        m_bZeroHex =false;
        return *this;
    }
    StringStream& Manip_dec()
    {
        m_bHex = false;
        m_bZeroHex = false;
        return *this;
    }
    StringStream& Manip_zerohex()
    {
        m_bHex = true;
        m_bZeroHex =true;
        return *this;
    }
#endif // !ENABLE_ZERO_FILL_HEX

    StringStream& Manip_showbase()
    {
        m_bShowBase = true;
        return *this;
    }
    StringStream& Manip_noshowbase()
    {
        m_bShowBase = false;
        return *this;
    }

    StringStream& Manip_boolalpha()
    {
        m_bBoolAlpha = true;
        return *this;
    }
    StringStream& Manip_noboolalpha()
    {
        m_bBoolAlpha = false;
        return *this;
    }

    const qChar& operator[](qUnsignedInt32 uiPos) const
    {
        return m_szBuffer[uiPos];
    }

    StringStream& BytesDump(const qByte *qBytes, qUnsignedInt len);

private:
    // Disable these methods.
    StringStream(const StringStream&);
    StringStream& operator=(const StringStream&);

private:
    void FreeBuffer();
    void FreeBuffer(qChar *szBuffer);
    void ResizeBuffer(qUnsignedInt32 uiSize);
    void AddBaseIfRequired();

    StringStream& StreamNumber(qByte uiValue);
    StringStream& StreamNumber(qUnsignedInt32 uiValue);
    StringStream& StreamNumber(qInt32 iValue);
    void TestFreeRoom(qUnsignedInt32 uiNewSize);

public:
#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif

private:
    qChar* m_szBuffer;
    qUnsignedInt32 m_uiSize;
    qChar* m_szCurrentPosition;

    qChar m_szInitialBuffer[STRINGSTREAM_INITIAL_SIZE];

    qBool m_bHex;
    qBool m_bShowBase;
    qBool m_bBoolAlpha;

#if ENABLE_ZERO_FILL_HEX
    //n2788
    qBool m_bZeroHex;
#else
    qByte m_padding1;  // to avoid compiler warning.
#endif // ENABLE_ZERO_FILL_HEX
};

// Manipulators.

/* Please see man pages for details 
 
 
 
*/
extern StringStream& hex(StringStream& oStringStream);
/* Please see man pages for details 
 
 
 
*/
extern StringStream& dec(StringStream& oStringStream);
/* Please see man pages for details 
 
 
 
*/
extern StringStream& endl(StringStream& oStringStream);
/* Please see man pages for details 
 
 
 
*/
extern StringStream& showbase(StringStream& oStringStream);
/* Please see man pages for details 
 
 
 
*/
extern StringStream& noshowbase(StringStream& oStringStream);
/* Please see man pages for details 
 
 
 
*/
extern StringStream& boolalpha(StringStream& oStringStream);
/* Please see man pages for details 
 
 
 
*/
extern StringStream& noboolalpha(StringStream& oStringStream);

#if ENABLE_ZERO_FILL_HEX
extern StringStream& zerohex(StringStream& oStringStream);
#endif // ENABLE_ZERO_FILL_HEX
END_NAMESPACE_NEX

#endif
