﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

//
//  Values are 32 bit values laid out as follows:
//
//   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
//   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
//  +---+-+-+-----------------------+-------------------------------+
//  |Sev|1|R|     Subsystem         |               Code            |
//  +---+-+-+-----------------------+-------------------------------+
//
//  where
//
//      Sev - is the severity code
//
//          00 - Success
//          01 - Informational
//          10 - Warning
//          11 - Error
//
//      R - is a reserved bit
//
//      Subsystem - is the Subsystem code
//
//      Code - is the Subsystem's status code
//
//
// Define the facility codes
//

#ifndef _SystemError_H_
#define _SystemError_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#ifdef CTR_TARGET
#include <nn/Result.h>
#endif



//n1589: The four SEV flags are the 2 bits of 30 - 32 bits
#define SEV_SUCCESS                         0x0
#define SEV_INFORMATIONAL                   0x1u
#define SEV_WARNING                         0x2u
#define SEV_ERROR                           0x3u

//n1589: The SUBSYSTEM flag is the 13 bits between bits 16 - 28
enum SubSystemType
{
    SUBSYSTEM_GENERAL = 0,
    // SUBSYSTEM_NETWORK, // not used
    // SUBSYSTEM_DC, // not used
    SUBSYSTEM_OBJDUP,
    // SUBSYSTEM_TRANSPORT_GENERAL, // not used
    // SUBSYSTEM_TRANSPORT_UDS, // not used
    // SUBSYSTEM_TRANSPORT_SOCKET, // not used
    NUMBER_OF_SUBSYSTEMS,
    SUBSYSTEM_NONE = 0x10
};

BEGIN_NAMESPACE_NEX

//n1589:?:ERROR_IS_WIN32 flag is always ON ? or reserved ?
#define SET_ERROR_CODE(SEV,SUBSYS,STATUS)   (NAMESPACE_NEX qUnsignedInt32)((SEV<<30)|(1<<29)|(SUBSYS<<16)|STATUS)
#define ERROR_IS_WIN32(ERRCODE)             ((ERRCODE&(0x1<<29))==0)
#define GET_SUBSYSTEM(ERRCODE)              ((ERRCODE>>16)&0xFFF)
#define GET_STATUS(ERRCODE)                 (ERRCODE&0xFFFF)
#define GET_SEVERITY(ERRCODE)               ((ERRCODE>>30)&0x3)

// {secret}
// SUBSYSTEM_NONE Error Codes
#define SYSTEMERROR_FACADE_NOT_INITIALIZED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_NONE,0)

#define SIGNAL_SYSTEMERROR(Code,ExtCode)            SystemError::SignalError(_T(__FILE__),__LINE__,Code,ExtCode);
// #define SIGNAL_CTRSDK_RESULT(result)                SystemError::SignalError(_T(__FILE__),__LINE__,result);
// #define CALL_ERROR_HANDLING_FUNCTION(result)        SystemError::CallErrorHandlingFunction(result,_T(__FILE__),__LINE__);

// group = System Errors

/*
Description: There was not enough memory to complete the operation.
*/
#define SYSTEMERROR_GEN_NOT_ENOUGH_MEM  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,0)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_ILLEGAL_ACCESS  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,1)

/*
Description: An underlying WIN_FAMILY operation has failed.  The WIN_FAMILY error code can be
retrieved with SystemError::GetLastExt
*/
#define SYSTEMERROR_GEN_WIN32_ERROR     SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,2)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_INDEX_OOB           SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,3)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_DUPLICATED_KEY  SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,4)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_ITEM_NOT_DELETED    SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,5)
/*
{Secret}
*/
#define SYSTEMERROR_GEN_ITEM_NOT_FOUND  SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,6)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_END_OF_COLLECTION   SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL,7)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_OBJ_NOT_INIT        SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,8)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_NOT_IMPLEMENTED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,9)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_OUT_OF_ID           SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL, 10)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_INTERNAL            SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL, 11)

/*
Description: The delay that allowed for an operation to occur has expired.
*/
#define SYSTEMERROR_GEN_TIMEOUT         SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_GENERAL, 12)

/*
{Secret}
*/
#define SYSTEMERROR_GEN_CHILDPROCESS_FAILED SET_ERROR_CODE(SEV_WARNING, SUBSYSTEM_GENERAL, 13)

/*
Description: An operation was called in an invalid context.
*/
#define SYSTEMERROR_GEN_INVALID_OPERATION   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,14)

/*
Description: An attempt was made at instantiating more than one $PRODUCTNAME$ object in the same process.
*/
#define SYSTEMERROR_GEN_MULTIPLE_INSTANCES SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,15)

/*
{secret}
Description: An attempt to register more than one ExceptionHandler was made.
*/
#define SYSTEMERROR_GEN_MULTIPLE_EXCEPTION_HANDLER SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,16)

/*
Description: An attempt to use an expired version of $PRODUCTNAME$ was made.
*/
#define SYSTEMERROR_GEN_EXPIRED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,17)  // Currently, this should never occur because the product key isn't checked.

/*
Description: A method was called with an out-of-range parameter.
*/
#define SYSTEMERROR_GEN_OUT_OF_RANGE SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,18)

//{secret}
#define SYSTEMERROR_GEN_ASSERTION_FAILED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,19)

//{secret}
#define SYSTEMERROR_GEN_DEADLOCK_DETECTED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,20)

/*
Description: $PRODUCTNAME$ has not been initialized.
*/
#define SYSTEMERROR_GEN_NOT_INITIALIZED SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,21)

/*
Description: An invalid parameter, which is typically NULL, was passed to a method.
*/
#define SYSTEMERROR_GEN_INVALID_PARAMETER SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,22)

//Description: An invalid input was passed to a method.
#define SYSTEMERROR_GEN_IO_ERROR SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,23)

/*
Description: The product key used for $PRODUCTNAME$ does not exist or is invalid.
*/
#define SYSTEMERROR_GEN_INVALID_PRODUCT_KEY   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,24)

/*
Description: A buffer extraction tried to go beyond the end of the buffer.
*/
#define SYSTEMERROR_GEN_BUFFER_EXTRACTION_OVERFLOW   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,25)

/*
Description: The timeout value set is invalid.
Remarks: This error is returned if DuplicatedObject::Create or DuplicatedObject::Publish are called
in a singlethread context with a timeout value other than 0.
See Also: DuplicatedObject::Create, DuplicatedObject::Publish
*/
#define SYSTEMERROR_GEN_INVALID_WAIT   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,26)

// Description: Access to the requested resource was refused.
#define SYSTEMERROR_GEN_ACCESS_DENIED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,27)

// Description: The operation was denied.
#define SYSTEMERROR_GEN_OP_DENIED  SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,28)

//n1769
#define SYSTEMERROR_GEN_INVALID_OPERATION_WARNING SET_ERROR_CODE(SEV_WARNING,SUBSYSTEM_GENERAL,29)

//n1769
#ifdef CTR_TARGET
#define SYSTEMERROR_GEN_CTR_ERROR   SET_ERROR_CODE(SEV_ERROR,SUBSYSTEM_GENERAL,30)
#endif

//n1589
//#define SYSTEMERROR_TRANSPORT_GENERAL_RELIABLE SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_TRANSPORT_GENERAL, 0)  //n1769: Deleted because not used.

//n1589
//#define SYSTEMERROR_TRANSPORT_UDS SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_TRANSPORT_UDS, 0)                   //n1769: Deleted because not used.

//n1769
//#define SYSTEMERROR_TRANSPORT_UDS_INVALID_STATE   SET_ERROR_CODE(SEV_ERROR, SUBSYSTEM_TRANSPORT_UDS, 1)   //n1769: Deleted because not used.

class UserContext;
class qResult;

// {Secret}
class ErrorDescriptionTable: public RootObject
{

public:

    ErrorDescriptionTable(qChar *aszStatusMessage[], SubSystemType uiSubsystem);
    virtual ~ErrorDescriptionTable();

    SubSystemType       m_uiSubsystem;
    qChar               **m_aszStatusMessage;
};


// Group=Results Classes

/*
Description: This class enables the retrieval of error information.
Remarks: $PRODUCTNAME$ uses error codes to signal errors. The error codes are used to
obtain more information about the error. If an error occurs
while calling a method from any $PRODUCTNAME$ interface, information about the error
is stored within this class. If a method causes a $PRODUCTNAME$ error, it is the return value
of that method that will signal the error. For example, Session::CreateSession will
return true if the call did not signal an error, and false if it did. In the case of a method that
returns a value such as DuplicatedObject::Create, NULL is returned if there is an error.
To retrieve the error information, use the methods of this class. For example:

        Avatar* pNewAvatar= Avatar::Create();
        if (pNewAvatar==NULL) {
            Display(_T("ERROR: Could not create Avatar."));
            qChar strError[256] = _T("");
            SystemError::GetErrorString(SystemError::GetLast(), strError, 256);
            Display(_T("%s!\n"), strError);
            return 0;
        }

*/
/* Please see man pages for details 
  

  
  
  

  
  

  
  

  
  

  
  

  
  
  

  
   */
class SystemError: public RootObject
{
    //{secret}
    class ImplAttributes;
    //Group=Methods
public:

    // {secret}
    SystemError(qChar *szFilename, qUnsignedInt LineNo);
    // {Secret}
    virtual ~SystemError() {}

    // {secret}
    SystemError SetErrorInfo(qUnsignedInt32 uiErrorCode, qUnsignedInt32 uiExtErrorCode);
    // {Secret}
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    /*
    Description: Returns a description of the error as a string.
    Parameters:
    uiErrorCode: The $PRODUCTNAME$ error code that caused the error.
    szStringBuf: A string buffer that will hold the result.
    uiBufSize: Specifies the maximum number of characters that can be copied into
    the buffer pointed to by szStringBuf, including a terminating null character.
    Returns     : A description of the error as a string.
    Remarks: This method fills a user supplied buffer with the description of an error.
    See Also: GetLast, GetLastExt
    */
    /* Please see man pages for details 
    

    
    
    

    
    */
    static void GetErrorString(qUnsignedInt32 uiErrorCode, qChar *szStringBuf, qUnsignedInt uiBufSize);

    // {Secret}
    static void SignalError(qChar *szFilename, qUnsignedInt uiLineNo, qUnsignedInt32 uiErrorCode, qUnsignedInt32 uiExtErrorCode);

    /*
    Summary: ClearLast will remove the stored error.
    Remarks: This method may be called before GetLast or GetLastExt to ensure that the
    returned error is generated by the most recently called method.
    See Also: GetLast, GetLastExt
    */
    /* Please see man pages for details 
    

    
    

    
    
    */
    static void ClearLast();

    //{secret}
    static void ResetSystemError();

    /*
    Summary: Returns the $PRODUCTNAME$ error code of the most recent error.
    Returns     : The specific error code of the most recent $PRODUCTNAME$
    error that occurred. 0 is returned if no error is encountered.
    See Also: GetLastExt
    */
    /* Please see man pages for details 
    

            
            
            

    

    
    */
    static qUnsignedInt32 GetLast();

    /*
    Summary: Returns the system error code of the most recent error.
    Returns     : The system error code of the most recent error that occurred.
    See Also: GetLast
    */
    /* Please see man pages for details 
    

            
            
            

            

    
    
    */
    static qUnsignedInt32 GetLastExt();

#ifdef CTR_TARGET
    //n1769
    /* Please see man pages for details 
     
    

            
            
            
            

            
            

    

    
    */
    static nn::Result GetLastCtr();
#endif

    //n1589
    // {Secret}
    //  Determines whether the severity of the most recent error is Error-level.
    //  Returns true if the severity of the most recent error is SEV_ERROR; returns false otherwise.
    static qBool IsError();

    //n1589
    // Determines whether the severity of the most recent error is Error-level.
    // Pass errors that can be obtained with GetLast function to uiCode.
    // Returns true if the severity of the most recent error is SEV_ERROR; returns false otherwise.
    static qBool IsError(qUnsignedInt32 uiCode);

    //n1769
    // {Secret}
    // Determines whether the severity of the most recent error is Warning-level.
    // Pass errors that can be obtained with GetLast function to uiCode.
    // Returns true if the severity of the most recent error is SEV_WARNING; returns false otherwise.
    static qBool IsWarning(qUnsignedInt32 uiCode);

    //n1769
    // {Secret}
    // Determines whether the severity of the passed error is Informational-level.
    // Pass errors that can be obtained with GetLast function to uiCode.
    // Returns true if the severity of the error is SEV_INFORMATIONAL; returns false otherwise.
    static qBool IsInformational(qUnsignedInt32 uiCode);

    //n1769
    // {Secret}
    // Determines whether the severity of the passed error is Success-level.
    // Pass errors that can be obtained with GetLast function to uiCode.
    // Returns true if the severity of the error is SEV_SUCCESS; returns false otherwise.
    static qBool IsSuccess(qUnsignedInt32 uiCode);

    /*
    Summary: Traces the most recent error.
    Parameters:
    uiTraceFlag: A flag that determines what data is traced. Refer to DataLogging for a list of valid trace flags.
    Remarks: This method outputs a string describing the most recent error to the display, the debugger, or a file.
    See Also: Data Logging
    */
    /* Please see man pages for details 
    
    

    
    */
    static void TraceLast(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

#ifdef TEST_ENABLED
    // Tests
    // {secret}
    static qBool Test();
#endif

    // {Secret}
    qUnsignedInt32 GetErrorCode()
    {
        return m_uiErrorCode;
    }
    // {Secret}
    qUnsignedInt32 GetExtErrorCode()
    {
        return m_uiExtErrorCode;
    }

    //n1769
    // {Secret}
    // Delete the element related to the error code corresponding to the specified thread ID..
    // This API is provided to prevent memory leaks when threads are repeatedly created and destroyed.
    // 
    // Avoid calling this API directly from the application because it is assumed it will be called from the NEX library.
    // 
    static void EraseErrorElements(qUnsignedInt32 threadID);

    //n1769 Different from the above, note that these are not {Secret} APIs.
    /* Please see man pages for details 
    

           
           
           

           
           
           
           
           

           
           
           

    
    */
    static void EraseErrorElements(void);

    //n1769
    // {Secret}
    // Deletes all recorded error information from the container.
    // Do not call this function directly from the application because it is assumed it will be called from within the NEX library.
    // 
    static void EraseAllErrorElements(void);

private:
    qUnsignedInt32      m_uiErrorCode;
    qUnsignedInt32      m_uiExtErrorCode;
    qChar               *m_szFilename;
    qUnsignedInt        m_uiLineNo;
};

END_NAMESPACE_NEX

#endif

