﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Time_H_
#define _Time_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>
#include <OnlineCore/src/Platform/Core/TimeDefs.h>
#ifdef UNIX_FAMILY
#include <sys/time.h>
#include <unistd.h>
#endif

//{secret}
BEGIN_NAMESPACE_NEX

class TimeProvider;

/*
$ifdef VOICE$
{secret}
$else$

Description: Expresses a time in milliseconds.
Remarks: The SystemClock and SessionClock classes can be used to get Time values.
The following example sets the value of the variable /tNow/ to
the number of milliseconds elapsed since the session started.

    Time tNow = SessionClock::GetTime();

See also: SystemClock, SessionClock, TimeInterval
$endif$
*/
/* Please see man pages for details 
 
 
 
 */
class Time: public RootObject
{
public:
    //Group=Methods

    /*
    Parameters:
    ui64Value: The initial time.
    tTime: The initial time.
    Remarks: The copy constructor, Time(const Time& tTime), is not valid on Win32 platforms.
    */
    /* Please see man pages for details 
     
     
     
     */
    Time() : m_ui64Value(0) {}

    /* Please see man pages for details 
     
     
     */
    Time(const qUnsignedInt64 ui64Value) : m_ui64Value(ui64Value) {}

#ifdef UNIX_FAMILY
    /* Please see man pages for details 
     
     
     
     
     */
    Time(const Time& tTime);

    // {secret}
    Time(const struct timeval oTimeval);
    // {secret}
    Time(const struct timespec oTimeSpec);

    // {Secret}
    void GetTime(struct timespec &oTimeSpec) const;
    // {secret}
    void GetTime(struct timeval &oTimevla) const;
#endif

    //n1589:memo:returns milli
    // {secret}
    /* Please see man pages for details 
     

            
            
            

     
     
     */
    static Time GetTime();
    // {secret}
    static inline Time GetCachedTime()
    {
#if NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION //n2343
        return s_cachedTime;
#else
        // return non cached time
        return Time::GetTime();
#endif
    }
    // {secret}
    static inline void SetCachedTime(Time& t)
    {
#if NEX_ENABLE_GET_TIME_CACHE_OPTIMIZATION //n2343
        s_cachedTime = t;
#else
        // Do nothing
        (void)t;
#endif
    }
    // {secret}
    static void Reset();
    // {secret}
    static Time GetSessionTime()
    {
        if (s_pfGetSessionTime!=NULL)
        {
            return s_pfGetSessionTime();
        }
        else
        {
            return 0;
        }
    }

    // {secret}
    static void  GetFrequency(Time* pliFrequency)
    {
        *pliFrequency=1000;
    }

    /* Please see man pages for details 
     
     
     
     */
    Time& operator=(const Time& tTime)
    {
        m_ui64Value=tTime.m_ui64Value;
        return *this;
    }
    /* Please see man pages for details 
     
     
     
     */
    Time& operator=(qUnsignedInt64 ui64Value)
    {
        m_ui64Value=ui64Value;
        return *this;
    }
    /* Please see man pages for details 
     
     
     */
    operator qUnsignedInt64() const
    {
        return m_ui64Value;
    }
    operator qUnsignedInt64()
    {
        return m_ui64Value;
    }

    /* Please see man pages for details 
     
     
     
     */
    Time operator-(TimeInterval tiTime) const
    {
        if (tiTime>0)
        {
#ifdef DEBUG_VERSION
            if (m_ui64Value>0)
            {
                SYSTEMCHECK(m_ui64Value>=(qUnsignedInt64)tiTime);
            }
#endif
            return m_ui64Value-tiTime;
        }
        else
        {
            // CodeWarrior promotes the tiTime to Unsigned prior to computing the -.
            return m_ui64Value+(-tiTime);
        }
    }

    /* Please see man pages for details 
     
     
     
     */
    Time operator+(TimeInterval tiTime) const
    {
        if (tiTime<0)
        {
#ifdef DEBUG_VERSION
            // We check if the operation is valid.
            if (tiTime==-tiTime)
            {
                // If tiTime is 0x80000000, the tiTime==-tiTime, and we cannot use the
                // regular check.
                SYSTEMCHECK(m_ui64Value>=0x8000000);
            }
            else
            {
                SYSTEMCHECK(m_ui64Value>=(qUnsignedInt64)-tiTime);
            }
#endif
            return m_ui64Value-(-tiTime);
        }
        else
        {
            return m_ui64Value+tiTime;
        }
    }

    /* Please see man pages for details 
     
     
     
     */
    Time operator-(LargeTimeInterval tiTime) const
    {
#ifdef DEBUG_VERSION
        if (tiTime>0 && m_ui64Value>0)
        {
            SYSTEMCHECK(m_ui64Value>=(qUnsignedInt64)tiTime);
        }
#endif
        return m_ui64Value-tiTime;
    }

    /* Please see man pages for details 
     
     
     
     */
    Time operator+(LargeTimeInterval tiTime) const
    {
#ifdef DEBUG_VERSION
        if (tiTime<0)
        {
            if (tiTime==-tiTime)
            {
                // for the lowest negative value, tiTime==-tiTime, and we cannot use the
                // regular check.
                // TODO... We should add a test, to be really safe...
            }
            else
            {
                SYSTEMCHECK(m_ui64Value>=qUnsignedInt64(-tiTime));
            }
        }
#endif
        return m_ui64Value+tiTime;
    }
    /* Please see man pages for details 
     
     
     
     */
    Time& operator-=(TimeInterval tiTime)
    {
        m_ui64Value-=tiTime;
        return *this;
    }
    /* Please see man pages for details 
     
     
     
     */
    Time& operator+=(TimeInterval tiTime)
    {
        m_ui64Value+=tiTime;
        return *this;
    }
    /* Please see man pages for details 
     
     
     
     */
    Time& operator-=(LargeTimeInterval tiTime)
    {
        m_ui64Value-=tiTime;
        return *this;
    }
    /* Please see man pages for details 
     
     
     
     */
    Time& operator+=(LargeTimeInterval tiTime)
    {
        m_ui64Value+=tiTime;
        return *this;
    }

#if NEX_ENABLE_TIME_SUBTRACT_OPERATOR_OPTIMIZATION
    LargeTimeInterval operator-(const Time& tTime) const
    {
        return static_cast<LargeTimeInterval>(m_ui64Value - tTime.m_ui64Value);
    }
#else
    /* Please see man pages for details 
     
     
     
     */
    LargeTimeInterval operator-(const Time& tTime) const;
#endif
    /* Please see man pages for details 
     
     
     
     */
    qBool operator==(const Time& tTime) const
    {
        return m_ui64Value==tTime.m_ui64Value;
    }
    /* Please see man pages for details 
     
     
     
     */
    qBool operator!=(const Time& tTime) const
    {
        return m_ui64Value!=tTime.m_ui64Value;
    }
    /* Please see man pages for details 
     
     
     
     */
    qBool operator<(const Time& tTime) const
    {
        return m_ui64Value<tTime.m_ui64Value;
    }
    /* Please see man pages for details 
     
     
     
     */
    qBool operator>(const Time& tTime) const
    {
        return m_ui64Value>tTime.m_ui64Value;
    }
    /* Please see man pages for details 
     
     
     
     */
    qBool operator<=(const Time& tTime) const
    {
        return m_ui64Value<=tTime.m_ui64Value;
    }
    /* Please see man pages for details 
     
     
     
     */
    qBool operator>=(const Time& tTime) const
    {
        return m_ui64Value>=tTime.m_ui64Value;
    }

    /*
    Description: Converts a time interval in seconds to milliseconds.
    Parameters:
    fTimeIntervalInSeconds: A time interval in seconds.
    Returns     : The given time interval in milliseconds.
    See Also: ConvertToSeconds
    */
    /* Please see man pages for details 
     
     
     
     */
    static TimeInterval ConvertFromSeconds(qReal fTimeIntervalInSeconds)
    {
        return TimeInterval(fTimeIntervalInSeconds * QREAL(1000.0));
    }

    /*
    Description: Converts a time interval in milliseconds to seconds.
    Parameters:
    tiTimeInterval: A time interval in milliseconds.
    Returns     : The given time interval in seconds.
    See Also: ConvertFromSeconds
    */
    /* Please see man pages for details 
     
     
     
     */
    static qReal ConvertToSeconds(TimeInterval tiTimeInterval)
    {
        return qReal(tiTimeInterval) / QREAL(1000.0);
    }

    //{secret}
    static Time ConvertTimeoutToDeadline(qUnsignedInt32 uiTimeout);
    //{secret}
    static qUnsignedInt32 ConvertDeadlineToTimeout(Time tDeadline);

    // {Secret}
    Time Scale(qReal fRatio) const;

    // {Secret}
    Time Multiply(qReal fFactor) const;
    // {Secret}
    Time Divide(qReal fDividend) const;

#ifdef TEST_ENABLED
    // {Secret}
    static qBool Test();
#endif

    // {secret}
    typedef qUnsignedInt64 (*SessionTimeCallback)();
    // {secret}
    static void RegisterSessionTimeCallback(SessionTimeCallback pfGetSessionTime)
    {
        s_pfGetSessionTime = pfGetSessionTime;
    }

    /*
    Description: This method has been deprecated. Use the SystemClock::RegisterTimeProvider method instead.
    Parameters:
    pTimeProvider: A pointer to the TimeProvider that will be registered to this Time object.
    By default the $PRODUCTNAME$ internal, platform specific TimeProvider is registered.
    Remarks: Registers the specified TimeProvider to this Time object. A custom TimeProvider may be used to
    provide the time that $PRODUCTNAME$ uses for time related computations, such as those associated with the
    SessionClock and SystemClock. Before the time can be retrieved from a TimeProvider other than the default,
    the TimeProvider must be registered to a Time object
    via this method. Note that RegisterTimeProvider must be called before the $PRODUCTNAME$ object is created as
    follows:

            // Call these two lines before you create the NetZ object
            MyTimeProvider oTimeProvider;
            Time::RegisterTimeProvider(&oTimeProvider);

    See Also: GetProviderTime
    */
    static void RegisterTimeProvider(TimeProvider *pTimeProvider);

    //n1769
    LargeTimeInterval fastSub(const Time& tTime) const;

private:

    qUnsignedInt64 m_ui64Value;
    static SessionTimeCallback s_pfGetSessionTime;
    static qUnsignedInt64 s_cachedTime;
};

// Time streaming.
//{secret}
extern StringStream& operator<<(StringStream& oStream, const Time& tValue);

//{secret}
END_NAMESPACE_NEX

#endif
