/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _UpdatableTimeProvider_H_
#define _UpdatableTimeProvider_H_

#include <OnlineCore/src/Platform/Core/HighResolutionClock.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>
#include <OnlineCore/src/Platform/Core/Platform.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/TimeProvider.h>

#ifdef CTR_TARGET
#include <nn/os.h>
#endif

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
 
 
 
 
 
 
 
 */
class UpdatableTimeProvider : public TimeProvider
{
public:
    /* Please see man pages for details */
    static const qUnsignedInt32 DEFAULT_THREAD_STACK_SIZE = 4 * 1024;
    /* Please see man pages for details */
    static const qUnsignedInt32 DEFAULT_SLEEP_TIME = 4;

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     
     
     
     
     
     */
    UpdatableTimeProvider(qUnsignedInt32 sleepTime = DEFAULT_SLEEP_TIME,
                          qUnsignedInt32 threadPriority = NN_NEX_PRIORITY_NORMAL,
                          qInt32 cpuCoreNo = ObjectThreadRoot::DEFAULT_CORE_NO,
                          qUnsignedInt32 threadStackSize = DEFAULT_THREAD_STACK_SIZE)
        : m_bRun(true), m_timeUpdaterThread(_T("TimeUpdatorThread"), threadPriority)
    {
        m_timeUpdaterThread.SetCoreNo(cpuCoreNo);
        m_timeUpdaterThread.Launch(this, &UpdatableTimeProvider::Update,
                                   sleepTime, threadStackSize);
    }

    /* Please see man pages for details 
     
     
     
     */
    virtual ~UpdatableTimeProvider()
    {
        Stop();
    }

    /* Please see man pages for details 
     
     
     
     
     
     */
    virtual Time GetTime()
    {
        return m_time;
    }

    /* Please see man pages for details 
     
     */
    void Stop()
    {
        m_bRun = false;
        m_timeUpdaterThread.Wait();
    }

    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    void Update(qUnsignedInt32 sleepTime)
    {
#ifdef CTR_TARGET
        QLOG(EventLog::Info, _T("Start UpdatableTimeProvider::Update CurrentProcessorNumber:") << nn::os::Thread::GetCurrentProcessorNumber());
#endif
        while (m_bRun)
        {
            m_time = HighResolutionClock::GetTime();
            Platform::Sleep(sleepTime);
        }
    }

#ifdef TEST_ENABLED
    static qBool Test();
#endif

private:
    Time m_time;  //
    volatile qBool m_bRun;  //
    ObjectThread<UpdatableTimeProvider, qUnsignedInt32> m_timeUpdaterThread;  //
};

END_NAMESPACE_NEX

#endif // _UpdatableTimeProvider_H_
