/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BerkeleySocketDriver_H_
#define _BerkeleySocketDriver_H_


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/Socket.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

#define SOCKET_IS_THREADSAFE

//n1589
#define BERKELEY_SOCKET_DRIVER_PEER_ADDRESS_MAX 15 // Maximum number of Station addresses to save, excluding own. Used as SendMulti address.
//n1589:todo2:We should document that port 0 is for SendMulti in the internal specifications.
#define BERKLEY_SOCKET_DRIVER_SEND_MULTI_PORT 0

BEGIN_NAMESPACE_NEX

#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
#else
#if defined(UNIX_FAMILY) || defined(NON_SUPPORT_A_TARGET) || defined(NON_SUPPORT_C_TARGET) || defined(NINTENDO_FAMILY)
typedef qInt32                  SocketHandle;
#else
typedef qUnsignedInt32          SocketHandle;
#endif
#endif

#if NEX_ENABLE_BERKLEY_SOCKET_PROFILING
typedef struct _BerkeleySocketAnalyzeInfo
{
    qUnsignedInt32 uiSendToCount;
    qUnsignedInt32 uiSendToMultiCount;
    qUnsignedInt32 uiRecvFromCount;
    qUnsignedInt32 uiRecvFromTryCount;
} BerkeleySocketAnalyzeInfo;
#endif

class BerkeleySocketDriver: public SocketDriver
{
public:
    class BerkeleySocket;

#if NEX_ENABLE_BERKLEY_SOCKET_PROFILING
    static void ResetAnalyzeInfo();
    static void LogAnalyzeInfo(EventLog::LogLevel logLevel);
    static BerkeleySocketAnalyzeInfo* GetAnalyzeInfo();
#endif

#if NEX_ENABLE_UNRELIABLE_BC
    static qBool IsSendMultiAddress(const InetAddress* address);
    static qUnsignedInt16 GetSendMultiPort();

    static void SetPeerAddress(qInt index, const nn::nex::InetAddress& address);
    static void GetPeerAddress(qInt index, nn::nex::InetAddress* address);

    static void SetPeerAddressCount(qInt peerAddressCount);
    static qInt GetPeerAddressCount();
    static nn::nex::CriticalSection& GetPeerAddressLock();
#endif // NEX_ENABLE_UNRELIABLE_BC

    BerkeleySocketDriver() {}
    virtual ~BerkeleySocketDriver() {}

    virtual Socket *Create();
    virtual void Delete(SocketDriver::Socket *pSocket);

    virtual qBool Poll(PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs);


    //n2967
    /* Please see man pages for details 
     */
    typedef void (*RecvFromCallback)(SocketDriver::Result result, const qByte *pBuffer,
                                     qUnsignedInt uiBufferSize,
                                     const nn::nex::SocketDriver::InetAddress *pSourceAddress, const qUnsignedInt *puiNbBytesRecvd);
    /* Please see man pages for details 
     */
    static void SetRecvFromCallback(RecvFromCallback cb);

    /* Please see man pages for details 
     */
    typedef void (*SendToCallback)(SocketDriver::Result result, const qByte *pBuffer,
                                   qUnsignedInt uiBufferSize,
                                   const nn::nex::SocketDriver::InetAddress &refDestinationAddress, const qUnsignedInt *puiNbBytesSent);
    /* Please see man pages for details 
     */
    static void SetSendToCallback(SendToCallback cb);

    /* Please see man pages for details 
     */
    typedef void (*RecvCallback)(SocketDriver::Result result, const qByte *pBuffer,
                                 qUnsignedInt uiBufferSize, const qUnsignedInt *puiNbBytesSent);
    /* Please see man pages for details 
     */
    static void SetRecvCallback(RecvCallback cb);

    /* Please see man pages for details 
     */
    typedef void (*SendCallback)(SocketDriver::Result result, const qByte *pBuffer,
                                 qUnsignedInt uiBufferSize, const qUnsignedInt *puiNbBytesSent);
    /* Please see man pages for details 
     */
    static void SetSendCallback(SendCallback cb);

private:

#ifdef NON_SUPPORT_C_TARGET
#error "Not supported platform."
#endif
};

END_NAMESPACE_NEX

#endif
