﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Socket_H_
#define _Socket_H_


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>

BEGIN_NAMESPACE_NEX

class IOCompletionContext;
class Buffer;
class InterfaceInfo;

class Socket: public RootObject
{
    friend class QueuingSocketTest;
    class ImplAttributes;

    friend StringStream& operator<<(StringStream& oStr, Socket& oSocket);

public:
    //n1589
    enum
    {
        STATE_OPEN,
        STATE_BOUND,
        STATE_DOWN,
        STATE_CLOSE
    };

    // With Winsock2, the value of GetLastSocketError gets used here.
    enum CALL_STATE
    {
        CALL_SUCCESS= 0,
        CALL_ERROR = 0xFFFFFFFFUL
    };

    static const qInt SOCKET_SUCCESS = 0; //Opposite of SOCKET_ERROR.
#if defined(CTR_TARGET) || defined(UNIX_FAMILY) //Defined as a Socket variable on other platforms.
    static const qInt SOCKET_ERR = -1; //Opposite of SOCKET_SUCCESS.
#else
    static const qInt SOCKET_ERR = SOCKET_ERROR; //Opposite of SOCKET_SUCCESS.
#endif
    //n1589
    static SocketDriver* GetDefaultSocketDriver()
    {
        return s_pDefaultSocketDriver;
    }

    Socket(qUnsignedInt uiTrafficType = DEFAULT_TRAFFIC_TYPE, qUnsignedInt32 uiMtu = DEFAULT_MTU);
    virtual ~Socket();

    qBool Open(qBool bUdpSocketType = true);

    void Shutdown();
    void Close();

    qBool Bind(InetAddress *pBindAddress, qBool bReuse = false);

    qInt Recv(qByte *pBuffer, qUnsignedInt uiBufferSize,
              InetAddress *pInetAddress, IOCompletionContext *pIOCompletionContext,
              SocketDriver::SocketFlag flag = SocketDriver::NonBlock);
    qInt Send(qByte *pBuffer, qUnsignedInt uiBufferSize,
              InetAddress *pInetAddress, IOCompletionContext *pIOCompletionContext);

    //  TCP socket
    qBool Connect(InetAddress *pConnectAddress, IOCompletionContext *pIOCompletionContext);

    Socket::CALL_STATE GetIOResult(IOCompletionContext *pIOCompletionContext);
    qBool SetMulticastAddress(qUnsignedInt32 uiAddress);
    InetAddress *GetAddress();

    qUnsignedInt16 GetBoundPort();

#ifdef WINSOCK2_STACK_SUPPORT
    qUnsignedInt32  GetHandle()
    {
        return m_hSocket;
    }
#endif

    qBool           IsUDP()
    {
        return m_bUdpSocketType;
    }
    qBool           IsVDP()
    {
        return m_bVDP;
    }
    qUnsignedInt    GetTrafficType()
    {
        return m_uiTrafficType;
    }


    qBool IsSocketValid() const
    {
        return m_isSocketValid;
    }

    static SocketDriver* GetSocketDriver()
    {
        return s_pSocketDriver;
    }
    static void SetSocketDriver(SocketDriver* pFactory)
    {
        s_pSocketDriver = pFactory;
    }
    SocketDriver::Socket* GetDriverSocket()
    {
        return m_pDriverSocket;
    }

    //n1589
    qByte GetState()
    {
        return m_eState;
    }

#ifdef WINSOCK2_STACK_SUPPORT
    qBool SetSendBufferSize(qUnsignedInt32 uiSize);
    qBool SetRecvBufferSize(qUnsignedInt32 uiSize);
    qBool SetAsync(qBool bAsyncMode);
    qBool SetBroadcastMode(qBool bBroadcast);
#endif

#ifdef TEST_ENABLED
    static qBool TestUDPSockets();
    static qBool TestTCPSockets();

    static qBool TestUDPSend();
    static qBool TestUDPRecv();

#ifdef WINSOCK2_STACK_SUPPORT
    void    SetHandle(qUnsignedInt32 hSocket)
    {
        m_hSocket = hSocket;
    }
#endif
#endif

    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

#if 0
    qBool ReuseAddress(qBool bReuse);
    qBool SetDebugMode(qBool bDebug);
#endif

    qBool SetTTL(qUnsignedInt8 uiTTL);

#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
    SocketHandle GetHandle();
#endif

#if NEX_ENABLE_SOCKET_TRANSPORT_SWITCH
    virtual void SetEnableTransport(qBool enable)
    {
        m_bIsEnableTransport = enable;
    }

    qBool IsEnableTransport()
    {
        return m_bIsEnableTransport;
    }
#endif

protected:
    //n1589
    static SocketDriver* s_pDefaultSocketDriver;

    //n1589
    //  enum { STATE_OPEN, BOUND, DOWN, CLOSE } m_eState;
    qByte m_eState;

    InetAddress         m_oInetAddress;
    qInt                m_iRecvAddrLen;
    qBool               m_bUdpSocketType;
    qUnsignedInt        m_uiTrafficType;
#ifdef WINSOCK2_STACK_SUPPORT
    qUnsignedInt32      m_hSocket;
#endif
    qUnsignedInt32      m_uiMTU;
    qBool               m_bVDP;

    static SocketDriver     *s_pSocketDriver;
    SocketDriver::Socket    *m_pDriverSocket;

#if NEX_ENABLE_SOCKET_TRANSPORT_SWITCH
    qBool m_bIsEnableTransport;
#endif

    qBool m_isSocketValid;

private:
    static qUnsignedInt32 GetLastSocketError();
    qBool ProcessDriverSocketResult(SocketDriver::Result eResult, IOCompletionContext *pIOCompletionContext);
#ifdef WINSOCK2_STACK_SUPPORT
    qBool ProcessWinsockResult(qInt iReturnCode, qUnsignedInt uiPendingCode, IOCompletionContext *pIOCompletionContext);
#endif
    qBool   _Bind(InetAddress *pBindAddress);

    static const qUnsignedInt DEFAULT_TRAFFIC_TYPE = 0;
    static const qUnsignedInt32 DEFAULT_MTU = 1000;

};

StringStream& operator<<(StringStream& oStr, Socket& oSocket);



END_NAMESPACE_NEX

#endif
