﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _GameStream_H_
#define _GameStream_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Core/Buffer.h>
#include <OnlineCore/src/Platform/Core/qQueue.h>
#include <OnlineCore/src/Transport/Interface/HighLevelStream.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Core/RefBuffer.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

class GameStream: public HighLevelStream
{
    // New type
public:
    class GameStreamMsg : public RootObject
    {
        // Methods
    public:
        GameStreamMsg() : m_idStream(0) {}
        GameStreamMsg(Buffer &oRecvBuffer, const InetAddress &oSourceInet, Stream::ID idStream)
        {
            m_oRecvBuffer = oRecvBuffer;
            m_oSourceInet = oSourceInet;
            m_idStream   = idStream;
        }

        // Data members
    public:
        RefBuffer m_oRecvBuffer;
        InetAddress m_oSourceInet;
        Stream::ID m_idStream;
    };

    // Methods
public:
    GameStream(qUnsignedInt32 maxRecvSize);
    virtual ~GameStream();

    qResult SendMsg(const InetAddress &oDestinationAddress, Stream::ID idStream, Buffer *pBuffer);
    qBool GetMsg(GameStreamMsg &gameMsg);

    // HighLevelStream overridden methods

    virtual void ReceiveIncomingMsg(const InetAddress &oSourceAddress, Stream::Type oType, Stream::ID idStream, Buffer *pBuffer, Time ioTime, qUnsignedInt16 ui16Port);

    // Data members
    static qBool s_specificHeaderCalc;
private:
    qQueue<GameStreamMsg> m_lstGameMsg;
    CriticalSection m_cslstGameMsg;
    qUnsignedInt32 m_maxRecvSize;
    qBool m_isRecvOverflow;
#ifdef DEBUG_VERSION
    qMap<InetAddress, Time> m_recvTimeMap;
#endif

};

END_NAMESPACE_NEX

#endif
