﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2014 Nintendo Co., Ltd.  All rights reserved.
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------
*/

#ifndef _NEX_RELAYSIGNATUREKEY_H_
#define _NEX_RELAYSIGNATUREKEY_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Core/qBuffer.h>
#include <OnlineCore/src/Platform/Core/DateTime.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>


BEGIN_NAMESPACE_NEX
enum RelayAddressType
{
    RELAY_ADDRESS_TYPE_USE_IP = 0, //IP Address from RV.
    RELAY_ADDRESS_TYPE_USE_NAMEADDRESS = 1 //Name Address from RV.
};

enum RelayMode
{
    RELAY_MODE_ERROR =  -1, //Cannot pass signature or address due to some error.
    RELAY_MODE_DISABLED = 0, //Relay mode off.
    RELAY_MODE_ENABLED = 1, //Relay mode on.
    RELAY_MODE_ENABLED_FORCE_EDM_RELAY = 2, //Force relay of EDM.
    RELAY_MODE_ENABLED_FORCE_ALL_RELAY = 3,  //Relay everything.

    RELAY_MODE_CONFIG_MIN = RELAY_MODE_DISABLED,
    RELAY_MODE_MIN = RELAY_MODE_ERROR,
    RELAY_MODE_MAX = RELAY_MODE_ENABLED_FORCE_ALL_RELAY
};

class RelaySignatureKey: public RootObject
{
public:
    DateTime currentUTCTime;
    String relayAddress;
    qUnsignedInt16 relayPort;
    qInt32 relayAddressType; //Is actually RelayAddressType.
    qInt32 relayMode; //Is actually RelayMode.
    qUnsignedInt32 gameServerID;

    RelayAddressType GetRelayAddressType() { return static_cast<RelayAddressType>(relayAddressType); }
    RelayMode GetRelayMode() { return static_cast<RelayMode>(relayMode); }

    RelaySignatureKey() : relayPort(0), relayAddressType(0), relayMode(0), gameServerID(QUNSIGNEDINT32_MAX) {}
};

END_NAMESPACE_NEX

#endif /* _NEX_RELAYSIGNATUREKEY_H_ */
