/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1589

#ifndef _SocketTransport_H_
#define _SocketTransport_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/InterfaceTable.h>
#include <OnlineCore/src/Platform/Stack/Core/IOCompletionNotifier.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>
#include <OnlineCore/src/Platform/Core/qSortedVector.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Core/SystemPorts.h>
#include <OnlineCore/src/Transport/Interface/Network.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferThread.h>
#include <OnlineCore/src/Transport/Interface/TransportPerfCounters.h>
#include <OnlineCore/src/Transport/Interface/VirtualPort.h>
#include <OnlineCore/src/Transport/Routing/Router.h>
#include <OnlineCore/src/Transport/Relay/Relay.h>
#include <OnlineCore/src/Transport/UDP/PacketQueue.h>
#include <OnlineCore/src/Transport/UDP/PacketHistoryChainDefine.h>

BEGIN_NAMESPACE_NEX

#if defined(WINSOCK2_STACK_SUPPORT)
#define SOCKET_HANDLE_SUPPORT
#endif

//n1589
#define NEX_ENABLE_SOCKET_TRANSPORT_SEND_TERMINATION 1

//n1589
#define NEX_ENABLE_SOCKET_RECEIVE_RECOGNIZER 1

#if NEX_ENABLE_SOCKET_TRANSPORT_PROFILING
typedef struct _SocketTransportAnalyzeInfo
{
    qUnsignedInt32 uiSendCount;
    qUnsignedInt32 uiReceiveCount;
    qUnsignedInt32 uiTransportJobLoopCount;
    qUnsignedInt32 uiOutgoingPacketQueueLatencyLimitDequeueCount;
    qUnsignedInt32 uiTransportBufferSendLatencyLimitDequeueCount;
} SocketTransportAnalyzeInfo;
#endif

class QueuingSocket;
class ICMPSocket;
class QueuingDevice;
class Event;
class Timeout;
class Buffer;
class PacketOut;
class PacketIn;
class Packet;
class Job;
class TransportBufferThreadInterface;
class QueuingSocketTransportBuffer; //n1589:for GetWellKnownTransportBufferSocket
class TransportBuffer;

/* Please see man pages for details 
 
 
 */
class SocketTransport: public RootTransport
{
public:
    friend class Network;
    friend class PRUDPStream;
    friend class QueuingSocketTest;
    class TransportJob;

    SocketTransport();
    virtual ~SocketTransport();

    /*
     * Inherited from RootTransport
     */
    virtual qResult Initialize();

    //n2343 start
    // Register callback to be called in each Transport thread loop
    typedef void (*TransportThreadLoopCallback) (void *);
    static void SetTransportThreadLoopCallback(TransportThreadLoopCallback cb);
    //n2343 end

    //n1589
    qBool TerminateTransport();

    virtual qBool StartListen(
        qUnsignedInt16 ui16Port,
        qUnsignedInt16 *pui16ListeningPort = NULL,
        qBool bPreferred = true,
        qUnsignedInt uiTrafficType = 0,
        qBool usePacketBufferPool=true
    );
    virtual qBool StartListenReuse(
        qUnsignedInt16 ui16Port,
        qUnsignedInt16 *pui16ListeningPort = NULL,
        qBool bPreferred = true,
        qUnsignedInt uiTrafficType = 0,
        qBool usePacketBufferPool=true
    );
    virtual qBool StopListen(qUnsignedInt16 ui16Port);
    virtual qBool StopListen();

    virtual qUnsignedInt32 GetNbListeningPorts();

    virtual qBool Send(
        qUnsignedInt16 ui16Port,
        Stream::Type oType,
        Stream::ID idSource,
        Stream::ID idDestination,
        PacketOut *pPacket,
        //n1589:memo: When FALSE, direct send process
        //n1589: Changed to be explicitly specified because it is an important flag
#if NEX_ENABLE_EXPLICIT_ARGUMENT_TRANSPORT_SEND_QUEUE
        qBool bQueue
#else
        qBool bQueue = false
#endif
    );

    virtual qBool Receive(
        qUnsignedInt16 uiPort,
        Buffer *pBuffer,
        const InetAddress *pSourceAddress
    );

#ifdef ROUTING_SUPPORT
    virtual Router* GetRouter();
#endif
    virtual Relay* GetRelay() { return &m_oRelay; }

    virtual qResult Send(StationURL *pStationURL, Buffer *pBuffer);

    void TransportJobMethod();

    QueuingSocket* GetWellKnownSocket();
    QueuingSocketTransportBuffer* GetWellKnownTransportBufferSocket();

    //n1769: Prepare to get the number of send and receive buffers for the transport buffer.
    // Return 0xffffffff when the transport buffer thread is not in a valid state.
    // We wanted to attach a const, but when we get the pointer it ended up being a non-const, so we gave up.
    qUnsignedInt32 GetTransportBufferSendBufferSize();
    qUnsignedInt32 GetTransportBufferRecvBufferSize();

#if NEX_ENABLE_SOCKET_HANDLE_EXPORT
    SocketHandle GetSocketHandle(qUnsignedInt16 ui16Port);
#endif

#if NEX_ENABLE_SOCKET_TRANSPORT_SWITCH
    qBool SetEnableTransport(qUnsignedInt16 port, qBool enable);
#endif

    virtual qUnsignedInt32 GetPacketQueueSize() const;
    virtual qUnsignedInt32 GetEstimatedPacketQueueMemoryUsage() const;

    TransportBufferThreadInterface *GetTransportBufferThread()
    {
        return m_pTransportBufferThread;
    }
    
#if defined(SOCKET_HANDLE_SUPPORT)
    //  uiFromPort == 0: use preferred port (GetPreferredPortNumber())
    //  Note (using both RendezVous and NetZ):
    //  Since both facades share the same transport layer, this is much safer to specify the port number
    //  to get the handle of the wanted socket.
    //  E.g.: using RendezVous listening port while Credentials for a given player are valid:
    //
    //  if ( logged_in ) {
    //    UDPTransport* pTheTransport = static_cast<UDPTransport*>(RootTransport::GetInstance());
    //    pTheTransport->GetSocketHandle( pRendezVous->GetStreamManager()->GetPort() );
    //  }
    //
    void* GetSocketHandle(qUnsignedInt16 uiFromPort=0);
#endif

    static const qUnsignedInt INVALID_THREAD_PRIORITY = 0xffffffff; //

    void SetTransportThreadCurrentPriority(qUnsignedInt priority) ;
    qUnsignedInt GetTransportThreadCurrentPriority(void) const ;

#if NEX_ENABLE_TRANSPORT_BUFFER_THREAD
    void SetTransportBufferThreadCurrentPriority(qUnsignedInt priority) ;
    qUnsignedInt GetTransportBufferThreadCurrentPriority(void) const ;
    void GetBufferSetEmptyDropCount(qUnsignedInt32& uiSendDropCount, qUnsignedInt32& uiRecvDropCount) const ;
#endif

    //n1589
    qUnsignedInt32 GetTransportProcessCount()
    {
        return m_uiTransportProcessCount;
    }

    qUnsignedInt32 GetServiceIOProcessCount()
    {
        return m_uiServiceIOProcessCount;
    }

    void ResetTransportProcessCount()
    {
        m_uiTransportProcessCount = 0;
        m_uiServiceIOProcessCount = 0;
    }

    //n1589
#if 1
    qUnsignedInt32 GetIncomingPacketQueueSize()
    {
        return m_oIncomingPacketQueue.GetPacketCount();
    }

    qUnsignedInt32 GetOutgoingPacketQueueSize()
    {
        if (mp_oOutgoingPacketQueue == NULL)
        {
            return 0;
        }
        return mp_oOutgoingPacketQueue->GetPacketCount();
    }
#endif

#if NEX_ENABLE_LATENCY_LIMIT_PACKET_QUEUE_SEND
    Time GetOutgoingPacketQueueBeginTime()
    {
        if (mp_oOutgoingPacketQueue == NULL)
        {
            return (Time)0;
        }

        if (mp_oOutgoingPacketQueue->GetPacketCount() == 0)
        {
            return (Time)0;
        }

        PacketQueue::iterator it = mp_oOutgoingPacketQueue->begin();
        Packet* pPacket = static_cast<Packet*>(*it);
        return pPacket->GetQueuedTime();
    }

    Time GetOutgoingPacketQueueLastTime()
    {
        if (mp_oOutgoingPacketQueue == NULL)
        {
            return (Time)0;
        }

        if (mp_oOutgoingPacketQueue->GetPacketCount() == 0)
        {
            return (Time)0;
        }

        PacketQueue::iterator it = mp_oOutgoingPacketQueue->last();
        Packet* pPacket = static_cast<Packet*>(*it);
        return pPacket->GetQueuedTime();
    }

    TimeInterval GetOutgoingPacketQueueLatency()
    {
        return (TimeInterval)(GetOutgoingPacketQueueLastTime() - GetOutgoingPacketQueueBeginTime());
    }
#endif

#if NEX_ENABLE_SOCKET_TRANSPORT_PROFILING
    void ResetAnalyzeInfo();
    void LogAnalyzeInfo(EventLog::LogLevel logLevel = EventLog::Debug);

    SocketTransportAnalyzeInfo* GetAnalyzeInfo()
    {
        return &m_AnalyzeInfo;
    }

    void SetOutgoingPacketQueueLatencyLimitDequeueCount(qUnsignedInt32 o)
    {
        m_AnalyzeInfo.uiOutgoingPacketQueueLatencyLimitDequeueCount = o;
    }

    void SetTransportBufferSendLatencyLimitDequeueCount(qUnsignedInt32 o);
#endif

#if NEX_ENABLE_BERKLEY_SOCKET_PROFILING
    qUnsignedInt32 GetBerkeleySocketSendToMultiCount();
    qUnsignedInt32 GetBerkeleySocketSendToCount();
    qUnsignedInt32 GetBerkeleySocketRecvFromCount();
    qUnsignedInt32 GetBerkeleySocketRecvFromTryCount();
    qUnsignedInt32 GetBerkeleySocketPeerAddressCount();
#endif

#if NEX_ENABLE_SOCKET_TRANSPORT_SEND_TERMINATION
    void SetSendTerminated(qBool b)
    {
        m_bSendTerminated = b;
    }
    qBool IsSendTerminated()
    {
        return m_bSendTerminated;
    }
#endif

#ifdef NEX_CLIENT_PBPOOL
    /* Please see man pages for details 
     
            
     */
    virtual void ClearPacketInPacketBufferPool();
#endif // NEX_CLIENT_PBPOOL

protected:
    //n1589:memo:RootTransport:virtual void StopTransportThreadImpl() = 0;
    virtual void    StopTransportThreadImpl()
    {
        m_bShutdown = true;
    }
    void            StartEventListener();

    qBool StartListenImpl(
        qUnsignedInt16 ui16Port,
        qUnsignedInt16 *pui16ListeningPort,
        qBool bPreferred,
        qUnsignedInt uiTrafficType,
        qBool usePacketBufferPool,
        qBool bReuse
    );
    qBool           BindSocket(
        qUnsignedInt16 ui16DesiredPort,
        qUnsignedInt16 *pui16ListeningPort,
        qUnsignedInt uiTrafficType,
        qBool usePacketBufferPool,
        qBool bReuse
    );

    void            DeliverOutgoing();
    void            DeliverOutgoingImpl();
    void            DeliverIncoming();

    //n1589
    void ProcessTransport();

    //n1589
#ifdef ROUTING_SUPPORT
#ifdef NEX_CLIENT_OPTIMIZATION
    qBool RouteRelaySendMultiMessage(Buffer *pBuffer, qUnsignedInt16 uiPort);
#endif // NEX_CLIENT_OPTIMIZATION
#endif

    QueuingSocket   *GetSocket(qUnsignedInt16 ui16Port);

    void            TransportThread(void *pDummyParam);

    void            ServiceIOCompletions();
    void            ServiceIOCompletionsImpl();
    qBool           FillPacketQueueFromBuffer(
        QueuingSocket *pSocket,
        Buffer *pBuffer,
        const InetAddress *pSourceAddress
    );

    qBool IsFinishPacketProcess();
    qBool m_IsPacketSended;

    ObjectThread<SocketTransport, void*>    *m_pTransportThread;
    TransportBufferThreadInterface *m_pTransportBufferThread;
    Job                                 *m_pTransportJob;

    //n1589
    qUnsignedInt32 m_uiTransportProcessCount;
    qUnsignedInt32 m_uiServiceIOProcessCount;

    typedef qSortedVector<qUnsignedInt16,QueuingSocket*> SocketPortMap;
    SocketPortMap               m_mapSocketPort;

    IOCompletionNotifier        m_oIOCompletionNotifier;

    volatile qBool                      m_bShutdown;
    InterfaceTable              m_oInterfaceTable;

    PacketQueue                 m_oIncomingPacketQueue;

    PacketQueue* mp_oOutgoingPacketQueue;

    PacketHistoryChain          m_oOutgoingHistoryPacketChain;

    ProfilingUnit               m_puTransportJob;
    ICContext                   m_icPseudoSingletonContext;
#ifdef ROUTING_SUPPORT
    Router                      m_oRouter;
#endif
    Relay                       m_oRelay;
    
    int                         m_TransportThreadSleepSpan;
    Time                        m_ForcedRetransmitCheckTimeout;

    TransportBuffer*            m_pTransportBuffer;

#if NEX_ENABLE_SOCKET_TRANSPORT_PROFILING
    SocketTransportAnalyzeInfo m_AnalyzeInfo;
#endif
#if NEX_ENABLE_SOCKET_TRANSPORT_SEND_TERMINATION
    qBool m_bSendTerminated;
#endif
};

END_NAMESPACE_NEX

#endif
