﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StationContactInfo_H_
#define _StationContactInfo_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Platform/Core/qList.h>

BEGIN_NAMESPACE_NEX

//group=Miscellaneous Classes

/*
Description: This class defines the contact information for a given Station.
Remarks: This class contains information on how to connect to a given Station, in particular the StationURL s via which
a connection may be made.
See Also: StationURL
*/
class StationContactInfo: public RootObject
{
    //Group=Methods
public:
    StationContactInfo();
    // Parameters:
    // lstURLs: A list of the StationURL s of the Station whose contact info will be stored in this StationContactInfo object.
    StationContactInfo(const qList<StationURL> & lstURLs);
    virtual ~StationContactInfo();

    //{secret}
    qBool           SetConnectionID(ConnectionID idConnection);
    //{secret}
    ConnectionID    GetConnectionID() const;

    /*
    Description: Adds the specified StationURL to the list of StationURL s held by this StationContactInfo object.
    Parameters:
    refURL: The StationURL that will be added to the list of StationURL s for the station to which this StationContactInfo object pertains.
    Returns: true if the new StationURL was successfully added to this StationContactInfo object, false if not or an error occurred.
    Remarks: Refer to ConnectivityManager::ProbeStations for an example of this method in use.
    */
    qBool           AddURL(const StationURL& refURL);

    //{secret}
    void            SortAndFilterTarget(StationContactInfo &refTargetStation) const;

    /*
    Description: Returns a list of StationURL s for the Station to which this StationContactInfo object pertains.
    Returns: A list of StationURL s for the Station to which this StationContactInfo object pertains.
    */
    qList<StationURL>& StationURLs()
    {
        return m_lstURLs;
    }
    const qList<StationURL>& StationURLs() const
    {
        return m_lstURLs;
    }

    /*
    Description: Traces the content of this StationContactInfo object.
    Remarks: The data traced is the list of StationURL s and the ConnectionID of the Station that pertains to this StationContactInfo object.
    */
    void            Trace(qUnsignedInt64 uiTraceFlag = TRACE_NAT);

#ifdef TEST_ENABLED
    //{secret}
    static qBool Test();
    //{secret}
    static void ForcePublicURLInSortAndFilter(qBool bForce);
    //{secret}
    static qBool PublicURLIsForced()
    {
        return s_bForcePublic;
    }
#endif

private:
    ConnectionID        m_idConnection;
    qList<StationURL>   m_lstURLs;
#ifdef TEST_ENABLED
    static qBool s_bForcePublic;
#endif
};

END_NAMESPACE_NEX

#endif
