﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StationProbeList_H_
#define _StationProbeList_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/StationProbe.h>

BEGIN_NAMESPACE_NEX

//group=Miscellaneous Classes

/*
Description: Contains a list of StationProbe s.
Remarks:
*/
class StationProbeList: public qList<StationProbe>
{
public:
    /*
    {group:Classes}
    Description: Contains the ping time statistics for the stations contained in this StationProbeList object.
    Remarks:
    */
    class PingStats
    {
        friend class StationProbeList;
    public:
        //group=Methods

        PingStats()
        {
            m_uiMin = Infinite;
            m_uiMax = Infinite;
            m_uiAverage = Infinite;
            m_uiMedian = Infinite;
        }

        /*
        Description: Returns the minimum ping time to the group of Station s in this StationProbeList.
        Returns: The minimum ping time in milliseconds to the group of Station s in this StationProbeList.
        Remarks: The default minimum ping time is infinite, which will also be returned if no connection can be made
        between the caller of this method and the listed stations.
        */
        qUnsignedInt32 GetMin() const
        {
            return m_uiMin;
        }

        /*
        Description: Returns the maximum ping time to the group of Station s in this StationProbeList.
        Returns: The maximum ping time in milliseconds to the group of Station s in this StationProbeList.
        Remarks: The default maximum ping time is infinite, which will also be returned if no connection can be made
        between the caller of this method and the listed stations.
        */
        qUnsignedInt32 GetMax() const
        {
            return m_uiMax;
        }

        /*
        Description: Returns the average ping time to the group of Station s in this StationProbeList.
        Returns: The average ping time in milliseconds to the group of Station s in this StationProbeList.
        Remarks: The default average ping time is infinite, which will also be returned if no connection can be made
        between the caller of this method and the listed stations.
        */
        qUnsignedInt32 GetAverage() const
        {
            return m_uiAverage;
        }

        /*
        Description: Returns the median ping time to the group of Station s in this StationProbeList.
        Returns: The median ping time in milliseconds to the group of Station s in this StationProbeList.
        Remarks: The default median ping time is infinite, which will also be returned if no connection can be made
        between the caller of this method and the listed stations.
        */
        qUnsignedInt32 GetMedian() const
        {
            return m_uiMedian;
        }

    private:
        qUnsignedInt32 m_uiMin;
        qUnsignedInt32 m_uiMax;
        qUnsignedInt32 m_uiAverage;
        qUnsignedInt32 m_uiMedian;
    };

    //Group=Methods
    /* Please see man pages for details 
     
     
     */
    StationProbeList();
    /*
    Parameters:
    reflstStationURL: The list of StationURL s that will be used to construct this StationProbeList.
    Note that you need to append the Rendez-Vous Connection ID to each StationURL in the StationProbeList.
    See Also: StationURL
    */
    /* Please see man pages for details 
     
     
     
     
     
     
     
     */
    StationProbeList(const qList<StationURL>& reflstStationURL);

    /* Please see man pages for details 
     
     
     */
    virtual ~StationProbeList();

    /* Please see man pages for details 
     
     

     

     
     
     */
    StationProbe* AddStationURL(const StationURL& refStationURL);


    //{secret}
    StationProbe *GetProbeByCID(ConnectionID idConnection);
    //{secret}
    void Update();

    /*
    Description: Returns the PingStats for the list of Station s in this StationProbeList.
    Returns: Returns the PingStats for the list of Station s in this StationProbeList.
    See Also: PingStats
    */
    PingStats       GetPingStats() const;

    /*
    Description: Returns whether all Station s in this StationProbeList could be reached.
    Returns: true if a connection could be made by the caller of this method to all Station s in this StationProbeList,
    false if not or an error occurred.
    */
    qBool           EveryoneReached() const;

    qBool           AnyoneFailed() const;

    /*
    Description: Traces the content of this StationProbeList object.
    Remarks: The data traced is the list of StationProbes currently used.
    */
    void Trace(qUnsignedInt64 uiTraceFlag);

    //{secret}
    void Reset();

#ifdef TEST_ENABLED
    //{secret}
    static qBool    Test();
#endif
};

END_NAMESPACE_NEX

#endif
