/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Stream_H_
#define _Stream_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/StreamSettings.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

BEGIN_NAMESPACE_NEX
class Packet;

#define MAX_STREAM_NB 16

//group=Transport Protocol

/*
    Description: This class provides access to the settings associated with the specified transport stream Type.
    See Also: StreamSettings
*/
/* Please see man pages for details 
    

           

    
 */
class Stream: public RootObject
{
public:
    /*
    n1589:memo
    DO
    RVAuthentication
    RVSecure
    SandBoxMgmt
    NAT
    SessionDiscovery
    NATEcho
    Routing
    Game
    */

    /* Please see man pages for details 
     
     
     */
    typedef enum
    {
        //   Description: do:/address=;vport=0;
        DO                  = 1,  //

        //   Description: auth:/;vport=0;
        RVAuthentication    = 2,  //

        //   Description: rv:/PID=x;CID=y;
        OldRVSecure         = 3,  //

        //   Description: rv:/PID=x;CID=y;
        SandBoxMgmt         = 4,  //

        //   Description: NAT:/
        NAT                 = 5,  //

        //   Description: LSD:/
        SessionDiscovery    = 6,  //

        //   Description: NATEcho:/
        NATEcho             = 7,  //

        //   Description: Routing:/
        Routing             = 8,  //

        //   Description: Game:/
        Game                = 9,  //

        //   Description: rv:/PID=x;CID=y; security update
        RVSecure            = 10, //

        Relay               = 11,
        
        //   Description: LSD:/
        LastStreamType      = 11 //
    } Type; // Description: The $PRODUCTNAME$ transport streams.

    // {secret}
    typedef qByte   ID;

public:
    /*
    Remarks: This object should never be directly instantiated. Instead, it is created
    by the system when $FACADENAME$ is created.
    */
    Stream(Type oStreamType);

    /*
    Remarks: This object should never be directly destroyed. It is deleted by
    $FACADENAME$.
    */
    virtual ~Stream();

    // {secret}
    virtual qBool ReceiveIncomingPacket(qUnsignedInt16 uiPort, ID idStream, Packet *pPacket) = 0;
    // {secret}
    virtual void DoWork() = 0;

    virtual qBool IsDuplicateReorderingPacket(Packet * /*pPacket*/)
    {
        return false;
    }

    // {secret}
    Type GetType() const
    {
        return m_oStreamType;
    }

    // {secret}
    const qChar* GetTypeName() const
    {
        return GetTypeName(GetType());
    }

    // {secret}
    static const qChar* GetTypeName(Type strType);

    // {secret}
    StreamSettings& GetThisSettings()
    {
        SYSTEMCHECK(m_oStreamType <= LastStreamType);
        return s_oStreamSettings[m_oStreamType].GetValue();
    }

#ifdef FOR_DOC_PURPOSE
    /* Description: Returns a data object encapsulating the transport settings associated with the specified stream Type.
        Returns: A data object of type StreamSettings.
        See Also: StreamSettings
    */
    /* Please see man pages for details 
    

           

    
    */
    static StreamSettings& GetSettings();
#else
    static StreamSettings& GetSettings(Type oStreamType = Stream::DO)
    {
        SYSTEMCHECK(oStreamType <= LastStreamType);
        return s_oStreamSettings[oStreamType].GetValue();
    }

#endif

private:
    Type m_oStreamType;
    static PseudoGlobalVariable<StreamSettings> s_oStreamSettings[MAX_STREAM_NB];
};

END_NAMESPACE_NEX

#endif
