/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _TransportBufferMultiThread_H_
#define _TransportBufferMultiThread_H_

#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/Condition.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferThreadInterface.h>

BEGIN_NAMESPACE_NEX

class QueuingSocket;


/* Please see man pages for details 
 
 
 
 
 */
class TransportBufferMultiThread: public TransportBufferThreadInterface
{
public:
    /* Please see man pages for details 
     
     */
    TransportBufferMultiThread();

    /* Please see man pages for details 
     
     */
    virtual ~TransportBufferMultiThread();

    /* Please see man pages for details 
     
     */
    void Start();

    /* Please see man pages for details 
     
     */
    void Stop();

    /* Please see man pages for details 
     
     
     */
    void TransportSendLoop( int nThreadIndex );

    /* Please see man pages for details 
     
     
     */
    void TransportRecvLoop( int nThreadIndex );

    /* Please see man pages for details 
     
     
     
     
     */
    qBool InsertSocketPort(qUnsignedInt16 port, QueuingSocket *pSocket);

    void PreparedEraseSocketPort(qUnsignedInt16 port, QueuingSocket *pSocket);

    /* Please see man pages for details 
     
     
     
     */
    void EraseSocketPort(qUnsignedInt16 port);

    void SetPriority(qUnsignedInt32 priority);
    qUnsignedInt32 GetPriority() const;

    Condition* GetSendCond()
    {
        return &m_sendCond;
    }

private:
    enum
    {
#ifndef CTR_TARGET
        //
        NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT = 3
#else
        //
        NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT = 2
#endif
    };

private:
    struct PortThreadSocket
    {
        qUnsignedInt16 first;
        int nThreadIndex;
        QueuingSocket *socket;
        PortThreadSocket(qUnsignedInt16 p, int i, QueuingSocket *s) :
            first(p), nThreadIndex(i), socket(s) {}
        PortThreadSocket() :
            first(0), nThreadIndex(0), socket(NULL) {}
    };
    typedef qVector< PortThreadSocket > PortThreadSocketMap;

    typedef std::pair<qUnsignedInt16, int> PortThreadPair;

    typedef std::pair<qUnsignedInt16, QueuingSocket *> PortSocketPair;
    typedef qVector< PortSocketPair > SocketPortMap;


    volatile qBool m_bShutdown;  //
    volatile qBool m_bNeedSleep; //

    //
    ForcedCriticalSection m_csTransportBufferSocketLock;

    //
    qBool m_bAssignedRecvSocketThread[ NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT ];

    //
    volatile qBool m_bPreparedCloseRecvSocket[ NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT ];

    //
    volatile qInt32 m_PreparedCloseSendSocketPort;

    //
    PortThreadSocketMap m_mapSocketPortThread;

    //
    ForcedCriticalSection* m_pRecvSocketLockPerThread[ NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT ];

    //
    PortSocketPair m_mapSocketPorts[ NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT ];

    SocketPortMap m_mapSendSocketPorts;

    ForcedCriticalSection m_SendSocketPortsLock;

    //
    ObjectThread<TransportBufferMultiThread, int >* m_pReceivingThreads[ NEX_TRANSPORTBUFFER_RECV_THREAD_COUNT ];

    //
    Condition m_sendCond;

    //
    ForcedCriticalSection m_sendCondLock;

    //
    ObjectThread<TransportBufferMultiThread, int >* m_pSendingThread;

};

END_NAMESPACE_NEX

#endif
