/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef _PRUDPMessageSelector_H_
#define _PRUDPMessageSelector_H_

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class RelayMessage;

/* Please see man pages for details 
 
 
 */
class PRUDPMessageSelector : public PRUDPMessageInterface, public RootObject
{
public:

    PRUDPMessageSelector();

    virtual ~PRUDPMessageSelector();

    static const qUnsignedInt MAX_VERSION = 1;


    virtual qBool CalcSignature(PacketOut *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                                TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);
    virtual void CalcSignature(Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                               TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    virtual SignatureMethod DecideSignatureMethod(const Packet *pPacket, Stream::Type type, qBool hasEndPoint);


    virtual SignatureMatchPattern CalcExpectedSignature(const Packet *pPacket, SignatureMethod method, const Key *pKey,
            TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &expectedSignature);

    virtual qBool Unpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt *pSizeUnpacked = NULL);

    virtual qBool FastUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt *pPayloadSize = NULL);

    virtual qBool Pack(PacketOut *pPacketOut, ByteStream *pByteStream);


    RelayMessage*  GetRelayMessage() { return m_pRelayMessage;}
    
protected:
    qVector<PRUDPMessageInterface *> mPRDUPMessageImps;
    
    RelayMessage* m_pRelayMessage;

};

END_NAMESPACE_NEX

#endif //_PRUDPMessageSelector_H_
