﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#ifndef _PRUDPMessageMessageV0_H_
#define _PRUDPMessageMessageV0_H_

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

BEGIN_NAMESPACE_NEX

class Packet;
class PacketIn;
class PacketOut;
class Key;
class ByteStream;
class TransportSignatureGenerator;

class PRUDPMessageV0 : public PRUDPMessageInterface, public RootObject
{
public:
    static const qUnsignedInt MAX_HEADER_SIZE =
        sizeof(qByte) + /* m_oSourceVPort */\
        sizeof(qByte) + /* m_oDestinationVPort */\
        sizeof(qUnsignedInt16) + /* m_uiPacketTypeFlags*/\
        sizeof(qByte) +  /* m_bySessionID */\
        sizeof(qUnsignedInt32) + /* m_uiSignature*/\
        sizeof(qUnsignedInt16) + /* m_lcSeqId*/\
        sizeof(qUnsignedInt32) + /* m_uiConnectionSignature or m_byPartNumber*/\
        sizeof(qUnsignedInt16) + /* m_uiSizePresent*/\
        sizeof(qByte) /* Checksum */\
        ;

    PRUDPMessageV0();

    virtual ~PRUDPMessageV0();

    virtual qBool CalcSignature(PacketOut *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                                TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);
    virtual void CalcSignature(Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                               TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature);

    virtual SignatureMethod DecideSignatureMethod(const Packet *pPacket, Stream::Type type, qBool hasEndPoint);

    virtual SignatureMatchPattern CalcExpectedSignature(const Packet *pPacket, SignatureMethod method, const Key *pKey,
            TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &expectedSignature);;

    virtual qBool Unpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt *pSizeUnpacked = NULL);

    virtual qBool FastUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, qUnsignedInt *pPayloadSize = NULL);

    virtual qBool Pack(PacketOut *pPacketOut, ByteStream *pByteStream);

    union Signature
    {
        qUnsignedInt32 signature;
        qByte signatureBytes[sizeof(qUnsignedInt32)];
    } ;

private:
    void CalcSignatureHelper(const Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                             TransportSignatureGenerator* pSignatureGenerator, SignatureBytes &calcSignature);


};

END_NAMESPACE_NEX

#endif //_PRUDPMessageMessageV0_H_
