﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _PRUDPStream_H_
#define _PRUDPStream_H_

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/ConnectionOrientedStream.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportSignatureGenerator.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPointMap.h>
#include <OnlineCore/src/Transport/PRUDP/TimeoutManager.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDP.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qSet.h>

#if defined(LIVE_VERSION) && ( (defined(WIN_FAMILY) && !defined(VS2008_COMPILER)) || defined(UNIX_FAMILY) )
#define NEX_ENABLE_SYN_CACHE
#define NEX_ENABLE_SYN_CACHE_RVONLY
#endif

BEGIN_NAMESPACE_NEX

class Buffer;
class PacketOut;
class PacketIn;
class Packet;
class PRUDPMessageSelector;

#ifdef NEX_ENABLE_SYN_CACHE
class EndPointUniqueIDSignatureHolder;
#endif

//n1589: Plan to dynamically reflect SilenceTime and also dynamically change KeepAliveTimeout
#define NEX_ENABLE_DYNAMIC_MAX_SILENCE_TIME 1

//n1589
#define NEX_ENABLE_PRUDP_STREAM_END_POINT_LOCK 1

#define NB_LISTENING_SOCKETS        2
#define PRUDP_ADDRESS_KEY           _T("address")
#define PRUDP_PORT_KEY              _T("port")

class PRUDPStream: public ConnectionOrientedStream
{
    friend class PRUDPEndPoint;
    friend class JobTerminateFacade; // Because we want to see TimeoutManager.
    friend class JobTerminateDOCore; // Because we want to see TimeoutManager.
public:
    PRUDPStream(Stream::Type oType, RootTransport *pTransport = NULL);
    virtual ~PRUDPStream();

    //{secret}
    virtual qResult         Initialize();
    //{secret}
    virtual qResult         Teardown();

    virtual qBool           ResponsibleForURL(const StationURL *pStationURL);
    virtual qChar*          GetURLType();

    virtual EndPoint        *OpenEndPoint(const StationURL *pURL);
    virtual EndPoint        *OpenEndPoint(const ConnectionID cidStation);
    virtual qBool           OpenEndPoint(EndPoint *pEndPoint);
    virtual void            CloseEndPoint(EndPoint *pEndPoint);
    virtual void            ForceCompleteIO();

    virtual qBool           IsCapable(qUnsignedInt32 uiCapability);

    //{secret}
    virtual void            AddPIDEndPointAssociation(const PrincipalID pidStation, EndPoint *pEndPoint);
    //{secret}
    virtual qBool           RemovePIDEndPointAssociation(const PrincipalID pidStation, EndPoint *pEndPoint);
    //{secret}
    virtual EndPoint        *FindEndPointByPID(const PrincipalID pidStation);
    //{secret}
    virtual qUnsignedInt32  GetEndPointNumber();
    //{secret}
    virtual void            AddCIDEndPointAssociation(const ConnectionID cidStation, EndPoint *pEndPoint);
    //{secret}
    virtual qBool           RemoveCIDEndPointAssociation(const ConnectionID cidStation, EndPoint *pEndPoint);
    //{secret}
    virtual EndPoint        *FindEndPointByCID(const ConnectionID cidStation);
    //{secret}
    virtual const qVector<ConnectionID> GetAllConnectionIDs() const;

    //{secret}
    virtual qResult         SendBroadcast(StationURL *pStationURL, Buffer *pBuffer);
    virtual qResult         Send(StationURL *pStationURL, Buffer *pBuffer);

    virtual qBool           StopListen();

    void    SetPreferredPortNumber(qUnsignedInt16 uiPreferredPortNumber)
    {
        m_uiPreferredPortNumber = uiPreferredPortNumber;
    }
    qUnsignedInt16  GetPreferredPortNumber();

#if NEX_ENABLE_DYNAMIC_MAX_SILENCE_TIME
    virtual void SetMaxSilenceTimeAll(qUnsignedInt32 uiMaxSilenceTime);
#endif

#ifdef TEST_ENABLED
    static qBool Test();
#endif

    //n1769 Whether queueing is executed for the UDPTransport::Send function
    static void  EnableQueuing   (void)
    {
        s_bQueuing = true;
    }
    static void  DisableQueuing  (void)
    {
        s_bQueuing = false;
    }
    static qBool IsEnabledQueuing(void)
    {
        return s_bQueuing;
    }

#if NEX_ENABLE_PRUDP_STREAM_END_POINT_LOCK
    virtual void LockEndPointMap();
    virtual void UnlockEndPointMap();
#endif

    void OnSetTerminateImmediately();

    static void SetInitialVersion(qByte version)
    {
        s_bInitialVersion = version;
    }
    static qByte GetInitialVersion()
    {
        return s_bInitialVersion;
    }

    static void SetInitialMinorVersion(qByte version)
    {
        s_bInitialSupportedFunctionsMinorVersion = version;
    }
    static qByte GetInitialMinorVersion()
    {
        return s_bInitialSupportedFunctionsMinorVersion;
    }

private:
    PRUDPEndPoint*  CreateEndPoint(const StationURL *pStationURL,
                                   qUnsignedInt16 uiOutgoingPort,
                                   qBool bOutgoing);
    inline void     CloseEndPointImpl(EndPoint *pEndPoint);
    void            ReleaseEndPoint(PRUDPEndPoint *pEndPoint);

    PRUDPEndPoint*  ServiceConnectionRequest(InetAddress *pInetAddress, Buffer *pBuffer,
            qUnsignedInt16 uiPort, Stream::ID idSource);
    void            ServiceUserMsg(InetAddress *pInetAddress, Buffer *pBuffer);

#if NEX_ENABLE_HMAC_USE_SECURE_KEY
    qBool           Send(qUnsignedInt16 uiPort, Stream::ID idDestination, PacketOut *pPacket, Key *pKey = 0);
#else
    qBool           Send(qUnsignedInt16 uiPort, Stream::ID idDestination, PacketOut *pPacket);
#endif
    void            SendReset(qUnsignedInt16 uiPort, InetAddress *pInetAddress);

    void            DeliverOutgoing();
    void            DeliverIncoming();
    //  void            DemuxIncomingICMP(const InetAddress *pInetAddress, VirtualPort oVPort);
    void            DemuxIncomingPacket(qUnsignedInt32 uiSocket, PacketIn *pPacket);

    //n1589:?:not used
    PRUDPEndPoint   *LocateEndPointByPID(const PrincipalID pidStation);
    PRUDPEndPoint   *LocateEndPointByCID(const ConnectionID cidStation);
#if 0
    qBool           SetInetAddressFromURL(const StationURL *pStationURL, InetAddress *pInetAddress);
    StationURL*     GetURLFromInetAddress(const InetAddress *pInetAddress);
#endif

    qUnsignedInt16  GetPortNumber(qUnsignedInt32 uiSocket);
    void            SetPortNumber(qUnsignedInt16 ui16PortNumber);
    TimeoutManager  *GetTimeoutManager()
    {
        return &m_oTimeoutManager;
    }

    void            ServiceIOCompletions();
    void            ServiceTimeouts();
    void            PurgeDeletedEndPoints(qBool force);

    virtual void            DoWork();
    virtual qBool   ReceiveIncomingPacket(qUnsignedInt16 uiPort, Stream::ID idSource, Packet *pPacket);

    //n1589:todo1:rename to m_pTransport
    RootTransport*  GetTransport()
    {
        return m_pUDPTransport;
    }

    virtual qBool IsDuplicateReorderingPacket(Packet *pPacket);

#ifdef TRACE_ENABLED    // n2370
    void            Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
#endif  // TRACE_ENABLED

    TransportSignatureGenerator*    GetSignatureGenerator()
    {
        return &m_oSignatureGenerator;
    }

    PRUDPMessageInterface::SignatureMatchPattern CheckSignature(InetAddress* pInetAddress, qUnsignedInt16 uiPort,
            Stream::ID idSource, Packet *pPacket);

    PRUDPMessageInterface::SignatureMatchPattern CheckSignature(InetAddress* pInetAddress, qUnsignedInt16 uiPort,
            Stream::ID idSource, Packet *pPacket, Key *pKey, SignatureBytes *pSB, PRUDPMessageInterface::SignatureMethod oMethod);

    inline void PurgeDeletedEndPointsImpl(qBool force);

private:
    // Illegal operations.
    PRUDPStream(const PRUDPStream&);
    PRUDPStream& operator=(const PRUDPStream&);

    void ReleaseEndPointImpl(PRUDPEndPoint *pEndPoint);

    void VersionNegotiation(Packet *pPacket, Packet *pAckPacket);

    PRUDPMessageInterface::SignatureMethod DecideSignatureMethod(const Packet *pConnectPacket, Stream::Type type, qBool hasEndPoint);

private:
    TimeoutManager              m_oTimeoutManager;
    PRUDPEndPointMap            m_oEndPointMap;
    qList<PRUDPEndPoint*>       m_oEndPointsToDelete;

    Time                        m_tiLastTimeoutServicingTime;
    qBool                       m_bShutdown;
    ICContext                   m_icPseudoSingletonContext;
#ifdef WATERMARK_ENABLED
    WaterMark                   m_wmConnections;
#endif

    qUnsignedInt16                      m_uiPreferredPortNumber;
    TransportSignatureGenerator         m_oSignatureGenerator;

    LogicalClock16              m_lcNextUnreliableBroadcastSeqIdToSend; //n2967: The sequence number for shared Broadcast between PRUDPEndPoints

    typedef qMultiMap<PrincipalID,EndPoint*>    MMapPID2EndPoint;
    MMapPID2EndPoint                    m_mmapPIDEndPoint;

    // TODO: This should be a map, bug check out bug 2463
    typedef qMultiMap<ConnectionID,EndPoint*>    MMapCID2EndPoint;
    MMapCID2EndPoint                    m_mmapCIDEndPoint;
    qVector<ConnectionID>               m_vecCachedCID;

    //n1769
    static qBool                        s_bQueuing;
    static qByte s_bInitialVersion;
    static PseudoGlobalVariable<qByte> s_bInitialSupportedFunctionsMinorVersion;
    static qUnsignedInt32 s_uiInitialSupportedFunctionsOptionMask;


    PRUDPMessageSelector* m_pPRUDPMessageSelector;
    qByte m_bInitialVersion;

#if NEX_ENABLE_PRUDP_STREAM_END_POINT_LOCK
    mutable CriticalSection m_csEndPointLock;
#endif

#ifdef NEX_ENABLE_SYN_CACHE
    EndPointUniqueIDSignatureHolder* m_pEpuisHolder;
    Time m_tFloodSynStart;
    Time m_tEpuisHolderTimeOutCheckTime;
    static const TimeInterval sCancelSynFloodWarningTimeInterval = 5000;
    static const TimeInterval sEpusiHolderTimeOutCheckTimeInterval = 2000;
#endif

};

END_NAMESPACE_NEX

#endif
