/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
//n1589

#ifndef _PRUDPUserProtocol_H_
#define _PRUDPUserProtocol_H_

#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>

BEGIN_NAMESPACE_NEX

// Type used in library
typedef void (*PRUDPUserReceiveCallback)(ConnectionID sourceId, qByte* buffer, qUnsignedInt32 size);//For DirectTransport compatibility.
typedef void (*PRUDPUserPrivateReceiveCallback)(ConnectionID sourceId, Buffer* pBuffer, qUnsignedInt32 uiFlags, qByte subStreamID);

class PRUDPUserProtocol: public EndPointEventHandler
{
public:
    static void AcquireInstance();
    static void ReleaseInstance();

    static PRUDPUserProtocol* GetInstance()
    {
        InstanceControl* pInstance = InstanceControl::FindInstance(IC_TYPE_PRUDPUSERPROTOCOL, PseudoSingleton::GetCurrentContext());
        if (pInstance != NULL)
        {
            return static_cast<PRUDPUserProtocol*>(pInstance->GetDelegatorInstance());
        }
        else
        {
            return NULL;
        }
    }

    PRUDPUserProtocol();
    virtual ~PRUDPUserProtocol();

    void RegisterUserReceiveCallback( PRUDPUserReceiveCallback callback );
    void UnregisterUserReceiveCallback();
    // Register/cancel internal callback
    /*
    
    
    */
    void RegisterUserPrivateReceiveCallback( PRUDPUserPrivateReceiveCallback callback );
    /*
    
    */
    void UnregisterUserPrivateReceiveCallback();

    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault);
    virtual void PeerDisconnected(EndPoint *pEndPoint);
    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qByte subStreamID);
    // Receive method by adding packet TypeFlags
    /*
    
    */
    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qUnsignedInt32 uiPacketTypeFlags, qByte subStreamID);
    qResult Send(EndPoint* pEndPoint, const qByte* buffer, qUnsignedInt32 size, qUnsignedInt32 uiFlags, qByte subStreamID, qUnsignedInt32 uiCallID = 0);

#ifdef TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif

protected:
    PseudoSingleton m_psInstance;
    //      static PRUDPUserProtocol* s_pUserProtocolInstance;
    PRUDPUserReceiveCallback m_pfReceiveCallback;
    // PrivateReceiveCallback storage location is handled with the Register/UnregisterPrivateReceiveCallback functions.
    PRUDPUserPrivateReceiveCallback m_pfPrivateReceiveCallback;
};

END_NAMESPACE_NEX

#endif
