﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _SlidingWindow_H_
#define _SlidingWindow_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Core/LogicalClock.h>

BEGIN_NAMESPACE_NEX

//n1589:test:off
#define NEX_EXPERIMENT_SLIDING_WINDOW_ALWAYS_PUSH_FAIL 0

class PacketOut;
class PacketEncDec;
class Stream;

class SlidingWindow: public RootObject
{

public:
    typedef qMap<LogicalClock16,PacketOut*>::iterator Iterator;

#if NEX_EXPERIMENT_SLIDING_WINDOW_ALWAYS_PUSH_FAIL
    static qBool s_bEnableAlwaysPushFail;
#endif

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    SlidingWindow(qUnsignedInt16 ui16WindowSize, qUnsignedInt32 ui32MaxMapSize);
#else
    SlidingWindow(qUnsignedInt16 ui16WindowSize);
#endif
    ~SlidingWindow();
    void                Purge();

    //n2690
#if NEX_ENABLE_ENCDEC_AT_ENDPOINT
    qBool               Push(PacketOut *pPacket, PacketEncDec* pPacketEncDec, Stream* pStream);
#else
    qBool               Push(PacketOut *pPacket);
#endif
    void                Acknowledged(LogicalClock16 lcSeqId, qMap<qUnsignedInt32, qUnsignedInt32>& unackedCount);
    PacketOut           *GetNextToSend();
    PacketOut           *GetPacket(LogicalClock16 lcSeqId);
    qBool               ReadyToSend();
    qBool               Empty();

    //n1589:memo:DataPending
    //  The purpose of keeping track of number of DATA packets
    //  is to make sure that we send all reliable data prior
    //  to send a DISCONNECT packet.
    qBool               DataPending();
    qUnsignedInt32      NbDataPending();
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    /* Please see man pages for details 
     
            
     */
    qInt32              GetFreeBufferNum() const; // How many remaining Reliable packets can be added?
#endif // NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION

    Iterator            AcquireIterator();
    PacketOut           *GetPacket(Iterator it);
    void                ReleaseIterator();
    LogicalClock16      LowestSeq() const
    {
        return m_lcLeftEdge;
    };

    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);

#ifdef TEST_ENABLED
    static qBool        Test();

    // Temp: to investigate a test failure in src/Services/Foundation/Service test 5 (r16sp)
#ifdef WATERMARK_ENABLED
    static qUnsignedInt32   TestGetSlidingWindowMapSize();
    static qUnsignedInt32   TestGetNextSlidingWindowID();
#endif

#endif

    //n1589
    qUnsignedInt32 GetCurrentPacketCount() const
    {
        return m_oPacketMap.size();
    }

    //n1589
protected:
    qMap<LogicalClock16,PacketOut*> m_oPacketMap;
    qUnsignedInt16                  m_ui16WindowSize;
    //n1589:memo:The next sequence ID to push is the Map position.
    LogicalClock16                  m_lcNextSeqId;
    LogicalClock16                  m_lcRightEdge;
    LogicalClock16                  m_lcLeftEdge;
    qUnsignedInt32                  m_uiNbDataPacketsPending;

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    qUnsignedInt32                  m_uiMaxMapSize;
#endif

#ifdef WATERMARK_ENABLED
    static WaterMark                            s_wmSlidingWindow;

private:

    qUnsignedInt32                              m_uiSlidingWindowID;

    static void CheckThreshold(qChar* szCallerName);
#endif

private:
    // ?? for some reason the compiler is trying to generate this method and
    //fails. This prevents it from generating it in the first place.
    SlidingWindow& operator= (SlidingWindow& oOther);

    //n1589
    mutable CriticalSection m_csPacketMap;
};

END_NAMESPACE_NEX

#endif
