﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.
  Copyright (C)2011-2014 Nintendo  All rights reserved.
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------

*/

#ifndef _PacketDefines_H_
#define _PacketDefines_H_

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

BEGIN_NAMESPACE_NEX



//n1589: Use enum instead of static const to put this in a namespace.
enum
{
    PACKET_TYPE_MASK_BIT = 4,
    PACKET_TYPE_MASK = ((qByte)0x0F),
    PACKET_FLAG_MASK = ((qUnsignedInt16)0xFFF0)
};

enum PacketType
{
    TYPE_SYN        = (qByte)0x00,
    TYPE_CONNECT    = (qByte)0x01,
    TYPE_DATA       = (qByte)0x02,
    TYPE_DISCONNECT = (qByte)0x03,
    TYPE_PING       = (qByte)0x04,
    TYPE_USER       = (qByte)0x05,
    //n1589:memo:no effect to behavior
    TYPE_ROUTE      = (qByte)0x06,
    TYPE_RAW        = (qByte)0x07,
    TYPE_RESERVED_MAX = (qByte)0x0F
};

enum PacketFlags
{
    //n1598
    FLAG_NONE           = (qUnsignedInt16)0x0000,
    FLAG_ACK            = (qUnsignedInt16)0x0010,
    FLAG_RELIABLE       = (qUnsignedInt16)0x0020,
    FLAG_ACK_WANTED     = (qUnsignedInt16)0x0040,
    FLAG_SIZE_PRESENT   = (qUnsignedInt16)0x0080,
    FLAG_VOICE          = (qUnsignedInt16)0x0100,
    FLAG_BROADCAST      = (qUnsignedInt16)0x0200,
    FLAG_SEND_MULTI     = (qUnsignedInt16)0x0400,
    //n1589
    FLAG_PRUDP_USER     = (qUnsignedInt16)0x0800,
    //n1589
    FLAG_EXTERNAL         = (qUnsignedInt16)0x1000,
    FLAG_AGGREGATE_ACK  = (qUnsignedInt16)0x2000,
    FLAG_RESERVED_MAX   = (qUnsignedInt16)0x8000
};

enum RelayType
{
    RELAYTYPE_RELAYDATA = (qByte)0,
    RELAYTYPE_PING=(qByte)1
};

enum RelayFlags
{
    RELAYFLAG_NONE = (qUnsignedInt16)0
};

//Store data for the relay protocol using part of the m_optionForSignatureCalc region.
struct RelayProtocol
{
    qByte pad[2];
    qUnsignedInt16 relayTypeFlags;
    qUnsignedInt32 srcPublicIPAddress;
    qUnsignedInt32 dstPublicIPAddress;
    qUnsignedInt16 srcPublicPort;
    qUnsignedInt16 dstPublicPort;
    qUnsignedInt32 gameServerID;
    qUnsignedInt32 relayGatheringID;
    qUnsignedInt32 timeStamp;

    static RelayType      GetType(const RelayProtocol &rp)
    {
        return static_cast<RelayType>(rp.relayTypeFlags & PACKET_TYPE_MASK);
    }

    static void SetType(RelayProtocol &rp, RelayType byPacketType)
    {
        rp.relayTypeFlags |= (byPacketType & PACKET_TYPE_MASK);
    }


    static void        SetFlag(RelayProtocol &rp, qUnsignedInt16 uiFlag)
    {
        rp.relayTypeFlags |= (uiFlag & PACKET_FLAG_MASK);
    }

    static void        ClearFlag(RelayProtocol &rp, qUnsignedInt16 uiFlag)
    {
        rp.relayTypeFlags &= ~(uiFlag & PACKET_FLAG_MASK);
    }
    static qBool       FlagSet(const RelayProtocol &rp, qUnsignedInt16 uiFlag)
    {
        return (rp.relayTypeFlags & uiFlag)!=0;
    }

    static qUnsignedInt16 GetFlags(RelayProtocol &rp)
    {
        return (rp.relayTypeFlags & PACKET_FLAG_MASK) >> PACKET_TYPE_MASK_BIT;
    }
};

enum RelayPingType
{
    RELAYPING_REPLY = 0,
    RELAYPING_REQUEST = 1,
    RELAYPING_CONNECTION_LEARNING = 2
};

struct RelayPingProtocolHeader
{
    qByte type;
    qByte replyType;
    qByte pad[2];
    qUnsignedInt32 seq;
    qUnsignedInt32 requestTimestamp;
};

struct RelayPingProtocol
{
    RelayProtocol relay;
    RelayPingProtocolHeader ping;
};

END_NAMESPACE_NEX


#endif /* _PacketDefines_H_ */
