/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _QueuingSocketTransportBuffer_H_
#define _QueuingSocketTransportBuffer_H_

#include <OnlineCore/src/Transport/UDP/QueuingSocket.h>

BEGIN_NAMESPACE_NEX

class Condition;
class TransportBuffer;

class QueuingSocketTransportBuffer: public QueuingSocket
{
public:
    QueuingSocketTransportBuffer(IOCompletionNotifier *pNotifier, qUnsignedInt uiTrafficType, RootTransport *pTransport,
                                 qUnsignedInt uiReceiveCount, qUnsignedInt uiSendCount, qUnsignedInt32 uiMtu, Condition *pCond,
                                 TransportBuffer *pTransportBuffer);
    virtual ~QueuingSocketTransportBuffer();

    virtual void Recv(qUnsignedInt32 uiSize);
    virtual qBool IsRecvDataExists() const;
    qBool SendBufferCore(); //Really send.
    qBool SendBufferCoreSingle(); //Really send 1 packet.
#ifdef WINSOCK2_STACK_SUPPORT
    qBool CheckRecvEvent(qUnsignedInt32 timeOut); // Receive event check.
#endif
    qBool RecvCore();       //Really receive.
    qBool RecvCoreSingle(qBool isNonBlock = true);       //Really receive 1 packet.

    qBool CanSend()
    {
        return m_TransportSend.IsBufferListExists() == true;
    }
    qBool CanRecv()
    {
        return m_TransportRecv.QueueIsFull() == false;
    }

    //n1589
    qUnsignedInt GetTranasportBufferReceiveCount()
    {
        return m_iTranasportBufferReceiveCount;
    }

    //n2850 Number of drops when Send/Recv TransportBufferSet was exhausted.
    qUnsignedInt GetSendBufferSetEmptyDropCount() const
    {
        return m_TransportSend.GetInfoPoolEmptyDropCount();
    }
    qUnsignedInt GetRecvBufferSetEmptyDropCount() const
    {
        return m_TransportRecv.GetInfoPoolEmptyDropCount();
    }

    //n1769: Prepare to get the number of send and receive buffers for the transport buffer.
    qUnsignedInt32 GetRecvBufferSize() const
    {
        return m_TransportRecv.GetInfoListSize();
    }
    qUnsignedInt32 GetSendBufferSize() const
    {
        return m_TransportSend.GetInfoListSize();
    }

#ifdef TEST_ENABLED
    static qBool Test();
#endif

protected:
    //Actual communication data / for receiving
    TransportBufferSet m_TransportRecv;
    Buffer* m_pRecvTmpBuffer;
    //Actual communication data / for sending
    TransportBufferSet m_TransportSend;
    //n1589
    qUnsignedInt m_iTranasportBufferReceiveCount;
    //n2370
    qUnsignedInt m_iTransportBufferSendCount;

    //n2937 For waking up sending thread
    NAMESPACE_NEX Condition* m_pCond;

    virtual qBool SendBuffer(Buffer *pBuffer, InetAddress *pAddress);

private:
    void RecvNewData(IOCompletionContext* pContext);
    void PushRecvBuffer(Buffer* pBuffer, const InetAddress& address);
    void SetReceived(bool isReceived, IOCompletionContext* pContext);
};

END_NAMESPACE_NEX

#endif
