/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _Credentials_H
#define _Credentials_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/Protocol.h>

BEGIN_NAMESPACE_NEX

class StreamManager;
class EndPoint;
template class OSDK_API MemAllocator<ProtocolID >;
template class OSDK_API qList<ProtocolID>;

/*
\ingroup group_rdv_core_client
\brief This class assigns a unique identifier to each Principal.
\remarks The Credentials of a given Principal are unique to that Principal and encompass a
variety of information used internally for verification and security purposes.
*/

/* Please see man pages for details 
 

 
 
 */
class OSDK_API Credentials: public RefCountedObject
{
public:
    /*
        \brief Constructor
        \param[in] id The PrincipalID of the Principal to which these Credentials belong.
        \param[in] pTransport A pointer to the service transport that the specified Principal uses.
        \note
        A Credentials object should never be directly instantiated. Instead, it is created
        by the lobby when a Principal logs in to the system.
    */
    Credentials(PrincipalID id, StreamManager *pTransport);

    Credentials();

    /*
        \note
        A Credentials object should never be directly destroyed. It is destroyed by the lobby
        when its associated Principal logs out of the system. */
    virtual ~Credentials();

    /*
    \brief Returns the PrincipalID of the Principal associated with this Credentials object.

    A PrincipalID is a unique positive integer used to identify each Principal in the lobby.
    PrincipalID s ranging from 1 to 99 are reserved for Quazal processes and cannot be
    assigned by the developer. IDs above and including 100 can be assigned by the developer however,
    IDs from 100 to 999 are reserved for server processes.

    \returns The ID of the Principal associated with this Credentials object.
    */
    /* Please see man pages for details 
     

     
     */
    PrincipalID GetPrincipalID() const
    {
        return m_idPrincipal;
    }

    /* Please see man pages for details */
    void SetPrincipalID(PrincipalID pid)
    {
        m_idPrincipal = pid;
    }

    /* Please see man pages for details */
    String GetPassword() const
    {
        return m_strPassword;
    }

    /* Please see man pages for details */
    void SetPassword(String password)
    {
        m_strPassword = password;
    }

    /*
        \brief Returns the ConnectionID associated with this Credentials object.

        A ConnectionID is a unique positive integer used to identify each secure connection.

        \returns The ID of the connection associated with this Credentials object.
    */
    ConnectionID GetConnectionID() const
    {
        return m_idConnection;
    }

    /* Please see man pages for details */
    void SetConnectionID(ConnectionID id)
    {
        m_idConnection = id;
    }

    /* Please see man pages for details */
    ConnectionID* GetPointerToConnectionID()
    {
        return &m_idConnection;
    }

    /* Please see man pages for details */
    StreamManager* MarkInvalid();

    /* Please see man pages for details */
    StreamManager* GetStreamManager() const
    {
        return m_pStreamManager;
    }

    /* Please see man pages for details */
    void SetStreamManager(StreamManager* pStreamManager);

    /* Please see man pages for details */
    EndPoint* GetConnection(ProtocolID uiProtocolID) const;

    /* Please see man pages for details */
    EndPoint* GetAuthenticationConnection() const
    {
        return m_pAuthenticationConnection;
    }

    /* Please see man pages for details */
    EndPoint* GetSecureConnection() const
    {
        return m_pRegularSecureConnection;
    }

    /* Please see man pages for details */
    void SetAuthenticationConnection(EndPoint* pConnection);

    /* Please see man pages for details */
    void SetSecureConnection(EndPoint* pConnection);

    /* Please see man pages for details */
    void Logout();

    /*
        \brief Gets the locally logged-on player's name.

        This function sets the locally logged on player's name. Of course, you can only call this
        function after a successful login (while the Credentials object is valid).
    */
    const qChar* GetUserName() const
    {
        return m_strUserName.CStr();
    }

    /* Please see man pages for details */
    void SetUserName(const String &sUserName)
    {
        m_strUserName = sUserName;
    }

private:
    Credentials(const Credentials&);
    Credentials& operator=(const Credentials&);

private:
    PrincipalID m_idPrincipal;
    ConnectionID m_idConnection;
    String m_strUserName;
    String m_strPassword;

    // TODO - to clean all this up, we could probably simply keep
    // both EndPoints in the credentials, rather than within the
    // transport, that has to be accessed via a function callback.

    StreamManager* m_pStreamManager;

    EndPoint* m_pAuthenticationConnection;
    EndPoint* m_pRegularSecureConnection;
};

END_NAMESPACE_NEX

#endif


