﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ServiceClient_H_
#define _ServiceClient_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Core/src/Common/Protocol.h>
#include <OnlineCore/src/Platform/Core/CoreConstants.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/Credentials.h>


BEGIN_NAMESPACE_NEX

class EndPoint;
class StationURL;
class Credentials;
class CallContext;
class UserContext;
class StreamManager;
class ProtocolRegistry;

/*
\ingroup group_rdv_core_client
\brief This is the root class of all lobby services.
\remarks This is the root class of all lobby services and the class via which a client links to
a given back-end or peer service. Before being able to connect to a given service, a client
must first log in to the lobby via the RendezVous class.

Because this is the root class of all lobby services, if you create your own lobby service it must
inherit from this class.
*/
/* Please see man pages for details 
 
 */
class OSDK_API ServiceClient: public RootObject
{
public:

    /* Please see man pages for details 
    
    
    
    
    
    */
    ServiceClient(qUnsignedInt32 uiServiceID);

    /* Please see man pages for details 
    
    
    
    
    */
    virtual ~ServiceClient();

    /* Please see man pages for details 
    
    
    
    */
    void SetDefaultCredentials(Credentials* pCredentials);
    Credentials* GetDefaultCredentials() const
    {
        return m_pCredentials;
    }

    ProtocolRegistry* GetProtocolRegistry() const;

    /* Please see man pages for details */
    virtual void UseLocalLoopback(PrincipalID pidLocal, ConnectionID cidLocal);
    /* Please see man pages for details */
    qBool IsUsingLocalLoopback() const
    {
        return m_bUseLocalLoopback;
    }

#ifdef OBSOLETE_API_SUPPORT
    /*
    \remarks As part of client side protocols refactoring, we removed a lot of uneeded functionality in ServiceClient
       for backward compatability, the methods are still implemented, but will eventually be made obsolete.
       We recommend that you keep your ServiceClient connection unaware, since ServiceClient are not required to
       bind anymore.
    */
    /*
    \brief Registers an additional protocol that the lobby services may use.

    \param pProtocol A pointer to the protocol to be registered.
    \param uiProtocolID The ID of the protocol to be registered, which must be greater than 100.
    This value is set by the user when the protocol is installed on a service, either via the
    Management Consoles or a configuration script.

    \returns true if the given protocol was successfully registered, false if not or an
    error occurred.
    \remarks Once you have declared and compiled your custom protocol, before it can be used by a given
    service you need to register the protocol to the service that will use it. For example, assuming the
    following is declared in the DDL:
    \code
        protocol MyMatchMakingProtocol {
            void FindByLevel(uint32 uiLevel, ResultRange resultRange,
            out std_list<GatheringHolder> lstGatherings);
        };
    \endcode

    then this custom protocol would be registered to the Matchmaking Service as follows:

    \code
        MyMatchMakingProtocolClient* MyMatchMakingClient() {
            if (m_pMyMatchMakingClient == NULL) {
                // Create a new ServiceClient, the name of which must
                // match the DDL protocol declaration
                m_pMyMatchMakingClient  = new MyMatchMakingProtocolClient();
                // Register the custom protocol
                GetMatchMakingClient()->RegisterExtraProtocol(m_pMyMatchMakingClient, 102);
            } else {
                return m_pMyMatchMakingClient;
            }
        }
    \endcode

    Refer to the Rendez-Vous User's Guide for further details of how to implement custom protocols.
    \sa UnregisterExtraProtocol
    */
    qBool RegisterExtraProtocol(Protocol* pProtocol, ProtocolID uiProtocolID);
    /* \internal */
    qBool RegisterExtraProtocol(Protocol* pProtocol);

    /*
    \brief Unregisters an additional protocol that the lobby services may have used.

    \param pProtocol A pointer to the protocol to be unregistered.
    \returns true if the given protocol was successfully unregistered, false if not or an
    error occurred.
    \remarks Make sure you call UnregisterExtraProtocol() before the Protocol object is
    deleted or goes out of scope. The ServiceClient destructor unregisters any protocols
    remaining in its list and thus may crash if the Protocol pointer has become invalid.
    \sa RegisterExtraProtocol
    */
    qBool UnregisterExtraProtocol(Protocol* pProtocol);

    /*
    \brief Links a specific Principal with this ServiceClient.

    \param pCredentials A pointer to the Credentials of the Principal who will be bound to this ServiceClient.
    \returns true if the specified Principal was successfully bound to this ServiceClient, false if not
    or an error occurred.
    \remarks Once a given Principal has logged in to the lobby and the ServiceClient objects have been created,
    the Principal must then be linked to each ServiceClient that it wishes to use before that Principal may actually
    use a given ServiceClient. For example, assuming the player has successfully logged in to the lobby he then needs
    to bind to the services as follows:
    \include RendezVous\Samples\SimpleMatch\SimpleMatch.cpp
    \sa Unbind
    */
    /* Please see man pages for details 
     

     
     
     

     

     
     
     */
    virtual qBool Bind(Credentials* pCredentials);

    /*
    \brief Unlinks a specific Principal from this ServiceClient.
    \returns true if the specified Principal was successfully unbound from this ServiceClient, false if not
    or an error occurred.
    \remarks If this method is not called before the ServiceClient is deleted, then it will automatically be called
    in the ServiceClient destructor. If there are any CallContexts in progress when a service is unbound, then
    those CallContexts will be automatically ended; that is, the CallContext::State will be set to CallFailure.
    \sa Bind
    */
    /* Please see man pages for details 
    

       
       

    
    
    */
    virtual qBool Unbind();

    /* Please see man pages for details */
    qBool BindLocal(Credentials* pCredentials);

    /*
    \brief Returns the Credentials of the client who is bound to this ServiceClient.
    \returns A pointer to the Credentials of the client who is bound to this ServiceClient.
    \sa Bind
    */
    /* Please see man pages for details 
     

     
     */
    Credentials* GetCredentials() const
    {
        return GetDefaultCredentials();
    }

    /*
    \brief Returns the ServiceID of a client associated service for a specified service.
    \returns The ServiceID of the client associated service.
    */
    qUnsignedInt32 GetServiceID() const
    {
        return m_uiServiceID;
    }

    /*
    \brief Returns whether or not a given client is connected to a given service.
    \returns true if the given client is currently connected to the given service, false if not
    or an error occurred.
    */
    virtual qBool IsConnected() const;

    virtual qBool IsFaulty() const;

    /* \internal */
    StreamManager *GetStreamManager() const;

    /* \internal */
    qBool PrepareForConnection(Credentials* pCredentials);

    /* \internal */
    qBool RegisterProtocol(Protocol *pProtocol);
    /* \internal */
    qBool UnregisterProtocol(Protocol *pProtocol);

protected:
    /* Please see man pages for details */
    EndPoint *GetConnection(ProtocolID uiProtocolID) const;

    /* Please see man pages for details */
    virtual void ConnectionStateHasChanged();

#endif // OBSOLETE_API_SUPPORT
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials) = 0;

    void SetCredentials(Credentials* pCredentials);

private:
    qUnsignedInt32 m_uiServiceID;

    Credentials* m_pCredentials;

    qBool m_bUseLocalLoopback;
};

END_NAMESPACE_NEX

#endif

