﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// We do not use Capabilities, as it clashes with the platform capabilities
#ifndef _AccountCapabilities_H_
#define _AccountCapabilities_H_

BEGIN_NAMESPACE_NEX

/**
\ingroup group_rdv_core_common
\brief Namespace for enum Capabilities.
*/
#ifdef SWIG
class Capabilities
{
public:
#else
namespace Capabilities
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Enumerates the capabilities that a given Principal may have.
    \remarks A Capability defines a power or an ability that a given Principal may or may not have.
    Capabilities are defined in order to restrict what a given Principal can do in the lobby and are
    assigned on a per Group basis to the groups defined in BuiltInGroups. You cannot modify the built-in
    Capabilities however, you may add an almost unlimited number of user-defined Capabilities.

    Configuration of access rights via Capabilities and Group s is performed via the
    Back-end Management Service. Refer to this service for details as to how to implement
    custom access-rights.

    \note In order to access these types you must include the RendezVous.h file.
    \sa Principal, PrincipalID, BuiltInGroups
    */
    enum Capabilities
    {

        /**
        The capability of a Principal to create player accounts. Accounts are created via AccountManagementClient::CreateAccount.
        */
        CreatePlayer        = 1,
        /**
        The capability of a Principal to change its own password. Account passwords are
        updated via AccountManagementClient::ChangePassword.
        */
        ChangePassword      = 2,
        /**
        The capability of a Principal to temporarily disable any player account. Player accounts are
        disabled via AccountManagementClient::DisableAccount, AccountManagementClient::UpdateAccountExpiryDate,
        or AccountManagementClient::UpdateAccountEffectiveDate.
        */
        DisableAccount      = 3,
        /**
        The capability of a Principal to update the user name, e-mail address, and custom data fields of an
        account. Account data is updated via the appropriate Update methods of the AccountManagementClient class.
        */
        UpdateAccount       = 4,
        /**
        The capability of a Principal to configure sandboxes.
        */
        ConfigureSandbox    = 10,
        /**
        The capability of a Principal to maintain more than one log-in at any given time. If a player has multiple login sessions,
        their status will be Online until the last logout - only after which will it change to Offline.
        */
        ConcurrentLogin     = 11,
        /**
        The capability of a Principal to overcome max connection
        restriction.
        */
        IgnoreMaxConnections = 12,

        /**
        The capability of a Principal to bypass authentication through Nintendo servers (Wii clients).
        */
        BypassNintendoValidation = 21,

        /**
        The capability of a Principal to bypass NEX authentication.
        */
        BypassNEXValidation = 22,

        BypassZZZAccountValidation = 23,
        /**
        The capability of a Principal to bypass the Unique Key validation.
        */
        BypassUniqueKeyValidation = 24,
        /**
        The capability of a Principal to do some match making
        */
        MatchMaking = 25,
        /**
        Allow to manage its own subaccounts
        */
        ManageSubAccountsForSelf = 26,

        /**
        Allow to manage subaccounts for any users
        */
        ManageSubAccountsForAllUsers = 27,

        /**
        The capability of a Principal to disconnect another Principal
        */
        DisconnectPrincipals = 28,

        /**
        The ID of the first user-defined Capability, which by default is equal to 1000.
        */
        FirstUserCapability = 1000

    };

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif
