﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _NotificationEvents_H_
#define _NotificationEvents_H_

BEGIN_NAMESPACE_NEX

/*
\brief Namespace for enum NotificationEvents.
*/
/* Please see man pages for details 
 
*/
#ifdef SWIG
class NotificationEvents
{
public:
#else
namespace NotificationEvents
{
#endif

    /*
    \ingroup group_rdv_core_common
    \brief Enumerates the types of Notification event s that may be sent to a given Principal.
    \remarks A NotificationEvent is a message sent from a back-end service to another process, which is
    typically a client process, that is triggered when a given event occurs on the back-end service.
    The purpose of these messages is to notify the remote process that a specific event has occurred
    on a given back-end service.
    You cannot modify the built-in Notification events however, you may add an almost unlimited
    number of user-defined Notification events via the NotificationEvent class.

    \note In order to access these types you must include the RendezVous.h file.
    \sa Principal, PrincipalID
    */
    /* Please see man pages for details 
     

     
     
     */
    enum NotificationEvents
    {

        /*
        A given player is in the process of adding another player to one of his/her Friends lists.
        This Notification event will be triggered when FriendsClient::AddFriend,
        FriendsClient::AcceptFriendship, or FriendsClient::DeclineFriendship is called and will have
        a different meaning according to which method triggers the event.

        AddFriend:
        This event is sent to the player with whom a given player wishes to be friends.
        The first parameter of the NotificationEvent indicates the ID of the player (PrincipalID) who
        wishes to be friends with another player (i.e., the ID of the player who called FriendsClient::AddFriend),
        the second parameter indicates the new PlayerRelationships, which is Pending, and the string parameter
        indicates the associated message, if any.

        AcceptFriendship:
        This event is sent to the player who wants to be friends with another player (i.e., the player who called
        FriendsClient::AddFriend).
        The first parameter of the NotificationEvent indicates the ID of the player (PrincipalID) who
        accepts to be friends with the other player (i.e., the ID of the player who called FriendsClient::AcceptFriendship),
        and the second parameter indicates the new PlayerRelationships, which is Friend.

        DeclineFriendship:
        This event is sent to the player who wants to be friends with another player (i.e., the player who called
        FriendsClient::AddFriend).
        The first parameter of the NotificationEvent indicates the ID of the player (PrincipalID) who
        declines to be friends with the other player (i.e., the ID of the player who called FriendsClient::DeclineFriendship),
        and the second parameter indicates the new PlayerRelationships, which is None.
        */
        /* Please see man pages for details 
         
        */
        FriendEvent = 1,
        /* Please see man pages for details 
        
        
        
        
        
        */
        /* Please see man pages for details 
         
        */
        SessionLaunched = 2,
        /* Please see man pages for details 
        
        
        
        
        
        
        
        */

        // The number of participants taken from the client that just joined is set for the third parameter.
        /* Please see man pages for details 
         
         
         
         
         */
        ParticipationEvent = 3,
        /* Please see man pages for details 
        
        
        
        
        
        */
        /* Please see man pages for details 
         
         
        */
        OwnershipChangeEvent = 4,
        /* Please see man pages for details 
         
         
         */
        /* Please see man pages for details 
         
        */
        FriendStatusChangeEvent = 5,
        /*
        rdv node forces user disconnection.
        This is used to implement preemptive login.
        The first parameter of the NotificationEvent is the PID of the principal to disconnect.
        The second parameter is currently set to 0
        */
        /* Please see man pages for details 
         
        */
        ForceDisconnectEvent = 6,
        /* Please see man pages for details 
        
        
        */
        /* Please see man pages for details 
         
        */
        GameSessionEvent = 7,
        /*
        RDV node forces user disconnection.
        This is used to implement kick out principal.
        The first parameter of the NotificationEvent is the DID of the deamon owner of the principal to disconnect.
        The second parameter of the NotificationEvent is the PID of the principal to disconnect.
        */
        /* Please see man pages for details 
         
        */
        ForceDisconnectPrincipalEvent = 8,
        /* Please see man pages for details 
        
        
        
        
        */
        /* Please see man pages for details 
         
        */
        ForceDisconnectAllPrincipalsEvent = 9,
        /*
          for UpdateNotificationData, GetFriendNotificationData
        */
        /* Please see man pages for details 
         
         
        */
        GameNotificationType1 = 101,
        /* Please see man pages for details 
         
        */
        GameNotificationType2 = 102,
        /* Please see man pages for details 
         
        */
        GameNotificationType3 = 103,
        /* Please see man pages for details 
         
        */
        GameNotificationType4 = 104,
        /* Please see man pages for details 
         
        */
        GameNotificationType5 = 105,
        /* Please see man pages for details 
         
        */
        GameNotificationType6 = 106,
        /* Please see man pages for details 
         
        */
        GameNotificationType7 = 107,
        /* Please see man pages for details 
         
        */
        GameNotificationType8 = 108,
        /* Please see man pages for details 
         
         
        */
        GatheringUnregistered = 109,
        /* Please see man pages for details 
         
         
        */
        HostChangeEvent = 110,
        /* Please see man pages for details 
         
         
         
        */
        GameNotificationLogout = 111,
        /* Please see man pages for details 
         
        */
        SubscriptionEvent = 112,
        /* Please see man pages for details 
         
        */
        GameServerMaintenance = 113,
        /* Please see man pages for details 
         
         
         
        */
        MaintenanceAnnouncement = 114,
        /* Please see man pages for details 
         
        */
        SystemEvent = 115,
        /* Please see man pages for details 
         
                   
        */
        RoundStarted = 116,
        /* Please see man pages for details 
         
                   
        */
        FirstRoundReportReceived = 117,
        /* Please see man pages for details 
         
                   
        */
        RoundSummarized = 118,
        /* Please see man pages for details 
         
        */
        MatchmakeSystemConfigurationNotification = 119,
        /* Please see man pages for details 
         
                   
                   
        */
        MatchmakeSessionSystemPasswordSet = 120,
        /* Please see man pages for details 
         
                   
        */
        MatchmakeSessionSystemPasswordClear = 121,
        /* Please see man pages for details 
         
         
         
         
         
         
         */
        AddedToGathering = 122,
        /* Please see man pages for details 
         
         */
        SharedLocalCacheUpdated = 123,
        /* Please see man pages for details 
         
        */
        FirstUserNotification = 1000
    };
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif
