﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ServiceID_H_
#define _ServiceID_H_

BEGIN_NAMESPACE_NEX


/**
    \ingroup group_rdv_core_common
    \brief Namespace for enum ServiceID.
*/
#ifdef SWIG
class ServiceID
{
public:
#else
namespace ServiceID
{
#endif

    /**
    \ingroup group_rdv_core_common
    \brief Enumerates the unique IDs assigned to each Rendez-Vous lobby service.
    \remarks A ServiceID is a unique ID assigned to each Rendez-Vous service process. ServiceID s
    in the range 0 to 99 are reserved for Quazal processes.

    \note In order to access these types you must include the RendezVous.h file.
    */
    enum ServiceID
    {

        /** \internal */
        Core                 = 0,
        Authentication       = 1,
        SecureConnection     = 2,
        NATTraversal         = 3,

        /*
        The ServiceID of the Back-end Management Service. This service is only accessed by Administrators and not
        by ordinary lobby clients.
        */
        BackEndManagement   = 4,
        /** \internal */
        CoreTestingFramework = 50,

        /** \internal */
        SandboxManagement    = 5,
        /** \internal */
        Clustering           = 6,
        /** \internal */
        Siege                = 7, // TODO:R17SP1 seige is now a package so there should be no need of a service ID anymore, we should be able to remove this after running the tests...
        Health               = 8,

        Friends              = 10,
        Messaging            = 11,
        MatchMaking          = 12,
        PersistentStore      = 13,
        AccountManagement    = 14,
        /*
        The ServiceID of the Competition Service.
        */
        Competition          = 15,
        /*
        The ServiceID of the Internet Servers Service.
        */
        InternetServers      = 16,
        Peer                 = 17,

        ZZZAccountManagement = 18,
        SubAccountManagement = 29,

        /**
        The highest valid ServiceID.
        */
        HighestServiceID    = 99,


        // Nintendo defined services
        NintendoFriendPresence = 101,

        //n2494
        NintendoUtility        = 110,
        //NintendoStorage        = 111,
        NintendoRanking        = 112,
        NintendoMessaging      = 113,
        NintendoReporting      = 114,
        //n2852
        NintendoDataStore      = 115,
        NintendoDebug          = 116,
        NintendoSubscription   = 117,
        NintendoRating         = 118,
        NintendoServiceItem    = 119,
        NintendoMatchmakeReferee  = 120,
        NintendoSubscriber     = 121
    };

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif
