﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _ProtocolRequestBroker_H_
#define _ProtocolRequestBroker_H_

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>
#include <RendezVous/Core/src/Common/Protocol.h>
#include <OnlineCore/src/Core/OperationManager.h>
#include <OnlineCore/src/Core/SecurityContextManager.h>
#include <OnlineCore/src/Platform/Core/BandwidthCounter.h>
#include <RendezVous/Core/src/Common/ProtocolRequestBrokerInterface.h>

BEGIN_NAMESPACE_NEX

class ConnectionOrientedStream;
class Message;


//==============================================================================
// Pack and unpack messages headers, and dispatch messages to the appropriate protocol.
//==============================================================================

// The ProtocolRequestBroker is a class that plays two roles.  It
// serves as a repository of all protocol (the master protocol), it can
// also be closed, where it will clone all the protocols.  An instance
// of the ProtocolRequestBroker will be used by WorkerThread, and this
// is what ensures that WorkerThreads work on independant instances of
// the protocols.

// This class is used both by the client and the server.  When used by
// the client, protocols will not need to be cloned since there are no
// worker threads.

// An exception is when we need to send messages (with the client),
// where

class OSDK_API ProtocolRequestBroker : public ProtocolRequestBrokerInterface
{

public:
    ProtocolRequestBroker(ProtocolRequestBroker* pPrototypeBroker = NULL);
    virtual ~ProtocolRequestBroker();

public:
    EndPoint *GetCurrentEndPoint()
    {
        return m_pCurrentEndPoint;
    }

    void TraceProtocols(qUnsignedInt64 uiTraceFlags);
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS)
    {
        TraceProtocols(uiTraceFlags);
    }

    virtual qBool RegisterProtocol(Protocol *pProtocol);
    virtual void UnregisterProtocol(Protocol *pProtocol);
    virtual Protocol *GetProtocol(ProtocolID uiProtocolID, Protocol::Type eType);

    virtual qBool RegisterServerProtocol(ProtocolID, ServerProtocol*);
    virtual qBool UnregisterServerProtocol(ProtocolID);
    virtual ServerProtocol *GetServerProtocol(ProtocolID);

    qBool IsMaster() const
    {
        return m_pPrototypeBroker==NULL;
    }

    virtual qBool CloneProtocols();

    static qBool SystemStateIsValid(qBool bBlockingCall);

public:
    CallProtocolMethodOperation* GetCurrentOperation();

    qResult DispatchLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint);
    qResult DispatchBlockingLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint);
    qResult DispatchThreadSafeLocalRMC(Message* pmsgRequest, ClientProtocol *pCaller, ServerProtocol *pCallee, EndPoint *pSourceEndPoint);

    // Called by the JobProcessProtocolMessage.
    virtual void ProcessMessage(CallProtocolMethodOperation* pOp, EndPoint *pEndPoint, Buffer *pBuffer);
    virtual void ProcessCallRequest(ProtocolID uiProtocolID, Message* pMessageReceived, Message* pmsgResponse, qBool *pbReturnsResponse, EndPoint *pEndPoint);
    virtual void ProcessCallResponse(ProtocolID uiProtocolID, Message* pMessageReceived, EndPoint *pEndPoint);

    // TODO - Should eventaully be merged.
    void ProcessLocalCallRequest(ServerProtocol* pProtocol, Message* pMessageReceived, Message* pmsgResponse, qBool *pbReturnsResponse, EndPoint *pEndPoint);

private:
    void DispatchCallToProtocol(ServerProtocol* pProtocol, Message* pMessage, Message* pResponse,
                                qBool* pbReturnsMessage, EndPoint* pEndPoint);
    void ProcessMessageCore(CallProtocolMethodOperation* pOp, EndPoint *pEndPoint, Buffer *pBuffer);

public:
    virtual void PeerDisconnected(EndPoint *pEndPoint);
    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault);

private:
    // Block default copy construction and assignement
    ProtocolRequestBroker(const ProtocolRequestBroker &);
    ProtocolRequestBroker & operator=(const ProtocolRequestBroker &);

private:
    typedef qMap<ProtocolID, Protocol*> ProtocolMap;
    ProtocolMap& GetProtocolMap(Protocol::Type eType)
    {
        return (eType == Protocol::Server)?m_mServerProtocols:m_mClientProtocols;
    }
    OSDK_DISABLE_EXPORT_WARNING(ProtocolMap m_mClientProtocols);
    OSDK_DISABLE_EXPORT_WARNING(ProtocolMap m_mServerProtocols);
    EndPoint *m_pCurrentEndPoint;

    ProtocolRequestBroker* m_pPrototypeBroker;
    SingleThreadCallPolicy m_oCallPolicy;
};

END_NAMESPACE_NEX

#endif

