﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _StreamManager_H_
#define _StreamManager_H_

// A RendezVousStream encapsulates a Stream, manages its connections to
// it via a ConnectionManager and forwards its messages to a or many
// ProtocolRequestBroker.

// There are several kinds of RendezVousStreams: the
// RendezVousClientStream, a RendezVousAuthenticationStream and a
// RendezVousSecureStream.

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/SystemSetting.h>
#include <OnlineCore/src/Transport/Interface/ConnectionManager.h>
#include <OnlineCore/src/Transport/Interface/Network.h>
#include <OnlineCore/src/Transport/Interface/EndPointEventHandler.h>

BEGIN_NAMESPACE_NEX

class ConnectionOrientedStream;
class PRUDPStream;
class ProtocolRequestBrokerInterface;
class Credentials;

class OSDK_API StreamManager: public EndPointEventHandler
{
public:
#ifndef SWIG
    StreamManager(Stream::Type oStreamType, RootTransport* pTransport);
    virtual ~StreamManager();

    typedef enum _Flags
    {
        AcceptIncomingConnections = (1<<0),
        MustUseSpecifiedPort = (1<<1)
    } Flags;

    qBool Initialize(qUnsignedInt16 uiPort, Stream::ID idStream, qUnsignedInt32 uiFlags);

    qBool InitializeAsService(qUnsignedInt16 uiPort)
    {
        return Initialize(uiPort, StreamID::WellKnown, AcceptIncomingConnections | MustUseSpecifiedPort);
    }

    void Terminate();
    qBool IsTerminated() const;

    qBool IsInitialized() const
    {
        return m_bInitialized;
    }

    ProtocolRequestBrokerInterface* GetProtocolRequestBroker()
    {
        return m_pBroker;
    }
    const ProtocolRequestBrokerInterface* GetProtocolRequestBroker() const
    {
        return m_pBroker;
    }

    ConnectionManager* GetConnectionManager()
    {
        return m_pConnectionManager;
    }
    const ConnectionManager* GetConnectionManager() const
    {
        return m_pConnectionManager;
    }

    PRUDPStream *GetPRUDPStream()
    {
        return m_pStream;
    }
    const PRUDPStream *GetPRUDPStream() const
    {
        return m_pStream;
    }

    ConnectionOrientedStream *GetSecureStream()
    {
        return m_pSecureStream;
    }
    void AssociateSecureStream(ConnectionOrientedStream *pTransport, ConnectionManager::ConnectionFilter pfConnectionFilter);
    void DeleteAssociatedSecureStream();
    void ClearAssociatedSecureStream();

    static void SetSandboxAccessKey(const String& strSandboxAccessKey);
    static String GetSandboxAccessKey();

    static void SetProtocolVersion(qUnsignedInt32 uiProtocolVersion)
    {
        s_uiProtocolVersion = uiProtocolVersion;
    }
#endif

    void SetMaxSilenceTimeout(qUnsignedInt32 uiTimeout);
    void SetKeepAliveTimeout(qUnsignedInt32 uiTimeout);
    void EmulateSymmetricNAT();
    qBool SimulateFault(PrincipalID pid);

    // used for test purposes
    static void SetPacketDropProbability(qFloat fProbability)
    {
        s_fPacketDropProbability = fProbability;
    }

    virtual void SetCredentials(Credentials *) {};

private:

    void SetStreamKey();

    virtual void Receive(EndPoint *pEndPoint, Buffer *pBuffer, qByte subStreamID)=0;
    virtual void PeerDisconnected(EndPoint *pEndPoint)=0;
    virtual void FaultDetection(EndPoint *pEndPoint, qUnsignedInt32 uiFault)=0;

protected:
    void SetProtocolRequestBroker(ProtocolRequestBrokerInterface* pBroker)
    {
        m_pBroker = pBroker;
    }

private:
    PRUDPStream *m_pStream;

    ConnectionOrientedStream *m_pSecureStream;

    ProtocolRequestBrokerInterface *m_pBroker;
    ConnectionManager *m_pConnectionManager;
    qBool m_bInitialized;

    //  Char array instead of String, to prevent memory leak
    enum { SANDBOX_ACCESS_KEY_SIZE = 128 };
    static qChar s_szSandboxAccessKey[SANDBOX_ACCESS_KEY_SIZE];

    static qUnsignedInt32 s_uiProtocolVersion;

    static qFloat s_fPacketDropProbability;

    RootTransport* m_pTransport;
    Stream::Type m_oStreamType;

    qUnsignedInt16 m_uiPort;

    static SystemSetting s_ssUseDistinctPort;
};

END_NAMESPACE_NEX

#endif

