﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.
  Copyright (C)2011-2014 Nintendo  All rights reserved.
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------

*/

#ifndef _DATA_STORE_DATA_ID_TRAITS_H
#define _DATA_STORE_DATA_ID_TRAITS_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreEventListener.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreClientInterface.h>

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
    
*/
enum DATASTORE_API_VERSION
{
    DATASTORE_API_VERSION_1,
    DATASTORE_API_VERSION_LATEST
};

/* Please see man pages for details 
    
    
*/
template<DATASTORE_API_VERSION>
struct DataStoreApiVersionTraits;

class DataStorePrepareGetParamV1;
class DataStoreReqGetInfoV1;
class DataStorePreparePostParamV1;
class DataStoreReqPostInfoV1;
class DataStoreCompletePostParamV1;
class DataStoreNotificationV1;
class DataStoreGetSpecificMetaParamV1;
class DataStoreSpecificMetaInfoV1;

class DataStorePrepareGetParam;
class DataStoreReqGetInfo;
class DataStorePreparePostParam;
class DataStoreReqPostInfo;
class DataStoreCompletePostParam;
class DataStoreNotification;
class DataStoreGetSpecificMetaParam;
class DataStoreSpecificMetaInfo;

/* Please see man pages for details 
    
    
*/
template<>
struct DataStoreApiVersionTraits<DATASTORE_API_VERSION_1>
{
    // Corresponding data ID type
    typedef qUnsignedInt32 TDataId;

    // Corresponding DataStoreReqGetInfo type
    typedef DataStoreReqGetInfoV1 TReqGetInfo;

    // Corresponding PrepareGetParam type
    typedef DataStorePrepareGetParamV1 TPrepareGetParam;

    // Corresponding PreparePostParam type
    typedef DataStorePreparePostParamV1 TPreparePostParam;

    // Corresponding ReqPostInfo type
    typedef DataStoreReqPostInfoV1 TReqPostInfo;

    // Corresponding CompletePostParam type
    typedef DataStoreCompletePostParamV1 TCompletePostParam;

    // Corresponding Notification type
    typedef DataStoreNotificationV1 TNotification;

    // Corresponding GetSpecificMetaParam type
    typedef DataStoreGetSpecificMetaParamV1 TGetSpecificMetaParam;

    // Corresponding SpecificMetaInfo type
    typedef DataStoreSpecificMetaInfoV1 TSpecificMetaInfo;

    // Corresponding GetObjectEventListener type
    typedef DataStoreGetObjectEventListenerTemplate<TDataId> TGetObjectEventListener;

    // Corresponding ReceiveNewArrivedObjectsEventListener type
    typedef DataStoreReceiveNewArrivedObjectsEventListenerTemplate<TDataId> TReceiveNewArrivedObjectsEventListener;

    // Corresponding DataStoreClientInterface::GetObject function pointer type and variable. Used with JobReceiveNewArrivedObjects.
    // Use as a function pointer so that we can change the function name.
    typedef qBool (DataStoreClientInterface::*GetObjectFunction)(ProtocolCallContext *pContext, const TPrepareGetParam& param, TGetObjectEventListener* eventListener);
    static GetObjectFunction pGetObject;
};

/* Please see man pages for details 
    
    
*/
template<>
struct DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST>
{
    // Corresponding data ID type
    typedef qUnsignedInt64 TDataId;

    // Corresponding PrepareGetParam type
    typedef DataStorePrepareGetParam TPrepareGetParam;

    // Corresponding DataStoreReqGetInfo type
    typedef DataStoreReqGetInfo TReqGetInfo;

    // Corresponding PreparePostParam type
    typedef DataStorePreparePostParam TPreparePostParam;

    // Corresponding ReqPostInfo type
    typedef DataStoreReqPostInfo TReqPostInfo;

    // Corresponding CompletePostParam type
    typedef DataStoreCompletePostParam TCompletePostParam;

    // Corresponding Notification type
    typedef DataStoreNotification TNotification;

    // Corresponding GetSpecificMetaParam type
    typedef DataStoreGetSpecificMetaParam TGetSpecificMetaParam;

    // Corresponding SpecificMetaInfo type
    typedef DataStoreSpecificMetaInfo TSpecificMetaInfo;

    // Corresponding GetObjectEventListener type
    typedef DataStoreGetObjectEventListenerTemplate<TDataId> TGetObjectEventListener;

    // Corresponding ReceiveNewArrivedObjectsEventListener type
    typedef DataStoreReceiveNewArrivedObjectsEventListenerTemplate<TDataId> TReceiveNewArrivedObjectsEventListener;

    // Corresponding DataStoreClientInterface::GetObject function pointer type and variable. Used with JobReceiveNewArrivedObjects.
    // Use as a function pointer so that we can change the function name.
    typedef qBool (DataStoreClientInterface::*GetObjectFunction)(ProtocolCallContext *pContext, const TPrepareGetParam& param, TGetObjectEventListener* eventListener);
    static GetObjectFunction pGetObject;
};

END_NAMESPACE_NEX

#endif
