﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef DATASTORE_CLIENT_H__
#define DATASTORE_CLIENT_H__

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreApiVersionTraits.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreClientInterface.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreEventListener.h>
#include <RendezVous/Services/Jugem/DataStore/src/Client/DataStoreLogicServerClient.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreErrors.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreDataCode.h>

BEGIN_NAMESPACE_NEX

#ifdef FOR_DOC_PURPOSE
/* Please see man pages for details 
    
*/
class DataStoreClient : public DataStoreLogicServerClient, public DataStoreClientInterface
#else
template<class TLogicServerClient>
class DataStoreClientTemplate : public TLogicServerClient, public DataStoreClientInterface
#endif  // FOR_DOC_PURPOSE
{
public:

#ifdef FOR_DOC_PURPOSE
    /* Please see man pages for details 
        
    */
    DataStoreClient();
#else
    DataStoreClientTemplate();
#endif  // FOR_DOC_PURPOSE

#ifdef FOR_DOC_PURPOSE
    /* Please see man pages for details 
        
    */
    virtual ~DataStoreClient() {};
#else
    virtual ~DataStoreClientTemplate() {};
#endif  // FOR_DOC_PURPOSE

    /* Please see man pages for details 
        

        

            
            
            
            
            
            
            
            
            

        
        
        

        
        
    */
    virtual qBool GetObject(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, eventListener, NULL, 0, false, NULL);
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            
            
            
            
            

        
        
        

        
        
    */
    virtual qBool GetObjectOrMetaBinary(ProtocolCallContext *pContext, const DataStoreGetParam& param, DataStoreGetObjectEventListener* eventListener)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, eventListener, NULL, 0, true, NULL);
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            
            

        
        
        

        
        
    */
    virtual qBool GetObjectV1(ProtocolCallContext *pContext, const DataStoreGetParamV1& param, DataStoreGetObjectEventListenerV1* eventListener)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_1> >(pContext, param, eventListener, NULL, 0, false, NULL);
    }

    /* Please see man pages for details 
        

        

            
            
            
            
            
            
            
            

        
        
        
        
        

        
        
    */
    virtual qBool GetObject(ProtocolCallContext *pContext, const DataStoreGetParam& param, void* buffer, qUnsignedInt32 bufferSize, qUnsignedInt32* pWrittenSize)
    {
        return GetObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, NULL, static_cast<qUnsignedInt8*>(buffer), bufferSize, false, pWrittenSize);
    }

    /* Please see man pages for details 

        

            
            
            
            
            

        
        
        
        

        
        
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId, DataStorePostObjectEventListener* eventListener)
    {
        return PostObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, dataId, eventListener, NULL, false);
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            

        
        
        
        

        
        
    */
    virtual qBool PostObjectV1(ProtocolCallContext *pContext, const DataStorePostParamV1& param, qUnsignedInt32* dataId, DataStorePostObjectEventListener* eventListener)
    {
        return PostObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_1> >(pContext, param, dataId, eventListener, NULL, false);
    }

    /* Please see man pages for details 

        

            
            
            
            
            

        
        
        
        
        
                                      
                                      

        
        
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId, const void* buffer, qBool useInternalBuffer=true)
    {
        return PostObjectImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, param, dataId, NULL, static_cast<const qUnsignedInt8*>(buffer), useInternalBuffer);
    }

    /* Please see man pages for details 

        

            

        
        
        

        
        
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const DataStorePostParam& param, qUnsignedInt64* dataId)
    {
        return TLogicServerClient::PostMetaBinary(pContext, param, dataId);
    }

    /* Please see man pages for details 

        

            
            
            
            

        
        
        

        
        
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const qUnsignedInt64 dataId, const DataStorePostParam& param)
    {
        return TLogicServerClient::PostMetaBinaryWithDataId(pContext, dataId, param);
    }

    /* Please see man pages for details 

        

            
            
            
            

            
            

        
        
        
        

        
        
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const qVector< std::pair<qUnsignedInt64, DataStorePostParam> >& params, const qBool transactional = true, qVector<qResult>* pResults = NULL)
    {
        return TLogicServerClient::PostMetaBinariesWithDataId(pContext, params, transactional, pResults);
    }

    /* Please see man pages for details 

        

            
            
            
            

           
            
            

        
        
        
        
        

        
        
    */
    virtual qBool PostObject(ProtocolCallContext *pContext, const qVector< qUnsignedInt64 >& dataIds, const DataStorePreparePostParam& param, const qBool transactional = true, qVector<qResult>* pResults = NULL)
    {
        return TLogicServerClient::PostMetaBinaryWithDataIds(pContext, dataIds, param, transactional, pResults);
    }


    /* Please see man pages for details 

        

            
            
            
            
            
            
            

        
        
        

        
        
    */
    virtual qBool UpdateObject(ProtocolCallContext *pContext, const DataStoreUpdateParam& param, DataStorePostObjectEventListener* eventListener)
    {
        return UpdateObjectImpl(pContext, param, eventListener, NULL, false);
    }

    /* Please see man pages for details 

        

            
            
            
            
            
            
            

        
        
        
        
                                      
                                      

        
        
    */
    virtual qBool UpdateObject(ProtocolCallContext *pContext, const DataStoreUpdateParam& param, const void* buffer, qBool useInternalBuffer=true)
    {
        return UpdateObjectImpl(pContext, param, NULL, static_cast<const qUnsignedInt8*>(buffer), useInternalBuffer);
    }

    /* Please see man pages for details 
        

            
            
            
            

        
    */
    void SetHttpThreadPriority(qUnsignedInt32 priority)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_HttpThreadPriority = priority;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetHttpThreadPriority() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_HttpThreadPriority;
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            

        
    */
    void SetUseHttps(qBool useHttps)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_UseHttps = useHttps;
    }

    /* Please see man pages for details 
        
        

        
    */
    qBool GetUseHttps() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_UseHttps;
    }

    /* Please see man pages for details 
        

            
            
            
            
            
            
            

        
    */
    void SetRelayBufferSize(qUnsignedInt32 relayBufferSize)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_RelayBufferSize = relayBufferSize;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetRelayBufferSize() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_RelayBufferSize;
    }

    /* Please see man pages for details 
        

            
             
            
            
            

        
    */
    void SetHttpBufferSize(qUnsignedInt32 httpBufferSize)
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_HttpBufferSize = httpBufferSize;
    }

    /* Please see man pages for details 
        

        
    */
    qUnsignedInt32 GetHttpBufferSize() const
    {
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        return m_HttpBufferSize;
    }

    /* Please see man pages for details 
        

            
            

            

            
            
            
            
            
            
            

        
        
    */
    void SetDataTransferTimeout(qUnsignedInt32 timeoutBytesPerSecond, TimeInterval minimumTimeoutMilliSecond=DataStoreConstants::DEFAULT_DATA_TRANSFER_MINIMUM_TIMEOUT)
    {
        // SystemLock is needed since referenced from Job within the ReceiveNewArrivedObjects function.
        ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
        m_TimeoutBytesPerSecond = timeoutBytesPerSecond;
        m_MinimumTimeoutMilliSecond = minimumTimeoutMilliSecond;
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            
            
            

        
        
        
                           
        
        
                                      
        
        
        
        
        

        
        
    */
    virtual qBool CheckNotification(ProtocolCallContext *pContext,
                                    const String& urlIn,
                                    String* urlOut,
                                    qUnsignedInt64 notificationIdIn,
                                    qUnsignedInt64* notificationIdOut,
                                    qUnsignedInt32 sandboxInstanceIdIn,
                                    qUnsignedInt32* sandboxInstanceIdOut,
                                    qUnsignedInt32 principalId,
                                    qBool* isFailedRetrieveNotificationUrl,
                                    qBool forceRetrieveUrl=false
                                   )
    {
        return CheckNotificationImpl(pContext, urlIn, urlOut, notificationIdIn, notificationIdOut,
                                     sandboxInstanceIdIn, sandboxInstanceIdOut, principalId, isFailedRetrieveNotificationUrl, forceRetrieveUrl);
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            
            
            
            
            
            

        
        
        
        
                    

        
        
        
                                                                
                                                                
    */
    qResult CheckNewArrival(ProtocolCallContext* pContext,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
            qBool* pNeedReceiving
                           )
    {
        return CheckNewArrivalImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, newArrivalCheckToken, receiveNewArrivedObjectsEventListener, pNeedReceiving);
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            
            

        
        
                                         
        
                                            
                                            
                                            
                                            
        
        
                            
                            
                            

        
        
        
                                                                
                                                                
    */
    virtual qResult ReceiveNewArrivedObjects(ProtocolCallContext* pContext,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            qVector<DataStoreGetObjectEventListener*> getObjectEventListeners,
            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
            qBool* hasNext
                                    )
    {
        return ReceiveNewArrivedObjectsImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_LATEST> >(pContext, newArrivalCheckToken, getObjectEventListeners,
                receiveNewArrivedObjectsEventListener, hasNext);
    }

    /* Please see man pages for details 
        
        

            
            
            
            
            
            

        
        
                                         
        
                                            
                                            
                                            
                                            
        
        
                            
                            
                            

        
        
        
                                                                
                                                                
    */
    virtual qResult ReceiveNewArrivedObjectsV1(ProtocolCallContext* pContext,
            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
            qVector<DataStoreGetObjectEventListenerV1*> getObjectEventListeners,
            DataStoreReceiveNewArrivedObjectsEventListenerV1* receiveNewArrivedObjectsEventListener,
            qBool* hasNext
                                      )
    {
        return ReceiveNewArrivedObjectsImpl<DataStoreApiVersionTraits<DATASTORE_API_VERSION_1> >(pContext, newArrivalCheckToken, getObjectEventListeners,
                receiveNewArrivedObjectsEventListener, hasNext);
    }

    /* Please see man pages for details 
        

        

            
            

        
        
        
        

        
        
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return TLogicServerClient::GetRating(pContext, target, pRating, accessPassword);
    }

    /*
        

        
        

            
            

        
        
        
        
        

        
        
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qVector< DataStoreRatingInfoWithSlot > > *pRatings, qVector<qResult>* pResults=NULL, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return TLogicServerClient::GetRating(pContext, dataIds, pRatings, pResults, accessPassword);
    }

    /* Please see man pages for details 
        

        

            
            

        
        
        
        

        
        
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qUnsignedInt64 dataId, qMap<qInt8, DataStoreRatingInfo> *pRatings, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return GetRatingImpl(pContext, dataId, pRatings, accessPassword);
    }

    /* Please see man pages for details 
        

        

            
            
            

        
        
        
        

        
        
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qMap<qInt8, DataStoreRatingInfo> > *pRatings, qVector<qResult>* pResults=NULL)
    {
        return GetRatingImpl(pContext, dataIds, pRatings, pResults);
    }

    /* Please see man pages for details 
        

        

            
            
            
            
            

        
        
        
        
        

        
        
    */
    virtual qBool GetRating(ProtocolCallContext* pContext, const DataStoreRatingTarget& target, DataStoreRatingInfo* pRating, DataStoreRatingLog* pRatingLog, qUnsignedInt64 accessPassword=DataStoreConstants::INVALID_PASSWORD)
    {
        return TLogicServerClient::GetRatingWithLog(pContext, target, pRating, pRatingLog, accessPassword);
    }

    /* Please see man pages for details 
        

        
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, _T("DataStoreClient: (")
               << _T("m_HttpThreadPriority=") << m_HttpThreadPriority
               << _T(", m_UseHttps=") << m_UseHttps
               << _T(", m_TimeoutBytesPerSecond=") << m_TimeoutBytesPerSecond
               << _T(", m_MinimumTimeoutMilliSecond=") << m_MinimumTimeoutMilliSecond
               << _T(", m_RelayBufferSize=") << m_RelayBufferSize
               << _T(", m_HttpBufferSize=") << m_HttpBufferSize
               << _T(")")
              );
    }
#ifdef TEST_ENABLED
    static qBool Test();
#endif
protected:
    qBool SetDependentRvConnection(ProtocolCallContext* pContext) const;
    template<class TVersionTraits>
    qBool GetObjectImpl(ProtocolCallContext *pContext, const typename TVersionTraits::TPrepareGetParam& param,
                        typename TVersionTraits::TGetObjectEventListener* eventListener,
                        qUnsignedInt8* buffer, qUnsignedInt32 bufferSize, qBool getMetaBinary, qUnsignedInt32* pWrittenSize);
    template<class TVersionTraits>
    qBool PostObjectImpl(ProtocolCallContext *pContext, const typename TVersionTraits::TPreparePostParam& param,
                         typename TVersionTraits::TDataId* dataId, DataStorePostObjectEventListener* eventListener,
                         const qUnsignedInt8* buffer, qBool useInternalBuffer);
    qBool UpdateObjectImpl(ProtocolCallContext *pContext, const DataStorePrepareUpdateParam& param,
                           DataStorePostObjectEventListener* eventListener, const qUnsignedInt8* buffer, qBool useInternalBuffer);
    qBool CheckNotificationImpl(ProtocolCallContext *pContext,
                                const String& urlIn,
                                String* urlOut,
                                qUnsignedInt64 notificationIdIn,
                                qUnsignedInt64* notificationIdOut,
                                qUnsignedInt32 sandboxInstanceIdIn,
                                qUnsignedInt32* sandboxInstanceIdOut,
                                qUnsignedInt32 principalId,
                                qBool* isFailedRetrieveNotificationUrl,
                                qBool forceRetrieveUrl=false
                               );
    template<class TVersionTraits>
    qResult CheckNewArrivalImpl(ProtocolCallContext* pContext,
                            const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
                            DataStoreReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
                            qBool* pNeedReceiving
                           );
    template<class TVersionTraits>
    qResult ReceiveNewArrivedObjectsImpl(ProtocolCallContext* pContext,
                                         const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
                                         qVector<typename TVersionTraits::TGetObjectEventListener*> getObjectEventListeners,
                                         typename TVersionTraits::TReceiveNewArrivedObjectsEventListener* receiveNewArrivedObjectsEventListener,
                                         qBool* hasNext
                                        );
    qBool GetRatingImpl(ProtocolCallContext* pContext, const qUnsignedInt64 dataId, qMap<qInt8, DataStoreRatingInfo> *pRatings, qUnsignedInt64 accessPassword);
    qBool GetRatingImpl(ProtocolCallContext* pContext, const qVector<qUnsignedInt64>& dataIds, qVector< qMap<qInt8, DataStoreRatingInfo> > *pRatings, qVector<qResult>* pResults);

    qUnsignedInt32 m_HttpThreadPriority;
    qBool m_UseHttps;
    qByte padding1[3];
    qUnsignedInt32 m_TimeoutBytesPerSecond;     // Timeout setting dependent on speed for data communications with the file server
    TimeInterval m_MinimumTimeoutMilliSecond;   // Minimum timeout time setting for data communications with the file server
    qUnsignedInt32 m_RelayBufferSize;           // Buffer size used in the HTTP communication relay
    qUnsignedInt32 m_HttpBufferSize;            // Buffer size for HTTP communication
};

typedef DataStoreClientTemplate<DataStoreLogicServerClient> DataStoreClient;

END_NAMESPACE_NEX

#endif // DATASTORE_CLIENT_H__
