﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _DATA_STORE_CONSTANTS_H
#define _DATA_STORE_CONSTANTS_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
    
*/
namespace DataStoreConstants
{
/// Invalid date and time. Defined as <tt>DateTime(9999, 12, 31, 23, 59, 59)</tt>.
extern const DateTime& INVALID_DATE_TIME;

/// Expiration set for perpetualized data. Defined as <tt>DateTime(9999, 12, 31, 0, 0, 0)</tt>. Note that this is different from DateTime::Never.
extern const DateTime& PERMANENT_DATE_TIME;

/// Invalid data ID.
static const qUnsignedInt64 INVALID_DATAID = 0;


/// The maximum data code string size. 
static const qUnsignedInt MAX_DATACODE_STR_LEN = 14; // LOG(18,446,744,073,709,551,615, 28) = 13.3

// Invalid data ID. (API for V1)
static const qUnsignedInt32 INVALID_DATAID_V1 = 0;

/// Invalid data type.
static const qUnsignedInt16 INVALID_DATA_TYPE = 0xffff;

/// Default expiration (in days). The default value is 90 days.
static const qUnsignedInt16 DEFAULT_PERIOD = 90;

/// The default value for the priority of HTTP communication threads used for sending/receiving data.
static const qUnsignedInt32 DEFAULT_HTTP_THREAD_PRIORITY = 16;

// Size of token used to check for new arrivals.
static const qUnsignedInt32 NEW_ARRIVAL_CHECK_TOKEN_SIZE = 352;

/// This is the maximum number of evaluation slots.
static const qInt8 RATING_SLOT_MAX = 15;

/// This is the number of evaluation slots.
static const qInt8 NUM_RATING_SLOT = RATING_SLOT_MAX + 1;

// This is the communication timeout value for HPP.
static const qUnsignedInt32 DEFAULT_HPP_TRANSFER_TIMEOUT = 60 * 1000;

/// The timeout setting depending on communication speed when downloading from a storage server. (Bytes per second.) When a 100 KB transmission takes 10 minutes and still does not finish, it times out.
static const TimeInterval DEFAULT_DATA_TRANSFER_TIMEOUT_BYTES_PER_SECOND = 167;

/// The default minimum timeout value when downloading from a storage server. (in milliseconds)
static const qUnsignedInt32 DEFAULT_DATA_TRANSFER_MINIMUM_TIMEOUT = 60 * 1000;

/// The default value for the maximum size when transferring data.
static const qUnsignedInt32 DEFAULT_RELAY_BUFFER_SIZE = 16 * 1024;

/// The default value of the buffer size used for HTTP communication.
static const qUnsignedInt32 DEFAULT_HTTP_BUFFER_SIZE = 32 * 1024;

// Invalid notification ID.
static const qUnsignedInt64 INVALID_NOTIFICATIONID = 0;

// An invalid SandboxInstanceID.
static const qUnsignedInt32 INVALID_SANDBOX_INSTANCEID = 0;

// The default value of the SandboxInstanceID.
static const qUnsignedInt32 INITIAL_SANDBOX_INSTANCEID = 1;

/// Invalid permanent slot ID.
static const qUnsignedInt16 INVALID_PERSISTENCE_SLOT_ID = 0xFFFF;

/// This is the number of permanent slots.
static const qUnsignedInt16 NUM_PERSISTENCE_SLOT = 16;

/// Invalid value for password. This value is never set to a password.
static const qUnsignedInt64 INVALID_PASSWORD = 0;

/// Maximum number of data IDs that can be specified at once for batch processing functions.
static const qUnsignedInt32 BATCH_PROCESSING_CAPACITY = 100;

/// Maximum number of data IDs that can be specified at once in batch POST processing.
static const qUnsignedInt32 BATCH_PROCESSING_CAPACITY_POST_OBJECT = 16;

/// Maximum number of search results that can be obtained at one time.
static const qUnsignedInt32 MAX_SEARCH_RESULT_SIZE = 100;

/// Maximum number of search results that can be retrieved at once with random searches using `RESULTRANGE_ANY_OFFSET`.
static const qUnsignedInt32 MAX_SEARCH_ANY_RESULT_SIZE = 20;

/// Number of tag slots.
static const qUnsignedInt32 NUM_TAG_SLOT = 16;

/// Maximum number of characters that can be specified in tags.
static const qUnsignedInt32 MAX_TAG_LENGTH = 24;

/// Maximum number of characters that can be specified in titles (summary info).
static const qUnsignedInt32 MAX_NAME_LENGTH = 64;

/* Please see man pages for details 
    
*/
enum Permission
{
    PERMISSION_PUBLIC           = 0, //
    PERMISSION_FRIEND           = 1, //
    PERMISSION_SPECIFIED        = 2, //
    PERMISSION_PRIVATE          = 3, //
    PERMISSION_SPECIFIED_FRIEND = 4  //
};

/* Please see man pages for details 
    
*/
enum DataFlag
{
    DATA_FLAG_NONE                      = 0x0000, //
    DATA_FLAG_NEED_REVIEW               = 0x0001, //
    DATA_FLAG_PERIOD_FROM_LAST_REFERRED = 0x0002, //
    DATA_FLAG_USE_READ_LOCK             = 0x0004, //
    DATA_FLAG_USE_NOTIFICATION_ON_POST  = 0x0008, //
    DATA_FLAG_USE_NOTIFICATION_ON_UPDATE= 0x0010, //
    DATA_FLAG_NOT_USE_FILESERVER        = 0x0020, //
    DATA_FLAG_NEED_COMPLETION           = 0x0040  //
};

/* Please see man pages for details 
    
*/
enum ModificationFlag
{
    MODIFICATION_FLAG_NONE              = 0x0000, //
    MODIFICATION_FLAG_NAME              = 0x0001, //
    MODIFICATION_FLAG_ACCESS_PERMISSION = 0x0002, //
    MODIFICATION_FLAG_UPDATE_PERMISSION = 0x0004, //
    MODIFICATION_FLAG_DELETE_PERMISSION = MODIFICATION_FLAG_UPDATE_PERMISSION, //
    MODIFICATION_FLAG_PERIOD            = 0x0008, //
    MODIFICATION_FLAG_METABINARY        = 0x0010, //
    MODIFICATION_FLAG_TAGS              = 0x0020, //
    MODIFICATION_FLAG_UPDATED_TIME      = 0x0040, //
    MODIFICATION_FLAG_DATA_TYPE         = 0x0080, //
    MODIFICATION_FLAG_REFERRED_CNT      = 0x0100, //
    MODIFICATION_FLAG_STATUS            = 0x0200  //
};

/* Please see man pages for details 
    
*/
enum ComparisonFlag
{
    COMPARISON_FLAG_NONE              = 0x0000, //
    COMPARISON_FLAG_NAME              = 0x0001, //
    COMPARISON_FLAG_ACCESS_PERMISSION = 0x0002, //
    COMPARISON_FLAG_UPDATE_PERMISSION = 0x0004, //
    COMPARISON_FLAG_PERIOD            = 0x0008, //
    COMPARISON_FLAG_METABINARY        = 0x0010, //
    COMPARISON_FLAG_TAGS              = 0x0020, //
    COMPARISON_FLAG_DATA_TYPE         = 0x0040, //
    COMPARISON_FLAG_REFERRED_CNT      = 0x0080, //
    COMPARISON_FLAG_STATUS            = 0x0100, //
    COMPARISON_FLAG_ALL               = 0xFFFF  //
};

/* Please see man pages for details 
    
*/
enum SearchType
{
    SEARCH_TYPE_PUBLIC                =  1, //
    SEARCH_TYPE_SEND_FRIEND           =  2, //
    SEARCH_TYPE_SEND_SPECIFIED        =  3, //
    SEARCH_TYPE_SEND_SPECIFIED_FRIEND =  4, //
    SEARCH_TYPE_SEND                  =  5, //
    SEARCH_TYPE_FRIEND                =  6, //
    SEARCH_TYPE_RECEIVED_SPECIFIED    =  7, //
    SEARCH_TYPE_RECEIVED              =  8, //
    SEARCH_TYPE_PRIVATE               =  9, //
    SEARCH_TYPE_OWN                   = 10, //
#ifndef FOR_DOC_PURPOSE
    SEARCH_TYPE_PUBLIC_EXCLUDE_FRIEND = 11, // Data was uploaded by someone other than the local user or the user's friends, and the access permission is <tt>DataStoreConstants::PERMISSION_PUBLIC</tt>.
#endif  // FOR_DOC_PURPOSE
    SEARCH_TYPE_OWN_PENDING           = 12, //
    SEARCH_TYPE_OWN_REJECTED          = 13, //
    SEARCH_TYPE_OWN_ALL               = 14  //
};

/* Please see man pages for details 
    
*/
enum SearchTarget
{
    SEARCH_TARGET_ANYBODY               = 0,    //
    SEARCH_TARGET_FRIEND                = 1,    //
#ifndef FOR_DOC_PURPOSE
    SEARCH_TARGET_EXCLUDE_SPECIFIED_IDS = 2     // Exclude data with the specified IDs from searches.
#endif  // FOR_DOC_PURPOSE
};

/* Please see man pages for details 
    
*/
enum SearchSortColumn
{
    SEARCH_SORT_COLUMN_DATAID           = 0, //
#ifndef FOR_DOC_PURPOSE
    SEARCH_SORT_COLUMN_SIZE_            = 1, // Size. Not available for use.
    SEARCH_SORT_COLUMN_NAME_            = 2, // Title. Not available for use.
    SEARCH_SORT_COLUMN_DATA_TYPE_       = 3, // Data type. Not available for use.
    SEARCH_SORT_COLUMN_REFERRED_COUNT_  = 4, // The number of times accessed. Not available for use.
#endif  // FOR_DOC_PURPOSE
    SEARCH_SORT_COLUMN_CREATED_TIME     = 5, //
    SEARCH_SORT_COLUMN_UPDATED_TIME     = 6, //

    SEARCH_SORT_COLUMN_RATING0          = 0x40, //
    SEARCH_SORT_COLUMN_RATING1          = 0x41, //
    SEARCH_SORT_COLUMN_RATING2          = 0x42, //
    SEARCH_SORT_COLUMN_RATING3          = 0x43, //
    SEARCH_SORT_COLUMN_RATING4          = 0x44, //
    SEARCH_SORT_COLUMN_RATING5          = 0x45, //
    SEARCH_SORT_COLUMN_RATING6          = 0x46, //
    SEARCH_SORT_COLUMN_RATING7          = 0x47, //
    SEARCH_SORT_COLUMN_RATING8          = 0x48, //
    SEARCH_SORT_COLUMN_RATING9          = 0x49, //
    SEARCH_SORT_COLUMN_RATING10         = 0x4a, //
    SEARCH_SORT_COLUMN_RATING11         = 0x4b, //
    SEARCH_SORT_COLUMN_RATING12         = 0x4c, //
    SEARCH_SORT_COLUMN_RATING13         = 0x4d, //
    SEARCH_SORT_COLUMN_RATING14         = 0x4e, //
    SEARCH_SORT_COLUMN_RATING15         = 0x4f, //

    SEARCH_SORT_COLUMN_RATING_AVERAGE0  = 0x60, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE1  = 0x61, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE2  = 0x62, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE3  = 0x63, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE4  = 0x64, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE5  = 0x65, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE6  = 0x66, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE7  = 0x67, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE8  = 0x68, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE9  = 0x69, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE10 = 0x6a, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE11 = 0x6b, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE12 = 0x6c, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE13 = 0x6d, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE14 = 0x6e, //
    SEARCH_SORT_COLUMN_RATING_AVERAGE15 = 0x6f  //
};

/* Please see man pages for details 
    
*/
enum SearchSortOrder
{
    SEARCH_SORT_ORDER_ASC    = 0,    //
    SEARCH_SORT_ORDER_DESC   = 1     //
};

/* Please see man pages for details 
    
*/
enum ResultFlag
{
    RESULT_FLAG_TAGS            = 0x0001, //
    RESULT_FLAG_RATINGS         = 0x0002, //
    RESULT_FLAG_METABINARY      = 0x0004, //
    RESULT_FLAG_PERMITTED_IDS   = 0x0008  //
};


/* Please see man pages for details 
    
*/
enum RatingLockType
{
    RATING_LOCK_NONE      = 0,   //
    RATING_LOCK_INTERVAL  = 1,   //
    RATING_LOCK_PERIOD    = 2,   //
    RATING_LOCK_PERMANENT = 3    //
};

/* Please see man pages for details 
    

        

*/
enum RatingFlag
{
    RATING_FLAG_MODIFIABLE          = 0x04,   //
    RATING_FLAG_ROUND_MINUS         = 0x08,   //
    RATING_FLAG_DISABLE_SELF_RATING = 0x10    //
};

/* Please see man pages for details 
    
    
*/
enum RatingInternalFlag
{
    RATING_INTERNAL_FLAG_USE_RANGE_MIN       = 0x02,   //
    RATING_INTERNAL_FLAG_USE_RANGE_MAX       = 0x04    //
};

/* Please see man pages for details 
    
*/
enum RatingLockPeriod
{
    RATING_LOCK_PERIOD_MON  = -0x01,    //
    RATING_LOCK_PERIOD_TUE  = -0x02,    //
    RATING_LOCK_PERIOD_WED  = -0x03,    //
    RATING_LOCK_PERIOD_THU  = -0x04,    //
    RATING_LOCK_PERIOD_FRI  = -0x05,    //
    RATING_LOCK_PERIOD_SAT  = -0x06,    //
    RATING_LOCK_PERIOD_SUN  = -0x07,    //
    RATING_LOCK_PERIOD_DAY1 = -0x11     //
};

/* Please see man pages for details 
    

        

*/
enum SearchResultTotalCountType
{
    SEARCH_RESULT_TOTAL_EXACT    = 0x00,    //
    SEARCH_RESULT_TOTAL_MINIMUM  = 0x01,    //
    SEARCH_RESULT_TOTAL_ESTIMATE = 0x02     //
};

/* Please see man pages for details 
    

        
        
        
        

*/
enum DataStatus
{
    DATA_STATUS_NONE       = 0,    //
    DATA_STATUS_PENDING    = 2,    //
    DATA_STATUS_REJECTED   = 5     //
};

}

END_NAMESPACE_NEX

#endif
