﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __MatchmakeConstants_h__
#define __MatchmakeConstants_h__

BEGIN_NAMESPACE_NEX

/// The number of attributes that can be specified in a matchmaking session
static const qUnsignedInt32 NUM_MATCHMAKE_SESSION_ATTRIBUTES = 6;

/// Index of base attribute value when selecting matchmaking session most closely matching conditions.
static const qUnsignedInt32 MATCHMAKE_SESSION_NEAREST_NEIGHBOR_ATTRIBUTE_INDEX = 1;

/// Index of base attribute value when selecting matchmaking session with range broadening.
static const qUnsignedInt32 MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX = MATCHMAKE_SESSION_NEAREST_NEIGHBOR_ATTRIBUTE_INDEX;

//Invalid persistent gathering code.
static const qUnsignedInt64 INVALID_PERSISTENT_GATHERING_CODE = 0;

//Upper limit of persistent gathering registrations per person.
static const qUnsignedInt32 PERSISTENT_GATHERING_CREATION_MAX = 4;

//Upper limit of persistent gathering participations per person.
static const qUnsignedInt32 PERSISTENT_GATHERING_PARTICIPATION_MAX = 16;

//Maximum number of chat participants in a persistent gathering. The maximum number of participants must be set at or below this number.
static const qUnsignedInt64 PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX = 32;

#if defined(CTR_TARGET) || !defined(FOR_DOC_PURPOSE)
/// Invalid community code
static const qUnsignedInt64 INVALID_COMMUNITY_CODE = INVALID_PERSISTENT_GATHERING_CODE;

/// Upper limit for community registrations for a single person
static const qUnsignedInt32 COMMUNITY_CREATION_MAX = PERSISTENT_GATHERING_CREATION_MAX;

/// Upper limit for community participation for a single person
static const qUnsignedInt32 COMMUNITY_PARTICIPATION_MAX = PERSISTENT_GATHERING_PARTICIPATION_MAX;

//Maximum number of chat participants in a community. The maximum number of participants must be set at or below this number.
static const qUnsignedInt64 COMMUNITY_CHAT_PARTICIPANTS_MAX = PERSISTENT_GATHERING_CHAT_PARTICIPANTS_MAX;
#endif

/// Maximum value of the number of attribute value elements when searching matchmaking sessions for multiple attribute values.
static const qUnsignedInt32 SETATTIBUTE_VECTOR_SIZE_MAX = 100;

/// Maximum game progress score.
static const qUnsignedInt8 MAX_PROGRESS_SCORE = 100;

/// Minimum update interval (in seconds) for game progress score.
static const qUnsignedInt32 UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME = 30;

// Maximum user password length for a matchmaking session.
static const qUnsignedInt32 MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH = 32;

// Matchmaking session system password length.
static const qUnsignedInt32 MATCHMAKE_SESSION_SYSTEM_PASSWORD_LENGTH = 16;


extern const qChar* MATCHMAKE_PARAM_KEY_SETTING_INDEX;
extern const qChar* MATCHMAKE_PARAM_KEY_NINTENDO_COUNTRY_CODE;
extern const qChar* MATCHMAKE_PARAM_KEY_OVERRIDE_IP_ADDRESS;
extern const qChar* MATCHMAKE_PARAM_KEY_USE_GEO_IP;
extern const qChar* MATCHMAKE_PARAM_KEY_COUNTRY_CODE;
extern const qChar* MATCHMAKE_PARAM_KEY_RATING_VALUE;
extern const qChar* MATCHMAKE_PARAM_KEY_DISCONNECTION_RATE;
extern const qChar* MATCHMAKE_PARAM_KEY_VIOLATION_RATE;
extern const qChar* MATCHMAKE_PARAM_KEY_LATITUDE;
extern const qChar* MATCHMAKE_PARAM_KEY_LONGITUDE;
extern const qChar* MATCHMAKE_PARAM_KEY_DISTANCE;
extern const qChar* MATCHMAKE_PARAM_KEY_TOTAL_SCORE;
extern const qChar* MATCHMAKE_PARAM_KEY_GEO_IP_RESULT;
extern const qChar* MATCHMAKE_PARAM_KEY_UPDATE_GEO_IP;
extern const qChar* MATCHMAKE_PARAM_KEY_SERVER_RETURNED;
extern const qChar* MATCHMAKE_PARAM_KEY_LAST_GID_FOR_PARTICIPATION_CHECK;

/* Please see man pages for details 
 

 
 */
typedef enum
{
    MATCHMAKE_SYSTEM_TYPE_INVALID,         //
    MATCHMAKE_SYSTEM_TYPE_ANYBODY,         //
    MATCHMAKE_SYSTEM_TYPE_FRIEND,          //
    MATCHMAKE_SYSTEM_TYPE_FRIEND_INVITE,   //
    MATCHMAKE_SYSTEM_TYPE_ANYBODY_INVITE,  //
    // Shift from Community -> PersistentGathering. Same as community match.
    MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING, //
    // If CTR (includes generating documents) || If not CTR, when not generating documents.
#if defined(CTR_TARGET) || !defined(FOR_DOC_PURPOSE)
    MATCHMAKE_SYSTEM_TYPE_COMMUNITY = MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING, //
#endif
    // If Cafe (includes generating documents) || If not Cafe, when not generating documents.
#if defined(CAFE_TARGET) || !defined(FOR_DOC_PURPOSE)
    MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY,   //
#endif
    MATCHMAKE_SYSTEM_TYPE_END              //
}
MatchmakeSystemType;

/* Please see man pages for details 
 

       
       
       
       
       
       

       

       
       
       
       
       
       
       

       
       
       
       
       
       
       
       
       
       
       
       
       
       
       
       
       
       

       
       
       
       
       
       
       
       
       
       
       
       

       
       
       
       
       
       
       
       
       
       

       
       
       
       
       
       

       
       
       
       
       
       
       
       

       
       
       
       
       
       
       
       
       
       
       

*/
typedef enum
{
    MATCHMAKE_SELECTION_METHOD_RANDOM = 0,        //
    MATCHMAKE_SELECTION_METHOD_NEAREST_NEIGHBOR,  //
    MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE,     //
    MATCHMAKE_SELECTION_METHOD_PROGRESS_SCORE,    //
    MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE,  //
    MATCHMAKE_SELECTION_METHOD_SCORE_BASED        //
}
MatchmakeSelectionMethod;

/* Please see man pages for details 
 
*/
typedef enum
{
    MATCHMAKE_GEO_IP_RESULT_INVALID = 0,    //
    MATCHMAKE_GEO_IP_RESULT_FOUND,          //
    MATCHMAKE_GEO_IP_RESULT_NOT_FOUND,      //
    MATCHMAKE_GEO_IP_RESULT_UNUSED          //
}
MATCHMAKE_GEO_IP_RESULT;


// CreateMatchmakeSessionOption, JoinMatchmakeSessionOption, and AutoMatchmakeOption look at the values defined in MatchmakeOption and use the same values for items with the same meaning.
/* Please see man pages for details 
 
 
*/
enum MatchmakeOption
{
    MATCHMAKE_OPTION_NONE                           = 0x00000000, //
    MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = 0x00000001, //
    MATCHMAKE_OPTION_DONT_CARE_MY_BLACKLIST         = 0x00000002 //
};

/* Please see man pages for details 
 
 
*/
enum CreateMatchmakeSessionOption
{
    CREATE_MATCHMAKE_SESSION_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //
    CREATE_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK //
};

/* Please see man pages for details 
 
 
*/
enum JoinMatchmakeSessionOption
{
    JOIN_MATCHMAKE_SESSION_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //
    JOIN_MATCHMAKE_SESSION_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK, //
    JOIN_MATCHMAKE_SESSION_OPTION_DONT_CARE_MY_BLACKLIST         = MATCHMAKE_OPTION_DONT_CARE_MY_BLACKLIST //
};

/* Please see man pages for details 
 
 
*/
enum AutoMatchmakeOption
{
    AUTO_MATCHMAKE_OPTION_NONE                           = MATCHMAKE_OPTION_NONE, //
    AUTO_MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK = MATCHMAKE_OPTION_RECORD_LAST_GID_FOR_PARTICIPATION_CHECK //
};

/* Please see man pages for details 
 
*/
enum JoinMatchmakeSessionBehavior
{
    JOIN_MATCHMAKE_SESSION_BEHAVIOR_JOIN_MYSELF,                      //
    JOIN_MATCHMAKE_SESSION_BEHAVIOR_IM_ALREADY_JOINED                 //
};

/* Please see man pages for details 
 
*/
enum MatchmakeSessionModificationFlag
{
    MATCHMAKE_SESSION_MODIFICATION_FLAG_NONE                        = 0x00000000,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_ATTRIBUTES                  = 0x00000001,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_OPEN_PARTICIPATION          = 0x00000002,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_APPLICATION_BUFFER          = 0x00000004,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_PROGRESS_SCORE              = 0x00000008,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_OPTION0                     = 0x00000010,   //   Option0 (not used).
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM             = 0x00000020,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE    = 0x00000040,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_STARTED_TIME                = 0x00000080,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_USER_PASSWORD               = 0x00000100,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_GAME_MODE                   = 0x00000200,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_DESCRIPTION                 = 0x00000400,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MIN_PARTICIPANTS            = 0x00000800,   //
    MATCHMAKE_SESSION_MODIFICATION_FLAG_MAX_PARTICIPANTS            = 0x00001000    //
};

namespace BuiltInParticipationPolicies
{
static const qUnsignedInt32 FriendOnlyParticipationPolicy = FirstUserPolicy - 2;        // Only friends can join
static const qUnsignedInt32 CommunityParticipationPolicy = FirstUserPolicy - 3;         // Community
static const qUnsignedInt32 MatchmakeSessionParticipationPolicy = FirstUserPolicy - 4;  // Matchmake Session
static const qUnsignedInt32 AnybodyParticipationPolicy          = FirstUserPolicy - 5;  // A matchmake session that anyone can join
}

/* Please see man pages for details 
 
 */
namespace AnybodyParticipationPolicyArgument
{
/* Please see man pages for details 
 
 */
typedef enum
{
    WithoutClose          = 0, //
    CloseOnOwnerMigration = 1, //
    End
}
AnybodyParticipationPolicyArgument;
}

/* Please see man pages for details 
 
 */
namespace FriendOnlyParticipationPolicyArgument
{
/* Please see man pages for details 
 
 */
typedef enum
{
    CloseOnOwnerMigration, //
    WithoutClose,          //
    End
}
FriendOnlyParticipationPolicyArgument;
}

END_NAMESPACE_NEX

#endif  // __MatchmakeConstants_h__
