﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __MatchmakeSession_h__
#define __MatchmakeSession_h__

#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSessionDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeFlags.h>

BEGIN_NAMESPACE_NEX

/* Please see man pages for details 
 
 */
#ifdef FOR_DOC_PURPOSE
class MatchmakeSession: public Gathering
#else
class MatchmakeSession: public DDLCLASS(MatchmakeSession)
#endif
{
public:
    /* Please see man pages for details 
    

       
    */
    MatchmakeSession();
    /* Please see man pages for details 
     
     */
    virtual ~MatchmakeSession() {};

    /* Please see man pages for details 
    

       

    
    */
    void SetGameMode(qUnsignedInt32 gameMode);

    /* Please see man pages for details 
    

       

    
    
    */
    void SetAttribute(qUnsignedInt32 uiIndex, qUnsignedInt32 val);

    /* Please see man pages for details 
    

       

       
       
       
       

       
       
       
       

       

       

       

    
    
    */
    void SetMatchmakeSystemType(MatchmakeSystemType matchmakeSystemType, qUnsignedInt32 policyArgument=0);

    /* Please see man pages for details 
    

       

    
    */
    void SetApplicationBuffer(const qVector<qUnsignedInt8>& applicationBuffer);

    /* Please see man pages for details 
    

       

     
    */
    void SetOpenParticipation(bool openParticipation);

    /* Please see man pages for details 
    

       
    */
    void Reset();

    /* Please see man pages for details 
    

       

    
    */
    qUnsignedInt32 GetGameMode() const
    {
        return m_GameMode;
    }

    /* Please see man pages for details 
    

       

    

    
    */
    qUnsignedInt32 GetAttribute(qUnsignedInt32 uiIndex) const;

    /* Please see man pages for details 
    

       

    
    */
    MatchmakeSystemType GetMatchmakeSystemType() const
    {
        return static_cast<MatchmakeSystemType>(m_MatchmakeSystemType);
    };

    /* Please see man pages for details 
    

       

    
    */
    void GetApplicationBuffer(qVector<qUnsignedInt8>* pApplicationBuffer) const
    {
        SYSTEMCHECK(pApplicationBuffer != NULL);
        pApplicationBuffer->assign(m_ApplicationBuffer.begin(), m_ApplicationBuffer.end());
    };

    /* Please see man pages for details 
    

       

    
    */
    bool GetOpenParticipation() const
    {
        return m_OpenParticipation;
    }

    /* Please see man pages for details 
    

       

    
    */
    qUnsignedInt32 GetParticipantCount() const
    {
        return m_ParticipationCount;
    }

    /* Please see man pages for details 
    

       
       
       
       
       
       

    

    
    */
    const qVector<qByte>& GetSessionKey() const
    {
        return m_SessionKey;
    }

    /* Please see man pages for details 
    

       

    
    
    */
    qBool SetProgressScore(qUnsignedInt8 progressScore);

    /* Please see man pages for details 
    

       

    
    */
    qUnsignedInt8 GetProgressScore() const
    {
        return m_ProgressScore;
    }

    /* Please see man pages for details 
    

        

    
    
    
    */
    void SetOption0Flag(qUnsignedInt32 uiFlag)
    {
        m_Option0 |= uiFlag;
    }

    /* Please see man pages for details 
    

        

    
    
    
    */
    void ClearOption0Flag(qUnsignedInt32 uiFlag)
    {
        m_Option0 &= ~uiFlag;
    }
    
    /* Please see man pages for details 
    

    
    
    
    
    
    */
    qBool GetOption0FlagSet(qUnsignedInt32 uiFlag) const
    {
        return (m_Option0 & uiFlag)!=0;
    }

    /* Please see man pages for details 
    

       
       

    
    */
    void SetMatchmakeParam(const MatchmakeParam& matchmakeParam)
    {
        m_MatchmakeParam = matchmakeParam;
    }

    /* Please see man pages for details 
    

       

    
    */
    const MatchmakeParam& GetMatchmakeParam() const
    {
        return m_MatchmakeParam;
    }

    /* Please see man pages for details 
    

       
       

    
    */
    void SetUserPassword(const String& userPassword)
    {
        m_UserPassword = userPassword;
    }

    /* Please see man pages for details 
    

       
       
       

    
    */
    const String& GetUserPassword() const
    {
        return m_UserPassword;
    }

    /*
    

       
       
       
       
       

    
    */
    void SetStartedTime(const DateTime& startedTime)
    {
        m_StartedTime = startedTime;
    };

    /* Please see man pages for details 
    

       

    
    */
    const DateTime& GetStartedTime() const
    {
        return m_StartedTime;
    }

    /* Please see man pages for details 
    

       

    
                              
                              
                              
                              
                              
    */
    void SetReferPersistentGatheringId(GatheringID referPersistentGatheringId)
    {
        m_ReferGid = referPersistentGatheringId;
    }

    /* Please see man pages for details 
    

       

    
    */
    GatheringID GetReferPersistentGatheringId() const
    {
        return m_ReferGid;
    }

    /* Please see man pages for details 
    

        

    
    */
    qBool GetUserPasswordEnabled() const
    {
        return static_cast<qBool>(m_UserPasswordEnabled);
    }

    /* Please see man pages for details 
    

        

    
    */
    qBool GetSystemPasswordEnabled() const
    {
        return static_cast<qBool>(m_SystemPasswordEnabled);
    }

    /*
      

      
      */
    const MatchmakeSession& GetRef() const
    {
        return *this;
    }

    /*
      

      
      */
    MatchmakeSession& GetRef()
    {
        return *this;
    }
};

END_NAMESPACE_NEX

#endif
