﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __ONLINESDK_RENDEZVOUS_CORE_SRC_COMMON_CONSTANTS_GATHERINGFLAGS_H__
#define __ONLINESDK_RENDEZVOUS_CORE_SRC_COMMON_CONSTANTS_GATHERINGFLAGS_H__

BEGIN_NAMESPACE_NEX

//group=Matchmaking Service

/*
Description: Enumerates the various flags that may be set for a Gathering.
Remarks: GatheringFlags control the persistence of a Gathering, or in other words
when the Gathering will be destroyed. By default a gathering will be destroyed when its
GatheringStates is set to /HasFinished/ or the Gathering owner logs out of the lobby.
If you want a Gathering to persist after either of these two events occurs, you can
set the Persistent flag for a specified gathering via Gathering::SetFlag. You may set
multiple flags for a specified gathering however, the CompetitionEvent flag should only be
set by Rendez-Vous and not by the user.
*/
#ifdef SWIG
class GatheringFlags
{
public:
#else  // Not SWIG
/* Please see man pages for details 
 
*/
namespace GatheringFlags
{
#endif // End of SWIG

    // group=Type Definitions

    //Description: Enumerates the various flags that may be set for a Gathering.
    //Remarks: GatheringFlags control the persistence of a Gathering, or in other words
    //when the Gathering will be destroyed. By default a gathering will be destroyed when its
    //GatheringStates is set to /HasFinished/ or the Gathering owner logs out of the lobby.
    //If you want a Gathering to persist after either of these two events occurs, you can
    //set the Persistent flag for a specified gathering via Gathering::SetFlag. You may set
    //multiple flags for a specified gathering however, the CompetitionEvent flag should only be
    //set by Rendez-Vous and not by the user.

    /* Please see man pages for details 
     

     
     
     
     */
    enum GatheringFlags
    {
        /* Please see man pages for details 
         
         */
        Nothing = 0x00000000,
        /*
        The Gathering is persistent and therefore, it will not automatically be destroyed when
        either the Gathering owner logs out of or disconnects from the lobby or the GatheringStates
        is set to /HasFinished/. A gathering with this flag set will only be destroyed when the Gathering
        is unregistered from the Matchmaking service, via MatchMakingClient::UnregisterGathering.
        */
        /* Please see man pages for details 
         
         */
        Persistent = 0x00000001,
        /*
        The Gathering is a Competition event, which is a sub-gathering of a Competition
        gathering. This flag should not be set by the user as it is automatically set when
        a competition event is registered to a competition via CompetitionClient::RegisterCompetitionEvent.
        A Gathering with this flag set will not be destroyed until after its associated Competition has
        finished all necessary processing regarding the Gathering. For example, a game session gathering
        will only be destroyed once its associated Competition has processed any relevant statistics.
        */
        /* Please see man pages for details 
         
         */
        CompetitionEvent = 0x00000002,
        /*
        All sub-gatherings of the Gathering for which this flag is set are persistent. In other words,
        all sub-gatherings of the gathering will not be destroyed until the parent gathering is destroyed.
        */
        /* Please see man pages for details 
         
         */
        PersistentEvents = 0x00000004,
        /*
        The gathering is private and will therefore, not be found by some FindBy functions. Note that this
        flag is currently not implemented.
        */
        /* Please see man pages for details 
         
         */
        Private = 0x00000008,
        /*
        When the current Gathering owner of a non-persistent gathering disconnects from the lobby, ownership of
        the Gathering will be transferred to one of the other participants, with preference given to participants
        who are active and online. If no participant is found, that is, all participants have left the session, then the
        gathering will be destroyed. If you wish to control to which Principal a Gathering is migrated, then you may
        use the MatchMakingClient::MigrateGatheringOwnership method.
        */
        /* Please see man pages for details 
         
         
         
         */
        MigrateOwner = 0x00000010,
        /* Please see man pages for details 
          
         */
        _UseBlackList = 0x00000020,
        /* Please see man pages for details 
          

                    
         */
        NoPersistentParticipation = 0x00000040,
        /* Please see man pages for details 
          
                    
                    
                    
                    

                    

                    
         */
        // This isn't guaranteed to work for a non-persistent gathering even if explicitly specified.
        AllowNoParticipant = 0x00000080,
        /* Please see man pages for details 
          
                    
                    

                    
         */
        RegistrationByServer = 0x00000100,

        /* Please see man pages for details 
         
         
         
         
         
         
         */
        ChangeOwnerByOtherHost = 0x00000200,

        /* Please see man pages for details 
         
                  

                  
         */
        NotifyParticipationEventsToAllParticipants = 0x00000400,

        /* Please see man pages for details 
         
                  
                  

                  
         */
        NotifyParticipationEventsToAllParticipantsReproducibly = 0x00000800
    };

#ifdef SWIG
};
#else // Not SWIG
}
#endif // End of SWIG

END_NAMESPACE_NEX

#endif  // __ONLINESDK_RENDEZVOUS_CORE_SRC_COMMON_CONSTANTS_GATHERINGFLAGS_H__
