﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef __ONLINESDK_RENDEZVOUS_CORE_SRC_COMMON_CONSTANTS_PARTICIPATIONEVENTS_H__
#define __ONLINESDK_RENDEZVOUS_CORE_SRC_COMMON_CONSTANTS_PARTICIPATIONEVENTS_H__

BEGIN_NAMESPACE_NEX

/*
    \ingroup group_rdv_services_gathering_client
    \brief   This class contains the enumeration of events associated with a change in the
                participation status of a Principal with regard to a given Gathering.
*/
/* Please see man pages for details 
 
 */
#ifdef SWIG
class ParticipationEvents
{
public:
#else
namespace ParticipationEvents
{
#endif
    /*
    \ingroup group_rdv_services_gathering_client
    \brief  Enumerates the events associated with a change in the participation status
            of a Principal with regard to a given Gathering.

    A Participation event is an event that is triggered when the participation status
    of a Principal changes with regard to a given Gathering. In other words, these types of
    events occur whenever a method that changes whether or not a Principal takes part in a
    Gathering completes. The occurence of a ParticipationEvents will trigger a ParticipationEvent
    NotificationEvents. Exactly what happens when a given ParticipationEvent occurs is determined
    by the Participation policy of the Gathering. You cannot modify the built-in Participation events however,
    you may add an almost unlimited number of user-defined Participation events. In order to access
    these types you must include the RendezVous.h file.
    */

    /* Please see man pages for details 
     

     
     
     
     
     
     */
    enum ParticipationEvents
    {
        /*
            A Principal has confirmed that he/she will participate in a Gathering.
            This event will be triggered when MatchMakingClient::Participate or
            MatchMakingClient::AddParticipants completes.
            The target Process of this event is the Gathering owner.
        */
        /* Please see man pages for details 
         
         
         
         */
        Participate     = 1,
        /*
            A Principal has cancelled his/her participation in a Gathering.
            This event will be triggered when MatchMakingClient::CancelParticipation completes or when a participant
            in a Gathering with the OngoingParticipation BuiltInParticipationPolicies disconnects from the gathering.
            The target Process of this event is the Gathering owner.
        */
        /* Please see man pages for details 
         
         */
        CancelParticipation     = 2,
        /*
            A Gathering owner has invited a Principal to the Gathering that it owns.
            This event will be triggered when MatchMakingClient::Invite completes.
            The target Process(es) of this event is the Process(es) that the Gathering owner invited to the Gathering.
        */
        /* Please see man pages for details 
         
         */
        Invite      = 3,
        /*
            A Principal has accepted an invitation to a Gathering.
            This event will be triggered when MatchMakingClient::AcceptInvitation completes.
            The target Process of this event is the Gathering owner.
        */
        /* Please see man pages for details 
         
         */
        AcceptInvitation        = 4,
        /*
            A Principal has declined an invitation to a Gathering.
            This event will be triggered when MatchMakingClient::DeclineInvitation completes.
            The target Process of this event is the Gathering owner.
        */
        /* Please see man pages for details 
         
         */
        DeclineInvitation       = 5,
        /*
            A Gathering owner has cancelled a previously sent invitation for a Principal to join the
            Gathering that it owns.
            This event will be triggered when MatchMakingClient::CancelInvitation completes.
            The target Process of this event is the Process that the Gathering owner invited to the Gathering.
        */
        /* Please see man pages for details 
         
         */
        CancelInvitation        = 6,
        /*
            One of the participants of a Gathering has disconnected from the lobby.
            This event is only triggered if the Participation policy of the Gathering uses the ParticipationStatus database
            table, which is every BuiltInParticipationPolicies except GroupParticipation.
            The target Process of this event is the Gathering owner.
        */
        /* Please see man pages for details 
         
         
         */
        Disconnect = 7,
        /*
            One of the participants of a Gathering has left the gathering session.
            This event will be triggered when MatchMakingClient::EndParticipation completes.
            The target Process of this event is the Gathering owner.
        */
        /* Please see man pages for details 
         
         
         */
        EndParticipation = 8,
        /*
            The ID of the first user-defined Participation event, which by default is equal to 100. Note that the
            ID of a user-defined event cannot be greater than 999.
        */
        /* Please see man pages for details 
         
         */
        FirstUserEvent  = 100
    };
#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif
