﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2014 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#ifndef _BinaryMessage_H
#define _BinaryMessage_H

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Services/Messaging/src/Common/BinaryMessageDDL.h>

BEGIN_NAMESPACE_NEX


#ifdef FOR_DOC_PURPOSE
/*
Description: Construct a message containing essentially a binary body.
See Also: Messaging Service
*/
/* Please see man pages for details 
 

 
 */
class BinaryMessage: public UserMessage
{

#else
class BinaryMessage: public DDLCLASS(BinaryMessage)
{

#endif

public:

    /*
     Description: Default constructor.
     */
    /* Please see man pages for details 
     

     
     */
    BinaryMessage()
    {
    }

    /*
     Description: Assignment constructor.
     Parameters:
     binaryBody: Binary body to attach to message.
     */
    /* Please see man pages for details 
     

     

     
     
     */
    BinaryMessage(const void* pBuffer, qUnsignedInt32 bufferSize)
    {
        SetBuffer(pBuffer, bufferSize);
    }

    /*
     Description: Accessor to set binary body.
     Parameters:
     binaryBody: Binary body to attach to message.
     */
    /* Please see man pages for details 
     
            
            

     
     
     */
    void SetBuffer(const void* pBuffer, qUnsignedInt32 bufferSize)
    {
        SYSTEMCHECK(pBuffer != NULL && _T("BinaryMessage::SetBuffer() pBuffer is NULL"));
        SYSTEMCHECK(bufferSize > 0 && _T("BinaryMessage::SetBuffer() bufferSize is 0"));
        if (!pBuffer || bufferSize == 0)
        {
            // The released version does nothing.
            return;
        }

        m_binaryBody.clear();
        m_binaryBody.push_back(static_cast<const qByte*>(pBuffer), bufferSize);
    }

    /*
     Description: Accessor to get binary body.
     Returns: Message binary body
     */
    /* Please see man pages for details 
     

            

     
     */
    const void* GetBufferPtr() const
    {
        // Branch using empty function because you get caught in SYSTEMCHECK due to front being empty == true.
        if (m_binaryBody.empty())
        {
            return NULL;
        }
        return static_cast<const void*>(&m_binaryBody.front());
    }
    /* Please see man pages for details 
     

            

     
     */
    qUnsignedInt32 GetBufferSize() const
    {
        return m_binaryBody.size();
    }

    /*
     For debug
     */
    virtual void Trace(qUnsignedInt64 traceFlag)
    {
        UserMessage::Trace(traceFlag);
        QTRACE(traceFlag, _T("BinaryMessage:") << this);
        QTRACE(traceFlag, _T("  BinaryBody Size:") << m_binaryBody.size());
        QTRACE(traceFlag, _T("  BinaryBody:"));
        StringStream ss;
        ss << noshowbase; // Do not attach 0x.
        ss << zerohex;    // Insufficient high-order digits will be filled in with 0.
        for (qUnsignedInt i = 0; i < m_binaryBody.size(); i++)
        {
            if (i % 64 == 0)
            {
                if (ss.GetLength() > 0)
                {
                    QTRACE(traceFlag, ss.CStr());
                }
                ss.Clear();
                ss << _T("    ");
            }
            ss << m_binaryBody[i];
        }
        if (ss.GetLength() > 0)
        {
            QTRACE(traceFlag, ss.CStr());
        }
    }
};

END_NAMESPACE_NEX

#endif
