﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2014, Technologies Quazal Inc. All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C) 2011-2014 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

////////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: Values of each element of the following enumeration must be
// specified so that translating this file for Python scripts works as expected.
// Also, the description of elements must use C-style /* */ comments where
// the opening and closing syntax must be on their own lines of code.
////////////////////////////////////////////////////////////////////////////////

#ifndef _RendezVousErrors_H_
#define _RendezVousErrors_H_

BEGIN_NAMESPACE_NEX

/*
    \ingroup group_rdv_errors
    \brief This enumeration defines the error codes that may be returned for the RendezVous facility.
*/
#ifdef SWIG
class RendezVousErrorCode
{
public:
#else
/* Please see man pages for details 
 
 */
namespace RendezVousErrorCode
{
#endif


    /*
        \brief Enumerates the error codes for the RendezVous facility that a call may return.
        \remark Details as to the cause of each ErrorCode are detailed in the documentation of each individual Rendez-Vous method that may return a result.
    */
    /* Please see man pages for details 
     
    */
    enum ErrorCode
    {
        /* Please see man pages for details 
            
        */
        ConnectionFailure                   = 1,    //

        /* Please see man pages for details 
            
        */
        NotAuthenticated                    = 2,    //

        /* Please see man pages for details 
            
        */
        InvalidUsername                     = 100,    //

        /* Please see man pages for details 
            
        */
        InvalidPassword                     = 101,    //

        /* Please see man pages for details 
            
        */
        UsernameAlreadyExists               = 102,    //

        /* Please see man pages for details 
            
        */
        AccountDisabled                     = 103,    //

        /* Please see man pages for details 
            
        */
        AccountExpired                      = 104,    //

        /* Please see man pages for details 
            
            
        */
        ConcurrentLoginDenied               = 105,    //

        /* Please see man pages for details 
            
        */
        EncryptionFailure                   = 106,    //

        /* Please see man pages for details 
            
        */
        InvalidPID                          = 107,    //

        /* Please see man pages for details 
            
        */
        MaxConnectionsReached               = 108,    //

        /* Please see man pages for details 
            
        */
        InvalidGID                          = 109,    //

        /* Please see man pages for details 
            
        */
        InvalidControlScriptID              = 110,    //

        /* Please see man pages for details 
            
        */
        InvalidOperationInLiveEnvironment   = 111,    //

        /* Please see man pages for details 
            
        */
        DuplicateEntry =                    112,    //

        /* Please see man pages for details 
            
        */
        ControlScriptFailure =              113,    //

        /* Please see man pages for details 
            
        */
        ClassNotFound =                     114,    //

        /* Please see man pages for details 
            
        */
        SessionVoid =                       115,    //
        /* Please see man pages for details 
            
        */
        DDLMismatch =                       117,    //

        /* Please see man pages for details 
            
        */
        InvalidConfiguration =              118,    //

        //
        // Nintendo defined under here.
        //

        /*
        The session gathering to join was full.
        */
        SessionFull                 = 200,    //

        /*
        Invalid gathering password.
        */
        InvalidGatheringPassword    = 201,    //

        WithoutParticipationPeriod  = 202,    //

        PersistentGatheringCreationMax  = 203,          //

        PersistentGatheringParticipationMax  = 204,     //

        DeniedByParticipants  = 205,          //

        ParticipantInBlackList  = 206,         //

        GameServerMaintenance  = 207,         //

        OperationPostpone  = 208,     //    Execution of the operation was postponed. (Internal use)

        OutOfRatingRange = 209,    //

        ConnectionDisconnected = 210, //

        InvalidOperation   = 211,    //

        NotParticipatedGathering = 212, //

        MatchmakeSessionUserPasswordUnmatch = 213, //

        MatchmakeSessionSystemPasswordUnmatch = 214, //

        UserIsOffline = 215,   //

        AlreadyParticipatedGathering = 216, //

        PermissionDenied = 217, //

        NotFriend = 218, //

        SessionClosed = 219 //
    };
    //note: When adding new results, it is necessary to change `NEX_QRESULT_TO_STR` in `OnlineCore/src/Platform/Core/Result.cpp`.
    //See operations done when adding error codes to the wiki.
    //If the errors the application should handle have increased, add them to the list of errors the application should handle in the`overall.rst` of the `common manual.`

#ifdef SWIG
};
#else
}
#endif

END_NAMESPACE_NEX

#endif // _RendezVousErrors_H_
